@@ -1,11 +1,11 @@
 /*
- * analysis.c
+ * Copyright (C) 2000-2004 Joern Thyssen <joern@thyssen.nu>
+ * Copyright (C) 2000-2019 the AUTHORS
  *
- * by Joern Thyssen <joern@thyssen.nu>, 2000
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: analysis.c,v 1.248 2018/04/21 20:12:08 plm Exp $
+ * $Id: analysis.c,v 1.260 2022/02/15 21:42:12 plm Exp $
  */
 
 #include "config.h"
@@ -106,7 +105,7 @@ LuckFirst(const TanBoard anBoard, const int n0, const int n1, cubeinfo * pci, co
             /* Find the best move for each roll at ply 0 only. */
             if (FindnSaveBestMoves(&ml, i + 1, j + 1, (ConstTanBoard) anBoardTemp, NULL, 0.0f,
                                    pci, pec, defaultFilters) < 0) {
-                free(ml.amMoves);
+                g_free(ml.amMoves);
                 return ERR_VAL;
             }
 
@@ -127,7 +126,7 @@ LuckFirst(const TanBoard anBoard, const int n0, const int n1, cubeinfo * pci, co
 
             } else {
                 aar[i][j] = ml.amMoves[0].rScore;
-                free(ml.amMoves);
+                g_free(ml.amMoves);
             }
 
             rMean += aar[i][j];
@@ -144,7 +143,7 @@ LuckFirst(const TanBoard anBoard, const int n0, const int n1, cubeinfo * pci, co
             /* Find the best move for each roll at ply 0 only. */
             if (FindnSaveBestMoves(&ml, i + 1, j + 1, (ConstTanBoard) anBoardTemp, NULL, 0.0f,
                                    &ciOpp, pec, defaultFilters) < 0) {
-                free(ml.amMoves);
+                g_free(ml.amMoves);
                 return ERR_VAL;
             }
 
@@ -165,7 +164,7 @@ LuckFirst(const TanBoard anBoard, const int n0, const int n1, cubeinfo * pci, co
 
             } else {
                 aar[i][j] = -ml.amMoves[0].rScore;
-                free(ml.amMoves);
+                g_free(ml.amMoves);
             }
 
             rMean += aar[i][j];
@@ -199,7 +198,7 @@ LuckNormal(const TanBoard anBoard, const int n0, const int n1, const cubeinfo *
             /* Find the best move for each roll at ply 0 only. */
             if (FindnSaveBestMoves(&ml, i + 1, j + 1, (ConstTanBoard) anBoardTemp, NULL, 0.0f,
                                    pci, pec, defaultFilters) < 0) {
-                free(ml.amMoves);
+                g_free(ml.amMoves);
                 return ERR_VAL;
             }
 
@@ -220,7 +219,7 @@ LuckNormal(const TanBoard anBoard, const int n0, const int n1, const cubeinfo *
 
             } else {
                 aar[i][j] = ml.amMoves[0].rScore;
-                free(ml.amMoves);
+                g_free(ml.amMoves);
             }
 
             rMean += (i == j) ? aar[i][j] : aar[i][j] * 2.0f;
@@ -298,7 +297,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
 {
     cubeinfo ci;
     static positionkey key;
-    float rSkill, rChequerSkill, rCost;
+    float rSkill, rCost;
     unsigned int i;
     float arDouble[4];
     const xmovegameinfo *pmgi = &((moverecord *) plGame->plNext->p)->g;
@@ -361,7 +360,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
 
                 rSkill = arDouble[OUTPUT_NODOUBLE] - arDouble[OUTPUT_OPTIMAL];
 
-                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * rSkill;
+                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * rSkill;
 
                 if (arDouble[OUTPUT_TAKE] > 1.0f) {
                     /* missed double above cash point */
@@ -387,7 +386,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
          */
         if (fAnalyseDice && pmr->rLuck != ERR_VAL) {
 
-            float r = pms->nMatchTo ? eq2mwc(pmr->rLuck, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * pmr->rLuck;
+            float r = pms->nMatchTo ? eq2mwc(pmr->rLuck, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * pmr->rLuck;
 
             psc->arLuck[pmr->fPlayer][0] += pmr->rLuck;
             psc->arLuck[pmr->fPlayer][1] += r;
@@ -416,10 +415,12 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
         if (fAnalyseMove && (pmr->esChequer.et != EVAL_NONE || pmr->n.anMove[0] < 0)) {
             /* find skill */
             TanBoard anBoardMove;
+            float rChequerSkill = 0.0f;
+
             memcpy(anBoardMove, pms->anBoard, sizeof(anBoardMove));
             ApplyMove(anBoardMove, pmr->n.anMove, FALSE);
             PositionKey((ConstTanBoard) anBoardMove, &key);
-            rChequerSkill = 0.0f;
+
             if (pmr->ml.amMoves) {
                 for (i = 0; i < pmr->ml.cMoves; i++)
 
@@ -432,7 +433,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
             }
             /* update statistics */
 
-            rCost = pms->nMatchTo ? eq2mwc(rChequerSkill, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * rChequerSkill;
+            rCost = pms->nMatchTo ? eq2mwc(rChequerSkill, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * rChequerSkill;
 
             psc->anMoves[pmr->fPlayer][Skill(rChequerSkill)]++;
 
@@ -471,7 +472,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
             if (rSkill < 0.0f) {
                 /* it was not a double */
 
-                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * rSkill;
+                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * rSkill;
 
                 if (arDouble[OUTPUT_NODOUBLE] > 1.0f) {
                     /* wrong double above too good point */
@@ -511,7 +512,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
 
                 rSkill = -arDouble[OUTPUT_TAKE] - -arDouble[OUTPUT_DROP];
 
-                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * rSkill;
+                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * rSkill;
 
                 psc->anCubeWrongTake[pmr->fPlayer]++;
                 psc->arErrorWrongTake[pmr->fPlayer][0] -= rSkill;
@@ -538,7 +539,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
 
                 rSkill = -arDouble[OUTPUT_DROP] - -arDouble[OUTPUT_TAKE];
 
-                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * rSkill;
+                rCost = pms->nMatchTo ? eq2mwc(rSkill, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * rSkill;
 
                 psc->anCubeWrongPass[pmr->fPlayer]++;
                 psc->arErrorWrongPass[pmr->fPlayer][0] -= rSkill;
@@ -558,7 +559,7 @@ updateStatcontext(statcontext * psc, const moverecord * pmr, const matchstate *
         GetMatchStateCubeInfo(&ci, pms);
         if (fAnalyseDice && pmr->rLuck != ERR_VAL) {
 
-            float r = pms->nMatchTo ? eq2mwc(pmr->rLuck, &ci) - eq2mwc(0.0f, &ci) : pms->nCube * pmr->rLuck;
+            float r = pms->nMatchTo ? eq2mwc(pmr->rLuck, &ci) - eq2mwc(0.0f, &ci) : (float) pms->nCube * pmr->rLuck;
 
             psc->arLuck[pmr->fPlayer][0] += pmr->rLuck;
             psc->arLuck[pmr->fPlayer][1] += r;
@@ -677,7 +678,7 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
             if (cmp_evalsetup(pesChequer, &pmr->esChequer) > 0) {
 
                 if (pmr->ml.cMoves) {
-                    free(pmr->ml.amMoves);
+                    g_free(pmr->ml.amMoves);
                     pmr->ml.amMoves = NULL;
                 }
 
@@ -690,13 +691,13 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
                                            pmr->anDice[1],
                                            (ConstTanBoard) pms->anBoard, &key,
                                            arSkillLevel[SKILL_DOUBTFUL], &ci, &pesChequer->ec, aamf) < 0) {
-                        free(ml.amMoves);
+                        g_free(ml.amMoves);
                         return -1;
                     }
                     MT_Exclusive();
                     CopyMoveList(&pmr->ml, &ml);
                     if (ml.cMoves)
-                        free(ml.amMoves);
+                        g_free(ml.amMoves);
                 }
 
             }
@@ -724,7 +725,7 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
 
         dt = DoubleType(pms->fDoubled, pms->fMove, pms->fTurn);
 
-        if (dt > DT_NORMAL)	/* TODO: analyse beavers */
+        if (dt > DT_NORMAL)     /* TODO: analyse beavers */
             break;
 
         /* cube action */
@@ -775,7 +776,7 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
             break;
 
         tt = (taketype) DoubleType(pms->fDoubled, pms->fMove, pms->fTurn);
-        if (tt > TT_NORMAL)	/* TODO: analyse beavers */
+        if (tt > TT_NORMAL)     /* TODO: analyse beavers */
             break;
 
         if (fAnalyseCube && pmgi->fCubeUse && doubleError && (*doubleError != ERR_VAL)) {
@@ -795,7 +796,7 @@ AnalyzeMove(moverecord * pmr, matchstate * pms, const listOLD * plParentGame,
             break;
 
         tt = (taketype) DoubleType(pms->fDoubled, pms->fMove, pms->fTurn);
-        if (tt > TT_NORMAL)	/* TODO: analyse beavers */
+        if (tt > TT_NORMAL)     /* TODO: analyse beavers */
             break;
 
         if (fAnalyseCube && pmgi->fCubeUse && doubleError && (*doubleError != ERR_VAL)) {
@@ -939,7 +940,6 @@ AnalyseMoveMT(Task * task)
 static int
 AnalyzeGame(listOLD * plGame, int wait)
 {
-    int result;
     unsigned int i;
     listOLD *pl = plGame->plNext;
     moverecord *pmr = pl->p;
@@ -1006,6 +1006,8 @@ AnalyzeGame(listOLD * plGame, int wait)
     g_assert(pl->plNext == plGame);
 
     if (wait) {
+        int result;
+
         multi_debug("wait for all task: analysis");
         result = MT_WaitForTasks(UpdateProgressBar, 250, fAutoSaveAnalysis);
 
@@ -1032,11 +1034,11 @@ UpdateVariance(float *prVariance, const float rSum, const float rSumAdd, const i
          * for formula */
 
         float rDelta = rSumAdd;
-        float rMuNew = rSum / nGames;
-        float rMuOld = (rSum - rDelta) / (nGames - 1);
+        float rMuNew = rSum / (float) nGames;
+        float rMuOld = (rSum - rDelta) / (float) (nGames - 1);
         float rDeltaMu = rMuNew - rMuOld;
 
-        *prVariance = *prVariance * (1.0f - 1.0f / (nGames - 1.0f)) + nGames * rDeltaMu * rDeltaMu;
+        *prVariance = *prVariance * (1.0f - 1.0f / ((float) (nGames - 1))) + (float) nGames * rDeltaMu * rDeltaMu;
 
         return;
 
@@ -1296,9 +1298,9 @@ IniStatcontext(statcontext * psc)
 extern float
 relativeFibsRating(float r, int n)
 {
-    float const x = -2000.0f / (float) sqrt(1.0 * n) * (float) log10(1.0 / r - 1.0);
+    float const x = -2000.0f / sqrtf((float) n) * log10f(1.0f / r - 1.0f);
 
-    return (x < -2100) ? -2100 : x;
+    return (x < -2100.0f) ? -2100.0f : x;
 }
 
 /*
@@ -1314,7 +1316,7 @@ extern float
 absoluteFibsRatingChequer(const float rChequer, const int n)
 {
 
-    return rChequer * (8798.0f + 25526.0f / (n));
+    return rChequer * (8798.0f + 25526.0f / (float) n);
 
 }
 
@@ -1332,7 +1334,7 @@ extern float
 absoluteFibsRatingCube(const float rCube, const int n)
 {
 
-    return rCube * (863.0f - 519.0f / n);
+    return rCube * (863.0f - 519.0f / (float) n);
 
 }
 
@@ -1370,7 +1372,7 @@ getMWCFromError(const statcontext * psc, float aaaar[3][2][2][2])
             aaaar[CHEQUERPLAY][TOTAL][i][j] = psc->arErrorCheckerplay[i][j];
 
             if (psc->anUnforcedMoves[i])
-                aaaar[CHEQUERPLAY][PERMOVE][i][j] = aaaar[0][0][i][j] / psc->anUnforcedMoves[i];
+                aaaar[CHEQUERPLAY][PERMOVE][i][j] = aaaar[0][0][i][j] / (float) psc->anUnforcedMoves[i];
             else
                 aaaar[CHEQUERPLAY][PERMOVE][i][j] = 0.0f;
 
@@ -1384,7 +1386,7 @@ getMWCFromError(const statcontext * psc, float aaaar[3][2][2][2])
                 + psc->arErrorWrongPass[i][j];
 
             if (psc->anCloseCube[i])
-                aaaar[CUBEDECISION][PERMOVE][i][j] = aaaar[CUBEDECISION][TOTAL][i][j] / psc->anCloseCube[i];
+                aaaar[CUBEDECISION][PERMOVE][i][j] = aaaar[CUBEDECISION][TOTAL][i][j] / (float) psc->anCloseCube[i];
             else
                 aaaar[CUBEDECISION][PERMOVE][i][j] = 0.0f;
 
@@ -1396,20 +1398,24 @@ getMWCFromError(const statcontext * psc, float aaaar[3][2][2][2])
 
             if (psc->anUnforcedMoves[i] + psc->anCloseCube[i])
                 aaaar[COMBINED][PERMOVE][i][j] =
-                    aaaar[COMBINED][TOTAL][i][j] / (psc->anUnforcedMoves[i] + psc->anCloseCube[i]);
+                    aaaar[COMBINED][TOTAL][i][j] / (float) (psc->anUnforcedMoves[i] + psc->anCloseCube[i]);
             else
                 aaaar[COMBINED][PERMOVE][i][j] = 0.0f;
 
         }
 }
 
+#define DSCFORMAT "%-40s %-23s %-23s\n"
+
 extern void
 DumpStatcontext(char *szOutput, const statcontext * psc, const char *player, const char *op, int nMatchTo)
 {
 
     /* header */
 
-    sprintf(szOutput, "%-40s %-23s %-23s\n\n", _("Player"), player, op);
+    sprintf(szOutput, DSCFORMAT, _("Player"), player, op);
+
+    strcat(szOutput, "\n");
 
     if (psc->fMoves) {
         GList *list = formatGS(psc, nMatchTo, FORMATGS_CHEQUER);
@@ -1422,7 +1428,7 @@ DumpStatcontext(char *szOutput, const statcontext * psc, const char *player, con
 
             char **asz = pl->data;
 
-            sprintf(strchr(szOutput, 0), "%-40s %-23s %-23s\n", asz[0], asz[1], asz[2]);
+            sprintf(strchr(szOutput, 0), DSCFORMAT, asz[0], asz[1], asz[2]);
 
         }
 
@@ -1444,7 +1450,7 @@ DumpStatcontext(char *szOutput, const statcontext * psc, const char *player, con
 
             char **asz = pl->data;
 
-            sprintf(strchr(szOutput, 0), "%-40s %-23s %-23s\n", asz[0], asz[1], asz[2]);
+            sprintf(strchr(szOutput, 0), DSCFORMAT, asz[0], asz[1], asz[2]);
 
         }
 
@@ -1466,7 +1472,7 @@ DumpStatcontext(char *szOutput, const statcontext * psc, const char *player, con
 
             char **asz = pl->data;
 
-            sprintf(strchr(szOutput, 0), "%-40s %-23s %-23s\n", asz[0], asz[1], asz[2]);
+            sprintf(strchr(szOutput, 0), DSCFORMAT, asz[0], asz[1], asz[2]);
 
         }
 
@@ -1488,7 +1494,7 @@ DumpStatcontext(char *szOutput, const statcontext * psc, const char *player, con
 
             char **asz = pl->data;
 
-            sprintf(strchr(szOutput, 0), "%-40s %-23s %-23s\n", asz[0], asz[1], asz[2]);
+            sprintf(strchr(szOutput, 0), DSCFORMAT, asz[0], asz[1], asz[2]);
 
         }
 
@@ -1500,7 +1506,7 @@ DumpStatcontext(char *szOutput, const statcontext * psc, const char *player, con
 extern void
 CommandShowStatisticsMatch(char *UNUSED(sz))
 {
-    char szOutput[4096];
+    char szOutput[STATCONTEXT_MAXSIZE];
 
     updateStatisticsMatch(&lMatch);
 
@@ -1530,7 +1536,7 @@ CommandShowStatisticsGame(char *UNUSED(sz))
 {
 
     moverecord *pmr;
-    char szOutput[4096];
+    char szOutput[STATCONTEXT_MAXSIZE];
 
     if (!CheckGameExists())
         return;
@@ -1641,7 +1647,6 @@ updateStatisticsGame(const listOLD * plGame)
 {
 
     listOLD *pl;
-    moverecord *pmr;
     moverecord *pmrx = plGame->plNext->p;
     matchstate msAnalyse;
 
@@ -1649,7 +1654,7 @@ updateStatisticsGame(const listOLD * plGame)
 
     for (pl = plGame->plNext; pl != plGame; pl = pl->plNext) {
 
-        pmr = pl->p;
+        moverecord *pmr = pl->p;
 
         updateStatisticsMove(pmr, &msAnalyse, plGame, &pmrx->g.sc);
 
@@ -1710,7 +1715,7 @@ AnalyseClearMove(moverecord * pmr)
         pmr->rLuck = ERR_VAL;
         pmr->lt = LUCK_NONE;
         if (pmr->ml.amMoves) {
-            free(pmr->ml.amMoves);
+            g_free(pmr->ml.amMoves);
             pmr->ml.amMoves = NULL;
         }
         pmr->ml.cMoves = 0;
@@ -2037,7 +2042,7 @@ static void
 cmark_move_show(GString * gsz, const matchstate * UNUSED(pms), const moverecord * pmr, int movenr)
 {
     guint i;
-    gchar sz[40];
+    gchar sz[FORMATEDMOVESIZE];
     int found = 0;
 
     g_return_if_fail(pmr);
@@ -2076,10 +2081,8 @@ cmark_game_show(GString * gsz, listOLD * game, int game_number)
             ApplyMoveRecord(&ms_local, game, pmr);
             break;
         case MOVE_NORMAL:
-            if (pmr->fPlayer != ms_local.fMove) {
+            if (pmr->fPlayer != ms_local.fMove)
                 SwapSides(ms_local.anBoard);
-                ms_local.fMove = pmr->fPlayer;
-            }
             ms_local.fTurn = ms_local.fMove = pmr->fPlayer;
             cmark_cube_show(gsz, &ms_local, pmr, movenr);
             ms_local.anDice[0] = pmr->anDice[0];
@@ -2140,7 +2143,7 @@ cmark_move_set(moverecord * pmr, gchar * sz, CMark cmark)
     while ((n = (int) strtoll(sz, &sz, 10)) != 0) {
 #endif
         if (n > c) {
-            outputerrf("Only %d moves in movelist\n", c);
+            outputerrf(_("Only %d legal moves, cannot mark move %d\n"), c, n);
             g_slist_free(list);
             return;
         }
@@ -2148,7 +2151,7 @@ cmark_move_set(moverecord * pmr, gchar * sz, CMark cmark)
             list = g_slist_append(list, GINT_TO_POINTER(n));
     }
     if (g_slist_length(list) == 0) {
-        outputerrf("Not a valid list of moves\n");
+        outputerrf(_("Not a valid list of moves\n"));
         return;
     }
 
@@ -2215,14 +2218,13 @@ cmark_match_clear(listOLD * match)
 static int
 cmark_move_rollout(moverecord * pmr, gboolean destroy)
 {
-    gchar(*asz)[40];
+    gchar(*asz)[FORMATEDMOVESIZE];
     cubeinfo ci;
     cubeinfo **ppci;
     GSList *pl = NULL;
     gint c;
     guint j;
     gint res;
-    move *m;
     move **ppm;
     void *p;
     GSList *list = NULL;
@@ -2241,14 +2243,14 @@ cmark_move_rollout(moverecord * pmr, gboolean destroy)
 
     ppm = g_new(move *, c);
     ppci = g_new(cubeinfo *, c);
-    asz = (char (*)[40]) g_malloc(40 * c);
+    asz = (char (*)[FORMATEDMOVESIZE]) g_malloc(FORMATEDMOVESIZE * c);
     if (pmr->n.iMove != UINT_MAX)
         CopyKey(pmr->ml.amMoves[pmr->n.iMove].key, key);
     GetMatchStateCubeInfo(&ci, &ms);
 
     for (pl = list, j = 0; pl; pl = g_slist_next(pl), j++) {
         gint i = GPOINTER_TO_INT(pl->data);
-        m = ppm[j] = &pmr->ml.amMoves[i];
+        move *m = ppm[j] = &pmr->ml.amMoves[i];
         ppci[j] = &ci;
         FormatMove(asz[j], msBoard(), m->anMove);
     }
@@ -2305,7 +2307,7 @@ cmark_cube_rollout(moverecord * pmr, gboolean destroy)
     float aarOutput[2][NUM_ROLLOUT_OUTPUTS];
     float aarStdDev[2][NUM_ROLLOUT_OUTPUTS];
     rolloutstat aarsStatistics[2][2];
-    gchar asz[2][40];
+    gchar asz[2][FORMATEDMOVESIZE];
     void *p;
     int res;
 
@@ -2626,7 +2628,8 @@ CommandAnalyseRolloutMove(char *sz)
         cmark_move_set(pmr, sz, CMARK_ROLLOUT);
 
     if (cmark_move_rollout(pmr, FALSE) == 0) {
-        outputerrf("No moves marked for rollout\n");
+        outputerrf(_("No moves marked for rollout"));
+        outputerrf("\n");
         return;
     }
 
