@@ -1,11 +1,11 @@
 /*
- * backgammon.h
+ * Copyright (C) 1999-2003 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 1999-2022 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 1999, 2000, 2001, 2002.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: backgammon.h,v 1.462 2018/06/10 18:17:59 plm Exp $
+ * $Id: backgammon.h,v 1.470 2022/10/16 19:05:38 plm Exp $
  */
 
 #ifndef BACKGAMMON_H
@@ -34,17 +33,14 @@
 
 #define MAX_CUBE ( 1 << 12 )
 #define MAX_NAME_LEN 32
-#ifndef BUILD_DATE
-#define BUILD_DATE_STR " " __DATE__
-#else
+
 #define BUILD_DATE_STR STRINGIZE(BUILD_DATE)
-#endif
 
 #define VERSION_STRING "GNU Backgammon " VERSION " " BUILD_DATE_STR
 #define GNUBG_CHARSET "UTF-8"
 
 extern const char *intro_string;
-typedef struct _command {
+typedef struct command {
     /* Command name (NULL indicates end of list) */
     const char *sz;
     /* Command handler; NULL to use default subcommand handler */
@@ -53,12 +49,12 @@ typedef struct _command {
     const char *szHelp;
     const char *szUsage;
     /* List of subcommands (NULL if none) */
-    struct _command *pc;
+    struct command *pc;
 } command;
 
-typedef struct _procrecorddata {
+typedef struct procrecorddata {
     /* Record handler */
-    int (*pfProcessRecord) (struct _procrecorddata *);
+    int (*pfProcessRecord) (struct procrecorddata *);
     void *pvUserData;
     void *avInputData[8];
     void *avOutputData[8];
@@ -73,11 +69,11 @@ typedef struct _procrecorddata {
 #define PROCREC_HINT_ARGOUT_INDEX 4
 
 
-typedef enum _playertype {
+typedef enum {
     PLAYER_HUMAN, PLAYER_GNU, PLAYER_EXTERNAL
 } playertype;
 
-typedef struct _player {
+typedef struct {
     /* For all player types: */
     char szName[MAX_NAME_LEN];
     playertype pt;
@@ -90,7 +86,7 @@ typedef struct _player {
     char *szSocket;
 } player;
 
-typedef enum _movetype {
+typedef enum {
     MOVE_GAMEINFO,
     MOVE_NORMAL,
     MOVE_DOUBLE,
@@ -103,7 +99,7 @@ typedef enum _movetype {
     MOVE_SETCUBEPOS
 } movetype;
 
-typedef struct _movegameinfo {
+typedef struct {
     /* ordinal number of the game within a match */
     int i;
     /* match length */
@@ -130,14 +126,14 @@ typedef struct _movegameinfo {
     statcontext sc;
 } xmovegameinfo;
 
-typedef struct _cubedecisiondata {
+typedef struct {
     float aarOutput[2][NUM_ROLLOUT_OUTPUTS];
     float aarStdDev[2][NUM_ROLLOUT_OUTPUTS];
     evalsetup esDouble;
     CMark cmark;
 } cubedecisiondata;
 
-typedef struct _movenormal {
+typedef struct {
     /* Move made. */
     int anMove[8];
     /* index into the movelist of the move that was made */
@@ -145,7 +141,7 @@ typedef struct _movenormal {
     skilltype stMove;
 } xmovenormal;
 
-typedef struct _moveresign {
+typedef struct {
     int nResigned;
     evalsetup esResign;
     float arResign[NUM_ROLLOUT_OUTPUTS];
@@ -153,19 +149,19 @@ typedef struct _moveresign {
     skilltype stAccept;
 } xmoveresign;
 
-typedef struct _movesetboard {
+typedef struct {
     positionkey key;            /* always stored as if player 0 was on roll */
 } xmovesetboard;
 
-typedef struct _movesetcubeval {
+typedef struct {
     int nCube;
 } xmovesetcubeval;
 
-typedef struct _movesetcubepos {
+typedef struct {
     int fCubeOwner;
 } xmovesetcubepos;
 
-typedef struct _moverecord {
+typedef struct {
     /*
      * Common variables
      */
@@ -194,6 +190,7 @@ typedef struct _moverecord {
     /* the evaluation and settings */
     cubedecisiondata *CubeDecPtr;
     cubedecisiondata CubeDec;
+    cubedecisiondata *MoneyCubeDecPtr; /* In case the cube has been evaluated at money. */
     /* skill for the cube decision */
     skilltype stCube;
     /* "private" data */
@@ -206,7 +203,7 @@ typedef struct _moverecord {
 } moverecord;
 
 
-typedef struct _matchinfo {     /* SGF match information */
+typedef struct {     /* SGF match information */
     char *pchRating[2];
     char *pchEvent;
     char *pchRound;
@@ -218,7 +215,7 @@ typedef struct _matchinfo {     /* SGF match information */
     unsigned int nDay;          /* 0 for nYear means date unknown */
 } matchinfo;
 
-typedef struct _decisionData {
+typedef struct {
     float aarOutput[2][NUM_ROLLOUT_OUTPUTS];
     float aarStdDev[2][NUM_ROLLOUT_OUTPUTS];
     rolloutstat aarsStatistics[2][2];
@@ -231,15 +228,15 @@ typedef struct _decisionData {
     int n;
 } decisionData;
 
-typedef struct _moveData {
+typedef struct {
     moverecord *pmr;
     matchstate *pms;
     const evalsetup *pesChequer;
     evalsetup *pesCube;
-     movefilter(*aamf)[MAX_FILTER_PLIES];
+    movefilter(*aamf)[MAX_FILTER_PLIES];
 } moveData;
 
-typedef struct _findData {
+typedef struct {
     movelist *pml;
     ConstTanBoard pboard;
     positionkey *keyMove;
@@ -247,10 +244,10 @@ typedef struct _findData {
     const cubeinfo *pci;
     const evalcontext *pec;
     int anDice[2];
-     movefilter(*aamf)[MAX_FILTER_PLIES];
+    movefilter(*aamf)[MAX_FILTER_PLIES];
 } findData;
 
-typedef struct _scoreData {
+typedef struct {
     move *pm;
     const cubeinfo *pci;
     const evalcontext *pec;
@@ -284,7 +281,7 @@ int RunAsyncProcess(AsyncFun fun, void *data, const char *msg);
  *
  */
 
-/*  List of moverecords representing the current game. One of the elements in
+/* List of moverecords representing the current game. One of the elements in
  * lMatch.
  * Typically the last game in the match).
  */
@@ -414,6 +411,7 @@ extern command acSetRolloutLatePlayer[];
 extern command acSetRolloutLimit[];
 extern command acSetRolloutPlayer[];
 extern command acSetTruncation[];
+extern command acShow[];
 extern command acTop[];
 
 extern int fInteractive;
@@ -424,8 +422,6 @@ extern int fInteractive;
 
 /* now we can include libintl.h */
 #include <glib/gi18n.h>
-/* Like strncpy, except it does the right thing */
-extern char *strcpyn(char *szDest, const char *szSrc, int cch);
 
 extern char *GetMatchCheckSum(void);
 extern char *CheckCommand(char *sz, command * ac);
@@ -466,11 +462,7 @@ extern int ParsePosition(TanBoard an, char **ppch, char *pchDesc);
 extern int SetToggle(const char *szName, int *pf, char *sz, const char *szOn, const char *szOff);
 extern moverecord *get_current_moverecord(int *pfHistory);
 extern moverecord *LinkToDouble(moverecord * pmr);
-#if defined(HAVE_FUNC_ATTRIBUTE_RETURNS_NONNULL)
-extern moverecord *NewMoveRecord(void) __attribute__((returns_nonnull));
-#else
 extern moverecord *NewMoveRecord(void);
-#endif
 extern void HandleInterrupt(int idSignal);
 extern void AddGame(moverecord * pmr);
 extern void AddMoveRecord(moverecord * pmr);
@@ -982,6 +974,7 @@ extern void CommandShowStatisticsGame(char *);
 extern void CommandShowStatisticsMatch(char *);
 extern void CommandShowStatisticsSession(char *);
 extern void CommandShowTemperatureMap(char *);
+extern void CommandShowScoreMap(char *);
 extern void CommandShowThorp(char *);
 extern void CommandShowThreads(char *);
 extern void CommandShowTurn(char *);
@@ -1001,12 +994,12 @@ extern void find_skills(moverecord * pmr, const matchstate * pms, int did_double
 extern int getGameNumber(const listOLD * plGame);
 extern int getMoveNumber(const listOLD * plGame, const void *p);
 extern int CheckGameExists(void);
-extern void pmr_cubedata_set(moverecord * pmr, evalsetup * pes, float output[2][NUM_ROLLOUT_OUTPUTS],
+extern void pmr_cubedata_set(moverecord * pmr, const evalsetup * pes, float output[2][NUM_ROLLOUT_OUTPUTS],
                              float stddev[2][NUM_ROLLOUT_OUTPUTS]);
-extern void pmr_movelist_set(moverecord * pmr, evalsetup * pes, movelist * pml);
+extern void pmr_movelist_set(moverecord * pmr, const evalsetup * pes, const movelist * pml);
 extern listOLD *game_add_pmr_hint(listOLD * plGame);
 extern void game_remove_pmr_hint(listOLD * pl_hint);
-extern gboolean game_is_last(listOLD * plGame);
+extern gboolean game_is_last(const listOLD * plGame);
 extern void pmr_hint_destroy(void);
 extern void StopAutomaticPlay(void);
 extern gboolean save_autosave(gpointer unused);
