@@ -1,13 +1,11 @@
 /*
- * gtkcolour3d.c
- * by Jon Kinsey, 2003
+ * Copyright (C) 2003-2019 Jon Kinsey <jonkinsey@gmail.com>
+ * Copyright (C) 2006-2022 the AUTHORS
  *
- * 3D colour selection dialog and preview area
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,16 +13,18 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkcolour3d.c,v 1.58 2018/05/10 17:51:31 plm Exp $
+ * $Id: gtkcolour3d.c,v 1.71 2022/08/30 18:38:35 plm Exp $
  */
 
 #include "config.h"
-#include "inc3d.h"
+#include "legacyGLinc.h"
+#include "fun3d.h"
 #include "gtklocdefs.h"
 
+#include <GL/glu.h>	/* Used for colour widget - todo: remove? */
+
 #include "gtkprefs.h"
 
 typedef struct UpdateDetails_T {
@@ -56,8 +56,8 @@ static int curDetail;
 #define PREVIEW_HEIGHT 40
 
 /* World sizes */
-#define STRIP_WIDTH 100
-#define STRIP_HEIGHT 10.f
+#define STRIP_WIDTH 100.0f
+#define STRIP_HEIGHT 10.0f
 
 static int previewLightLevels[3];
 
@@ -102,9 +102,9 @@ SetupLight(void)
 static void
 Draw(Material * pMat)
 {
-    Texture texture;
+    static Texture texture;
     int tempShin = pMat->shine;
-    float edge = (1 / (float) PREVIEW_HEIGHT) * STRIP_HEIGHT;
+    const float edge = STRIP_HEIGHT / PREVIEW_HEIGHT;
     /* Accentuate shiness - so visible in preview */
     pMat->shine = tempShin / 3;
 
@@ -144,8 +144,7 @@ Draw(Material * pMat)
     glTranslatef(edge * 2, STRIP_HEIGHT / 2, 0.f);
     glRotatef(90.f, 0.f, 1.f, 0.f);
     /* -edge to give a black outline */
-    gluCylinder(qobj, (double) STRIP_HEIGHT / 2 - edge, (double) STRIP_HEIGHT / 2 - edge,
-                (double) STRIP_WIDTH - edge * 3, 36, 36);
+    gluCylinder(qobj, STRIP_HEIGHT / 2 - edge, STRIP_HEIGHT / 2 - edge, STRIP_WIDTH - edge * 3, 36, 36);
     glPopMatrix();
 
     if (pMat->textureInfo) {
@@ -197,7 +196,7 @@ SetupColourPreview(void)
     glViewport(0, 0, PREVIEW_WIDTH, PREVIEW_HEIGHT);
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
-    glOrtho(0.0, (double) STRIP_WIDTH, 0.0, (double) STRIP_HEIGHT, (double) -STRIP_HEIGHT, 0.0);
+    glOrtho(0.0, STRIP_WIDTH, 0.0, STRIP_HEIGHT, -STRIP_HEIGHT, 0.0);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
 }
@@ -219,55 +218,26 @@ TextureChange(GtkComboBoxText * combo, gpointer UNUSED(data))
 }
 
 static gboolean
-expose_event_preview3d(GtkWidget * widget, GdkEventExpose * UNUSED(eventDetails), Material * pMat)
+exposePreviewCB(GtkWidget* UNUSED(widget), GdkEventExpose* UNUSED(eventData), void* data)
 {
-    GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);
-
-    if (!gdk_gl_drawable_gl_begin(gldrawable, gtk_widget_get_gl_context(widget)))
-        return TRUE;
-
-    CheckOpenglError();
-
+	Material* pMat = (Material*)data;
     SetupLight();
     Draw(pMat);
-
-    gdk_gl_drawable_swap_buffers(gldrawable);
-
-    gdk_gl_drawable_gl_end(gldrawable);
-
     return TRUE;
 }
 
 static void
-realize_preview3d(GtkWidget * widget, void *UNUSED(data))
+realizePreviewCB(void *UNUSED(data))
 {
-    GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);
-    if (!gdk_gl_drawable_gl_begin(gldrawable, gtk_widget_get_gl_context(widget)))
-        return;
-
     SetupColourPreview();
-
-    gdk_gl_drawable_gl_end(gldrawable);
 }
 
 static GtkWidget *
 CreateGLPreviewWidget(Material * pMat)
-{                               /* Rename this (and the one above to CreateGLBoardWidget) */
-    GtkWidget *p3dWidget = gtk_drawing_area_new();
-
-    /* Set OpenGL-capability to the widget - no list sharing */
-    if (!gtk_widget_set_gl_capability(p3dWidget, getGlConfig(), NULL, TRUE, GDK_GL_RGBA_TYPE)) {
-        outputerrf("Can't create opengl capable widget\n");
+{
+	GtkWidget* p3dWidget = GLWidgetCreate(realizePreviewCB, NULL, exposePreviewCB, pMat);
+	if (p3dWidget == NULL)
         return NULL;
-    }
-
-    if (p3dWidget == NULL) {
-        outputerrf("Can't create opengl drawing widget\n");
-        return NULL;
-    }
-
-    g_signal_connect(G_OBJECT(p3dWidget), "realize", G_CALLBACK(realize_preview3d), NULL);
-    g_signal_connect(G_OBJECT(p3dWidget), "expose_event", G_CALLBACK(expose_event_preview3d), pMat);
 
     return p3dWidget;
 }
@@ -301,71 +271,135 @@ texture_set_active(void)
 static void
 AddWidgets(GtkWidget * window)
 {
-    GtkWidget *table, *label, *scale;
+#if GTK_CHECK_VERSION(3,0,0)
+    GtkWidget *grid;
+#else
+    GtkWidget *table;
+#endif
+    GtkWidget *label, *scale;
+
+#if GTK_CHECK_VERSION(3,0,0)
+    grid = gtk_grid_new();
+    gtk_grid_set_column_homogeneous(GTK_GRID(grid), TRUE);
+    gtk_grid_set_row_homogeneous(GTK_GRID(grid), TRUE);
+    gtk_container_add(GTK_CONTAINER(window), grid);
+#else
     table = gtk_table_new(5, 4, TRUE);
     gtk_container_add(GTK_CONTAINER(window), table);
+#endif
 
     label = gtk_label_new(_("Ambient colour:"));
-    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
     pcpAmbient = gtk_color_button_new();
     g_signal_connect(G_OBJECT(pcpAmbient), "color-set", UpdateColourPreview, NULL);
+#if GTK_CHECK_VERSION(3,0,0)
+    /* coordinates are: left, top, width, height */
+    gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), pcpAmbient, 1, 0, 1, 1);
+#else
+    /* coordinates are: left, right, top, bottom */
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
     gtk_table_attach_defaults(GTK_TABLE(table), pcpAmbient, 1, 2, 0, 1);
+#endif
 
     label = gtk_label_new(_("Diffuse colour:"));
-    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
     pcpDiffuse = gtk_color_button_new();
     g_signal_connect(G_OBJECT(pcpDiffuse), "color-set", UpdateColourPreview, NULL);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), pcpDiffuse, 1, 1, 1, 1);
+#else
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
     gtk_table_attach_defaults(GTK_TABLE(table), pcpDiffuse, 1, 2, 1, 2);
+#endif
 
     label = gtk_label_new(_("Specular colour:"));
-    gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 0, 1);
     pcpSpecular = gtk_color_button_new();
     g_signal_connect(G_OBJECT(pcpSpecular), "color-set", UpdateColourPreview, NULL);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(grid), label, 2, 0, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), pcpSpecular, 3, 0, 1, 1);
+#else
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 0, 1);
     gtk_table_attach_defaults(GTK_TABLE(table), pcpSpecular, 3, 4, 0, 1);
+#endif
 
     label = gtk_label_new(_("Shine:"));
-    gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 1, 2);
     padjShine = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 128.0, 1.0, 10.0, 0.0));
     g_signal_connect(G_OBJECT(padjShine), "value-changed", G_CALLBACK(UpdateColourPreview), NULL);
+#if GTK_CHECK_VERSION(3,0,0)
+    scale = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, padjShine);
+#else
     scale = gtk_hscale_new(padjShine);
+#endif
     gtk_scale_set_digits(GTK_SCALE(scale), 0);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(grid), label, 2, 1, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), scale, 3, 1, 1, 1);
+#else
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 1, 2);
     gtk_table_attach_defaults(GTK_TABLE(table), scale, 3, 4, 1, 2);
+#endif
 
     pOpacitylabel = gtk_label_new(_("Opacity:"));
-    gtk_table_attach_defaults(GTK_TABLE(table), pOpacitylabel, 0, 1, 2, 3);
     padjOpacity = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 1.0, 100.0, 1.0, 10.0, 0.0));
     g_signal_connect(G_OBJECT(padjOpacity), "value-changed", G_CALLBACK(UpdateColourPreview), NULL);
+#if GTK_CHECK_VERSION(3,0,0)
+    psOpacity = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, padjOpacity);
+#else
     psOpacity = gtk_hscale_new(padjOpacity);
+#endif
     gtk_scale_set_digits(GTK_SCALE(psOpacity), 0);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(grid), pOpacitylabel, 0, 2, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), psOpacity, 1, 2, 1, 1);
+#else
+    gtk_table_attach_defaults(GTK_TABLE(table), pOpacitylabel, 0, 1, 2, 3);
     gtk_table_attach_defaults(GTK_TABLE(table), psOpacity, 1, 2, 2, 3);
+#endif
 
     pTexturelabel = gtk_label_new(_("Texture:"));
-    gtk_table_attach_defaults(GTK_TABLE(table), pTexturelabel, 2, 3, 2, 3);
     textureComboBox = GTK_WIDGET(gtk_combo_box_text_new());
     texture_set_active();
     gtk_widget_set_sensitive(textureComboBox, FALSE);
     g_signal_connect(textureComboBox, "changed", G_CALLBACK(TextureChange), NULL);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(grid), pTexturelabel, 2, 2, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), textureComboBox, 2, 3, 2, 1);
+#else
+    gtk_table_attach_defaults(GTK_TABLE(table), pTexturelabel, 2, 3, 2, 3);
     gtk_table_attach_defaults(GTK_TABLE(table), textureComboBox, 2, 4, 3, 4);
+#endif
 
     label = gtk_label_new(_("Preview:"));
-    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4);
 
     pwPreview = CreateGLPreviewWidget(&col3d);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(grid), label, 0, 3, 1, 1);
+    gtk_grid_attach(GTK_GRID(grid), pwPreview, 0, 4, 2, 1);
+#else
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4);
     gtk_table_attach_defaults(GTK_TABLE(table), pwPreview, 0, 2, 4, 5);
+#endif
+
 }
 
 static void
 DialogClose(GtkDialog * UNUSED(dialog), gint response, void *UNUSED(data))
 {
     if ((GtkResponseType) response == GTK_RESPONSE_OK) {        /* Apply new settings */
-        char *texStr;
 
         /* Copy new settings to preview material */
         curDetails->mat = col3d;
         *curDetails->pBoardMat = col3d;
 
         if (useTexture) {
-            texStr = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(textureComboBox));
+            char *texStr = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(textureComboBox));
 
             if (!strcmp(texStr, NO_TEXTURE_STRING))
                 col3d.textureInfo = 0;
@@ -414,7 +448,8 @@ UpdateColour3d(GtkButton * UNUSED(button), UpdateDetails * pDetails)
     gtk_adjustment_set_value(padjShine, (double) col3d.shine);
     if (IsFlagSet(pDetails->opacity, DF_VARIABLE_OPACITY)) {
         useOpacity = 1;
-        gtk_adjustment_set_value(padjOpacity, col3d.alphaBlend ? (col3d.ambientColour[3] + .001) * 100 : 100.0);
+        gtk_adjustment_set_value(padjOpacity,
+                                 col3d.alphaBlend ? ((double) col3d.ambientColour[3] + .001) * 100 : 100.0);
     } else {
         useOpacity = 0;
         if (IsFlagSet(pDetails->opacity, DF_FULL_ALPHA))
@@ -491,8 +526,8 @@ gtk_colour_picker_new3d(Material * pMat, int opacity, TextureType textureType)
 {
     GtkWidget *previewWidget, *button = gtk_button_new();
 
-    if (curDetail == MAX_DETAILS) {
-        outputerrf("Error: Too many 3D colour previews\n");
+    if (curDetail >= MAX_DETAILS) {
+        outputerrf(_("Error: Too many 3D colour previews\n"));
         return 0;
     }
     details[curDetail].mat = *pMat;
