@@ -1,13 +1,10 @@
 /*
- * matrix.c
- * by Jon Kinsey, 2003
+ * Copyright (C) 2003-2019 Jon Kinsey <jonkinsey@gmail.com>
  *
- * Simple 3d matrix functions
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,14 +12,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: matrix.c,v 1.24 2016/05/11 21:33:09 plm Exp $
+ * $Id: matrix.c,v 1.29 2019/12/29 14:55:26 plm Exp $
  */
 
 #include "config.h"
-#include "inc3d.h"
+#include "fun3d.h"
 #include <glib/gstdio.h>
 
 void
@@ -44,10 +40,6 @@ mult_matrix_vec(const float mat[4][4], const float src[4], float dst[4])
     dst[3] = (src[0] * mat[3][0] + src[1] * mat[3][1] + src[2] * mat[3][2] + src[3] * mat[3][3]);
 }
 
-/*
- * void matrixmult(float m[4][4], const float b[4][4])
- * Causes compiler warnings on gcc/linux
- */
 void
 matrixmult(float m[4][4], const float b[4][4])
 {
@@ -78,7 +70,7 @@ makeInverseTransposeMatrix(float m[4][4], const float v[3])
 void
 makeInverseRotateMatrixX(float m[4][4], float degRot)
 {
-    float radRot = -(degRot * (float) G_PI) / 180.0f;
+    float radRot = -(degRot * F_PI) / 180.0f;
     float cosRot = cosf(radRot);
     float sinRot = sinf(radRot);
 
@@ -92,7 +84,7 @@ makeInverseRotateMatrixX(float m[4][4], float degRot)
 void
 makeInverseRotateMatrixY(float m[4][4], float degRot)
 {
-    float radRot = -(degRot * (float) G_PI) / 180.0f;
+    float radRot = -(degRot * F_PI) / 180.0f;
     float cosRot = cosf(radRot);
     float sinRot = sinf(radRot);
 
@@ -106,7 +98,7 @@ makeInverseRotateMatrixY(float m[4][4], float degRot)
 void
 makeInverseRotateMatrixZ(float m[4][4], float degRot)
 {
-    float radRot = -(degRot * (float) G_PI) / 180.0f;
+    float radRot = -(degRot * F_PI) / 180.0f;
     float cosRot = cosf(radRot);
     float sinRot = sinf(radRot);
 
@@ -123,7 +115,7 @@ makeInverseRotateMatrixZ(float m[4][4], float degRot)
 void
 makeInverseRotateMatrix(float m[4][4], float degRot, float x, float y, float z)
 {
-    float radRot = -(degRot * (float) G_PI) / 180.0f;
+    float radRot = -(degRot * F_PI) / 180.0f;
     float sqnorm = x * x + y * y + z * z;
     float sin_theta;
     float q[4];
@@ -191,9 +183,9 @@ dumpMatrix(const float m[4][4])
 
     if (create) {
         create = 0;
-        fp = gnubg_g_fopen("test.txt", "w");
+        fp = g_fopen("test.txt", "w");
     } else {
-        fp = gnubg_g_fopen("test.txt", "a");
+        fp = g_fopen("test.txt", "a");
     }
     if (!fp)
         return;
