@@ -1,13 +1,10 @@
 /*
- * model.c
- * by Jon Kinsey, 2003
+ * Copyright (C) 2003-2021 Jon Kinsey <jonkinsey@gmail.com>
  *
- * Shadow model representation
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,26 +12,27 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: model.c,v 1.22 2018/04/28 21:44:35 plm Exp $
+ * $Id: model.c,v 1.32 2022/01/22 21:39:27 plm Exp $
  */
 
 #include "config.h"
-#include "inc3d.h"
+
+#include "legacyGLinc.h"
+#include "fun3d.h"
 
 #define TOP_EDGE (-2)
 
-typedef struct _position {
+typedef struct {
     float x, y, z;
 } position;
 
-typedef struct _plane {
+typedef struct {
     float a, b, c, d;
 } plane;
 
-typedef struct _winged_edge {
+typedef struct {
     unsigned int e[2];          /* vertex index */
     int w[2];                   /* plane index */
 } winged_edge;
@@ -42,12 +40,14 @@ typedef struct _winged_edge {
 void
 initOccluder(Occluder * pOcc)
 {
-    pOcc->handle = (OccModel *) malloc(sizeof(OccModel));
+    pOcc->handle = (OccModel *) g_malloc(sizeof(OccModel));
     pOcc->handle->planes = g_array_new(FALSE, FALSE, sizeof(plane));
     pOcc->handle->edges = g_array_new(FALSE, FALSE, sizeof(winged_edge));
     pOcc->handle->points = g_array_new(FALSE, FALSE, sizeof(position));
 
+#if !GTK_CHECK_VERSION(3,0,0)
     pOcc->shadow_list = glGenLists(1);
+#endif
     pOcc->rotator = 0;
     pOcc->show = 1;
 }
@@ -55,14 +55,18 @@ initOccluder(Occluder * pOcc)
 void
 freeOccluder(Occluder * pOcc)
 {
+#if !GTK_CHECK_VERSION(3,0,0)
     if (pOcc->handle) {
         g_array_free(pOcc->handle->planes, TRUE);
         g_array_free(pOcc->handle->edges, TRUE);
         g_array_free(pOcc->handle->points, TRUE);
-        free(pOcc->handle);
+        g_free(pOcc->handle);
         pOcc->handle = NULL;
         glDeleteLists(pOcc->shadow_list, 1);
     }
+#else
+    (void)pOcc; /* suppress unused parameter compiler warning */
+#endif
 }
 
 void
@@ -71,12 +75,15 @@ copyOccluder(const Occluder * fromOcc, Occluder * toOcc)
     toOcc->handle = fromOcc->handle;
     toOcc->show = fromOcc->show;
     toOcc->rotator = fromOcc->rotator;
+#if !GTK_CHECK_VERSION(3,0,0)
     toOcc->shadow_list = glGenLists(1);
+#endif
 }
 
 void
 moveToOcc(const Occluder * pOcc)
 {
+#if !GTK_CHECK_VERSION(3,0,0)
     glTranslatef(pOcc->trans[0], pOcc->trans[1], pOcc->trans[2]);
 
     if (pOcc->rotator) {
@@ -84,6 +91,9 @@ moveToOcc(const Occluder * pOcc)
         glRotatef(pOcc->rot[1], 1.f, 0.f, 0.f);
         glRotatef(pOcc->rot[2], 0.f, 0.f, 1.f);
     }
+#else
+    (void)pOcc;	/* suppress unused parameter compiler warning */
+#endif
 }
 
 static unsigned int
@@ -125,7 +135,7 @@ CreatePlane(plane * p, const position * p1, const position * p2, const position
     cr[2] = v0[0] * v1[1] - v0[1] * v1[0];
 
     l = sqrtf(cr[0] * cr[0] + cr[1] * cr[1] + cr[2] * cr[2]);
-    if (l == 0) {               /* degenerate triangle */
+    if (l <= 0) {               /* degenerate triangle */
         p->a = p->b = p->c = p->d = 0;
         return;
     }
@@ -158,6 +168,7 @@ AddPlane(GArray * planes, const position * a, const position * b, const position
 void
 GenerateShadowEdges(const Occluder * pOcc)
 {
+#if !GTK_CHECK_VERSION(3,0,0)
     unsigned int i, numEdges = pOcc->handle->edges->len;
     for (i = 0; i < numEdges; i++) {
         winged_edge *we = &g_array_index(pOcc->handle->edges, winged_edge, i);
@@ -190,11 +201,12 @@ GenerateShadowEdges(const Occluder * pOcc)
             }
         }
     }
+#endif
 }
 #endif
 
 static float
-sqdDist(const GArray * planes, int pIndex, const float point[4])
+sqdDist(const GArray * planes, const int pIndex, const float point[4])
 {
     plane *p = &g_array_index(planes, plane, pIndex);
     return (p->a * point[0] + p->b * point[1] + p->c * point[2] + p->d * point[3]);
@@ -229,6 +241,7 @@ GenerateShadowVolume(const Occluder * pOcc, const float olight[4])
             continue;
         }
 
+#if !GTK_CHECK_VERSION(3,0,0)
         {
             position *pn0 = &g_array_index(pOcc->handle->points, position, edgeOrder[0]);
             position *pn1 = &g_array_index(pOcc->handle->points, position, edgeOrder[1]);
@@ -243,6 +256,7 @@ GenerateShadowVolume(const Occluder * pOcc, const float olight[4])
             glVertex4f(pn0->x * olight[3] - olight[0],
                        pn0->y * olight[3] - olight[1], pn0->z * olight[3] - olight[2], 0.f);
         }
+#endif
     }
 }
 
@@ -456,14 +470,14 @@ addCubeCentered(Occluder * pOcc, float x, float y, float z, float w, float h, fl
 void
 addCylinder(Occluder * pOcc, float x, float y, float z, float r, float d, unsigned int numSteps)
 {
-    float step = (2 * (float) G_PI) / numSteps;
-    float *xPts = (float *) malloc(sizeof(float) * numSteps);
-    float *yPts = (float *) malloc(sizeof(float) * numSteps);
+    float step = (2 * F_PI) / (float) numSteps;
+    float *xPts = (float *) g_malloc(sizeof(float) * numSteps);
+    float *yPts = (float *) g_malloc(sizeof(float) * numSteps);
     unsigned int i;
     g_assert(xPts && yPts);
 
     for (i = 0; i < numSteps; i++) {
-        float ang = step * i + (step / 2.0f);
+        float ang = step * ((float) i + 0.5f);
         xPts[i] = sinf(ang) * r;
         yPts[i] = cosf(ang) * r;
     }
@@ -482,8 +496,8 @@ addCylinder(Occluder * pOcc, float x, float y, float z, float r, float d, unsign
         addLine(pOcc, x + xPts[i], y + yPts[i], z, x + xPts[i], y + yPts[i], z + d, x + xPts[prev], y + yPts[prev], z);
         addLine(pOcc, x + xPts[i], y + yPts[i], z + d, x + xPts[i], y + yPts[i], z, x + xPts[next], y + yPts[next], z);
     }
-    free(xPts);
-    free(yPts);
+    g_free(yPts);
+    g_free(xPts);
 }
 
 void
@@ -498,14 +512,14 @@ addHalfTube(Occluder * pOcc, float r, float h, unsigned int numSteps)
         return;
     }
 
-    step = ((2 * (float) G_PI) / numSteps) / 2.0f;
+    step = F_PI / (float) numSteps;
 
-    xPts = (float *) malloc(sizeof(float) * (numSteps + 1));
-    yPts = (float *) malloc(sizeof(float) * (numSteps + 1));
+    xPts = (float *) g_malloc(sizeof(float) * (numSteps + 1));
+    yPts = (float *) g_malloc(sizeof(float) * (numSteps + 1));
     g_assert(xPts && yPts);
 
     for (i = 0; i <= numSteps; i++) {
-        float ang = step * i - (float) G_PI_2;
+        float ang = step * (float) i - F_PI_2;
         xPts[i] = sinf(ang) * r;
         yPts[i] = cosf(ang) * r;
     }
@@ -526,8 +540,8 @@ addHalfTube(Occluder * pOcc, float r, float h, unsigned int numSteps)
     addLine(pOcc, xPts[i], 0.f, yPts[i], xPts[i], h, yPts[i], xPts[i - 1], 0.f, yPts[i - 1]);
     addLine(pOcc, xPts[i], h, yPts[i], xPts[i], 0.f, yPts[i], xPts[i], 0.f, yPts[i] - .1f);
 
-    free(xPts);
-    free(yPts);
+    g_free(yPts);
+    g_free(xPts);
 }
 
 static float
@@ -566,14 +580,14 @@ void
 addDice(Occluder * pOcc, float size)
 {                               /* Hard-coded numSteps to keep model simple + doesn't work correctly when > 8... */
     unsigned int numSteps = 8;
-    float step = (2 * (float) G_PI) / numSteps;
-    float *xPts = (float *) malloc(sizeof(float) * numSteps);
-    float *yPts = (float *) malloc(sizeof(float) * numSteps);
+    float step = (2.0f * F_PI) / (float) numSteps;
+    float *xPts = (float *) g_malloc(sizeof(float) * numSteps);
+    float *yPts = (float *) g_malloc(sizeof(float) * numSteps);
     unsigned int i, c, f;
     g_assert(xPts && yPts);
 
     for (i = 0; i < numSteps; i++) {
-        float ang = step * i;
+        float ang = step * (float) i;
         xPts[i] = sinf(ang) * size;
         yPts[i] = cosf(ang) * size;
     }
@@ -608,6 +622,6 @@ addDice(Occluder * pOcc, float size)
         }
     }
 
-    free(xPts);
-    free(yPts);
+    g_free(yPts);
+    g_free(xPts);
 }
