@@ -1,13 +1,11 @@
 /*
- * shadow.c
- * by Jon Kinsey, 2003
+ * Copyright (C) 2003-2020 Jon Kinsey <jon_kinsey@hotmail.com>
+ * Copyright (C) 2003-2018 the AUTHORS
  *
- * 3d shadow functions
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,16 +13,19 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: shadow.c,v 1.23 2018/05/01 20:25:45 plm Exp $
+ * $Id: shadow.c,v 1.34 2022/03/06 22:45:43 plm Exp $
  */
 
 #include "config.h"
-#include "inc3d.h"
+#include "legacyGLinc.h"
+#include "fun3d.h"
+#include "render.h"
 
+#if !GTK_CHECK_VERSION(3,0,0)
 static int midStencilVal;
+#endif
 
 extern int
 ShadowsInitilised(const BoardData3d * bd3d)
@@ -35,28 +36,30 @@ ShadowsInitilised(const BoardData3d * bd3d)
 void
 shadowInit(BoardData3d * bd3d, renderdata * prd)
 {
-    int i;
-    GLint stencilBits;
-
     if (bd3d->shadowsInitialised)
         return;
 
+#if !GTK_CHECK_VERSION(3,0,0)
+    int stencilBits;
+	
     /* Darkness as percentage of ambient light */
-    prd->dimness = ((prd->lightLevels[1] / 100.0f) * (100 - prd->shadowDarkness)) / 100;
+    prd->dimness = (float)(prd->lightLevels[1] * (100 - prd->shadowDarkness)) / (100.0f * 100.0f);
 
-    for (i = 0; i < NUM_OCC; i++)
-        bd3d->Occluders[i].handle = NULL;
+    initOccluders(bd3d);
 
     /* Check the stencil buffer is present */
     glGetIntegerv(GL_STENCIL_BITS, &stencilBits);
     if (!stencilBits) {
-        g_print("No stencil buffer - no shadows\n");
+        g_warning(_("No stencil buffer, no shadows\n"));
         return;
     }
-    midStencilVal = 1u << (stencilBits - 1);
+    midStencilVal = (int) (1u << (stencilBits - 1));
     glClearStencil(midStencilVal);
 
     bd3d->shadowsInitialised = TRUE;
+#else
+    (void)prd;	/* suppress unused parameter compiler warning */
+#endif
 }
 
 #ifdef TEST_HARNESS
@@ -65,13 +68,16 @@ extern void
 draw_shadow_volume_edges(Occluder * pOcc)
 {
     if (pOcc->show) {
+#if !GTK_CHECK_VERSION(3,0,0)
         glPushMatrix();
         moveToOcc(pOcc);
         GenerateShadowEdges(pOcc);
         glPopMatrix();
+#endif
     }
 }
 #endif
+
 extern void
 draw_shadow_volume_extruded_edges( /*lint -e{818} */ Occluder * pOcc, const float light_position[4], unsigned int prim)
 {
@@ -79,6 +85,7 @@ draw_shadow_volume_extruded_edges( /*lint -e{818} */ Occluder * pOcc, const floa
         float olight[4];
         mult_matrix_vec((ConstMatrix) pOcc->invMat, light_position, olight);
 
+#if !GTK_CHECK_VERSION(3,0,0)
         glNewList(pOcc->shadow_list, GL_COMPILE);
         glPushMatrix();
         moveToOcc(pOcc);
@@ -87,29 +94,13 @@ draw_shadow_volume_extruded_edges( /*lint -e{818} */ Occluder * pOcc, const floa
         glEnd();
         glPopMatrix();
         glEndList();
+#else
+        (void)prim;	/* suppress unused parameter compiler warning */
+#endif
     }
 }
 
-int renderingBase = FALSE;
-
-static void
-DrawShadows(const BoardData3d * bd3d)
-{
-    int i;
-    if (renderingBase) {
-        glCallList(bd3d->Occluders[OCC_BOARD].shadow_list);
-        if (bd3d->Occluders[OCC_HINGE1].show) {
-            glCallList(bd3d->Occluders[OCC_HINGE1].shadow_list);
-            glCallList(bd3d->Occluders[OCC_HINGE2].shadow_list);
-        }
-    } else {
-        for (i = 0; i < NUM_OCC; i++) {
-            if (bd3d->Occluders[i].show)
-                glCallList(bd3d->Occluders[i].shadow_list);
-        }
-    }
-}
-
+#if !GTK_CHECK_VERSION(3,0,0)
 static void
 draw_shadow_volume_to_stencil(const BoardData3d * bd3d)
 {
@@ -122,7 +113,7 @@ draw_shadow_volume_to_stencil(const BoardData3d * bd3d)
     glDepthMask(GL_FALSE);
 
     /* Z-pass approach */
-    glStencilFunc(GL_ALWAYS, midStencilVal, (GLuint) ~ 0u);
+    glStencilFunc(GL_ALWAYS, midStencilVal, (unsigned int) ~ 0u);
 
     glCullFace(GL_FRONT);
     glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
@@ -139,25 +130,23 @@ draw_shadow_volume_to_stencil(const BoardData3d * bd3d)
     glEnable(GL_LIGHTING);
     glDepthMask(GL_TRUE);
 }
+#endif
 
 void
-shadowDisplay(void (*drawScene) (const BoardData *, const BoardData3d *, const renderdata *), const BoardData * bd,
-              const BoardData3d * bd3d, const renderdata * prd)
+shadowDisplay(const BoardData * bd, const BoardData3d * bd3d, const renderdata * prd)
 {
-    /* Pass 1: Draw model, ambient light only (some diffuse to vary shadow darkness) */
+#if !GTK_CHECK_VERSION(3,0,0)
     float zero[4] = { 0, 0, 0, 0 };
     float d1[4];
     float specular[4];
     float diffuse[4];
 
-    drawScene(bd, bd3d, prd);
-
-    /* Create shadow volume in stencil buffer */
+	/* Create shadow volume in stencil buffer */
     glEnable(GL_STENCIL_TEST);
     draw_shadow_volume_to_stencil(bd3d);
 
-    /* Pass 2: Redraw model, full light in non-shadowed areas */
-    glStencilFunc(GL_NOTEQUAL, midStencilVal, (GLuint) ~ 0);
+    /* Pass 2: Redraw model, dim light in shadowed areas */
+    glStencilFunc(GL_NOTEQUAL, midStencilVal, (unsigned int) ~ 0);
     glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
 
     glGetLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
@@ -167,13 +156,15 @@ shadowDisplay(void (*drawScene) (const BoardData *, const BoardData3d *, const r
     glGetLightfv(GL_LIGHT0, GL_SPECULAR, specular);
     glLightfv(GL_LIGHT0, GL_SPECULAR, zero);
 
-    if (renderingBase)
-        drawScene(bd, bd3d, prd);
-    else
-        drawBoard(bd, bd3d, prd);
+	drawBoard(bd, bd3d, prd);
 
     glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
     glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
 
     glDisable(GL_STENCIL_TEST);
+#else
+    (void)bd;	/* suppress unused parameter compiler warning */
+    (void)bd3d;	/* suppress unused parameter compiler warning */
+    (void)prd;	/* suppress unused parameter compiler warning */
+#endif
 }
