@@ -1,4 +1,4 @@
-/* $Id: tr.c,v 1.10 2014/05/11 15:15:10 plm Exp $ */
+/* $Id: tr.c,v 1.17 2021/10/30 13:48:05 plm Exp $ */
 
 /*
  * $originalLog: tr.c,v $
@@ -55,48 +55,14 @@
  */
 
 #include "config.h"
-#include "inc3d.h"
+#include "legacyGLinc.h"
+#include "fun3d.h"
 #include "tr.h"
 
 #define DEFAULT_TILE_WIDTH  256
 #define DEFAULT_TILE_HEIGHT 256
 #define DEFAULT_TILE_BORDER 0
 
-
-struct _TRctx {
-    /* Final image parameters */
-    GLint ImageWidth, ImageHeight;
-    GLenum ImageFormat, ImageType;
-    GLvoid *ImageBuffer;
-
-    /* Tile parameters */
-    GLint TileWidth, TileHeight;
-    GLint TileWidthNB, TileHeightNB;
-    GLint TileBorder;
-    GLenum TileFormat, TileType;
-    GLvoid *TileBuffer;
-
-    /* Projection parameters */
-    GLboolean Perspective;
-    GLdouble Left;
-    GLdouble Right;
-    GLdouble Bottom;
-    GLdouble Top;
-    GLdouble Near;
-    GLdouble Far;
-
-    /* Misc */
-    TRenum RowOrder;
-    GLint Rows, Columns;
-    GLint CurrentTile;
-    GLint CurrentTileWidth, CurrentTileHeight;
-    GLint CurrentRow, CurrentColumn;
-
-    GLint ViewportSave[4];
-};
-
-
-
 /*
  * Misc setup including computing number of tiles (rows and columns).
  */
@@ -145,7 +111,7 @@ trDelete(TRcontext * tr)
 
 
 void
-trTileSize(TRcontext * tr, GLint width, GLint height, GLint border)
+trTileSize(TRcontext * tr, int width, int height, int border)
 {
     if (!tr)
         return;
@@ -180,7 +146,7 @@ trTileBuffer(TRcontext * tr, GLenum format, GLenum type, GLvoid * image)
 
 
 void
-trImageSize(TRcontext * tr, GLuint width, GLuint height)
+trImageSize(TRcontext * tr, unsigned int width, unsigned int height)
 {
     if (!tr)
         return;
@@ -203,7 +169,7 @@ trImageBuffer(TRcontext * tr, GLenum format, GLenum type, GLvoid * image)
 }
 
 #if 0
-GLint
+int
 trGet(const TRcontext * tr, TRenum param)
 {
     if (!tr)
@@ -239,7 +205,7 @@ trGet(const TRcontext * tr, TRenum param)
     case TR_CURRENT_TILE_HEIGHT:
         return tr->CurrentTileHeight;
     case TR_ROW_ORDER:
-        return (GLint) tr->RowOrder;
+        return (int) tr->RowOrder;
     case TR_TOP_TO_BOTTOM:
         return tr->RowOrder == TR_TOP_TO_BOTTOM;
     case TR_BOTTOM_TO_TOP:
@@ -308,8 +274,9 @@ trPerspective(TRcontext * tr, GLdouble fovy, GLdouble aspect, GLdouble znear, GL
 void
 trBeginTile(TRcontext * tr)
 {
-    GLint matrixMode;
-    GLint tileWidth, tileHeight, border;
+#if !GTK_CHECK_VERSION(3,0,0)
+    int matrixMode;
+    int tileWidth, tileHeight, border;
     GLdouble left, right, bottom, top;
 
     if (!tr)
@@ -359,10 +326,10 @@ trBeginTile(TRcontext * tr)
 
     /* compute projection parameters */
     left = tr->Left + (tr->Right - tr->Left)
-        * (int) (tr->CurrentColumn * tr->TileWidthNB - border) / tr->ImageWidth;
+        * (tr->CurrentColumn * tr->TileWidthNB - border) / tr->ImageWidth;
     right = left + (tr->Right - tr->Left) * tileWidth / tr->ImageWidth;
     bottom = tr->Bottom + (tr->Top - tr->Bottom)
-        * (int) (tr->CurrentRow * tr->TileHeightNB - border) / tr->ImageHeight;
+        * (tr->CurrentRow * tr->TileHeightNB - border) / tr->ImageHeight;
     top = bottom + (tr->Top - tr->Bottom) * tileHeight / tr->ImageHeight;
 
     if (tr->Perspective)
@@ -372,6 +339,9 @@ trBeginTile(TRcontext * tr)
 
     /* restore user's matrix mode */
     glMatrixMode((GLenum) matrixMode);
+#else
+    (void)tr;	/* suppress unused parameter compiler warning */
+#endif
 }
 
 
@@ -379,7 +349,7 @@ trBeginTile(TRcontext * tr)
 int
 trEndTile(TRcontext * tr)
 {
-    GLint prevRowLength, prevSkipRows, prevSkipPixels;  /*, prevAlignment; */
+    int prevRowLength, prevSkipRows, prevSkipPixels;  /*, prevAlignment; */
 
     if (!tr)
         return 0;
@@ -396,20 +366,20 @@ trEndTile(TRcontext * tr)
     /*glGetIntegerv(GL_PACK_ALIGNMENT, &prevAlignment); */
 
     if (tr->TileBuffer) {
-        GLint srcX = tr->TileBorder;
-        GLint srcY = tr->TileBorder;
-        GLint srcWidth = tr->TileWidthNB;
-        GLint srcHeight = tr->TileHeightNB;
+        int srcX = tr->TileBorder;
+        int srcY = tr->TileBorder;
+        int srcWidth = tr->TileWidthNB;
+        int srcHeight = tr->TileHeightNB;
         glReadPixels(srcX, srcY, srcWidth, srcHeight, tr->TileFormat, tr->TileType, tr->TileBuffer);
     }
 
     if (tr->ImageBuffer) {
-        GLint srcX = tr->TileBorder;
-        GLint srcY = tr->TileBorder;
-        GLint srcWidth = tr->CurrentTileWidth - 2 * tr->TileBorder;
-        GLint srcHeight = tr->CurrentTileHeight - 2 * tr->TileBorder;
-        GLint destX = tr->TileWidthNB * tr->CurrentColumn;
-        GLint destY = tr->TileHeightNB * tr->CurrentRow;
+        int srcX = tr->TileBorder;
+        int srcY = tr->TileBorder;
+        int srcWidth = tr->CurrentTileWidth - 2 * tr->TileBorder;
+        int srcHeight = tr->CurrentTileHeight - 2 * tr->TileBorder;
+        int destX = tr->TileWidthNB * tr->CurrentColumn;
+        int destY = tr->TileHeightNB * tr->CurrentRow;
 
         /* setup pixel store for glReadPixels */
         glPixelStorei(GL_PACK_ROW_LENGTH, tr->ImageWidth);
@@ -451,7 +421,7 @@ trRasterPos3d(const TRcontext * tr, GLdouble x, GLdouble y, GLdouble z)
         glRasterPos3d(x, y, z);
     } else {
         GLdouble modelview[16], proj[16];
-        GLint viewport[4];
+        int viewport[4];
         GLdouble winX, winY, winZ;
 
         /* Get modelview, projection and viewport */
