@@ -1,7 +1,11 @@
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * Copyright (C) 2006-2009 Jon Kinsey <jonkinsey@gmail.com>
+ * Copyright (C) 2006-2017 the AUTHORS
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -9,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: common.h,v 1.34 2017/11/02 21:16:33 plm Exp $
+ * $Id: common.h,v 1.35 2019/12/02 10:56:00 plm Exp $
  */
 
 /*! \file common.h
@@ -29,6 +32,8 @@
 #define __attribute__(Spec) /* Empty */
 #endif /* HAVE___ATTRIBUTE__ */
 
+#define F_PI (float)G_PI
+#define F_PI_2 (float)G_PI_2
 
 /*! \brief Safe error value
  */
