@@ -1,14 +1,11 @@
 /*
- * dbprovider.c
+ * Copyright (C) 2008-2009 Jon Kinsey <jonkinsey@gmail.com>
+ * Copyright (C) 2008-2019 the AUTHORS
  *
- * by Jon Kinsey, 2008
- *
- * Database providers
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: dbprovider.c,v 1.54 2017/11/02 21:59:58 plm Exp $
+ * $Id: dbprovider.c,v 1.61 2022/03/22 16:55:46 plm Exp $
  */
 
 #include "config.h"
@@ -48,12 +44,10 @@ static void PyDisconnect(void);
 static RowSet *PySelect(const char *str);
 static int PyUpdateCommand(const char *str);
 static void PyCommit(void);
-#if !defined(WIN32)
 static int PyPostgreConnect(const char *dbfilename, const char *user, const char *password, const char *hostname);
 static GList *PyPostgreGetDatabaseList(const char *user, const char *password, const char *hostname);
 static int PyPostgreDeleteDatabase(const char *dbfilename, const char *user, const char *password,
                                    const char *hostname);
-#endif
 static GList *PyMySQLGetDatabaseList(const char *user, const char *password, const char *hostname);
 static int PyMySQLDeleteDatabase(const char *dbfilename, const char *user, const char *password, const char *hostname);
 #endif
@@ -72,28 +66,26 @@ static DBProvider providers[NUM_PROVIDERS] = {
 #if defined(USE_SQLITE)
     {SQLiteConnect, SQLiteDisconnect, SQLiteSelect, SQLiteUpdateCommand, SQLiteCommit, SQLiteGetDatabaseList,
      SQLiteDeleteDatabase,
-     "SQLite", "SQLite", "Direct SQLite3 connection", FALSE, TRUE, "gnubg", "", "", ""},
+     "SQLite", "SQLite", N_("Direct SQLite3 connection"), FALSE, TRUE, "gnubg", "", "", ""},
 #endif
 #if defined(USE_PYTHON)
 #if !defined(USE_SQLITE)
     {PySQLiteConnect, PyDisconnect, PySelect, PyUpdateCommand, PyCommit, SQLiteGetDatabaseList, SQLiteDeleteDatabase,
-     "SQLite (Python)", "PythonSQLite", "SQLite3 connection included in latest Python version", FALSE, TRUE, "gnubg",
+     "SQLite (Python)", "PythonSQLite", N_("SQLite3 connection via Python"), FALSE, TRUE, "gnubg",
      "", "", ""},
 #endif
     {PyMySQLConnect, PyDisconnect, PySelect, PyUpdateCommand, PyCommit, PyMySQLGetDatabaseList, PyMySQLDeleteDatabase,
-     "MySQL (Python)", "PythonMySQL", "MySQL connection via MySQLdb Python module", TRUE, TRUE, "gnubg", "", "",
+     "MySQL (Python)", "PythonMySQL", N_("MySQL/MariaDB connection via MySQLdb Python module"), TRUE, TRUE, "gnubg", "", "",
      "localhost:3306"},
-#if !defined(WIN32)
     {PyPostgreConnect, PyDisconnect, PySelect, PyUpdateCommand, PyCommit, PyPostgreGetDatabaseList,
      PyPostgreDeleteDatabase,
-     "Postgres (Python)", "PythonPostgre", "PostgreSQL connection via PyGreSQL Python module", TRUE, TRUE, "gnubg", "",
+     "PostgreSQL (Python)", "PythonPostgre", N_("PostgreSQL connection via PyGreSQL Python module"), TRUE, TRUE, "gnubg", "",
      "", "localhost:5432"},
 #endif
-#endif
 };
 
 #else
-DBProvider providers[1] = { {0, 0, 0, 0, 0, 0, 0, "No Providers", "No Providers", "No Providers", 0, 0, 0, 0, 0, 0} };
+DBProvider providers[1] = { {0, 0, 0, 0, 0, 0, 0, "No Providers", "No Providers", N_("No database providers"), 0, 0, 0, 0, 0, 0} };
 #endif
 
 #if defined(USE_PYTHON) || defined(USE_SQLITE)
@@ -101,15 +93,14 @@ static RowSet *
 MallocRowset(size_t rows, size_t cols)
 {
     size_t i;
-    RowSet *pRow = malloc(sizeof(RowSet));
-    g_assert(pRow);
+    RowSet *pRow = g_malloc(sizeof(RowSet));
 
-    pRow->widths = (size_t *) malloc(cols * sizeof(size_t));
+    pRow->widths = (size_t *) g_malloc(cols * sizeof(size_t));
     memset(pRow->widths, 0, cols * sizeof(size_t));
 
-    pRow->data = malloc(rows * sizeof(char **));
+    pRow->data = g_malloc(rows * sizeof(char **));
     for (i = 0; i < rows; i++) {
-        pRow->data[i] = malloc(cols * sizeof(char *));
+        pRow->data[i] = g_malloc(cols * sizeof(char *));
         memset(pRow->data[i], 0, cols * sizeof(char *));
     }
 
@@ -126,8 +117,7 @@ SetRowsetData( /*lint -e{818} */ RowSet * rs, size_t row, size_t col, const char
     if (!data)
         data = "";
 
-    rs->data[row][col] = malloc(strlen(data) + 1);
-    strcpy(rs->data[row][col], data);
+    rs->data[row][col] = g_strdup(data);
 
     size = strlen(data);
     if (row == 0 || size > rs->widths[col])
@@ -139,23 +129,23 @@ extern void
 FreeRowset(RowSet * pRow)
 {
     if (pRow != NULL) {
-        free(pRow->widths);
+        g_free(pRow->widths);
 
         if (pRow->data != NULL) {
             unsigned int i, j;
 
             for (i = 0; i < pRow->rows; i++) {
                 for (j = 0; j < pRow->cols; j++) {
-                    free(pRow->data[i][j]);
+                    g_free(pRow->data[i][j]);
                 }
-            free(pRow->data[i]);
+            g_free(pRow->data[i]);
             }
         }
 
-        free(pRow->data);
+        g_free(pRow->data);
     }   
 
-    free(pRow);
+    g_free(pRow);
 }
 
 int
@@ -313,7 +303,6 @@ PyMySQLConnect(const char *dbfilename, const char *user, const char *password, c
     return 1;
 }
 
-#if !defined(WIN32)
 int
 PyPostgreConnect(const char *dbfilename, const char *user, const char *password, const char *hostname)
 {
@@ -333,7 +322,6 @@ PyPostgreConnect(const char *dbfilename, const char *user, const char *password,
     }
     return 1;
 }
-#endif
 
 #if !defined(USE_SQLITE)
 static int
@@ -468,9 +456,9 @@ ConvertPythonToRowset(PyObject * v)
                     continue;
                 }
                 if (PyUnicode_Check(e2))
-                    strcpy(buf, PyBytes_AsString(PyUnicode_AsUTF8String(e2)));
+                    g_strlcpy(buf, PyBytes_AsString(PyUnicode_AsUTF8String(e2)), sizeof(buf));
                 else if (PyBytes_Check(e2))
-                    strcpy(buf, PyBytes_AsString(e2));
+                    g_strlcpy(buf, PyBytes_AsString(e2), sizeof(buf));
                 else if (PyInt_Check(e2) || PyLong_Check(e2)
                          || !StrCaseCmp(e2->ob_type->tp_name, "Decimal"))       /* Not sure how to check for decimal type directly */
                     sprintf(buf, "%ld", PyInt_AsLong(e2));
@@ -519,7 +507,6 @@ PyMySQLGetDatabaseList(const char *user, const char *password, const char *hostn
     }
 }
 
-#if !defined(WIN32)
 GList *
 PyPostgreGetDatabaseList(const char *user, const char *password, const char *hostname)
 {
@@ -541,7 +528,6 @@ PyPostgreGetDatabaseList(const char *user, const char *password, const char *hos
         return NULL;
     }
 }
-#endif
 
 int
 PyMySQLDeleteDatabase(const char *dbfilename, const char *user, const char *password, const char *hostname)
@@ -557,7 +543,6 @@ PyMySQLDeleteDatabase(const char *dbfilename, const char *user, const char *pass
     return ret;
 }
 
-#if !defined(WIN32)
 int
 PyPostgreDeleteDatabase(const char *dbfilename, const char *user, const char *password, const char *hostname)
 {
@@ -572,7 +557,6 @@ PyPostgreDeleteDatabase(const char *dbfilename, const char *user, const char *pa
     g_free(buf);
     return ret;
 }
-#endif
 
 #endif
 
@@ -614,21 +598,31 @@ SQLiteDisconnect(void)
 RowSet *
 SQLiteSelect(const char *str)
 {
-    size_t i, row;
     int ret;
-    char *buf = g_strdup_printf("Select %s;", str);
+    char *buf = g_strdup_printf("SELECT %s;", str);
     RowSet *rs = NULL;
 
     sqlite3_stmt *pStmt;
+#if SQLITE_VERSION_NUMBER >= 3003011
+    ret = sqlite3_prepare_v2(connection, buf, -1, &pStmt, NULL);
+#else
     ret = sqlite3_prepare(connection, buf, -1, &pStmt, NULL);
+#endif
     g_free(buf);
     if (ret == SQLITE_OK) {
+        size_t i, row;
         size_t numCols = (size_t) sqlite3_column_count(pStmt);
         size_t numRows = 0;
+
         while ((ret = sqlite3_step(pStmt)) == SQLITE_ROW)
             numRows++;
+
+        if (ret != SQLITE_DONE)
+            outputerrf("SQL error: %s in sqlite3_step()", sqlite3_errmsg(connection));
+
         if (sqlite3_reset(pStmt) != SQLITE_OK)
             outputerrf("SQL error: %s in sqlite3_reset()", sqlite3_errmsg(connection));
+
         rs = MallocRowset(numRows + 1, numCols);        /* first row is headings */
 
         for (i = 0; i < numCols; i++)
