@@ -1,11 +1,11 @@
 /*
- * dice.c
+ * Copyright (C) 1999-2003 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 1999-2019 the AUTHORS
  *
- * by Gary Wong, 1999
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: dice.c,v 1.99 2017/12/13 21:25:21 plm Exp $
+ * $Id: dice.c,v 1.106 2021/07/04 22:38:44 plm Exp $
  */
 
 #include "config.h"
@@ -76,7 +75,7 @@ const char *aszRNGTip[NUM_RNGS] = {
 rng rngCurrent = RNG_MERSENNE;
 rngcontext *rngctxCurrent = NULL;
 
-struct _rngcontext {
+struct rngcontext {
 
     /* RNG_FILE */
     FILE *fDice;
@@ -513,16 +512,16 @@ InitRNGSeedMP(mpz_t n, rng rng, rngcontext * rngctx)
     switch (rng) {
 
     case RNG_MERSENNE:{
-            gint32 *achState;
-            unsigned long tempmtkey[MT_ARRAY_N];
-            size_t cb;
-            unsigned int i;
-
             if (mpz_cmp_ui(n, UINT_MAX) > 0) {
+                gint32 *achState;
+                unsigned long tempmtkey[MT_ARRAY_N];
+                size_t cb;
+                unsigned int i;
 
                 achState = mpz_export(NULL, &cb, -1, sizeof(gint32), 0, 0, n);
                 for (i = 0; i < MT_ARRAY_N && i < cb; i++) {
-                    tempmtkey[i] = achState[i];
+                    /* FIXME?  Initializing an unsigned long with a guint32 */
+                    tempmtkey[i] = (unsigned long) achState[i];
                 }
                 for (; i < MT_ARRAY_N; i++) {
                     tempmtkey[i] = 0;
@@ -612,6 +611,7 @@ CloseRNG(const rng rngx, rngcontext * rngctx)
     case RNG_FILE:
         /* close file */
         CloseDiceFile(rngctx);
+        break;
 
     default:
         /* no-op */
@@ -630,40 +630,46 @@ RNGSystemSeed(const rng rngx, void *p, unsigned long *pnSeed)
     unsigned int n = 0;
 
 #if defined(HAVE_LIBGMP)
-#if !defined(WIN32)
-    int h;
-#endif
     if (!pnSeed) {
 #if defined(WIN32)
         /* Can be amended to support seeds > 32 bit */
         guint32 achState;
-        mpz_t n;
+        mpz_t mpzn;
 
+#if GLIB_CHECK_VERSION (2,28,0)
+        gint64 tv;
+        tv = g_get_real_time();
+        achState = (unsigned int) (((guint64)tv >> 32) ^ ((guint64)tv & 0xFFFFFFFF));
+#else
         GTimeVal tv;
         g_get_current_time(&tv);
         achState = (unsigned int) tv.tv_sec ^ (unsigned int) tv.tv_usec;
+#endif
 
-        mpz_init(n);
-        mpz_import(n, 1, -1, sizeof(guint32), 0, 0, &achState);
-        InitRNGSeedMP(n, rngx, rngctx);
-        mpz_clear(n);
+        mpz_init(mpzn);
+        mpz_import(mpzn, 1, -1, sizeof(guint32), 0, 0, &achState);
+        InitRNGSeedMP(mpzn, rngx, rngctx);
+        mpz_clear(mpzn);
 
         return TRUE;
 #else
         /* We can use long seeds and don't have to save the seed anywhere,
          * so try 512 bits of state instead of 32. */
+
+        int h;
+
         if ((h = open("/dev/urandom", O_RDONLY)) >= 0) {
             char achState[64];
 
             if (read(h, achState, 64) == 64) {
-                mpz_t n;
+                mpz_t mpzn;
 
                 close(h);
 
-                mpz_init(n);
-                mpz_import(n, 16, -1, 4, 0, 0, achState);
-                InitRNGSeedMP(n, rngx, rngctx);
-                mpz_clear(n);
+                mpz_init(mpzn);
+                mpz_import(mpzn, 16, -1, 4, 0, 0, achState);
+                InitRNGSeedMP(mpzn, rngx, rngctx);
+                mpz_clear(mpzn);
 
                 return TRUE;
             } else
@@ -684,9 +690,15 @@ RNGSystemSeed(const rng rngx, void *p, unsigned long *pnSeed)
 #endif
 
     if (!f) {
+#if GLIB_CHECK_VERSION (2,28,0)
+        gint64 tv;
+        tv = g_get_real_time();
+        n = (unsigned int) (((guint64)tv >> 32) ^ ((guint64)tv & 0xFFFFFFFF));
+#else
         GTimeVal tv;
         g_get_current_time(&tv);
         n = (unsigned int) tv.tv_sec ^ (unsigned int) tv.tv_usec;
+#endif
     }
 
     InitRNGSeed(n, rngx, rngctx);
@@ -792,8 +804,8 @@ RollDice(unsigned int anDice[2], rng * prng, rngcontext * rngctx)
                 rngctx->nMD5++; /* useful ? indispensable ? */
             }
 
-            anDice[0] = h.an[0] / exp232_q + 1;
-            anDice[1] = h.an[1] / exp232_q + 1;
+            anDice[0] = (unsigned int) (h.an[0] / exp232_q + 1);
+            anDice[1] = (unsigned int) (h.an[1] / exp232_q + 1);
 
             rngctx->nMD5++;
             rngctx->c += 2;
@@ -825,6 +837,8 @@ RollDice(unsigned int anDice[2], rng * prng, rngcontext * rngctx)
         anDice[1] = ReadDiceFile(rngctx);
         rngctx->c += 2;
 
+        break;
+
     default:
         break;
 
@@ -843,7 +857,7 @@ OpenDiceFile(rngcontext * rngctx, const char *sz)
     g_free(rngctx->szDiceFilename);     /* initialized to NULL */
     rngctx->szDiceFilename = g_strdup(sz);
 
-    return (rngctx->fDice = gnubg_g_fopen(sz, "r"));
+    return (rngctx->fDice = g_fopen(sz, "r"));
 }
 
 static unsigned int
@@ -853,6 +867,9 @@ ReadDiceFile(rngcontext * rngctx)
     unsigned char uch;
     size_t n;
 
+    if (rngctx->fDice == NULL)
+        return (unsigned int) (-1);
+
   uglyloop:
     {
 
@@ -883,7 +900,7 @@ GetDiceFileName(rngcontext * rngctx)
 rngcontext *
 CopyRNGContext(rngcontext * rngctx)
 {
-    rngcontext *newCtx = (rngcontext *) malloc(sizeof(rngcontext));
+    rngcontext *newCtx = (rngcontext *) g_malloc(sizeof(rngcontext));
     *newCtx = *rngctx;
     return newCtx;
 }
