@@ -1,11 +1,11 @@
 /*
- * dice.h
+ * Copyright (C) 1999-2002 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 1999-2016 the AUTHORS
  *
- * by Gary Wong, 1999
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: dice.h,v 1.36 2018/06/16 14:57:41 plm Exp $
+ * $Id: dice.h,v 1.38 2021/07/04 22:17:54 plm Exp $
  */
 
 #ifndef DICE_H
@@ -24,13 +23,13 @@
 
 #include <stdio.h>
 
-typedef enum _rng {
+typedef enum {
     RNG_BBS, RNG_ISAAC, RNG_MD5, RNG_MERSENNE,
     RNG_MANUAL, RNG_RANDOM_DOT_ORG, RNG_FILE,
     NUM_RNGS
 } rng;
 
-typedef struct _rngcontext rngcontext;
+typedef struct rngcontext rngcontext;
 
 extern const char *aszRNG[NUM_RNGS];
 extern const char *aszRNGTip[NUM_RNGS];
