@@ -1,11 +1,11 @@
 /*
- * drawboard.c
+ * Copyright (C) 1999-2002 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2000-2020 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 1999, 2000, 2001, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: drawboard.c,v 1.78 2017/02/19 15:43:26 plm Exp $
+ * $Id: drawboard.c,v 1.83 2021/09/20 21:08:25 plm Exp $
  */
 
 #include "config.h"
@@ -61,7 +60,6 @@ DrawBoardStd(char *sz, const TanBoard anBoard, int fRoll, char *asz[], char *szM
     char *pch = sz, *pchIn;
     unsigned int x, y;
     unsigned int cOffO = nChequers, cOffX = nChequers;
-    TanBoard an;
     static char achX[17] = "     X6789ABCDEF", achO[17] = "     O6789ABCDEF";
 
     for (x = 0; x < 25; x++) {
@@ -74,6 +72,8 @@ DrawBoardStd(char *sz, const TanBoard anBoard, int fRoll, char *asz[], char *szM
     if (fRoll)
         strcpy(pch, PositionID((ConstTanBoard) anBoard));
     else {
+        TanBoard an;
+
         for (x = 0; x < 25; x++) {
             an[0][x] = anBoard[1][x];
             an[1][x] = anBoard[0][x];
@@ -82,7 +82,7 @@ DrawBoardStd(char *sz, const TanBoard anBoard, int fRoll, char *asz[], char *szM
         strcpy(pch, PositionID((ConstTanBoard) an));
     }
 
-    pch += 14;
+    pch += L_POSITIONID;
     *pch++ = '\n';
 
     /* match id */
@@ -284,7 +284,6 @@ DrawBoardCls(char *sz, const TanBoard anBoard, int fRoll, char *asz[], char *szM
 
     char *pch = sz, *pchIn;
     unsigned int x, y, cOffO = nChequers, cOffX = nChequers;
-    TanBoard an;
     static char achX[17] = "     X6789ABCDEF", achO[17] = "     O6789ABCDEF";
 
     for (x = 0; x < 25; x++) {
@@ -297,6 +296,8 @@ DrawBoardCls(char *sz, const TanBoard anBoard, int fRoll, char *asz[], char *szM
     if (fRoll)
         strcpy(pch, PositionID((ConstTanBoard) anBoard));
     else {
+        TanBoard an;
+
         for (x = 0; x < 25; x++) {
             an[0][x] = anBoard[1][x];
             an[1][x] = anBoard[0][x];
@@ -305,7 +306,7 @@ DrawBoardCls(char *sz, const TanBoard anBoard, int fRoll, char *asz[], char *szM
         strcpy(pch, PositionID((ConstTanBoard) an));
     }
 
-    pch += 14;
+    pch += L_POSITIONID;
     *pch++ = '\n';
 
     /* match id */
@@ -507,9 +508,9 @@ FormatPoint(char *pch, int n)
         strcpy(pch, ("bar"));
         return pch + strlen(("bar"));
     } else if (n > 9)
-        *pch++ = (char) (n / 10) + '0';
+        *pch++ = (char) ((n / 10) + '0');
 
-    *pch++ = (n % 10) + '0';
+    *pch++ = (char) ((n % 10) + '0');
 
     return pch;
 }
@@ -521,15 +522,15 @@ FormatPointPlain(char *pch, int n)
     g_assert(n >= 0);
 
     if (n > 9)
-        *pch++ = (char) (n / 10) + '0';
+        *pch++ = (char) ((n / 10) + '0');
 
-    *pch++ = (n % 10) + '0';
+    *pch++ = (char) ((n % 10) + '0');
 
     return pch;
 }
 
 extern char *
-FormatMovePlain(char *sz, TanBoard anBoard, int anMove[8])
+FormatMovePlain(char *sz, const TanBoard anBoard, const int anMove[8])
 {
 
     char *pch = sz;
@@ -582,7 +583,7 @@ CanonicalMoveOrder(int an[])
 }
 
 extern char *
-FormatMove(char *sz, const TanBoard anBoard, int anMove[8])
+FormatMove(char *sz, const TanBoard anBoard, const int anMove[8])
 {
 
     char *pch = sz;
@@ -702,7 +703,7 @@ FormatMove(char *sz, const TanBoard anBoard, int anMove[8])
 
         if (anCount[i] > 1) {
             *pch++ = '(';
-            *pch++ = '0' + (char) anCount[i];
+            *pch++ = (char) ('0' + anCount[i]);
             *pch++ = ')';
         }
     }
@@ -720,7 +721,7 @@ ParseMove(char *pch, int an[8])
      * when instrumented by clang's address sanitizer. Gcc 5 complains
      * ("array subscript is above array bounds") until it is raised to 12
      * then gcc 6 does the same until the size is 17.
-     */ 
+     */
     int i, j, iBegin, iEnd, n, c = 0, anUser[17];
     unsigned fl = 0;
 
@@ -908,21 +909,19 @@ ParseMove(char *pch, int an[8])
  *
  * (see <URL:http://www.fibs.com/fibs_interface.html#board_state>)
  *
- * gnubg extends the meaning of the "doubled" field, so the sign of
+ * GNUbg extends the meaning of the "doubled" field, so the sign of
  * the field indicates who doubled. This is necessary to correctly
  * detects beavers and raccoons.
- *
  */
 
 extern char *
 FIBSBoard(char *pch, TanBoard anBoard, int fRoll,
           const char *szPlayer, const char *szOpp, int nMatchTo,
           int nScore, int nOpponent, int nDice0, int nDice1,
-          int nCube, int fCubeOwner, int fDoubled, int fTurn, int fCrawford, int nChequers, int UNUSED(fPostCrawford))
+          int nCube, int fCubeOwner, int fDoubled, int fTurn, int fCrawford, int nChequers, int fPostCrawford)
 {
     char *sz = pch;
     int i, anOff[2];
-    /* int fNonCrawford = !(!fPostCrawford && (nScore == nMatchTo - 1 || nOpponent == nMatchTo - 1)); */
 
     /* Names and match length/score */
     strcpy(sz, "board:");
@@ -957,21 +956,9 @@ FIBSBoard(char *pch, TanBoard anBoard, int fRoll,
     }
 
     sprintf(strchr(sz, 0), "%d:%d:%d:%d:%d:%d:%d:%d:1:-1:0:25:%d:%d:0:0:0:"
-            "0:%d:0", nDice0, nDice1, nDice0, nDice1, fTurn < 0 ? 1 : nCube,
+            "0:%d:%d", nDice0, nDice1, nDice0, nDice1, fTurn < 0 ? 1 : nCube,
             fTurn < 0 || fCubeOwner != 0, fTurn < 0 || fCubeOwner != 1,
-            fDoubled ? (fTurn ? -1 : 1) : 0, anOff[1], anOff[0], fCrawford);
-
-#if 0
-/*  Temporarily remove this code as it breaks the GUI by preventing a player
-    from hitting an opponents blot. It appears to the user as an illegal play
-    Reported by Wolfgang Nelles */
-
-    sprintf(strchr(sz, 0), "%d:%d:%d:%d:%d:%d:%d:%d:1:-1:%d:%d:%d:%d:0:0:0:"
-            "0:%d:0", nDice0, nDice1, nDice0, nDice1, fTurn < 0 ? 1 : nCube,
-            fTurn < 0 || fCubeOwner != 0, fTurn < 0 || fCubeOwner != 1,
-            fDoubled ? (fTurn ? -1 : 1) : 0, fNonCrawford, fPostCrawford,
-            anOff[1], anOff[0], fCrawford);
-#endif
+            fDoubled ? (fTurn ? -1 : 1) : 0, anOff[1], anOff[0], fPostCrawford, !fCrawford);
 
     return pch;
 }
@@ -981,7 +968,7 @@ ProcessFIBSBoardInfo(FIBSBoardInfo * brdInfo, ProcessedFIBSBoard * procBrd)
 {
 
     int i, n, fCanDouble, fOppCanDouble, anOppDice[2];
-    int nTmp, fNonCrawford, fPostCrawford;
+    int fNonCrawford, fPostCrawford;
     int nTurn, nColor, nDirection;
     int anFIBSBoard[26];
     int fMustSwap = 0;
@@ -1058,6 +1045,8 @@ ProcessFIBSBoardInfo(FIBSBoardInfo * brdInfo, ProcessedFIBSBoard * procBrd)
 
     /* Opponent's turn? */
     if (fMustSwap) {
+        int nTmp;
+
         nTmp = procBrd->nScore;
         procBrd->nScore = procBrd->nScoreOpp;
         procBrd->nScoreOpp = nTmp;
@@ -1125,7 +1114,7 @@ ProcessFIBSBoardInfo(FIBSBoardInfo * brdInfo, ProcessedFIBSBoard * procBrd)
      *
      * Since we cannot find out whether the Crawford rule is in use or not
      * in the pre-Crawford games, the cubeful evaluation can be slightly
-     * biased.  But since we the vast majority of matches on FIBS is played
+     * biased.  But since the vast majority of matches on FIBS is played
      * with the Crawford rule and we assume usage of the Crawford rule
      * as a default, the bias is negligible.
      */
