@@ -1,11 +1,11 @@
 /*
- * drawboard.h
+ * Copyright (C) 1999-2001 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2000-2015 the AUTHORS
  *
- * by Gary Wong <gary@cs.arizona.edu>, 1998-2001.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: drawboard.h,v 1.24 2015/06/26 06:39:37 mdpetch Exp $
+ * $Id: drawboard.h,v 1.27 2021/09/20 21:08:25 plm Exp $
  */
 
 #ifndef DRAWBOARD_H
@@ -31,8 +30,9 @@ extern char *DrawBoard(char *pch, const TanBoard anBoard, int fRoll, char *asz[]
 /* Fill the buffer pch with a representation of the move anMove, assuming
  * the board looks like anBoard.  pch must have room for 28 characters plus
  * a trailing 0 (consider the move `bar/24* 23/22* 21/20* 19/18*'). */
-extern char *FormatMove(char *pch, const TanBoard anBoard, int anMove[8]);
-extern char *FormatMovePlain(char *pch, TanBoard anBoard, int anMove[8]);
+#define FORMATEDMOVESIZE 29
+extern char *FormatMove(char *pch, const TanBoard anBoard, const int anMove[8]);
+extern char *FormatMovePlain(char *pch, const TanBoard anBoard, const int anMove[8]);
 extern int ParseMove(char *pch, int an[8]);
 extern void CanonicalMoveOrder(int an[]);
 /* Fill the buffer pch with a FIBS "boardstyle 3" description of the game. */
