@@ -1,11 +1,11 @@
 /*
- * export.c
+ * Copyright (C) 2002-2003 Joern Thyssen <jthyssen@dk.ibm.com>
+ * Copyright (C) 2002-2019 the AUTHORS
  *
- * by Joern Thyssen  <jthyssen@dk.ibm.com>, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: export.c,v 1.90 2018/06/15 20:04:18 plm Exp $
+ * $Id: export.c,v 1.98 2022/01/02 22:57:51 plm Exp $
  */
 
 #include "config.h"
@@ -27,7 +26,6 @@
 #include <glib/gprintf.h>
 #include <stdlib.h>
 #include <string.h>
-#include "glib-ext.h"
 
 #if defined(HAVE_LIBPNG)
 #include <png.h>
@@ -75,11 +73,11 @@ filename_from_iGame(const char *szBase, const int iGame)
     else {
         char *sz = g_malloc(strlen(szBase) + 5);
         char *szExtension = strrchr(szBase, '.');
-        char *pc;
         if (!szExtension) {
             sprintf(sz, "%s_%03d", szBase, iGame + 1);
             return sz;
         } else {
+            char *pc;
             strcpy(sz, szBase);
             pc = strrchr(sz, '.');
             sprintf(pc, "_%03d%s", iGame + 1, szExtension);
@@ -178,10 +176,8 @@ export_boards(moverecord * pmr, matchstate * msExport, int *iMove, int iGame, ca
     FixMatchState(msExport, pmr);
     switch (pmr->mt) {
     case MOVE_NORMAL:
-        if (pmr->fPlayer != msExport->fMove) {
+        if (pmr->fPlayer != msExport->fMove)
             SwapSides(msExport->anBoard);
-            msExport->fMove = pmr->fPlayer;
-        }
         msExport->fTurn = msExport->fMove = pmr->fPlayer;
         msExport->anDice[0] = pmr->anDice[0];
         msExport->anDice[1] = pmr->anDice[1];
@@ -225,7 +221,7 @@ draw_cairo_pages(cairo_t * cairo, listOLD * game_ptr)
     psc = &pmr->g.sc;
     AddStatcontext(psc, &scTotal);
     iGame = getGameNumber(game_ptr);
-    if (game_is_last(plGame))
+    if (game_is_last(game_ptr))
         pl_hint = game_add_pmr_hint(game_ptr);
     for (pl = pl->plNext, page = 1; pl != game_ptr; pl = pl->plNext, page++) {
         export_boards(pl->p, &msExport, &iMove, iGame, cairo, SIZE_2PERPAGE);
@@ -256,7 +252,6 @@ CommandExportPositionSVG(char *sz)
     int game_nr;
     moverecord *pmr;
     cairo_surface_t *surface;
-    cairo_t *cairo;
 
     if (!CheckGameExists())
         return;
@@ -264,16 +259,19 @@ CommandExportPositionSVG(char *sz)
     filename = export_get_filename(sz);
     if (!filename)
         return;
+
     pmr = export_get_moverecord(&game_nr, &move_nr);
     if (!pmr) {
         g_free(filename);
         outputerrf(_("Cannot create export for this move"));
         return;
     }
+
     surface = cairo_svg_surface_create(filename, SIMPLE_BOARD_WIDTH, SIMPLE_BOARD_HEIGHT / 2.0);
     g_free(filename);
+
     if (surface) {
-        cairo = cairo_create(surface);
+        cairo_t *cairo = cairo_create(surface);
         draw_simple_board_on_cairo(&ms, pmr, move_nr, game_nr, cairo, SIZE_1PERPAGE);
         cairo_surface_destroy(surface);
         cairo_destroy(cairo);
@@ -291,7 +289,6 @@ CommandExportPositionPDF(char *sz)
     int game_nr;
     moverecord *pmr;
     cairo_surface_t *surface;
-    cairo_t *cairo;
 
     if (!CheckGameExists())
         return;
@@ -299,16 +296,20 @@ CommandExportPositionPDF(char *sz)
     filename = export_get_filename(sz);
     if (!filename)
         return;
+
     pmr = export_get_moverecord(&game_nr, &move_nr);
     if (!pmr) {
         g_free(filename);
         outputerrf(_("Cannot create export for this move"));
         return;
     }
+
     surface = cairo_pdf_surface_create(filename, SIMPLE_BOARD_WIDTH, SIMPLE_BOARD_HEIGHT);
     g_free(filename);
+
     if (surface) {
-        cairo = cairo_create(surface);
+        cairo_t *cairo = cairo_create(surface);
+
         draw_simple_board_on_cairo(&ms, pmr, move_nr, game_nr, cairo, SIZE_1PERPAGE);
         cairo_surface_destroy(surface);
         cairo_destroy(cairo);
@@ -326,7 +327,6 @@ CommandExportPositionPS(char *sz)
     int game_nr;
     moverecord *pmr;
     cairo_surface_t *surface;
-    cairo_t *cairo;
 
     if (!CheckGameExists())
         return;
@@ -334,6 +334,7 @@ CommandExportPositionPS(char *sz)
     filename = export_get_filename(sz);
     if (!filename)
         return;
+
     pmr = export_get_moverecord(&game_nr, &move_nr);
     if (!pmr) {
         g_free(filename);
@@ -343,8 +344,10 @@ CommandExportPositionPS(char *sz)
 
     surface = cairo_ps_surface_create(filename, SIMPLE_BOARD_WIDTH, SIMPLE_BOARD_HEIGHT);
     g_free(filename);
+
     if (surface) {
-        cairo = cairo_create(surface);
+        cairo_t *cairo = cairo_create(surface);
+
         draw_simple_board_on_cairo(&ms, pmr, move_nr, game_nr, cairo, SIZE_1PERPAGE);
         cairo_surface_destroy(surface);
         cairo_destroy(cairo);
@@ -359,7 +362,6 @@ CommandExportGamePDF(char *sz)
 #if defined(HAVE_PANGOCAIRO)
     gchar *filename;
     cairo_surface_t *surface;
-    cairo_t *cairo;
 
     if (!CheckGameExists())
         return;
@@ -369,8 +371,10 @@ CommandExportGamePDF(char *sz)
         return;
     surface = cairo_pdf_surface_create(filename, SIMPLE_BOARD_WIDTH, SIMPLE_BOARD_HEIGHT);
     g_free(filename);
+
     if (surface) {
-        cairo = cairo_create(surface);
+        cairo_t *cairo = cairo_create(surface);
+
         draw_cairo_pages(cairo, plGame);
         cairo_surface_destroy(surface);
         cairo_destroy(cairo);
@@ -385,7 +389,6 @@ CommandExportGamePS(char *sz)
 #if defined(HAVE_PANGOCAIRO)
     gchar *filename;
     cairo_surface_t *surface;
-    cairo_t *cairo;
 
     if (!CheckGameExists())
         return;
@@ -395,8 +398,10 @@ CommandExportGamePS(char *sz)
         return;
     surface = cairo_ps_surface_create(filename, SIMPLE_BOARD_WIDTH, SIMPLE_BOARD_HEIGHT);
     g_free(filename);
+
     if (surface) {
-        cairo = cairo_create(surface);
+        cairo_t *cairo = cairo_create(surface);
+
         draw_cairo_pages(cairo, plGame);
         cairo_surface_destroy(surface);
         cairo_destroy(cairo);
@@ -410,11 +415,8 @@ CommandExportMatchPDF(char *sz)
 {
 #if defined(HAVE_PANGOCAIRO)
     listOLD *pl;
-    int nGames = 0;
-    int i;
     char *filename;
     cairo_surface_t *surface;
-    cairo_t *cairo;
 
     filename = export_get_filename(sz);
 
@@ -422,7 +424,12 @@ CommandExportMatchPDF(char *sz)
         return;
     surface = cairo_pdf_surface_create(filename, SIMPLE_BOARD_WIDTH, SIMPLE_BOARD_HEIGHT);
     g_free(filename);
+
     if (surface) {
+        cairo_t *cairo;
+        int nGames = 0;
+        int i;
+
         for (pl = lMatch.plNext; pl != &lMatch; pl = pl->plNext, nGames++);
 
         cairo = cairo_create(surface);
@@ -441,11 +448,8 @@ CommandExportMatchPS(char *sz)
 {
 #if defined(HAVE_PANGOCAIRO)
     listOLD *pl;
-    int nGames = 0;
-    int i;
     char *filename;
     cairo_surface_t *surface;
-    cairo_t *cairo;
 
     filename = export_get_filename(sz);
 
@@ -453,7 +457,12 @@ CommandExportMatchPS(char *sz)
         return;
     surface = cairo_ps_surface_create(filename, SIMPLE_BOARD_WIDTH, SIMPLE_BOARD_HEIGHT);
     g_free(filename);
+
     if (surface) {
+        cairo_t *cairo;
+        int nGames = 0;
+        int i;
+
         for (pl = lMatch.plNext; pl != &lMatch; pl = pl->plNext, nGames++);
 
         cairo = cairo_create(surface);
@@ -469,7 +478,7 @@ CommandExportMatchPS(char *sz)
 
 #if defined(HAVE_LIBPNG)
 
-/* size of html images in steps of BOARD_WIDTH x BOARD_HEIGHT
+/* size of HTML images in steps of BOARD_WIDTH x BOARD_HEIGHT
  * as defined in boarddim.h */
 
 extern int
@@ -480,9 +489,8 @@ WritePNG(const char *sz, unsigned char *puch, unsigned int nStride, unsigned int
     png_structp ppng;
     png_infop pinfo;
     png_text atext[3];
-    unsigned int i;
 
-    if (!(pf = gnubg_g_fopen(sz, "wb")))
+    if (!(pf = g_fopen(sz, "wb")))
         return -1;
 
     if (!(ppng = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL))) {
@@ -515,7 +523,7 @@ WritePNG(const char *sz, unsigned char *puch, unsigned int nStride, unsigned int
     atext[0].text = "Backgammon board";
     atext[0].compression = PNG_TEXT_COMPRESSION_NONE;
 
-    atext[1].key = "author";
+    atext[1].key = "Author";
     atext[1].text = VERSION_STRING;
     atext[1].compression = PNG_TEXT_COMPRESSION_NONE;
 
@@ -528,7 +536,9 @@ WritePNG(const char *sz, unsigned char *puch, unsigned int nStride, unsigned int
     png_write_info(ppng, pinfo);
 
     {
-        png_bytep *aprow = (png_bytep *) g_alloca(nSizeY * sizeof(png_bytep));;
+        png_bytep *aprow = (png_bytep *) g_alloca(nSizeY * sizeof(png_bytep));
+        unsigned int i;
+
         for (i = 0; i < nSizeY; ++i)
             aprow[i] = puch + nStride * i;
 
@@ -809,7 +819,7 @@ CommandExportPositionSnowieTxt(char *sz)
 
     if (!strcmp(sz, "-"))
         pf = stdout;
-    else if (!(pf = gnubg_g_fopen(sz, "w"))) {
+    else if (!(pf = g_fopen(sz, "w"))) {
         outputerr(sz);
         return;
     }
@@ -864,7 +874,7 @@ CommandExportPositionJF(char *sz)
 
     if (!strcmp(sz, "-"))
         fp = stdout;
-    else if (!(fp = gnubg_g_fopen(sz, "wb"))) {
+    else if (!(fp = g_fopen(sz, "wb"))) {
         outputerr(sz);
         return;
     }
@@ -1043,7 +1053,15 @@ ExportGameJF(FILE * pf, listOLD * plGame, int iGame, int withScore, int fSst)
     InitBoard(anBoard, ms.bgv);
 
     for (pl = plGame->plNext; pl != plGame; pl = pl->plNext) {
+        if (pl == NULL) {
+            g_assert_not_reached();
+            break;
+        }
         pmr = pl->p;
+        if (pmr == NULL) {
+            g_assert_not_reached();
+            break;
+        }
         switch (pmr->mt) {
         case MOVE_GAMEINFO:
             if (withScore) {
@@ -1060,24 +1078,23 @@ ExportGameJF(FILE * pf, listOLD * plGame, int iGame, int withScore, int fSst)
                 fprintf(pf, " %-31s%s\n", ap[0].szName, ap[1].szName);
             msExport.fCubeOwner = -1;
             /* FIXME what about automatic doubles? */
-            continue;	/* branch out of the switch and continue for (pl ...) */
+            continue;           /* branch out of the switch and continue for (pl ...) */
         case MOVE_NORMAL:
             diceRolled = 0;
             sprintf(sz, "%u%u: ", pmr->anDice[0], pmr->anDice[1]);
             if (fSst) {         /* Snowie standard text */
-                moverecord *pnextmr;
                 if (pl->plNext && pl->plNext->p) {
-                    pnextmr = pl->plNext->p;
+                    moverecord *pnextmr = pl->plNext->p;
                     if (pnextmr->mt == MOVE_SETBOARD)
                         /* Illegal move entered in gnubg as bogus move followed by
                          * editing position : don't export the move, only the dice */
                         diceRolled = 1;
                     else
-                        FormatMovePlain(sz + 4, anBoard, pmr->n.anMove);
+                        FormatMovePlain(sz + 4, (ConstTanBoard)anBoard, pmr->n.anMove);
                 } else
-                    FormatMovePlain(sz + 4, anBoard, pmr->n.anMove);
+                    FormatMovePlain(sz + 4, (ConstTanBoard)anBoard, pmr->n.anMove);
             } else
-                FormatMovePlain(sz + 4, anBoard, pmr->n.anMove);
+                FormatMovePlain(sz + 4, (ConstTanBoard)anBoard, pmr->n.anMove);
             ApplyMove(anBoard, pmr->n.anMove, FALSE);
             SwapSides(anBoard);
             /*   if (( sz[ strlen(sz)-1 ] == ' ') && (strlen(sz) > 5 ))
@@ -1150,23 +1167,36 @@ ExportGameJF(FILE * pf, listOLD * plGame, int iGame, int withScore, int fSst)
                 SwapSides(anBoard);
 
             if (fSst) {         /* Snowie standard text */
-                moverecord *pnextmr = pl->plNext->p;
+                moverecord *pnextmr;
                 char *ct;
 
-                msExport.nMatchTo = ms.nMatchTo;
-                msExport.nCube = nFileCube;
-                msExport.fMove = (i & 1);
-                msExport.fTurn = (i & 1);
-                PositionFromKey(msExport.anBoard, &pmr->sb.key);
-                msExport.anDice[0] = pnextmr->anDice[0];
-                msExport.anDice[1] = pnextmr->anDice[1];
-                if (i & 1)
-                    SwapSides(msExport.anBoard);
-                ExportSnowieTxt(buffer, &msExport);
+                if (pl->plNext == NULL) {
+                    fprintf(pf, "Unexportable play (%s)\n", buffer);
+                    g_assert_not_reached();
+                } else if (pl->plNext->p == NULL) {
+                    fprintf(pf, "Unexportable play (%s)\n", buffer);
+                    g_assert_not_reached();
+                } else {
+                    pnextmr = pl->plNext->p;
+                    msExport.nMatchTo = ms.nMatchTo;
+                    msExport.nCube = nFileCube;
+                    msExport.fMove = (i & 1);
+                    msExport.fTurn = (i & 1);
+                    PositionFromKey(msExport.anBoard, &pmr->sb.key);
+                    msExport.anDice[0] = pnextmr->anDice[0];
+                    msExport.anDice[1] = pnextmr->anDice[1];
+                    if (i & 1)
+                        SwapSides(msExport.anBoard);
+                    ExportSnowieTxt(buffer, &msExport);
 
-                /* I don't understand why we need to swap this field! */
-                ct = strstr(buffer, ";");
-                ct[7] = (ct[7] == '0') ? '1' : '0';
+                    /* I don't understand why we need to swap this field! */
+                    ct = strstr(buffer, ";");
+                    if (ct == NULL || strlen(ct) < 8) {
+                        g_assert_not_reached();
+                        fprintf(pf, "Corrupt Snowie position string (%s)\n", buffer);
+                    } else
+                        ct[7] = (ct[7] == '0') ? '1' : '0';
+                }
             }
             if (!(i & 1))
                 if (fSst)
@@ -1215,6 +1245,7 @@ ExportGameGam(char *sz, int fSst)
 {
 
     FILE *pf;
+    int fDontClose = FALSE;
 
     sz = NextToken(&sz);
 
@@ -1229,16 +1260,17 @@ ExportGameGam(char *sz, int fSst)
     if (!confirmOverwrite(sz, fConfirmSave))
         return;
 
-    if (!strcmp(sz, "-"))
+    if (!strcmp(sz, "-")) {
         pf = stdout;
-    else if (!(pf = gnubg_g_fopen(sz, "w"))) {
+        fDontClose = TRUE;
+    } else if (!(pf = g_fopen(sz, "w"))) {
         outputerr(sz);
         return;
     }
 
     ExportGameJF(pf, plGame, -1, FALSE, fSst);
 
-    if (pf != stdout)
+    if (!fDontClose)
         fclose(pf);
 
     setDefaultFileName(sz);
@@ -1263,8 +1295,9 @@ ExportMatchMat(char *sz, int fSst)
 {
 
     FILE *pf;
-    int i;
     listOLD *pl;
+    int i;
+    int fDontClose = FALSE;
 
     /* FIXME what should be done if nMatchTo == 0? */
 
@@ -1281,9 +1314,10 @@ ExportMatchMat(char *sz, int fSst)
     if (!confirmOverwrite(sz, fConfirmSave))
         return;
 
-    if (!strcmp(sz, "-"))
+    if (!strcmp(sz, "-")) {
         pf = stdout;
-    else if (!(pf = gnubg_g_fopen(sz, "w"))) {
+        fDontClose = TRUE;
+    } else if (!(pf = g_fopen(sz, "w"))) {
         outputerr(sz);
         return;
     }
@@ -1337,7 +1371,7 @@ ExportMatchMat(char *sz, int fSst)
     for (i = 0, pl = lMatch.plNext; pl != &lMatch; i++, pl = pl->plNext)
         ExportGameJF(pf, pl->p, i, TRUE, fSst);
 
-    if (pf != stdout)
+    if (!fDontClose)
         fclose(pf);
 
     setDefaultFileName(sz);
