@@ -1,11 +1,11 @@
 /*
- * export.h
+ * Copyright (C) 2002 Joern Thyssen <jthyssen@dk.ibm.com>
+ * Copyright (C) 2002-2019 the AUTHORS
  *
- * by Joern Thyssen  <jthyssen@dk.ibm.com>, 2002.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,16 +13,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: export.h,v 1.40 2018/06/05 19:24:05 plm Exp $
+ * $Id: export.h,v 1.44 2022/01/02 22:56:36 plm Exp $
  */
 
-#include "render.h"
-#include "list.h"
 #include <glib.h>
 
+#include "backgammon.h"
+#include "list.h"
+
 #ifndef EXPORT_H
 #define EXPORT_H
 
@@ -30,15 +30,14 @@
 #define EXPORT_CUBE_MISSED   5
 #define EXPORT_CUBE_CLOSE    6
 
-
-typedef enum _htmlexporttype {
+typedef enum {
     HTML_EXPORT_TYPE_GNU,
     HTML_EXPORT_TYPE_BBS,
     HTML_EXPORT_TYPE_FIBS2HTML,
     NUM_HTML_EXPORT_TYPES
 } htmlexporttype;
 
-typedef enum _htmlexportcss {
+typedef enum {
     HTML_EXPORT_CSS_HEAD,
     HTML_EXPORT_CSS_INLINE,
     HTML_EXPORT_CSS_EXTERNAL,
@@ -49,7 +48,7 @@ extern const char *aszHTMLExportType[];
 extern const char *aszHTMLExportCSS[];
 extern const char *aszHTMLExportCSSCommand[];
 
-typedef struct _exportsetup {
+typedef struct {
 
     int fIncludeAnnotation;
     int fIncludeAnalysis;
