@@ -1,11 +1,11 @@
 /*
- * external.c
+ * Copyright (C) 2001-2002 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2001-2021 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 2001, 2002.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: external.c,v 1.97 2018/04/23 21:36:20 plm Exp $
+ * $Id: external.c,v 1.108 2022/03/12 20:38:18 plm Exp $
  */
 
 #include "config.h"
@@ -55,6 +54,7 @@
 #include <winsock2.h>
 #include <ws2tcpip.h>
 
+#ifndef _MSC_VER
 #define EWOULDBLOCK             WSAEWOULDBLOCK
 #define EINPROGRESS             WSAEINPROGRESS
 #define EALREADY                WSAEALREADY
@@ -94,6 +94,7 @@
 #define EREMOTE                 WSAEREMOTE
 #define EINVAL                  WSAEINVAL
 #define EINTR                   WSAEINTR
+#endif
 
 #define inet_aton(ip,addr)  (addr)->s_addr = inet_addr(ip), 1
 #define inet_pton(fam,ip,addr) (addr)->s_addr = inet_addr(ip), 1
@@ -128,7 +129,7 @@ OutputWin32SocketError(const char *action)
                       (LPTSTR) & lpMsgBuf, 0, NULL) != 0) {
         outputerrf("%s (%s):\n%s", _("Windows socket error"), action, (LPCTSTR) lpMsgBuf);
         if (LocalFree(lpMsgBuf) != NULL)
-            g_print("LocalFree() failed\n");
+            g_print("win32 LocalFree() failed\n");
     }
 }
 #endif
@@ -157,9 +158,12 @@ ExternalSocket(struct sockaddr **ppsa, int *pcb, char *sz)
 #else
         if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &f, sizeof f))
 #endif                          /* WIN32 */
-            return -1;
+            {
+                close(sock);
+                return -1;
+            }
 
-        psin = malloc(*pcb = sizeof(struct sockaddr_in));
+        psin = g_malloc(*pcb = sizeof(struct sockaddr_in));
         memset(psin, 0, sizeof(*psin));
 
         psin->sin_family = AF_INET;
@@ -178,10 +182,10 @@ ExternalSocket(struct sockaddr **ppsa, int *pcb, char *sz)
             if ((phe = gethostbyname(sz)) == 0) {
                 *pch = ':';
                 errno = EINVAL;
-                free(psin);
+                g_free(psin);
                 return -1;
             }
-			memcpy(&(psin->sin_addr), phe->h_addr, phe->h_length); 
+            memcpy(&(psin->sin_addr), phe->h_addr, phe->h_length);
         }
 
         *pch++ = ':';
@@ -197,7 +201,7 @@ ExternalSocket(struct sockaddr **ppsa, int *pcb, char *sz)
 
         /* yuck... there's no portable way to obtain the necessary
          * sockaddr_un size, but this is a conservative estimate */
-        psun = malloc(*pcb = 16 + (int) strlen(sz));
+        psun = g_malloc(*pcb = 16 + (int) strlen(sz));
 
         psun->sun_family = AF_LOCAL;
         strcpy(psun->sun_path, sz);
@@ -298,7 +302,7 @@ ExternalWrite(int h, char *pch, size_t cch)
 #else
     int n;
 #endif
-	/* outputf("%s", pch); */
+    /* outputf("%s", pch); */
 
     while (cch) {
         ProcessEvents();
@@ -396,45 +400,42 @@ ExtEvaluation(scancontext * pec)
     evalcontext ec;
 
     if (ProcessFIBSBoardInfo(&pec->bi, &processedBoard)) {
-        szResponse = g_strdup_printf("Error: badly formed board\n");
-    } else {
-
-        anScore[0] = processedBoard.nScore;
-        anScore[1] = processedBoard.nScoreOpp;
-        /* If the session isn't using Crawford rule, set crawford flag to false */
-        processedBoard.fCrawford = pec->fCrawfordRule ? processedBoard.fCrawford : FALSE;
-        /* Set the Jacoby flag appropriately from the external interface settings */
-        processedBoard.fJacoby = pec->fJacobyRule;
-
-        /* printf ("Jacoby Setting: %d\n", fJacoby); */
-        /* printf ("Crawford Setting: %d\n", fCrawford); */
-
-        SetCubeInfo(&ci, processedBoard.nCube, processedBoard.fCubeOwner, 1, processedBoard.nMatchTo,
-                    anScore, processedBoard.fCrawford, processedBoard.fJacoby, nBeavers, bgvDefault);
-
-        ec.fCubeful = pec->fCubeful;
-        ec.nPlies = pec->nPlies;
-        ec.fUsePrune = pec->fUsePrune;
-        ec.fDeterministic = pec->fDeterministic;
-        ec.rNoise = pec->rNoise;
-
-        if (GeneralEvaluationE(arOutput, (ConstTanBoard) processedBoard.anBoard, &ci, &ec))
-            return NULL;
-
-        if (processedBoard.nMatchTo) {
-            if (ec.fCubeful)
-                r = arOutput[OUTPUT_CUBEFUL_EQUITY];
-            else
-                r = eq2mwc(arOutput[OUTPUT_EQUITY], &ci);
-        } else
-            r = ec.fCubeful ? arOutput[6] : arOutput[5];
-
-        szResponse = g_strdup_printf("%f %f %f %f %f %f\n",
-                                     arOutput[0], arOutput[1], arOutput[2], arOutput[3], arOutput[4], r);
+        return g_strdup_printf("Error: badly formed board\n");
     }
 
-    return szResponse;
+    anScore[0] = processedBoard.nScore;
+    anScore[1] = processedBoard.nScoreOpp;
+    /* If the session isn't using Crawford rule, set crawford flag to false */
+    processedBoard.fCrawford = pec->fCrawfordRule ? processedBoard.fCrawford : FALSE;
+    /* Set the Jacoby flag appropriately from the external interface settings */
+    processedBoard.fJacoby = pec->fJacobyRule;
 
+    /* printf ("Jacoby Setting: %d\n", fJacoby); */
+    /* printf ("Crawford Setting: %d\n", fCrawford); */
+
+    SetCubeInfo(&ci, processedBoard.nCube, processedBoard.fCubeOwner, 1, processedBoard.nMatchTo,
+                anScore, processedBoard.fCrawford, processedBoard.fJacoby, nBeavers, bgvDefault);
+
+    ec.fCubeful = pec->fCubeful;
+    ec.nPlies = pec->nPlies;
+    ec.fUsePrune = pec->fUsePrune;
+    ec.fDeterministic = pec->fDeterministic;
+    ec.rNoise = pec->rNoise;
+
+    if (GeneralEvaluationE(arOutput, (ConstTanBoard) processedBoard.anBoard, &ci, &ec))
+        return NULL;
+
+    if (processedBoard.nMatchTo) {
+        if (ec.fCubeful)
+            r = arOutput[OUTPUT_CUBEFUL_EQUITY];
+        else
+            r = eq2mwc(arOutput[OUTPUT_EQUITY], &ci);
+    } else
+        r = ec.fCubeful ? arOutput[6] : arOutput[5];
+
+    szResponse = g_strdup_printf("%f %f %f %f %f %f\n",
+                                 arOutput[0], arOutput[1], arOutput[2], arOutput[3], arOutput[4], r);
+    return szResponse;
 }
 
 static char *
@@ -448,105 +449,151 @@ ExtFIBSBoard(scancontext * pec)
     cubeinfo ci;
     char *szResponse;
 
-    if (ProcessFIBSBoardInfo(&pec->bi, &processedBoard)) {
-        szResponse = g_strdup_printf("Error: badly formed board\n");
-    } else {
+    if (ProcessFIBSBoardInfo(&pec->bi, &processedBoard))
+        return g_strdup_printf("Error: badly formed board\n");
 
-        anScore[0] = processedBoard.nScore;
-        anScore[1] = processedBoard.nScoreOpp;
+    anScore[0] = processedBoard.nScore;
+    anScore[1] = processedBoard.nScoreOpp;
 
-        /* If the session isn't using Crawford rule, set crawford flag to false */
-        processedBoard.fCrawford = pec->fCrawfordRule ? processedBoard.fCrawford : FALSE;
-        /* Set the Jacoby flag appropriately from the external interface settings */
-        processedBoard.fJacoby = pec->fJacobyRule;
+    /* If the session isn't using Crawford rule, set crawford flag to false */
+    processedBoard.fCrawford = pec->fCrawfordRule ? processedBoard.fCrawford : FALSE;
+    /* Set the Jacoby flag appropriately from the external interface settings */
+    processedBoard.fJacoby = pec->fJacobyRule;
 
-        /* printf ("Crawford Setting: %d\n", fCrawford); */
-        /* printf ("Jacoby Setting: %d\n", fJacoby); */
+    /* printf ("Crawford Setting: %d\n", fCrawford); */
+    /* printf ("Jacoby Setting: %d\n", fJacoby); */
 
-        fTurn = 1;
-        SetCubeInfo(&ci, processedBoard.nCube, processedBoard.fCubeOwner, fTurn, processedBoard.nMatchTo,
-                    anScore, processedBoard.fCrawford, processedBoard.fJacoby, nBeavers, bgvDefault);
+    fTurn = 1;
+    SetCubeInfo(&ci, processedBoard.nCube, processedBoard.fCubeOwner, fTurn, processedBoard.nMatchTo,
+                anScore, processedBoard.fCrawford, processedBoard.fJacoby, nBeavers, bgvDefault);
 
-        memcpy(anBoardOrig, processedBoard.anBoard, sizeof(processedBoard.anBoard));
+    memcpy(anBoardOrig, processedBoard.anBoard, sizeof(processedBoard.anBoard));
 
-        if (processedBoard.fDoubled) {
+    if (processedBoard.fDoubled) {
+        /* take decision */
 
-            /* take decision */
-            if (GeneralCubeDecision(aarOutput, aarStdDev,
-                                    aarsStatistics, (ConstTanBoard) processedBoard.anBoard, &ci, GetEvalCube(), NULL,
-                                    NULL) < 0)
-                return NULL;
+        SwapSides(processedBoard.anBoard);
+        processedBoard.fCubeOwner = (processedBoard.fCubeOwner == -1 ? -1 : !processedBoard.fCubeOwner);
 
-            switch (FindCubeDecision(arDouble, aarOutput, &ci)) {
-            case DOUBLE_PASS:
-            case TOOGOOD_PASS:
-            case REDOUBLE_PASS:
-            case TOOGOODRE_PASS:
-                szResponse = g_strdup("drop\n");
-                break;
+        SetCubeInfo(&ci, processedBoard.nCube, processedBoard.fCubeOwner, fTurn, processedBoard.nMatchTo, anScore,
+                    processedBoard.fCrawford, processedBoard.fJacoby, nBeavers, bgvDefault);
 
-            case NODOUBLE_BEAVER:
-            case DOUBLE_BEAVER:
-            case NO_REDOUBLE_BEAVER:
-                szResponse = g_strdup("beaver\n");
-                break;
+        if (GeneralCubeDecision(aarOutput, aarStdDev,
+                                aarsStatistics, (ConstTanBoard) processedBoard.anBoard, &ci, GetEvalCube(), NULL,
+                                NULL) < 0)
+            return NULL;
 
-            default:
-                szResponse = g_strdup("take\n");
-            }
+        switch (FindCubeDecision(arDouble, aarOutput, &ci)) {
 
-        } else if (pec->nResignation) {
+        case DOUBLE_TAKE:
+        case NODOUBLE_TAKE:
+        case TOOGOOD_TAKE:
+        case REDOUBLE_TAKE:
+        case NO_REDOUBLE_TAKE:
+        case TOOGOODRE_TAKE:
+        case NODOUBLE_DEADCUBE:
+        case NO_REDOUBLE_DEADCUBE:
+        case OPTIONAL_DOUBLE_TAKE:
+        case OPTIONAL_REDOUBLE_TAKE:
+            szResponse = g_strdup("take\n");
+            break;
 
-            /* if opp wants to resign (extension to FIBS board) */
+        case DOUBLE_PASS:
+        case TOOGOOD_PASS:
+        case REDOUBLE_PASS:
+        case TOOGOODRE_PASS:
+        case OPTIONAL_DOUBLE_PASS:
+        case OPTIONAL_REDOUBLE_PASS:
+            szResponse = g_strdup("drop\n");
+            break;
 
-            float arOutput[NUM_ROLLOUT_OUTPUTS];
-            float rEqBefore, rEqAfter;
-            const float epsilon = 1.0e-6f;
+        case NODOUBLE_BEAVER:
+        case DOUBLE_BEAVER:
+        case NO_REDOUBLE_BEAVER:
+        case OPTIONAL_DOUBLE_BEAVER:
+            szResponse = g_strdup("beaver\n");
+            break;
 
-            getResignation(arOutput, processedBoard.anBoard, &ci, &esEvalCube);
-
-            getResignEquities(arOutput, &ci, pec->nResignation, &rEqBefore, &rEqAfter);
-
-            /* if opponent gives up equity by resigning */
-            if ((rEqAfter - epsilon) < rEqBefore)
-                szResponse = g_strdup("accept\n");
-            else
-                szResponse = g_strdup("reject\n");
-
-        } else if (processedBoard.anDice[0]) {
-            /* move */
-            char szMove[64];
-            if (FindBestMove(anMove, processedBoard.anDice[0], processedBoard.anDice[1],
-                             processedBoard.anBoard, &ci, &GetEvalChequer()->ec, *GetEvalMoveFilter()) < 0)
-                return NULL;
-
-            FormatMovePlain(szMove, anBoardOrig, anMove);
-            szResponse = g_strconcat(szMove, "\n", NULL);
-        } else {
-            /* double decision */
-            if (GeneralCubeDecision(aarOutput, aarStdDev,
-                                    aarsStatistics, (ConstTanBoard) processedBoard.anBoard, &ci, GetEvalCube(),
-                                    NULL, NULL) < 0)
-                return NULL;
-
-            switch (FindCubeDecision(arDouble, aarOutput, &ci)) {
-            case DOUBLE_TAKE:
-            case DOUBLE_PASS:
-            case DOUBLE_BEAVER:
-            case REDOUBLE_TAKE:
-            case REDOUBLE_PASS:
-                szResponse = g_strdup("double\n");
-                break;
-
-            default:
-                szResponse = g_strdup("roll\n");
-            }
+        default:
+            g_assert_not_reached();
+            szResponse = g_strdup("take\n");
         }
 
+    } else if (pec->nResignation) {
+
+        /* if opp wants to resign (extension to FIBS board) */
+
+        float arOutput[NUM_ROLLOUT_OUTPUTS];
+        float rEqBefore, rEqAfter;
+        const float epsilon = 1.0e-6f;
+
+        getResignation(arOutput, processedBoard.anBoard, &ci, &esEvalCube);
+
+        getResignEquities(arOutput, &ci, pec->nResignation, &rEqBefore, &rEqAfter);
+
+        /* if opponent gives up equity by resigning */
+        if ((rEqAfter - epsilon) < rEqBefore)
+            szResponse = g_strdup("accept\n");
+        else
+            szResponse = g_strdup("reject\n");
+
+    } else if (processedBoard.anDice[0]) {
+        /* move */
+        char szMove[FORMATEDMOVESIZE];
+        if (FindBestMove(anMove, processedBoard.anDice[0], processedBoard.anDice[1],
+                         processedBoard.anBoard, &ci, &GetEvalChequer()->ec, *GetEvalMoveFilter()) < 0)
+            return NULL;
+
+        FormatMovePlain(szMove, (ConstTanBoard)anBoardOrig, anMove);
+        szResponse = g_strconcat(szMove, "\n", NULL);
+    } else {
+        /* double decision */
+        if (GeneralCubeDecision(aarOutput, aarStdDev,
+                                aarsStatistics, (ConstTanBoard) processedBoard.anBoard, &ci, GetEvalCube(),
+                                NULL, NULL) < 0)
+            return NULL;
+
+        switch (FindCubeDecision(arDouble, aarOutput, &ci)) {
+        case DOUBLE_TAKE:
+        case DOUBLE_PASS:
+        case DOUBLE_BEAVER:
+        case REDOUBLE_TAKE:
+        case REDOUBLE_PASS:
+            szResponse = g_strdup("double\n");
+            break;
+
+        case NODOUBLE_TAKE:
+        case TOOGOOD_TAKE:
+        case NO_REDOUBLE_TAKE:
+        case TOOGOODRE_TAKE:
+        case TOOGOOD_PASS:
+        case TOOGOODRE_PASS:
+        case NODOUBLE_BEAVER:
+        case NO_REDOUBLE_BEAVER:
+        case NODOUBLE_DEADCUBE:
+        case NO_REDOUBLE_DEADCUBE:
+            szResponse = g_strdup("roll\n");
+            break;
+
+        case OPTIONAL_DOUBLE_BEAVER:
+        case OPTIONAL_DOUBLE_TAKE:
+        case OPTIONAL_REDOUBLE_TAKE:
+        case OPTIONAL_DOUBLE_PASS:
+        case OPTIONAL_REDOUBLE_PASS:
+            if (pec->nPlies == 0 && aarOutput[fTurn][0] > 0.001f)
+                /* double if 0-ply except when about to lose game */
+                szResponse = g_strdup("double\n");
+            else
+                szResponse = g_strdup("roll\n");
+            break;
+
+        default:
+            g_assert_not_reached();
+            szResponse = g_strdup("roll\n");
+        }
     }
 
     return szResponse;
-
 }
 #endif
 
@@ -555,7 +602,7 @@ CommandExternal(char *sz)
 {
 
 #if !defined(HAVE_SOCKETS)
-    (void) sz;		/* silence compiler warning */
+    (void) sz;                  /* silence compiler warning */
     outputl(_("This installation of GNU Backgammon was compiled without\n"
               "socket support, and does not implement external controllers."));
 #else
@@ -595,12 +642,12 @@ CommandExternal(char *sz)
         if (bind(h, psa, cb) < 0) {
             SockErr(sz);
             closesocket(h);
-            free(psa);
+            g_free(psa);
             ExtDestroyParse(scanctx.scanner);
             return;
         }
 
-        free(psa);
+        g_free(psa);
 
         if (listen(h, 1) < 0) {
             SockErr("listen");
@@ -647,7 +694,7 @@ CommandExternal(char *sz)
 
         while (!fExit && !(retval = ExternalRead(hPeer, szCommand, sizeof(szCommand)))) {
             /* To keep lexer happy terminate each line with \n */
-            if (szCommand[strlen(szCommand)-1] != '\n')
+            if (szCommand[strlen(szCommand) - 1] != '\n')
                 strcat(szCommand, "\n");
 
             if ((ExtParse(&scanctx, szCommand)) == 0) {
@@ -655,16 +702,6 @@ CommandExternal(char *sz)
                 szResponse = scanctx.szError;
             } else {
                 ProcessedFIBSBoard processedBoard;
-                GValue *optionsmapgv;
-                GValue *boarddatagv;
-                GString *dbgStr;
-                int anScore[2];
-                int fCrawford, fJacoby;
-                char *asz[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
-                char szBoard[10000];
-                char **aszLines;
-                char **aszLinesOrig;
-                char *szMatchID;
                 gchar *szOptStr;
 
                 switch (scanctx.ct) {
@@ -701,6 +738,16 @@ CommandExternal(char *sz)
                 case COMMAND_FIBSBOARD:
                 case COMMAND_EVALUATION:
                     if (scanctx.fDebug) {
+                        GValue *optionsmapgv;
+                        GValue *boarddatagv;
+                        GString *dbgStr;
+                        int anScore[2];
+                        int fcrawford, fjacoby;
+                        char *asz[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+                        char szBoard[10000];
+                        char **aszLines, **aszLinesOrig;
+                        char *szMatchID;
+
                         optionsmapgv = (GValue *) g_list_nth_data(g_value_get_boxed(scanctx.pCmdData), 1);
                         boarddatagv = (GValue *) g_list_nth_data(g_value_get_boxed(scanctx.pCmdData), 0);
                         dbgStr = g_string_new(DEBUG_PREFIX);
@@ -713,14 +760,14 @@ CommandExternal(char *sz)
 
                         anScore[0] = processedBoard.nScoreOpp;
                         anScore[1] = processedBoard.nScore;
-                        /* If the session isn't using Crawford rule, set crawford flag to false */
-                        fCrawford = scanctx.fCrawfordRule ? processedBoard.fCrawford : FALSE;
+                        /* If the session isn't using Crawford rule, set Crawford flag to false */
+                        fcrawford = scanctx.fCrawfordRule ? processedBoard.fCrawford : FALSE;
                         /* Set the Jacoby flag appropriately from the external interface settings */
-                        fJacoby = scanctx.fJacobyRule;
+                        fjacoby = scanctx.fJacobyRule;
 
                         szMatchID = MatchID((unsigned int *) processedBoard.anDice, 1, processedBoard.nResignation,
-                                            processedBoard.fDoubled, 1, processedBoard.fCubeOwner, fCrawford,
-                                            processedBoard.nMatchTo, anScore, processedBoard.nCube, fJacoby,
+                                            processedBoard.fDoubled, 1, processedBoard.fCubeOwner, fcrawford,
+                                            processedBoard.nMatchTo, anScore, processedBoard.nCube, fjacoby,
                                             GAME_PLAYING);
 
                         DrawBoard(szBoard, (ConstTanBoard) & processedBoard.anBoard, 1, asz, szMatchID, 15);
@@ -742,7 +789,7 @@ CommandExternal(char *sz)
                                                    scanctx.fCrawfordRule ? "with" : "without");
                             g_string_append_printf(dbgStr, DEBUG_PREFIX "Score: %d-%d/%d%s, ", processedBoard.nScore,
                                                    processedBoard.nScoreOpp, processedBoard.nMatchTo,
-                                                   fCrawford ? "*" : "");
+                                                   fcrawford ? "*" : "");
                         } else {
                             g_string_append_printf(dbgStr, DEBUG_PREFIX "Money Session %s Jacoby Rule, %s Beavers\n",
                                                    scanctx.fJacobyRule ? "with" : "without",
@@ -784,7 +831,7 @@ CommandExternal(char *sz)
             }
 
             if (szResponse) {
-                /* outputf("%s", szResponse);*/
+                /* outputf("%s", szResponse); */
                 if (ExternalWrite(hPeer, szResponse, strlen(szResponse)))
                     break;
 
