@@ -1,10 +1,11 @@
 /*
- * file.c
+ * Copyright (C) 2007-2008 Christian Anthon <anthon@kiku.dk>
+ * Copyright (C) 2007-2019 the AUTHORS
  *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: file.c,v 1.33 2017/09/25 19:05:52 plm Exp $
+ * $Id: file.c,v 1.38 2022/02/06 15:34:32 plm Exp $
  */
 
 #include "config.h"
@@ -23,7 +23,6 @@
 #include <glib/gstdio.h>
 #include "file.h"
 #include <stdlib.h>
-#include "glib-ext.h"
 
 ExportFormat export_format[] = {
     {EXPORT_SGF, ".sgf", N_("GNU Backgammon File"), "sgf", {TRUE, TRUE, TRUE}
@@ -96,7 +95,7 @@ ImportFormat import_format[] = {
     {N_IMPORT_TYPES, NULL, N_("Unknown file format"), NULL}
 };
 
-typedef struct _FileHelper {
+typedef struct {
     FILE *fp;
     size_t dataRead;
     unsigned int dataPos;
@@ -113,7 +112,7 @@ OpenFileHelper(const char *filename)
         return NULL;            /* File not found */
 
     fh = g_new(FileHelper, 1);
-    fh->fp = gnubg_g_fopen(filename, "r");
+    fh->fp = g_fopen(filename, "r");
     if (!fh->fp) {              /* Failed to open file */
         g_free(fh);
         return NULL;
@@ -129,6 +128,7 @@ CloseFileHelper( /*lint -e{818} */ FileHelper * fh)
 {
     fclose(fh->fp);
     free(fh->data);
+    g_free(fh);
 }
 
 static void
@@ -146,7 +146,7 @@ fhDataGetChar(FileHelper * fh)
 
 #define BLOCK_SIZE 1024
 #define MAX_READ_SIZE 5000
-    fh->data = realloc(fh->data, fh->dataRead + BLOCK_SIZE);
+    fh->data = g_realloc(fh->data, fh->dataRead + BLOCK_SIZE);
     if (fh->dataRead > MAX_READ_SIZE)
         numRead = 0;            /* Too big - should have worked things out by now! */
     else
@@ -423,7 +423,7 @@ IsBGRFile(FileHelper * fh)
     if (fhReadStringNC(fh, "bgf version")) {
         fhSkipWS(fh);
         if (!fhReadString(fh, "1.2"))
-            printf("Unexpected version of bgroom file!\n");
+            g_print(_("Unexpected version of BGRoom file!\n"));
 
         return TRUE;
     }
@@ -469,21 +469,22 @@ extern char *
 GetFilename(int CheckForCurrent, ExportType type)
 {
     char *sz;
-    time_t t;
 
     if (CheckForCurrent && szCurrentFileName && *szCurrentFileName)
         sz = g_strdup_printf("%s%s", szCurrentFileName, export_format[type].extension);
     else {
-        char tstr[15];
-
         if (mi.nYear)
-            sprintf(tstr, "%04u-%02u-%02u", mi.nYear, mi.nMonth, mi.nDay);
+            sz = g_strdup_printf("%s-%s_%dp_%04u-%02u-%02u.sgf", ap[0].szName, ap[1].szName, ms. nMatchTo, mi.nYear, mi.nMonth, mi.nDay);
         else {
-            t = time(NULL);
+            time_t t = time(NULL);
+            char tstr[15];
+
             if (strftime(tstr, 14, "%Y-%m-%d-%H%M", localtime(&t)) == 0)
                 *tstr = '\0';
+
+            sz = g_strdup_printf("%s-%s_%dp_%s.sgf", ap[0].szName, ap[1].szName, ms.nMatchTo, tstr);
         }
-        sz = g_strdup_printf("%s-%s_%dp_%s.sgf", ap[0].szName, ap[1].szName, ms.nMatchTo, tstr);
     }
+
     return sz;
 }
