@@ -1,11 +1,11 @@
 /*
- * format.c
+ * Copyright (C) 2003 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2003-2021 the AUTHORS
  *
- * by Joern Thyssen  <jth@gnubg.org>, 2003
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: format.c,v 1.58 2018/06/18 11:07:22 plm Exp $
+ * $Id: format.c,v 1.69 2022/01/11 21:48:34 plm Exp $
  */
 
 #include "config.h"
@@ -45,7 +44,7 @@ typedef int (*classdumpfunc) (const TanBoard anBoard, char *szOutput, const bgva
 
 extern char *
 OutputRolloutResult(const char *szIndent,
-                    char asz[][1024],
+                    char asz[][FORMATEDMOVESIZE],
                     float aarOutput[][NUM_ROLLOUT_OUTPUTS],
                     float aarStdDev[][NUM_ROLLOUT_OUTPUTS],
                     const cubeinfo aci[], const int alt, const int cci, const int fCubeful)
@@ -73,11 +72,15 @@ OutputRolloutResult(const char *szIndent,
 
         strcat(sz, "  ");
         strcat(sz, OutputPercents(aarOutput[ici], TRUE));
-        strcat(sz, " CL ");
+        strcat(sz, " ");
+        strcat(sz, _("CL"));
+        strcat(sz, " ");
         strcat(sz, OutputEquityScale(aarOutput[ici][OUTPUT_EQUITY], &aci[alt + ici], &aci[0], TRUE));
 
         if (fCubeful) {
-            strcat(sz, " CF ");
+            strcat(sz, " ");
+            strcat(sz, _("CF"));
+            strcat(sz, " ");
             strcat(sz, OutputMWC(aarOutput[ici][OUTPUT_CUBEFUL_EQUITY], &aci[0], TRUE));
         }
 
@@ -90,11 +93,15 @@ OutputRolloutResult(const char *szIndent,
 
         strcat(sz, " [");
         strcat(sz, OutputPercents(aarStdDev[ici], FALSE));
-        strcat(sz, " CL ");
+        strcat(sz, " ");
+        strcat(sz, _("CL"));
+        strcat(sz, " ");
         strcat(sz, OutputEquityScale(aarStdDev[ici][OUTPUT_EQUITY], &aci[alt + ici], &aci[0], FALSE));
 
         if (fCubeful) {
-            strcat(sz, " CF ");
+            strcat(sz, " ");
+            strcat(sz, _("CF"));
+            strcat(sz, " ");
             strcat(sz, OutputMWC(aarStdDev[ici][OUTPUT_CUBEFUL_EQUITY], &aci[0], FALSE));
         }
 
@@ -117,7 +124,7 @@ OutputEvalContext(const evalcontext * pec, const int fChequer)
     sprintf(sz, "%u-%s %s", pec->nPlies, _("ply"), (!fChequer || pec->fCubeful) ? _("cubeful") : _("cubeless"));
 
     if (pec->fUsePrune) {
-        sprintf(strchr(sz, 0), " prune");
+        sprintf(strchr(sz, 0), " %s", _("prune"));
     }
 
     if (fChequer && pec->nPlies) {
@@ -125,7 +132,7 @@ OutputEvalContext(const evalcontext * pec, const int fChequer)
     }
 
     if (pec->rNoise > 0.0f)
-        sprintf(strchr(sz, 0), ", noise %0.3g (%s)", pec->rNoise, pec->fDeterministic ? "d" : "nd");
+        sprintf(strchr(sz, 0), ", %s %0.3g (%s)", _("noise"), pec->rNoise, pec->fDeterministic ? _("d") : _("nd"));
 
     for (i = 0; i < NUM_SETTINGS; i++)
 
@@ -339,10 +346,12 @@ OutputRolloutContext(const char *szIndent, const rolloutcontext * prc)
     if (szIndent && *szIndent)
         strcat(sz, szIndent);
 
-    sprintf(strchr(sz, 0), "%u games", prc->nGamesDone);
+    sprintf(strchr(sz, 0), _("%u games"), prc->nGamesDone);
 
-    if (prc->fInitial)
-        strcat(sz, ", rollout as initial position");
+    if (prc->fInitial) {
+        strcat(sz, ", ");
+        strcat(sz, _("rollout as initial position"));
+    }
 
     strcat(sz, ", ");
     if (prc->fRotate)
@@ -417,15 +426,16 @@ OutputEquity(const float r, const cubeinfo * pci, const int f)
 
     if (!pci->nMatchTo || !fOutputMWC) {
         if (f)
-            sprintf(sz, "%+*.*f", fOutputDigits + 3, fOutputDigits, r);
+            snprintf(sz, OUTPUT_SZ_LENGTH, "%+*.*f", fOutputDigits + 3, fOutputDigits, r);
         else
-            sprintf(sz, "% *.*f", fOutputDigits + 2, fOutputDigits, r);
+            snprintf(sz, OUTPUT_SZ_LENGTH, "% *.*f", fOutputDigits + 2, fOutputDigits, r);
     } else {
         if (fOutputMatchPC) {
-            sprintf(sz, "%*.*f%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0,
-                    100.0f * (f ? eq2mwc(r, pci) : se_eq2mwc(r, pci)));
+            snprintf(sz, OUTPUT_SZ_LENGTH, "%*.*f%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0,
+                     100.0f * (f ? eq2mwc(r, pci) : se_eq2mwc(r, pci)));
         } else {
-            sprintf(sz, "%*.*f", fOutputDigits + 3, fOutputDigits + 1, f ? eq2mwc(r, pci) : se_eq2mwc(r, pci));
+            snprintf(sz, OUTPUT_SZ_LENGTH, "%*.*f", fOutputDigits + 3, fOutputDigits + 1,
+                     f ? eq2mwc(r, pci) : se_eq2mwc(r, pci));
         }
     }
 
@@ -443,9 +453,9 @@ OutputMoneyEquity(const float ar[], const int f)
         ar[OUTPUT_LOSEGAMMON] - ar[OUTPUT_LOSEBACKGAMMON];
 
     if (f)
-        sprintf(sz, "%+*.*f", fOutputDigits + 3, fOutputDigits, eq);
+        snprintf(sz, OUTPUT_SZ_LENGTH, "%+*.*f", fOutputDigits + 3, fOutputDigits, eq);
     else
-        sprintf(sz, "% *.*f", fOutputDigits + 2, fOutputDigits, eq);
+        snprintf(sz, OUTPUT_SZ_LENGTH, "% *.*f", fOutputDigits + 2, fOutputDigits, eq);
 
 
     return sz;
@@ -475,25 +485,30 @@ OutputEquityScale(const float r, const cubeinfo * pci, const cubeinfo * pciBase,
 
     if (!pci->nMatchTo) {
         if (f)
-            sprintf(sz, "%+*.*f", fOutputDigits + 3, fOutputDigits, pci->nCube / pciBase->nCube * r);
+            snprintf(sz, OUTPUT_SZ_LENGTH, "%+*.*f", fOutputDigits + 3, fOutputDigits,
+                     (float) (pci->nCube / pciBase->nCube) * r);
         else
-            sprintf(sz, "% *.*f", fOutputDigits + 2, fOutputDigits, pci->nCube / pciBase->nCube * r);
+            snprintf(sz, OUTPUT_SZ_LENGTH, "% *.*f", fOutputDigits + 2, fOutputDigits,
+                     (float) (pci->nCube / pciBase->nCube) * r);
     } else {
 
         if (fOutputMWC) {
 
             if (fOutputMatchPC) {
-                sprintf(sz, "%*.*f%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0,
-                        100.0f * (f ? eq2mwc(r, pci) : se_eq2mwc(r, pci)));
+                snprintf(sz, OUTPUT_SZ_LENGTH, "%*.*f%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0,
+                         100.0f * (f ? eq2mwc(r, pci) : se_eq2mwc(r, pci)));
             } else {
-                sprintf(sz, "%*.*f", fOutputDigits + 3, fOutputDigits + 1, f ? eq2mwc(r, pci) : se_eq2mwc(r, pci));
+                snprintf(sz, OUTPUT_SZ_LENGTH, "%*.*f", fOutputDigits + 3, fOutputDigits + 1,
+                         f ? eq2mwc(r, pci) : se_eq2mwc(r, pci));
             }
 
         } else {
             if (f)
-                sprintf(sz, "%+*.*f", fOutputDigits + 3, fOutputDigits, mwc2eq(eq2mwc(r, pci), pciBase));
+                snprintf(sz, OUTPUT_SZ_LENGTH, "%+*.*f", fOutputDigits + 3, fOutputDigits,
+                         mwc2eq(eq2mwc(r, pci), pciBase));
             else
-                sprintf(sz, "% *.*f", fOutputDigits + 2, fOutputDigits, se_mwc2eq(se_eq2mwc(r, pci), pciBase));
+                snprintf(sz, OUTPUT_SZ_LENGTH, "% *.*f", fOutputDigits + 2, fOutputDigits,
+                         se_mwc2eq(se_eq2mwc(r, pci), pciBase));
         }
 
 
@@ -508,10 +523,8 @@ OutputEquityScale(const float r, const cubeinfo * pci, const cubeinfo * pciBase,
  * Return formatted string with equity or MWC for an equity difference.
  *
  * Input:
- *    r: equity (either money equity for normalised money eq. for match play
+ *    r1, r2: equities (either money equity for normalised money eq. for match play
  *    pci: cubeinfo
- *    f: indicates equity (TRUE) or std. error (FALSE)
- *    
  *
  * Important: function is not re-entrant. Caller must save output
  * if needed.
@@ -523,14 +536,36 @@ OutputEquityDiff(const float r1, const float r2, const cubeinfo * pci)
 
     static char sz[OUTPUT_SZ_LENGTH];
 
+    if (fOutputDigits > MAX_OUTPUT_DIGITS) {
+        g_assert_not_reached();
+        return NULL;
+    }
+
+    /* If the difference is 0, print it as -0 in the right format.
+     * Using IEEE754 negative zero is tricky and would not work
+     * if compiled with -ffast-math (which we tend to do for performance
+     * reasons). Make a special case and print the minus sign explicitely.
+     */
+
     if (!pci->nMatchTo || !fOutputMWC) {
-        sprintf(sz, "%+*.*f", fOutputDigits + 3, fOutputDigits, r1 - r2);
+        if (r1 == r2)
+            snprintf(sz, OUTPUT_SZ_LENGTH, "-%*.*f", fOutputDigits + 2, fOutputDigits, 0.0);
+        else
+            snprintf(sz, OUTPUT_SZ_LENGTH, "%+*.*f", fOutputDigits + 3, fOutputDigits, r1 - r2);
     } else {
         if (fOutputMatchPC) {
-            sprintf(sz, "%*.*f%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0,
-                    100.0f * eq2mwc(r1, pci) - 100.0f * eq2mwc(r2, pci));
+            if (r1 == r2)
+                snprintf(sz, OUTPUT_SZ_LENGTH, " -%*.*f%%", fOutputDigits + 1,
+                         fOutputDigits > 1 ? fOutputDigits - 1 : 0, 0.0);
+            else
+                snprintf(sz, OUTPUT_SZ_LENGTH, "%+*.*f%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0,
+                         100.0f * eq2mwc(r1, pci) - 100.0f * eq2mwc(r2, pci));
         } else {
-            sprintf(sz, "%*.*f", fOutputDigits + 3, fOutputDigits + 1, eq2mwc(r1, pci) - eq2mwc(r2, pci));
+            if (r1 == r2)
+                snprintf(sz, OUTPUT_SZ_LENGTH, "-%*.*f", fOutputDigits + 2, fOutputDigits + 1, 0.0);
+            else
+                snprintf(sz, OUTPUT_SZ_LENGTH, "+%*.*f", fOutputDigits + 3, fOutputDigits + 1,
+                         eq2mwc(r1, pci) - eq2mwc(r2, pci));
         }
     }
 
@@ -556,23 +591,24 @@ OutputMWC(const float r, const cubeinfo * pci, const int f)
 
     if (!pci->nMatchTo) {
         if (f)
-            sprintf(sz, "%+*.*f", fOutputDigits + 3, fOutputDigits, r);
+            snprintf(sz, OUTPUT_SZ_LENGTH, "%+*.*f", fOutputDigits + 3, fOutputDigits, r);
         else
-            sprintf(sz, "% *.*f", fOutputDigits + 2, fOutputDigits, r);
+            snprintf(sz, OUTPUT_SZ_LENGTH, "% *.*f", fOutputDigits + 2, fOutputDigits, r);
     } else {
 
         if (!fOutputMWC) {
             if (f)
-                sprintf(sz, "%+*.*f", fOutputDigits + 3, fOutputDigits, mwc2eq(r, pci));
+                snprintf(sz, OUTPUT_SZ_LENGTH, "%+*.*f", fOutputDigits + 3, fOutputDigits, mwc2eq(r, pci));
             else
-                sprintf(sz, "% *.*f", fOutputDigits + 3, fOutputDigits, se_mwc2eq(r, pci));
+                snprintf(sz, OUTPUT_SZ_LENGTH, "% *.*f", fOutputDigits + 3, fOutputDigits, se_mwc2eq(r, pci));
         } else if (fOutputMatchPC) {
-            sprintf(sz, "%*.*f%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0, 100.0f * r);
+            snprintf(sz, OUTPUT_SZ_LENGTH, "%*.*f%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0,
+                     100.0f * r);
         } else {
             if (f)
-                sprintf(sz, "%+*.*f", fOutputDigits + 3, fOutputDigits + 1, r);
+                snprintf(sz, OUTPUT_SZ_LENGTH, "%+*.*f", fOutputDigits + 3, fOutputDigits + 1, r);
             else
-                sprintf(sz, "% *.*f", fOutputDigits + 3, fOutputDigits + 1, r);
+                snprintf(sz, OUTPUT_SZ_LENGTH, "% *.*f", fOutputDigits + 3, fOutputDigits + 1, r);
         }
     }
 
@@ -588,9 +624,10 @@ OutputPercent(const float r)
     static char sz[OUTPUT_SZ_LENGTH];
 
     if (fOutputWinPC) {
-        sprintf(sz, "%*.*f", fOutputDigits + 2, fOutputDigits > 2 ? fOutputDigits - 2 : 0, 100.0 * r);
+        snprintf(sz, OUTPUT_SZ_LENGTH, "%*.*f", fOutputDigits + 2, fOutputDigits > 2 ? fOutputDigits - 2 : 0,
+                 100.0f * r);
     } else {
-        sprintf(sz, "%*.*f", fOutputDigits + 2, fOutputDigits, r);
+        snprintf(sz, OUTPUT_SZ_LENGTH, "%*.*f", fOutputDigits + 2, fOutputDigits, r);
     }
 
     return sz;
@@ -758,7 +795,7 @@ OutputCubeAnalysisFull(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
 
     }
 
-    strcat(sz, OutputCubeAnalysis(aarOutput, aarStdDev, pes, pci));
+    strcat(sz, OutputCubeAnalysis(aarOutput, aarStdDev, pes, pci, fTake));
 
     return sz;
 
@@ -766,7 +803,7 @@ OutputCubeAnalysisFull(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
 
 extern char *
 OutputCubeAnalysis(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
-                   float aarStdDev[2][NUM_ROLLOUT_OUTPUTS], const evalsetup * pes, const cubeinfo * pci)
+                   float aarStdDev[2][NUM_ROLLOUT_OUTPUTS], const evalsetup * pes, const cubeinfo * pci, int fTake)
 {
 
     static char sz[4096];
@@ -803,6 +840,15 @@ OutputCubeAnalysis(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
         break;
     }
 
+    if (fTake >= 0) { /* take or drop */
+        InvertEvaluationR(aarOutput[0], pci);
+        InvertEvaluationR(aarOutput[1], pci);
+        if (pes->et == EVAL_ROLLOUT) {
+            InvertStdDev(aarStdDev[0]);
+            InvertStdDev(aarStdDev[1]);
+        }
+    }
+
     if (pci->nMatchTo)
         sprintf(strchr(sz, 0), " %s %s (%s: %s)\n",
                 (!pci->nMatchTo || !fOutputMWC) ?
@@ -842,10 +888,17 @@ OutputCubeAnalysis(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
 
         sprintf(strchr(sz, 0), "%d. %-20s", i + 1, gettext(aszCube[ai[i]]));
 
-        strcat(sz, OutputEquity(arDouble[ai[i]], pci, TRUE));
+        if (fTake == -1) { /* double */
+            strcat(sz, OutputEquity(arDouble[ai[i]], pci, TRUE));
 
-        if (i)
-            sprintf(strchr(sz, 0), "  (%s)", OutputEquityDiff(arDouble[ai[i]], arDouble[OUTPUT_OPTIMAL], pci));
+            if (i)
+                sprintf(strchr(sz, 0), "  (%s)", OutputEquityDiff(arDouble[ai[i]], arDouble[OUTPUT_OPTIMAL], pci));
+        } else { /* take or drop */
+            strcat(sz, OutputEquity(-arDouble[ai[i]], pci, TRUE));
+
+            if (i)
+                sprintf(strchr(sz, 0), "  (%s)", OutputEquityDiff(-arDouble[ai[i]], -arDouble[OUTPUT_OPTIMAL], pci));
+        }
         strcat(sz, "\n");
 
     }
@@ -856,7 +909,7 @@ OutputCubeAnalysis(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
 
     sprintf(strchr(sz, 0), "%s: %s", _("Proper cube action"), GetCubeRecommendation(cd));
 
-    if ((r = getPercent(cd, arDouble)) >= 0.0)
+    if ((r = getPercent(cd, arDouble)) >= 0.0f)
         sprintf(strchr(sz, 0), " (%.1f%%)", 100.0f * r);
 
     strcat(sz, "\n");
@@ -865,7 +918,7 @@ OutputCubeAnalysis(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
 
     if (pes->et == EVAL_ROLLOUT && exsExport.fCubeDetailProb) {
 
-        char asz[2][1024];
+        char asz[2][FORMATEDMOVESIZE];
         cubeinfo aci[2];
 
         for (i = 0; i < 2; i++) {
@@ -888,6 +941,15 @@ OutputCubeAnalysis(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
 
     }
 
+    if (fTake >= 0) { /* take or drop */
+        InvertEvaluationR(aarOutput[0], pci);
+        InvertEvaluationR(aarOutput[1], pci);
+        if (pes->et == EVAL_ROLLOUT) {
+            InvertStdDev(aarStdDev[0]);
+            InvertStdDev(aarStdDev[1]);
+        }
+    }
+
     if (pes->et == EVAL_ROLLOUT && exsExport.afCubeParameters[1])
         strcat(strchr(sz, 0), OutputRolloutContext(NULL, &pes->rc));
 
@@ -896,7 +958,7 @@ OutputCubeAnalysis(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
 
 
 extern void
-FormatCubePositions(const cubeinfo * pci, char asz[2][40])
+FormatCubePositions(const cubeinfo * pci, char asz[2][FORMATEDMOVESIZE])
 {
 
     cubeinfo aci[2];
@@ -916,11 +978,18 @@ FormatCubePositions(const cubeinfo * pci, char asz[2][40])
 extern char *
 FormatCubePosition(char *sz, cubeinfo * pci)
 {
-
+    /* FIXME: functions showing rollout progress can call either
+     * FormatMove() from drawboard.c where we know that sz is 
+     * shorter than FORMATEDMOVESIZE, or this one. Moreover they
+     * depend on it to be this short.
+     * Truncate the result if necessary to avoid crashes with long
+     * player names or some translations. Remove the initial
+     * "Player " to save a few characters.
+     */
     if (pci->fCubeOwner == -1)
-        sprintf(sz, _("Centered %d-cube"), pci->nCube);
+        snprintf(sz, FORMATEDMOVESIZE, _("Centered %d-cube"), pci->nCube);
     else
-        sprintf(sz, _("Player %s owns %d-cube"), ap[pci->fCubeOwner].szName, pci->nCube);
+        snprintf(sz, FORMATEDMOVESIZE, _("%s owns %d-cube"), ap[pci->fCubeOwner].szName, pci->nCube);
 
     return sz;
 
@@ -935,15 +1004,15 @@ DumpOver(const TanBoard anBoard, char *pchOutput, const bgvariation bgv)
     if (EvalOver(anBoard, ar, bgv, NULL))
         return -1;
 
-    if (ar[OUTPUT_WIN] > 0.0)
+    if (ar[OUTPUT_WIN] > 0.0f)
         strcpy(pchOutput, _("Win"));
     else
         strcpy(pchOutput, _("Loss"));
     strcat(pchOutput, " ");
 
-    if (ar[OUTPUT_WINBACKGAMMON] > 0.0 || ar[OUTPUT_LOSEBACKGAMMON] > 0.0)
+    if (ar[OUTPUT_WINBACKGAMMON] > 0.0f || ar[OUTPUT_LOSEBACKGAMMON] > 0.0f)
         sprintf(pchOutput, "(%s)\n", _("backgammon"));
-    else if (ar[OUTPUT_WINGAMMON] > 0.0 || ar[OUTPUT_LOSEGAMMON] > 0.0)
+    else if (ar[OUTPUT_WINGAMMON] > 0.0f || ar[OUTPUT_LOSEGAMMON] > 0.0f)
         sprintf(pchOutput, "(%s)\n", _("gammon"));
     else
         sprintf(pchOutput, "(%s)\n", _("single"));
@@ -1171,7 +1240,7 @@ DumpPosition(const TanBoard anBoard, char *szOutput,
         es.ec = *pec;
 
         strcat(szOutput, "\n\n");
-        strcat(szOutput, OutputCubeAnalysis(aarOutput, NULL, &es, pci));
+        strcat(szOutput, OutputCubeAnalysis(aarOutput, NULL, &es, pci, -1));
 
     }
 
