@@ -1,11 +1,11 @@
 /*
- * format.h
+ * Copyright (C) 2003 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2003-2021 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2003
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,15 +13,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: format.h,v 1.15 2015/10/16 19:59:06 plm Exp $
+ * $Id: format.h,v 1.19 2021/06/08 20:56:30 plm Exp $
  */
 
 #ifndef FORMAT_H
 #define FORMAT_H
 
+#include "drawboard.h"
+
 #define MAX_OUTPUT_DIGITS 6
 
 extern int fOutputMWC, fOutputWinPC, fOutputMatchPC;
@@ -47,7 +48,7 @@ extern char *OutputEquityDiff(const float r1, const float r2, const cubeinfo * p
 extern char *OutputEquityScale(const float r, const cubeinfo * pci, const cubeinfo * pciBase, const int f);
 
 extern char *OutputRolloutResult(const char *szIndent,
-                                 char asz[][1024],
+                                 char asz[][FORMATEDMOVESIZE],
                                  float aarOutput[][NUM_ROLLOUT_OUTPUTS],
                                  float aarStdDev[][NUM_ROLLOUT_OUTPUTS],
                                  const cubeinfo aci[], const int alt, const int cci, const int fCubeful);
@@ -58,12 +59,11 @@ extern char *OutputCubeAnalysisFull(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
                                     int fDouble, int fTake, skilltype stDouble, skilltype stTake);
 
 extern char *OutputCubeAnalysis(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
-                                float aarStdDev[2][NUM_ROLLOUT_OUTPUTS], const evalsetup * pes, const cubeinfo * pci);
+                                float aarStdDev[2][NUM_ROLLOUT_OUTPUTS], const evalsetup * pes, const cubeinfo * pci, int fTake);
 
 extern char *OutputMoneyEquity(const float ar[], const int f);
 
 extern char *FormatCubePosition(char *sz, cubeinfo * pci);
-extern void
- FormatCubePositions(const cubeinfo * pci, char asz[2][40]);
+extern void FormatCubePositions(const cubeinfo * pci, char asz[2][FORMATEDMOVESIZE]);
 
 #endif                          /* FORMAT_H */
