@@ -1,11 +1,10 @@
 /*
- * glib-ext.c -- Map/GList extensions and utility functions for GLIB
+ * Copyright (C) 2014-2015 Michael Petch <mpetch@gnubg.org>
  *
- * by Michael Petch <mpetch@gnubg.org>, 2014.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,57 +12,20 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: glib-ext.c,v 1.12 2015/07/31 23:57:55 mdpetch Exp $
+ * $Id: glib-ext.c,v 1.16 2020/11/08 22:26:24 plm Exp $
  */
 
+/* Map/GList extensions and utility functions for GLIB */
 
+#include <config.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "glib-ext.h"
 
-/*  Based on glib's g_fopen to work around issues with WIN32 
- *  versions of g_fopen 
- */
-
-FILE *
-gnubg_g_fopen(const gchar * filename, const gchar * mode)
-{
-#if defined(WIN32)
-    wchar_t *wfilename = g_utf8_to_utf16(filename, -1, NULL, NULL, NULL);
-    wchar_t *wmode;
-    FILE *retval;
-    int save_errno;
-
-    if (wfilename == NULL) {
-        errno = EINVAL;
-        return NULL;
-    }
-
-    wmode = g_utf8_to_utf16(mode, -1, NULL, NULL, NULL);
-
-    if (wmode == NULL) {
-        g_free(wfilename);
-        errno = EINVAL;
-        return NULL;
-    }
-
-    retval = _wfopen(wfilename, wmode);
-    save_errno = errno;
-
-    g_free(wfilename);
-    g_free(wmode);
-
-    errno = save_errno;
-    return retval;
-#else
-    return fopen(filename, mode);
-#endif
-}
 
 #if ! GLIB_CHECK_VERSION(2,14,0)
 
@@ -144,9 +106,12 @@ glib_ext_init(void)
     return;
 }
 
+/* GNU indent misformats the next 3 lines, but adding ; is inadequate */
+
 GLIBEXT_DEFINE_BOXED_TYPE(GListBoxed, g_list_boxed, g_list_copy, g_list_gv_boxed_free)
-    GLIBEXT_DEFINE_BOXED_TYPE(GMapBoxed, g_map_boxed, g_list_copy, g_list_gv_boxed_free)
-    GLIBEXT_DEFINE_BOXED_TYPE(GMapEntryBoxed, g_mapentry_boxed, g_list_copy, g_list_gv_boxed_free)
+GLIBEXT_DEFINE_BOXED_TYPE(GMapBoxed, g_map_boxed, g_list_copy, g_list_gv_boxed_free)
+GLIBEXT_DEFINE_BOXED_TYPE(GMapEntryBoxed, g_mapentry_boxed, g_list_copy, g_list_gv_boxed_free)
+
 #if ! GLIB_CHECK_VERSION(2,28,0)
 void
 g_list_free_full(GList * list, GDestroyNotify free_func)
@@ -228,24 +193,6 @@ create_str2gvalue_tuple(char *str, GValue * gv)
     return g_list_prepend(g_list_prepend(NULL, gv), gvstr);
 }
 
-GList *
-create_str2double_tuple(char *str, double value)
-{
-    GString *tmpstr = g_string_new(str);
-    GVALUE_CREATE(G_TYPE_DOUBLE, double, value, gvdouble);
-    GVALUE_CREATE(G_TYPE_GSTRING, boxed, tmpstr, gvstr);
-    g_string_free(tmpstr, TRUE);
-    return g_list_prepend(g_list_prepend(NULL, gvdouble), gvstr);
-}
-
-void
-free_strmap_tuple(GList * tuple)
-{
-    g_string_free(g_list_nth_data(tuple, 0), TRUE);
-    g_free(g_list_nth_data(tuple, 1));
-    g_list_free(tuple);
-}
-
 void
 g_value_list_tostring(GString * str, GList * list, int depth)
 {
