@@ -1,11 +1,11 @@
 /*
- * gnubgmodule.h
+ * Copyright (C) 2003 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2003-2014 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2003
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,18 +13,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gnubgmodule.h,v 1.31 2014/08/07 21:51:48 mdpetch Exp $
+ * $Id: gnubgmodule.h,v 1.32 2019/11/05 21:17:14 plm Exp $
  */
 
 #ifndef GNUBGMODULE_H
 #define GNUBGMODULE_H
 
-#if USE_PYTHON
+#if defined(USE_PYTHON)
 
-#ifdef WIN32
+#if defined(WIN32)
 /* needed for mingw inclusion of Python.h */
 #include <stdint.h>
 #endif
