@@ -1,27 +1,26 @@
-/* gtk-multiview.c
- * Copyright (C) 2000  Jonathan Blandford
+/*
+ * Copyright (C) 2000 Jonathan Blandford
+ * Copyright (C) 2001-2019 the AUTHORS
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * License changed from the GNU LGPL to the GNU GPL (as permitted under
+ * Term 3 of the GNU LGPL) by Gary Wong for distribution with GNU Backgammon. 
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtk-multiview.c,v 1.17 2016/08/19 22:57:24 plm Exp $
+ * $Id: gtk-multiview.c,v 1.21 2022/09/30 19:42:23 plm Exp $
  */
 
-/* License changed from the GNU LGPL to the GNU GPL (as permitted
- * under Term 3 of the GNU LGPL) by Gary Wong for distribution
- * with GNU Backgammon. */
-
 #include "config.h"
 #include "common.h"
 #include "gtklocdefs.h"
@@ -107,7 +106,11 @@ gtk_multiview_size_request(GtkWidget * widget, GtkRequisition * requisition)
         tmp_list = tmp_list->next;
 
         if (gtk_widget_get_visible(child)) {
+#if GTK_CHECK_VERSION(3,0,0)
+            gtk_widget_get_preferred_size(child, &child_requisition, NULL);
+#else
             gtk_widget_size_request(child, &child_requisition);
+#endif
             requisition->width = MAX(requisition->width, child_requisition.width);
             requisition->height = MAX(requisition->height, child_requisition.height);
             if (gtk_widget_get_mapped(child) && child != multiview->current) {
@@ -140,7 +143,6 @@ gtk_multiview_size_allocate(GtkWidget * widget, GtkAllocation * allocation)
 {
     GtkMultiview *multiview;
     GList *tmp_list;
-    GtkWidget *child;
 
     g_return_if_fail(widget != NULL);
     g_return_if_fail(GTK_IS_MULTIVIEW(widget));
@@ -150,7 +152,8 @@ gtk_multiview_size_allocate(GtkWidget * widget, GtkAllocation * allocation)
 
     tmp_list = multiview->children;
     while (tmp_list) {
-        child = GTK_WIDGET(tmp_list->data);
+        GtkWidget *child = GTK_WIDGET(tmp_list->data);
+
         tmp_list = tmp_list->next;
 
         if (gtk_widget_get_visible(child)) {
@@ -201,7 +204,6 @@ static void
 gtk_multiview_forall(GtkContainer * container,
                      gboolean UNUSED(include_internals), GtkCallback callback, gpointer callback_data)
 {
-    GtkWidget *child;
     GtkMultiview *multiview;
     GList *tmp_list;
 
@@ -213,7 +215,8 @@ gtk_multiview_forall(GtkContainer * container,
 
     tmp_list = multiview->children;
     while (tmp_list) {
-        child = GTK_WIDGET(tmp_list->data);
+        GtkWidget *child = GTK_WIDGET(tmp_list->data);
+
         tmp_list = tmp_list->next;
         (*callback) (GTK_WIDGET(child), callback_data);
     }
