@@ -1,11 +1,11 @@
 /*
- * gtkboard.h
+ * Copyright (C) 2000-2003 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2002-2016 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 1997-2001.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,27 +13,24 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkboard.h,v 1.107 2018/05/05 20:52:19 plm Exp $
+ * $Id: gtkboard.h,v 1.111 2021/10/24 15:10:41 plm Exp $
  */
 
 #ifndef GTKBOARD_H
 #define GTKBOARD_H
 
 #include "backgammon.h"
+
 #include <gtk/gtk.h>
+
 #include "eval.h"
 #include "gtkpanels.h"
 #include "common.h"
 #include "render.h"
 #include "gtklocdefs.h"
 
-#if defined(USE_BOARD3D)
-#include "types3d.h"
-#endif
-
 #define TYPE_BOARD			(board_get_type ())
 #define BOARD(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BOARD, Board))
 #define BOARD_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BOARD, BoardClass))
@@ -41,26 +38,16 @@
 #define IS_BOARD_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE ((obj), TYPE_BOARD))
 #define BOARD_GET_CLASS(obj)  		(G_TYPE_INSTANCE_GET_CLASS((obj), TYPE_BOARD, BoardClass))
 
-typedef struct _BoardData BoardData;    /* Forward declaration for use in Board */
-typedef struct _Board Board;
-typedef struct _BoardClass BoardClass;
-
-struct _BoardClass {
+typedef struct {
     GtkVBoxClass parent_class;
-};
+} BoardClass;
 
-struct _Board {
-    GtkVBox vbox;
-    /* private data */
-    BoardData *board_data;
-};
-
-typedef enum _DiceShown {
+typedef enum {
     DICE_NOT_SHOWN = 0, DICE_BELOW_BOARD, DICE_ON_BOARD, DICE_ROLLING
 } DiceShown;
 
 /* private data */
-struct _BoardData {
+typedef struct {
     GtkWidget *drawing_area, *dice_area, *table, *wmove,
         *reset, *edit, *name0, *name1, *score0, *score1,
         *crawford, *jacoby, *widget, *key0, *key1, *stop, *stopparent,
@@ -115,14 +102,19 @@ struct _BoardData {
     BoardData3d *bd3d;          /* extra members for 3d board */
 #endif
     renderdata *rd;             /* The board colour settings */
-};
+} BoardData;
 
+typedef struct {
+    GtkVBox vboxxx;
+    /* private data */
+    BoardData *board_data;
+} Board;
 
-typedef enum _animation {
+typedef enum {
     ANIMATE_NONE, ANIMATE_BLINK, ANIMATE_SLIDE
 } animation;
 
-typedef enum _GuiShowPips {
+typedef enum {
     GUI_SHOW_PIPS_NONE, GUI_SHOW_PIPS_PIPS, GUI_SHOW_PIPS_EPC, GUI_SHOW_PIPS_WASTAGE, N_GUI_SHOW_PIPS
 } GuiShowPips;
 
@@ -139,7 +131,7 @@ extern int fGUIGrayEdit;
 extern unsigned int nGUIAnimSpeed;
 
 extern GType board_get_type(void);
-extern GtkWidget *board_new(renderdata * prd);
+extern GtkWidget *board_new(renderdata * prd, int inPreview);
 extern GtkWidget *board_cube_widget(Board * board);
 extern void DestroySetCube(GObject * po, GtkWidget * pw);
 extern void Copy3dDiceColour(renderdata * prd);
@@ -174,7 +166,7 @@ extern void RollDice2d(BoardData * bd);
 extern void DestroyPanel(gnubgwindow window);
 extern void
 DrawDie(cairo_t * pd,
-        unsigned char *achDice[2], unsigned char *achPip[2],
+        unsigned char *achDice[], unsigned char *achPip[],
         const int s, int x, int y, int fColour, int n, int alpha);
 
 extern int UpdateMove(BoardData * bd, TanBoard anBoard);
