@@ -1,11 +1,11 @@
 /*
- * gtkchequer.c
+ * Copyright (C) 2002-2003 Joern Thyssen <jthyssen@dk.ibm.com>
+ * Copyright (C) 2002-2022 the AUTHORS
  *
- * by Joern Thyssen <jthyssen@dk.ibm.com>, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkchequer.c,v 1.121 2016/08/24 21:52:22 plm Exp $
+ * $Id: gtkchequer.c,v 1.134 2022/08/30 18:38:35 plm Exp $
  */
 
 #include "config.h"
@@ -35,15 +34,12 @@
 #include "gtkgame.h"
 #include "gtkchequer.h"
 #include "gtktempmap.h"
+#include "gtkscoremap.h"
 #include "gtkwindows.h"
 #include "progress.h"
 #include "format.h"
-#if defined(USE_BOARD3D)
-#include "fun3d.h"
-#endif
 #include "gtklocdefs.h"
 
-
 int showMoveListDetail = 1;
 moverecord *pmrCurAnn;
 
@@ -51,11 +47,11 @@ static void
 MoveListRolloutClicked(GtkWidget * pw, hintdata * phd)
 {
     cubeinfo ci;
-    int i, c, res;
-    move *m;
     int *ai;
     void *p;
     GList *pl, *plSelList = MoveListGetSelectionList(phd);
+    int c;
+
     if (!plSelList)
         return;
 
@@ -65,12 +61,14 @@ MoveListRolloutClicked(GtkWidget * pw, hintdata * phd)
 
     /* setup rollout dialog */
     {
-        move **ppm = (move **) malloc(c * sizeof(move *));
-        cubeinfo **ppci = (cubeinfo **) malloc(c * sizeof(cubeinfo *));
-        char (*asz)[40] = (char (*)[40]) malloc(40 * c);
+        move **ppm = (move **) g_malloc(c * sizeof(move *));
+        cubeinfo **ppci = (cubeinfo **) g_malloc(c * sizeof(cubeinfo *));
+        char (*asz)[FORMATEDMOVESIZE] = (char (*)[FORMATEDMOVESIZE]) g_malloc(FORMATEDMOVESIZE * c);
+        int i, res;
 
         for (i = 0, pl = plSelList; i < c; pl = pl->next, i++) {
-            m = ppm[i] = MoveListGetMove(phd, pl);
+            move *m = ppm[i] = MoveListGetMove(phd, pl);
+
             ppci[i] = &ci;
             FormatMove(asz[i], msBoard(), m->anMove);
         }
@@ -83,9 +81,9 @@ MoveListRolloutClicked(GtkWidget * pw, hintdata * phd)
 
         RolloutProgressEnd(&p, FALSE);
 
-        free(asz);
-        free(ppm);
-        free(ppci);
+        g_free(asz);
+        g_free(ppm);
+        g_free(ppci);
 
         if (res < 0)
             return;
@@ -102,13 +100,13 @@ MoveListRolloutClicked(GtkWidget * pw, hintdata * phd)
 
     MoveListClearSelection(0, 0, phd);
 
-    ai = (int *) malloc(phd->pml->cMoves * sizeof(int));
+    ai = (int *) g_malloc(phd->pml->cMoves * sizeof(int));
     RefreshMoveList(phd->pml, ai);
 
     if (phd->piHighlight && phd->pml->cMoves)
         *phd->piHighlight = ai[*phd->piHighlight];
 
-    free(ai);
+    g_free(ai);
 
     find_skills(phd->pmr, &ms, -1, -1);
     MoveListUpdate(phd);
@@ -121,11 +119,11 @@ MoveListRolloutClicked(GtkWidget * pw, hintdata * phd)
 extern void
 ShowMove(hintdata * phd, const int f)
 {
-    char *sz;
-    TanBoard anBoard;
     if (f) {
         move *pm;
         GList *plSelList = MoveListGetSelectionList(phd);
+        TanBoard anBoard;
+
         if (!plSelList)
             return;
 
@@ -140,21 +138,18 @@ ShowMove(hintdata * phd, const int f)
         UpdateMove((BOARD(pwBoard))->board_data, anBoard);
     } else {
 
-        sz = g_strdup("show board");
+        gchar *sz = g_strdup("show board");
         UserCommand(sz);
         g_free(sz);
 
     }
-#if defined(USE_BOARD3D)
-    RestrictiveRedraw();
-#endif
 }
 
 static void
 MoveListTempMapClicked(GtkWidget * pw, hintdata * phd)
 {
     GList *pl;
-    char szMove[100];
+    char szMove[FORMATEDMOVESIZE];
     matchstate *ams;
     int i, c;
     gchar **asz;
@@ -201,6 +196,16 @@ MoveListTempMapClicked(GtkWidget * pw, hintdata * phd)
     g_free(asz);
 }
 
+static void
+MoveListScoreMapClicked(GtkWidget * UNUSED(pw), hintdata * UNUSED(phd))
+{
+
+    char *sz = g_strdup("show scoremap =move"); //cf keyword -> gtkgame.c: CMD_SHOW_SCORE_MAP
+    UserCommand(sz);
+    g_free(sz);  
+
+}
+
 static void
 MoveListCmarkClicked(GtkWidget * UNUSED(pw), hintdata * phd)
 {
@@ -229,14 +234,15 @@ MoveListCmarkClicked(GtkWidget * UNUSED(pw), hintdata * phd)
 static void
 MoveListMWC(GtkWidget * pw, hintdata * phd)
 {
-    char sz[80];
     int f = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pw));
 
     if (f != fOutputMWC) {
-        sprintf(sz, "set output mwc %s", fOutputMWC ? "off" : "on");
+        gchar *sz = g_strdup_printf("set output mwc %s", fOutputMWC ? "off" : "on");
 
         UserCommand(sz);
         UserCommand("save settings");
+
+        g_free(sz);
     }
 
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pw), fOutputMWC);
@@ -279,13 +285,13 @@ EvalMoves(hintdata * phd, evalcontext * pec)
 
     MoveListClearSelection(0, 0, phd);
 
-    ai = (int *) malloc(phd->pml->cMoves * sizeof(int));
+    ai = (int *) g_malloc(phd->pml->cMoves * sizeof(int));
     RefreshMoveList(phd->pml, ai);
 
     if (phd->piHighlight && phd->pml->cMoves)
         *phd->piHighlight = ai[*phd->piHighlight];
 
-    free(ai);
+    g_free(ai);
 
     find_skills(phd->pmr, &ms, -1, -1);
     MoveListUpdate(phd);
@@ -363,9 +369,8 @@ MoveListRolloutPresets(GtkWidget * pw, hintdata * phd)
 typedef int (*cfunc) (const void *, const void *);
 
 static int
-CompareInts(int *p0, int *p1)
+CompareInts(const int *p0, const int *p1)
 {
-
     return *p0 - *p1;
 }
 
@@ -379,10 +384,10 @@ MoveListCopyData(hintdata * phd)
     GList *plSelList = MoveListGetSelectionList(phd);
     c = g_list_length(plSelList);
 
-    an = (int *) malloc(c * sizeof(an[0]));
+    an = (int *) g_malloc(c * sizeof(an[0]));
     /* TODO: This needs to be cleaned up since the maximum number of
      * lines or length of a string can vary depending on settings */
-    sz = (char *) malloc(c * 25 * 80);
+    sz = (char *) g_malloc(c * 25 * 80);
 
     *sz = 0;
 
@@ -401,7 +406,7 @@ MoveListCopyData(hintdata * phd)
     for (i = 0, pch = sz; i < c; i++, pch = strchr(pch, 0))
         FormatMoveHint(pch, &ms, phd->pml, an[i], TRUE, TRUE, TRUE);
 
-    free(an);
+    g_free(an);
 
     return sz;
 }
@@ -428,10 +433,6 @@ MoveListMove(GtkWidget * pw, hintdata * phd)
 
     FormatMove(szMove, msBoard(), m.anMove);
     UserCommand(szMove);
-
-#if defined(USE_BOARD3D)
-    RestrictiveRedraw();
-#endif
 }
 
 static void
@@ -450,7 +451,7 @@ MoveListCopy(GtkWidget * UNUSED(pw), hintdata * phd)
     char *pc = MoveListCopyData(phd);
     if (pc) {
         GTKTextWindow(pc, _("Move details"), DT_INFO, NULL);
-        free(pc);
+        g_free(pc);
     }
 }
 
@@ -463,14 +464,13 @@ CreateMoveListTools(hintdata * phd)
     GtkWidget *pwRollout = gtk_button_new_with_label(_("Rollout"));
     GtkWidget *pwRolloutSettings = gtk_button_new_with_label(_("..."));
     GtkWidget *pwMWC = gtk_toggle_button_new_with_label(_("MWC"));
-    GtkWidget *pwMove = gtk_button_new_with_label(_("Move"));
+    GtkWidget *pwMove = gtk_button_new_with_label(Q_("verb|Move"));
     GtkWidget *pwShow = gtk_toggle_button_new_with_label(_("Show"));
     GtkWidget *pwCopy = gtk_button_new_with_label(_("Copy"));
     GtkWidget *pwTempMap = gtk_button_new_with_label(_("TM"));
     GtkWidget *pwCmark = gtk_button_new_with_label(_("Cmark"));
-    GtkWidget *pwply;
+    GtkWidget *pwScoreMap = gtk_button_new_with_label(_("Score Map"));     
     int i;
-    char *sz;
 
     pwDetails = phd->fDetails ? NULL : gtk_toggle_button_new_with_label(_("Details"));
     phd->pwRollout = pwRollout;
@@ -482,6 +482,7 @@ CreateMoveListTools(hintdata * phd)
     phd->pwCopy = pwCopy;
     phd->pwTempMap = pwTempMap;
     phd->pwCmark = pwCmark;
+    phd->pwScoreMap = pwScoreMap;
 
 #if GTK_CHECK_VERSION(3,0,0)
     gtk_style_context_add_class(gtk_widget_get_style_context(pwEval), "gnubg-analysis-button");
@@ -498,22 +499,38 @@ CreateMoveListTools(hintdata * phd)
 
     /* toolbox on the left with buttons for eval, rollout and more */
 
-    pwTools = gtk_table_new(2, phd->fDetails ? 6 : 7, FALSE);
+#if GTK_CHECK_VERSION(3,0,0)
+    pwTools = gtk_grid_new();
+
+    gtk_grid_attach(GTK_GRID(pwTools), pwEval, 0, 0, 1, 1);
+
+    gtk_grid_attach(GTK_GRID(pwTools), pwEvalSettings, 1, 0, 1, 1);
+#else
+    pwTools = gtk_table_new(2, 7, FALSE);
 
     gtk_table_attach(GTK_TABLE(pwTools), pwEval, 0, 1, 0, 1,
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
 
     gtk_table_attach(GTK_TABLE(pwTools), pwEvalSettings, 1, 2, 0, 1,
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+#endif
 
+#if GTK_CHECK_VERSION(3,0,0)
+    phd->pwEvalPly = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+
+    gtk_grid_attach(GTK_GRID(pwTools), phd->pwEvalPly, 2, 0, 1, 1);
+#else
     phd->pwEvalPly = gtk_hbox_new(FALSE, 0);
+
     gtk_table_attach(GTK_TABLE(pwTools), phd->pwEvalPly, 2, 3, 0, 1,
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+#endif
 
     for (i = 0; i < 5; ++i) {
 
-        sz = g_strdup_printf("%d", i);  /* string is freed by set_data_full */
-        pwply = gtk_button_new_with_label(sz);
+        gchar *sz = g_strdup_printf("%d", i);  /* string is freed by set_data_full */
+        GtkWidget *pwply = gtk_button_new_with_label(sz);
+
 #if GTK_CHECK_VERSION(3,0,0)
         gtk_style_context_add_class(gtk_widget_get_style_context(pwply), "gnubg-analysis-button");
 #endif
@@ -529,6 +546,18 @@ CreateMoveListTools(hintdata * phd)
 
     }
 
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(pwTools), pwShow, 3, 0, 1, 1);
+
+    gtk_grid_attach(GTK_GRID(pwTools), pwMWC, 4, 0, 1, 1);
+
+    if (!phd->fDetails)
+        gtk_grid_attach(GTK_GRID(pwTools), pwDetails, 5, 0, 1, 1);
+
+    gtk_grid_attach(GTK_GRID(pwTools), pwRollout, 0, 1, 1, 1);
+
+    gtk_grid_attach(GTK_GRID(pwTools), pwRolloutSettings, 1, 1, 1, 1);
+#else
     gtk_table_attach(GTK_TABLE(pwTools), pwShow, 3, 4, 0, 1,
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
 
@@ -536,7 +565,7 @@ CreateMoveListTools(hintdata * phd)
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
 
     if (!phd->fDetails)
-        gtk_table_attach(GTK_TABLE(pwTools), pwDetails, 5, 7, 0, 1,
+        gtk_table_attach(GTK_TABLE(pwTools), pwDetails, 5, 6, 0, 1,
                          (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);
 
     gtk_table_attach(GTK_TABLE(pwTools), pwRollout, 0, 1, 1, 2,
@@ -544,15 +573,21 @@ CreateMoveListTools(hintdata * phd)
 
     gtk_table_attach(GTK_TABLE(pwTools), pwRolloutSettings, 1, 2, 1, 2,
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+#endif
 
+#if GTK_CHECK_VERSION(3,0,0)
+    phd->pwRolloutPresets = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+    gtk_grid_attach(GTK_GRID(pwTools), phd->pwRolloutPresets, 2, 1, 1, 1);
+#else
     phd->pwRolloutPresets = gtk_hbox_new(FALSE, 0);
     gtk_table_attach(GTK_TABLE(pwTools), phd->pwRolloutPresets, 2, 3, 1, 2,
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+#endif
 
     for (i = 0; i < 5; ++i) {
-        GtkWidget *ro_preset;
-        sz = g_strdup_printf("%c", i + 'a');    /* string is freed by set_data_full */
-        ro_preset = gtk_button_new_with_label(sz);
+        gchar *sz = g_strdup_printf("%c", i + 'a');    /* string is freed by set_data_full */
+        GtkWidget *ro_preset = gtk_button_new_with_label(sz);
+
 #if GTK_CHECK_VERSION(3,0,0)
         gtk_style_context_add_class(gtk_widget_get_style_context(ro_preset), "gnubg-analysis-button");
 #endif
@@ -568,19 +603,32 @@ CreateMoveListTools(hintdata * phd)
 
     }
 
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(pwTools), pwMove, 3, 1, 1, 1);
+
+    gtk_grid_attach(GTK_GRID(pwTools), pwCopy, 4, 1, 1, 1);
+
+    gtk_grid_attach(GTK_GRID(pwTools), pwCmark, 5, 1, 1, 1);
+
+    gtk_grid_attach(GTK_GRID(pwTools), pwTempMap, 6, 0, 1, 1);
+
+    gtk_grid_attach(GTK_GRID(pwTools), pwScoreMap, 6, 1, 1, 1);
+#else
     gtk_table_attach(GTK_TABLE(pwTools), pwMove, 3, 4, 1, 2,
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
 
-
     gtk_table_attach(GTK_TABLE(pwTools), pwCopy, 4, 5, 1, 2,
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
 
     gtk_table_attach(GTK_TABLE(pwTools), pwCmark, 5, 6, 1, 2,
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
 
-    gtk_table_attach(GTK_TABLE(pwTools), pwTempMap, 6, 7, 1, 2,
+    gtk_table_attach(GTK_TABLE(pwTools), pwTempMap, 6, 7, 0, 1,
                      (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
 
+    gtk_table_attach(GTK_TABLE(pwTools), pwScoreMap, 6, 7, 1, 2,
+                     (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0); 
+#endif
 
     gtk_widget_set_sensitive(pwMWC, ms.nMatchTo);
     gtk_widget_set_sensitive(pwMove, FALSE);
@@ -602,27 +650,34 @@ CreateMoveListTools(hintdata * phd)
     g_signal_connect(G_OBJECT(pwShow), "toggled", G_CALLBACK(MoveListShowToggledClicked), phd);
     g_signal_connect(G_OBJECT(pwCopy), "clicked", G_CALLBACK(MoveListCopy), phd);
     g_signal_connect(G_OBJECT(pwTempMap), "clicked", G_CALLBACK(MoveListTempMapClicked), phd);
+    g_signal_connect(G_OBJECT(pwScoreMap), "clicked", G_CALLBACK(MoveListScoreMapClicked), phd);
     g_signal_connect(G_OBJECT(pwCmark), "clicked", G_CALLBACK(MoveListCmarkClicked), phd);
     if (!phd->fDetails)
         g_signal_connect(G_OBJECT(pwDetails), "clicked", G_CALLBACK(MoveListDetailsClicked), phd);
 
     /* tool tips */
 
-    gtk_widget_set_tooltip_text(pwRollout, _("Rollout chequer play with current settings"));
+    gtk_widget_set_tooltip_text(pwRollout, _("Rollout selected moves with current settings"));
 
-    gtk_widget_set_tooltip_text(pwEval, _("Evaluate chequer play with current settings"));
+    gtk_widget_set_tooltip_text(pwEval, _("Evaluate selected moves with current settings"));
 
     gtk_widget_set_tooltip_text(pwRolloutSettings, _("Modify rollout settings"));
 
     gtk_widget_set_tooltip_text(pwEvalSettings, _("Modify evaluation settings"));
 
+    gtk_widget_set_tooltip_text(pwShow, _("Show position after selected move"));
+
     gtk_widget_set_tooltip_text(pwMWC, _("Toggle output as MWC or equity"));
 
     gtk_widget_set_tooltip_text(pwCopy, _("Copy selected moves to clipboard"));
 
-    gtk_widget_set_tooltip_text(pwMove, _("Move the selected move"));
+    gtk_widget_set_tooltip_text(pwMove, _("Play the selected move"));
 
-    gtk_widget_set_tooltip_text(pwTempMap, _("Show Sho Sengoku Temperature Map of position " "after selected move"));
+    gtk_widget_set_tooltip_text(pwCmark, _("Mark selected moves for later rollout"));
+
+    gtk_widget_set_tooltip_text(pwTempMap, _("Show Sho Sengoku Temperature Map of position" " after selected move"));
+
+    gtk_widget_set_tooltip_text(pwScoreMap, _("Show map of best moves at different scores"));
 
 
     return pwTools;
@@ -657,6 +712,8 @@ HintSelect(GtkTreeSelection * selection, hintdata * phd)
 extern int
 CheckHintButtons(hintdata * phd)
 {
+    BoardData *bd;
+
     GList *plSelList = MoveListGetSelectionList(phd);
     int c = g_list_length(plSelList);
     MoveListFreeSelectionList(plSelList);
@@ -670,6 +727,9 @@ CheckHintButtons(hintdata * phd)
     gtk_widget_set_sensitive(phd->pwEval, c && phd->fButtonsValid);
     gtk_widget_set_sensitive(phd->pwEvalPly, c && phd->fButtonsValid);
 
+    bd = BOARD(pwBoard)->board_data;
+    gtk_widget_set_sensitive(phd->pwScoreMap, (bd->diceShown == DICE_ON_BOARD));
+
     return c;
 }
 
@@ -679,7 +739,7 @@ CreateMoveList(moverecord * pmr, const int fButtonsValid, const int fDestroyOnMo
     GtkWidget *pw;
     GtkWidget *pwVBox, *mlt;
 
-    hintdata *phd = (hintdata *) malloc(sizeof(hintdata));
+    hintdata *phd = (hintdata *) g_malloc(sizeof(hintdata));
 
     /* This function should only be called when the game state matches
      * the move list. */
@@ -703,11 +763,15 @@ CreateMoveList(moverecord * pmr, const int fButtonsValid, const int fDestroyOnMo
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
     gtk_container_add(GTK_CONTAINER(pw), phd->pwMoves);
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pwVBox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
     pwVBox = gtk_vbox_new(FALSE, 0);
+#endif
     gtk_box_pack_start(GTK_BOX(pwVBox), pw, TRUE, TRUE, 0);
     gtk_box_pack_end(GTK_BOX(pwVBox), mlt, FALSE, FALSE, 0);
 
-    g_object_set_data_full(G_OBJECT(pwVBox), "user_data", phd, free);
+    g_object_set_data_full(G_OBJECT(pwVBox), "user_data", phd, g_free);
 
     CheckHintButtons(phd);
 
