@@ -1,11 +1,11 @@
 /*
- * gtkexport.c
+ * Copyright (C) 2002-2003 Joern Thyssen <jthyssen@dk.ibm.com>
+ * Copyright (C) 2002-2022 the AUTHORS
  *
- * by Joern Thyssen <jthyssen@dk.ibm.com>, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkexport.c,v 1.55 2018/07/21 22:04:35 plm Exp $
+ * $Id: gtkexport.c,v 1.61 2022/09/06 21:02:05 plm Exp $
  */
 
 #include "config.h"
@@ -65,7 +64,7 @@ static char *aszCubeDisplay[] = {
 
 #define NUM_CUBES (sizeof(aszCubeDisplay)/sizeof(aszCubeDisplay[0]))
 
-typedef struct _exportwidget {
+typedef struct {
 
     /* export settings */
 
@@ -396,17 +395,30 @@ static void
 ExportHTMLImages(void)
 {
     GtkWidget *fc;
-    gchar *message, *expfolder, *folder, *command;
+    gchar *message, *expfolder, *command;
     gint ok = FALSE;
     fc = gtk_file_chooser_dialog_new(_
                                      ("Select top folder for HTML export"),
                                      NULL,
                                      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
+#if GTK_CHECK_VERSION(3,0,0)
+                                     "_Cancel",
+#else
+                                     GTK_STOCK_CANCEL,
+#endif
+                                     GTK_RESPONSE_CANCEL,
+#if GTK_CHECK_VERSION(3,0,0)
+                                     "_Open",
+#else
+                                     GTK_STOCK_OPEN,
+#endif
+                                     GTK_RESPONSE_ACCEPT, NULL);
     gtk_window_set_modal(GTK_WINDOW(fc), TRUE);
     gtk_window_set_transient_for(GTK_WINDOW(fc), GTK_WINDOW(pwMain));
 
     while (!ok) {
+        gchar *folder;
+
         if (gtk_dialog_run(GTK_DIALOG(fc)) == GTK_RESPONSE_CANCEL) {
             gtk_widget_destroy(fc);
             return;
@@ -461,8 +473,13 @@ GTKShowExport(exportsetup * pexs)
 
     GtkWidget *pwVBox;
     GtkWidget *pwFrame;
+#if GTK_CHECK_VERSION(3,0,0)
+    GtkWidget *pwGrid;
+    GtkWidget *pwGridX;
+#else
     GtkWidget *pwTable;
     GtkWidget *pwTableX;
+#endif
     GtkWidget *pwHBox;
     GtkWidget *pwHScale;
     GtkWidget *genHtml;
@@ -473,7 +490,7 @@ GTKShowExport(exportsetup * pexs)
 
     exportwidget *pew;
 
-    pew = (exportwidget *) malloc(sizeof(exportwidget));
+    pew = (exportwidget *) g_malloc(sizeof(exportwidget));
 
     pew->pexs = pexs;
 
@@ -487,15 +504,33 @@ GTKShowExport(exportsetup * pexs)
 
     /* first tab */
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pwGrid = gtk_grid_new();
+
+    gtk_notebook_append_page(GTK_NOTEBOOK(pwNotebook), pwGrid, gtk_label_new(_("Content")));
+#else
     pwTable = gtk_table_new(2, 2, FALSE);
 
     gtk_notebook_append_page(GTK_NOTEBOOK(pwNotebook), pwTable, gtk_label_new(_("Content")));
+#endif
 
     /* include stuff */
 
     pwFrame = gtk_frame_new(_("Include"));
     gtk_container_set_border_width(GTK_CONTAINER(pwFrame), 8);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(pwGrid), pwFrame, 0, 0, 1, 1);
+#if GTK_CHECK_VERSION(3,12,0)
+    gtk_widget_set_margin_start(pwFrame, 8);
+    gtk_widget_set_margin_end(pwFrame, 8);
+#else
+    gtk_widget_set_margin_left(pwFrame, 8);
+    gtk_widget_set_margin_right(pwFrame, 8);
+#endif
+#else
     gtk_table_attach(GTK_TABLE(pwTable), pwFrame, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 8, 0);
+#endif
 
 
 #if GTK_CHECK_VERSION(3,0,0)
@@ -516,18 +551,42 @@ GTKShowExport(exportsetup * pexs)
     pwFrame = gtk_frame_new(_("Board"));
 
     gtk_container_set_border_width(GTK_CONTAINER(pwFrame), 8);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(pwGrid), pwFrame, 1, 0, 1, 1);
+#if GTK_CHECK_VERSION(3,12,0)
+    gtk_widget_set_margin_start(pwFrame, 2);
+    gtk_widget_set_margin_end(pwFrame, 2);
+#else
+    gtk_widget_set_margin_left(pwFrame, 2);
+    gtk_widget_set_margin_right(pwFrame, 2);
+#endif
+    gtk_widget_set_margin_top(pwFrame, 2);
+    gtk_widget_set_margin_bottom(pwFrame, 2);
+#else
     gtk_table_attach(GTK_TABLE(pwTable), pwFrame, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 2, 2);
+#endif
 
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pwGridX = gtk_grid_new();
+    gtk_container_add(GTK_CONTAINER(pwFrame), pwGridX);
 
+    gtk_grid_attach(GTK_GRID(pwGridX), pw = gtk_label_new(_("Board")), 0, 0, 1, 1);
+#if GTK_CHECK_VERSION(3,12,0)
+    gtk_widget_set_margin_start(pw, 4);
+    gtk_widget_set_margin_end(pw, 4);
+#else
+    gtk_widget_set_margin_left(pw, 4);
+    gtk_widget_set_margin_right(pw, 4);
+#endif
+    gtk_widget_set_halign(pw, GTK_ALIGN_START);
+    gtk_widget_set_valign(pw, GTK_ALIGN_CENTER);
+#else
     pwTableX = gtk_table_new(2, 3, FALSE);
     gtk_container_add(GTK_CONTAINER(pwFrame), pwTableX);
 
     gtk_table_attach(GTK_TABLE(pwTableX), pw = gtk_label_new(_("Board")), 0, 1, 0, 1, GTK_FILL, GTK_FILL, 4, 0);
-#if GTK_CHECK_VERSION(3,0,0)
-    gtk_widget_set_halign(pw, GTK_ALIGN_START);
-    gtk_widget_set_valign(pw, GTK_ALIGN_CENTER);
-#else
     gtk_misc_set_alignment(GTK_MISC(pw), 0, 0.5);
 #endif
 
@@ -543,17 +602,44 @@ GTKShowExport(exportsetup * pexs)
 
     gtk_box_pack_start(GTK_BOX(pw), gtk_label_new(_("move(s) between board shown")), TRUE, TRUE, 0);
 
-    gtk_table_attach(GTK_TABLE(pwTableX), pw, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 4, 0);
-
-
-    gtk_table_attach(GTK_TABLE(pwTableX), pw = gtk_label_new(_("Players")), 0, 1, 1, 2, GTK_FILL, GTK_FILL, 4, 0);
 #if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(pwGridX), pw, 1, 0, 1, 1);
+
+    gtk_grid_attach(GTK_GRID(pwGridX), pw = gtk_label_new(_("Players")), 0, 1, 1, 1);
+#if GTK_CHECK_VERSION(3,12,0)
+    gtk_widget_set_margin_start(pw, 4);
+    gtk_widget_set_margin_end(pw, 4);
+#else
+    gtk_widget_set_margin_left(pw, 4);
+    gtk_widget_set_margin_right(pw, 4);
+#endif
     gtk_widget_set_halign(pw, GTK_ALIGN_START);
     gtk_widget_set_valign(pw, GTK_ALIGN_CENTER);
+
+    gtk_grid_attach(GTK_GRID(pwGridX), pw = pew->apwSide[0] = gtk_check_button_new_with_label(ap[0].szName), 1, 1, 1, 1);
+#if GTK_CHECK_VERSION(3,12,0)
+    gtk_widget_set_margin_start(pw, 4);
+    gtk_widget_set_margin_end(pw, 4);
 #else
-    gtk_misc_set_alignment(GTK_MISC(pw), 0, 0.5);
+    gtk_widget_set_margin_left(pw, 4);
+    gtk_widget_set_margin_right(pw, 4);
 #endif
 
+    gtk_grid_attach(GTK_GRID(pwGridX), pw = pew->apwSide[1] = gtk_check_button_new_with_label(ap[1].szName), 1, 2, 1, 1);
+#if GTK_CHECK_VERSION(3,12,0)
+    gtk_widget_set_margin_start(pw, 4);
+    gtk_widget_set_margin_end(pw, 4);
+#else
+    gtk_widget_set_margin_left(pw, 4);
+    gtk_widget_set_margin_right(pw, 4);
+#endif
+
+#else
+    gtk_table_attach(GTK_TABLE(pwTableX), pw, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 4, 0);
+
+    gtk_table_attach(GTK_TABLE(pwTableX), pw = gtk_label_new(_("Players")), 0, 1, 1, 2, GTK_FILL, GTK_FILL, 4, 0);
+    gtk_misc_set_alignment(GTK_MISC(pw), 0, 0.5);
+
     gtk_table_attach(GTK_TABLE(pwTableX),
                      pew->apwSide[0] =
                      gtk_check_button_new_with_label(ap[0].szName), 1, 2, 1, 2, GTK_FILL, GTK_FILL, 4, 0);
@@ -561,13 +647,28 @@ GTKShowExport(exportsetup * pexs)
     gtk_table_attach(GTK_TABLE(pwTableX),
                      pew->apwSide[1] =
                      gtk_check_button_new_with_label(ap[1].szName), 1, 2, 2, 3, GTK_FILL, GTK_FILL, 4, 0);
+#endif
 
     /* moves */
 
     pwFrame = gtk_frame_new(_("Output move analysis"));
 
     gtk_container_set_border_width(GTK_CONTAINER(pwFrame), 8);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(pwGrid), pwFrame, 0, 1, 1, 1);
+#if GTK_CHECK_VERSION(3,12,0)
+    gtk_widget_set_margin_start(pwFrame, 2);
+    gtk_widget_set_margin_end(pwFrame, 2);
+#else
+    gtk_widget_set_margin_left(pwFrame, 2);
+    gtk_widget_set_margin_right(pwFrame, 2);
+#endif
+    gtk_widget_set_margin_top(pwFrame, 2);
+    gtk_widget_set_margin_bottom(pwFrame, 2);
+#else
     gtk_table_attach(GTK_TABLE(pwTable), pwFrame, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 2, 2);
+#endif
 
 #if GTK_CHECK_VERSION(3,0,0)
     pwVBox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
@@ -616,7 +717,20 @@ GTKShowExport(exportsetup * pexs)
     pwFrame = gtk_frame_new(_("Output cube decision analysis"));
 
     gtk_container_set_border_width(GTK_CONTAINER(pwFrame), 8);
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(pwGrid), pwFrame, 1, 1, 1, 1);
+#if GTK_CHECK_VERSION(3,12,0)
+    gtk_widget_set_margin_start(pwFrame, 2);
+    gtk_widget_set_margin_end(pwFrame, 2);
+#else
+    gtk_widget_set_margin_left(pwFrame, 2);
+    gtk_widget_set_margin_right(pwFrame, 2);
+#endif
+    gtk_widget_set_margin_top(pwFrame, 2);
+    gtk_widget_set_margin_bottom(pwFrame, 2);
+#else
     gtk_table_attach(GTK_TABLE(pwTable), pwFrame, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 2, 2);
+#endif
 
 #if GTK_CHECK_VERSION(3,0,0)
     pwVBox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
@@ -650,16 +764,34 @@ GTKShowExport(exportsetup * pexs)
 
     /* second tab */
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pwGrid = gtk_grid_new();
+    gtk_notebook_append_page(GTK_NOTEBOOK(pwNotebook), pwGrid, gtk_label_new(_("Style")));
+#else
     pwTable = gtk_table_new(1, 2, FALSE);
-
     gtk_notebook_append_page(GTK_NOTEBOOK(pwNotebook), pwTable, gtk_label_new(_("Style")));
+#endif
 
     /* HTML */
 
     pwFrame = gtk_frame_new(_("HTML export options"));
 
     gtk_container_set_border_width(GTK_CONTAINER(pwFrame), 8);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(pwGrid), pwFrame, 0, 0, 1, 1);
+#if GTK_CHECK_VERSION(3,12,0)
+    gtk_widget_set_margin_start(pwFrame, 2);
+    gtk_widget_set_margin_end(pwFrame, 2);
+#else
+    gtk_widget_set_margin_left(pwFrame, 2);
+    gtk_widget_set_margin_right(pwFrame, 2);
+#endif
+    gtk_widget_set_margin_top(pwFrame, 2);
+    gtk_widget_set_margin_bottom(pwFrame, 2);
+#else
     gtk_table_attach(GTK_TABLE(pwTable), pwFrame, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 2, 2);
+#endif
 
 #if GTK_CHECK_VERSION(3,0,0)
     pwVBox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
@@ -721,7 +853,21 @@ GTKShowExport(exportsetup * pexs)
     pwFrame = gtk_frame_new(_("Export sizes"));
 
     gtk_container_set_border_width(GTK_CONTAINER(pwFrame), 8);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_attach(GTK_GRID(pwGrid), pwFrame, 1, 0, 1, 1);
+#if GTK_CHECK_VERSION(3,12,0)
+    gtk_widget_set_margin_start(pwFrame, 2);
+    gtk_widget_set_margin_end(pwFrame, 2);
+#else
+    gtk_widget_set_margin_left(pwFrame, 2);
+    gtk_widget_set_margin_right(pwFrame, 2);
+#endif
+    gtk_widget_set_margin_top(pwFrame, 2);
+    gtk_widget_set_margin_bottom(pwFrame, 2);
+#else
     gtk_table_attach(GTK_TABLE(pwTable), pwFrame, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 2, 2);
+#endif
 
 #if GTK_CHECK_VERSION(3,0,0)
     pwVBox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
@@ -731,7 +877,7 @@ GTKShowExport(exportsetup * pexs)
     gtk_container_add(GTK_CONTAINER(pwFrame), pwVBox);
     gtk_container_set_border_width(GTK_CONTAINER(pwVBox), 4);
 
-    /* Png size */
+    /* PNG size */
 #if GTK_CHECK_VERSION(3,0,0)
     pwHBox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 #else
@@ -790,7 +936,7 @@ GTKShowExport(exportsetup * pexs)
     g_signal_connect(G_OBJECT(genHtml), "clicked", G_CALLBACK(GenHtmlImages), pew->adjHtmlSize);
     g_signal_connect(G_OBJECT(pew->adjHtmlSize), "value-changed", G_CALLBACK(SizeChanged), pew->pwHtmlSize);
 
-    g_object_set_data_full(G_OBJECT(pwDialog), "exportwidget", pew, free);
+    g_object_set_data_full(G_OBJECT(pwDialog), "exportwidget", pew, g_free);
 
     /* show dialog */
 
