@@ -1,11 +1,11 @@
 /*
- * gtkmet.c
+ * Copyright (C) 2002-2003 Joern Thyssen <jthyssen@dk.ibm.com>
+ * Copyright (C) 2002-2022 the AUTHORS
  *
- * by Joern Thyssen <jth@freeshell.org>, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkmet.c,v 1.31 2017/05/29 20:05:19 plm Exp $
+ * $Id: gtkmet.c,v 1.36 2022/08/30 18:38:35 plm Exp $
  */
 
 #include "config.h"
@@ -25,16 +24,21 @@
 #include "matchequity.h"
 #include "gtkmet.h"
 #include "gtkwindows.h"
+#include "gtklocdefs.h"
 
-typedef struct _mettable {
+typedef struct {
+#if GTK_CHECK_VERSION(3,0,0)
+    GtkWidget *pwGrid;
+#else
     GtkWidget *pwTable;
+#endif
     GtkWidget *pwName;
     GtkWidget *pwFileName;
     GtkWidget *pwDescription;
     GtkWidget *aapwLabel[MAXSCORE][MAXSCORE];
 } mettable;
 
-typedef struct _metwidget {
+typedef struct {
     GtkWidget *apwPostCrawford[2];
     GtkWidget *pwPreCrawford;
     unsigned int nMatchTo;
@@ -44,7 +48,7 @@ typedef struct _metwidget {
 
 static void
 UpdateTable(const mettable * pmt,
-            /*lint -e{818} */ float met[MAXSCORE][MAXSCORE],
+            /*lint -e{818} */ float met[][MAXSCORE],
             const metinfo * pmi, const unsigned int nRows, const unsigned int nCols, const int fInvert)
 {
 
@@ -88,6 +92,7 @@ UpdateAllTables(const metwidget * pmw)
 
     for (i = 0; i < 2; ++i) {
         pmt = (const mettable *) g_object_get_data(G_OBJECT(pmw->apwPostCrawford[i]), "mettable");
+        /* Ugly cast to pass the the aafMETPostCrawford[i] vector as a [1][] array */
         UpdateTable(pmt, (float (*)[MAXSCORE]) (void *) aafMETPostCrawford[i], &miCurrent, pmw->nMatchTo, 1, TRUE);
     }
 }
@@ -98,7 +103,13 @@ GTKWriteMET(const unsigned int nRows, const unsigned int nCols, const unsigned i
     unsigned int i, j;
     char sz[16];
     GtkWidget *pwScrolledWindow = gtk_scrolled_window_new(NULL, NULL);
+#if GTK_CHECK_VERSION(3,0,0)
+    GtkWidget *pwGrid = gtk_grid_new();
+    gtk_grid_set_column_homogeneous(GTK_GRID(pwGrid), TRUE);
+    gtk_grid_set_row_homogeneous(GTK_GRID(pwGrid), TRUE);
+#else
     GtkWidget *pwTable = gtk_table_new(nRows + 1, nCols + 1, TRUE);
+#endif
     GtkWidget *pw, *pwBox;
     mettable *pmt;
 
@@ -110,7 +121,11 @@ GTKWriteMET(const unsigned int nRows, const unsigned int nCols, const unsigned i
 #endif
 
     pmt = (mettable *) g_malloc(sizeof(mettable));
+#if GTK_CHECK_VERSION(3,0,0)
+    pmt->pwGrid = pwGrid;
+#else
     pmt->pwTable = pwTable;
+#endif
 
     gtk_box_pack_start(GTK_BOX(pwBox), pmt->pwName = gtk_label_new((char *) miCurrent.szName), FALSE, FALSE, 4);
     gtk_box_pack_start(GTK_BOX(pwBox), pmt->pwFileName = gtk_label_new((char *) miCurrent.szFileName), FALSE, FALSE, 4);
@@ -119,8 +134,8 @@ GTKWriteMET(const unsigned int nRows, const unsigned int nCols, const unsigned i
 
     gtk_box_pack_start(GTK_BOX(pwBox), pwScrolledWindow, TRUE, TRUE, 0);
 
-#if GTK_CHECK_VERSION(3, 8, 0)
-    gtk_container_add(GTK_CONTAINER(pwScrolledWindow), pwTable);
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_container_add(GTK_CONTAINER(pwScrolledWindow), pwGrid);
 #else
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pwScrolledWindow), pwTable);
 #endif
@@ -129,7 +144,11 @@ GTKWriteMET(const unsigned int nRows, const unsigned int nCols, const unsigned i
 
     for (i = 0; i < nCols; i++) {
         sprintf(sz, _("%u-away"), i + 1);
+#if GTK_CHECK_VERSION(3,0,0)
+        gtk_grid_attach(GTK_GRID(pwGrid), pw = gtk_label_new(sz), i + 1, 0, 1, 1);
+#else
         gtk_table_attach_defaults(GTK_TABLE(pwTable), pw = gtk_label_new(sz), i + 1, i + 2, 0, 1);
+#endif
         if (i == nAway1) {
             gtk_widget_set_name(GTK_WIDGET(pw), "gnubg-met-matching-score");
         }
@@ -139,7 +158,11 @@ GTKWriteMET(const unsigned int nRows, const unsigned int nCols, const unsigned i
 
     for (i = 0; i < nRows; i++) {
         sprintf(sz, _("%u-away"), i + 1);
+#if GTK_CHECK_VERSION(3,0,0)
+        gtk_grid_attach(GTK_GRID(pwGrid), pw = gtk_label_new(sz), 0, i + 1, 1, 1);
+#else
         gtk_table_attach_defaults(GTK_TABLE(pwTable), pw = gtk_label_new(sz), 0, 1, i + 1, i + 2);
+#endif
         if (i == nAway0) {
             gtk_widget_set_name(GTK_WIDGET(pw), "gnubg-met-matching-score");
         }
@@ -154,7 +177,11 @@ GTKWriteMET(const unsigned int nRows, const unsigned int nCols, const unsigned i
 
             pmt->aapwLabel[i][j] = gtk_label_new(NULL);
 
+#if GTK_CHECK_VERSION(3,0,0)
+            gtk_grid_attach(GTK_GRID(pmt->pwGrid), pmt->aapwLabel[i][j], j + 1, i + 1, 1, 1);
+#else
             gtk_table_attach_defaults(GTK_TABLE(pmt->pwTable), pmt->aapwLabel[i][j], j + 1, j + 2, i + 1, i + 2);
+#endif
 
             if (i == nAway0 && j == nAway1) {
                 gtk_widget_set_name(GTK_WIDGET(pmt->aapwLabel[i][j]), "gnubg-met-the-score");
@@ -163,14 +190,17 @@ GTKWriteMET(const unsigned int nRows, const unsigned int nCols, const unsigned i
             }
         }
 
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_grid_set_column_spacing(GTK_GRID(pwGrid), 4);
+#else
     gtk_table_set_col_spacings(GTK_TABLE(pwTable), 4);
+#endif
 
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pwScrolledWindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
     g_object_set_data_full(G_OBJECT(pwBox), "mettable", pmt, g_free);
 
     return pwBox;
-
 }
 
 static void
@@ -210,6 +240,13 @@ GTKShowMatchEquityTable(const unsigned int nMatchTo, const int anScore[2])
     GtkWidget *pwLoad = gtk_button_new_with_label(_("Load table..."));
 
     GtkWidget *pwInvertButton = gtk_toggle_button_new_with_label(_("Invert table"));
+
+    /* similar tooltip is used in gtkoptions.c:append_match_options() */
+    gtk_widget_set_tooltip_text(pwInvertButton,
+                                _("Use the specified match equity table "
+                                  "around the other way (i.e., swap the players before "
+                                  "looking up equities in the table)."));
+
     metwidget mw;
 
     mw.nMatchTo = nMatchTo;
