@@ -1,13 +1,11 @@
 /*
- * gtkpanels.c
- * by Jon Kinsey, 2004
+ * Copyright (C) 2004-2019 Jon Kinsey <jonkinsey@gmail.com>
+ * Copyright (C) 2005-2018 the AUTHORS
  *
- * Panels/window code
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkpanels.c,v 1.84 2017/09/30 22:09:54 plm Exp $
+ * $Id: gtkpanels.c,v 1.95 2022/10/22 18:34:13 plm Exp $
  */
 
 #include "config.h"
@@ -33,7 +30,7 @@
 #include "gtktoolbar.h"
 #include "positionid.h"
 #if defined(USE_BOARD3D)
-#include "fun3d.h"
+#include "inc3d.h"
 #endif
 
 static int fDisplayPanels = TRUE;
@@ -42,7 +39,7 @@ static int fDockPanels = TRUE;
 #define NUM_CMD_HISTORY 10
 #define KEY_TAB -247
 
-typedef struct _CommandEntryData_T {
+typedef struct {
     GtkWidget *pwEntry, *pwHelpText, *cmdEntryCombo;
     int showHelp;
     int numHistory;
@@ -74,7 +71,7 @@ static gboolean ShowAnalysis(void);
 static gboolean ShowTheoryWindow(void);
 static gboolean ShowCommandWindow(void);
 
-typedef struct _windowobject {
+typedef struct {
     const char *winName;
     int showing;
     int docked;
@@ -102,7 +99,7 @@ static windowobject woPanel[NUM_WINDOWS] = {
      TRUE, TRUE, TRUE, TRUE,
      ShowGameWindow, DeleteGame,
      0,
-     {250, 200, 20, 20, FALSE}
+     {400, 200, 20, 100, FALSE}
      },
     /* analysis */
     {
@@ -110,7 +107,7 @@ static windowobject woPanel[NUM_WINDOWS] = {
      TRUE, TRUE, TRUE, TRUE,
      ShowAnalysis, DeleteAnalysis,
      0,
-     {0, 400, 20, 20, FALSE}
+     {600, 300, 40, 120, FALSE}
      },
     /* annotation */
     {
@@ -118,7 +115,7 @@ static windowobject woPanel[NUM_WINDOWS] = {
      FALSE, TRUE, TRUE, FALSE,
      ShowAnnotation, DeleteAnnotation,
      0,
-     {0, 400, 20, 20, FALSE}
+     {400, 200, 50, 50, FALSE}
      },
     /* hint */
     {
@@ -134,7 +131,7 @@ static windowobject woPanel[NUM_WINDOWS] = {
      FALSE, TRUE, TRUE, TRUE,
      ShowMessage, DeleteMessage,
      0,
-     {0, 500, 20, 20, FALSE}
+     {300, 200, 20, 20, FALSE}
      },
     /* command */
     {
@@ -150,7 +147,7 @@ static windowobject woPanel[NUM_WINDOWS] = {
      FALSE, TRUE, TRUE, TRUE,
      ShowTheoryWindow, DeleteTheoryWindow,
      0,
-     {0, 0, 20, 20, FALSE}
+     {160, 0, 20, 20, FALSE}
      }
 };
 
@@ -342,9 +339,10 @@ UpdateTheoryData(BoardData * bd, int UpdateType, const TanBoard points)
     }
 
     if (UpdateType & TT_RETURNHITS) {
-        TanBoard anBoard;
         pc = NULL;
         if (bd->valid_move) {
+            TanBoard anBoard;
+
             PositionFromKey(anBoard, &bd->valid_move->key);
             pc = ReturnHits(anBoard);
         }
@@ -365,7 +363,7 @@ UpdateTheoryData(BoardData * bd, int UpdateType, const TanBoard points)
             PipCount(points, anPip);
 
             fKC = KleinmanCount(anPip[1], anPip[0]);
-            if (fKC != -1) {
+            if (fKC >= 0) {
                 pc = g_strdup_printf("%.4f", fKC);
                 gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store), &iter, NULL, 3);
                 gtk_list_store_set(store, &iter, 1, pc, -1);
@@ -418,8 +416,7 @@ ShowHelp(GtkWidget * pwText, char *pStr)
     GtkTextIter iter;
 
     /* Copy string as token striping corrupts string */
-    pTemp = malloc(strlen(pStr) + 1);
-    strcpy(pTemp, pStr);
+    pTemp = strdup(pStr);
     cc = CheckCommand(pTemp, acTop);
 
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pwText));
@@ -443,10 +440,10 @@ ShowHelp(GtkWidget * pwText, char *pStr)
                 }
             }
 
-            sprintf(szBuf, "Command: %s\n", szCommand);
+            sprintf(szBuf, "%s %s\n", _("Command:"), szCommand);
             gtk_text_buffer_insert(buffer, &iter, szBuf, -1);
             gtk_text_buffer_insert(buffer, &iter, gettext(pc->szHelp), -1);
-            sprintf(szBuf, "\n\nUsage: %s", szUsage);
+            sprintf(szBuf, "\n\n%s %s", _("Usage:"), szUsage);
             gtk_text_buffer_insert(buffer, &iter, szBuf, -1);
 
             if (!(pc->pc && pc->pc->sz))
@@ -510,8 +507,7 @@ CommandOK(GtkWidget * UNUSED(pw), CommandEntryData_T * pData)
     for (i = pData->numHistory; i > 0; i--)
         pData->cmdHistory[i] = pData->cmdHistory[i - 1];
 
-    pData->cmdHistory[0] = malloc(strlen(pData->cmdString) + 1);
-    strcpy(pData->cmdHistory[0], pData->cmdString);
+    pData->cmdHistory[0] = strdup(pData->cmdString);
     pData->numHistory++;
     PopulateCommandHistory(pData);
     gtk_entry_set_text(GTK_ENTRY(pData->pwEntry), "");
@@ -567,14 +563,12 @@ Capitalize(char *str)
     int cap = 1;
     while (*str) {
         if (cap) {
-            if (*str >= 'a' && *str <= 'z')
-                *str += 'A' - 'a';
+            *str = g_ascii_toupper(*str);
             cap = 0;
         } else {
             if (*str == ' ')
                 cap = 1;
-            if (*str >= 'A' && *str <= 'Z')
-                *str -= 'A' - 'a';
+            *str = g_ascii_tolower(*str);
         }
         str++;
     }
@@ -665,7 +659,13 @@ CreateAnalysisWindow(void)
     GtkWidget *pHbox, *sw;
     GtkTextBuffer *buffer;
     if (!woPanel[WINDOW_ANALYSIS].docked) {
-        GtkWidget *pwPaned = gtk_vpaned_new();
+        GtkWidget *pwPaned;
+
+#if GTK_CHECK_VERSION(3,0,0)
+        pwPaned = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
+#else
+        pwPaned = gtk_vpaned_new();
+#endif
 
         woPanel[WINDOW_ANALYSIS].pwWin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
@@ -770,8 +770,8 @@ GTKRegenerateGames(void)
     GL_SetNames();
     GTKPopGame(0);
     for (pl = lMatch.plNext; pl->p; pl = pl->plNext) {
-        listOLD *plGame = pl->p;
-        GTKAddGame(plGame->plNext->p);
+        listOLD *plg = pl->p;
+        GTKAddGame(plg->plNext->p);
     }
 
     GTKSetGame(i);
@@ -836,7 +836,7 @@ CreateGameWindow(void)
     g_signal_connect(G_OBJECT(game_select_combo), "changed", G_CALLBACK(SelectGame), NULL);
     gtk_box_pack_start(GTK_BOX(phbox), game_select_combo, TRUE, TRUE, 4);
 
-    gtk_container_add(GTK_CONTAINER(pvbox), psw);
+    gtk_box_pack_end(GTK_BOX(pvbox), psw, TRUE, TRUE, 0);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(psw), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
 
     gtk_widget_set_size_request(psw, -1, 150);
@@ -864,7 +864,7 @@ CreateHeadWindow(gnubgwindow panel, const char *sz, GtkWidget * pwWidge)
     pwHbox = gtk_hbox_new(FALSE, 0);
 #endif
 
-    gtk_button_set_image(GTK_BUTTON(pwX), gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU));
+    gtk_button_set_image(GTK_BUTTON(pwX), gtk_image_new_from_icon_name("window-close", GTK_ICON_SIZE_MENU));
     g_signal_connect(G_OBJECT(pwX), "clicked", G_CALLBACK(woPanel[panel].hideFun), NULL);
 
     gtk_box_pack_start(GTK_BOX(pwVbox), pwHbox, FALSE, FALSE, 0);
@@ -1707,14 +1707,16 @@ IsPanelDocked(gnubgwindow window)
     return woPanel[window].docked;
 }
 
+#if ! GTK_CHECK_VERSION(3,0,0)
 int
 GetPanelWidth(gnubgwindow panel)
 {
     return woPanel[panel].wg.nWidth;
 }
+#endif
 
 int
-IsPanelShowVar(gnubgwindow panel, void *p)
+IsPanelShowVar(gnubgwindow panel, void * const p)
 {
     return (p == &woPanel[panel].showing);
 }
