@@ -1,11 +1,11 @@
 /*
- * gtkrolls.c
+ * Copyright (C) 2002-2003 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2003-2019 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkrolls.c,v 1.44 2018/04/10 22:11:41 plm Exp $
+ * $Id: gtkrolls.c,v 1.47 2021/06/27 20:55:09 plm Exp $
  */
 
 #include "config.h"
@@ -32,7 +31,7 @@
 #include "gtkwindows.h"
 #include "gtkrolls.h"
 
-typedef struct _rollswidget {
+typedef struct {
 
     GtkWidget *psw;             /* the scrolled window */
     GtkWidget *ptv;             /* the tree widget */
@@ -60,7 +59,7 @@ add_level(GtkTreeStore * model, GtkTreeIter * iter,
     int anMove[8];
     int i;
 
-    char szRoll[3], szMove[100], *szEquity;
+    char szRoll[3], szMove[FORMATEDMOVESIZE], *szEquity;
 
     /* cubeinfo for opponent on roll */
 
@@ -194,7 +193,6 @@ RollsTree(const int n, evalcontext * pec, const matchstate * pms)
     GtkTreeModel *pm;
     GtkWidget *ptv;
     int i;
-    GtkCellRenderer *renderer;
     static const char *aszColumn[] = {
         N_("Roll"),
         N_("noun|Move"),
@@ -212,7 +210,8 @@ RollsTree(const int n, evalcontext * pec, const matchstate * pms)
 
     for (i = 0; i < 3; ++i) {
 
-        renderer = gtk_cell_renderer_text_new();
+        GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
+
         g_object_set(G_OBJECT(renderer), "xalign", 0.0, NULL);
 
         gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(ptv),
