@@ -1,11 +1,11 @@
 /*
- * gtksplash.c
+ * Copyright (C) 2002-2003 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2003-2017 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtksplash.c,v 1.35 2017/05/27 19:49:24 plm Exp $
+ * $Id: gtksplash.c,v 1.36 2021/06/27 20:59:04 plm Exp $
  */
 
 #include "config.h"
@@ -26,7 +25,7 @@
 #include "util.h"
 #include "gtksplash.h"
 
-typedef struct _gtksplash {
+typedef struct {
     GtkWidget *pwWindow;
     GtkWidget *apwStatus[2];
 } gtksplash;
