@@ -1,11 +1,11 @@
 /*
- * gtkgame.c
+ * Copyright (C) 2002-2003 Joern Thyssen <jthyssen@dk.ibm.com>
+ * Copyright (C) 2002-2022 the AUTHORS
  *
- * by Joern Thyssen <jthyssen@dk.ibm.com>, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtktheory.c,v 1.69 2018/07/22 13:53:43 plm Exp $
+ * $Id: gtktheory.c,v 1.79 2022/09/11 12:38:03 plm Exp $
  */
 
 #include "config.h"
@@ -36,7 +35,7 @@
 
 #define MAXPLY 4
 
-typedef struct _theorywidget {
+typedef struct {
 
     cubeinfo ci;
 
@@ -217,14 +216,14 @@ static void
 do_mw_views(theorywidget * ptw)
 {
     int i;
-    GtkListStore *store;
-    GtkCellRenderer *renderer;
 
     for (i = 0; i < 2; i++) {
-        store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+        GtkListStore *store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+        GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
+
         ptw->apwMW[i] = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
         g_object_unref(store);
-        renderer = gtk_cell_renderer_text_new();
+
         gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(ptw->apwMW[i]), -1, "", renderer, "text", 0, NULL);
         gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(ptw->apwMW[i]), -1, _("Dead cube"), renderer,
                                                     "text", 1, NULL);
@@ -237,10 +236,11 @@ static void
 remove_mw_rows(theorywidget * ptw)
 {
     int i;
-    GtkTreeModel *model;
-    GtkTreeIter iter;
+
     for (i = 0; i < 2; ++i) {
-        model = gtk_tree_view_get_model(GTK_TREE_VIEW(ptw->apwMW[i]));
+        GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(ptw->apwMW[i]));
+        GtkTreeIter iter;
+
         if (gtk_tree_model_iter_nth_child(model, &iter, NULL, 0))
             while (gtk_list_store_remove(GTK_LIST_STORE(model), &iter)) {
             };
@@ -253,8 +253,6 @@ add_mw_money_rows(theorywidget * ptw, const cubeinfo * pci, float aarRates[2][2]
     int i, j, k;
     gchar *asz[3];
     float aaarPoints[2][7][2];
-    GtkListStore *store;
-    GtkTreeIter iter;
     const char *aszMoneyPointLabel[] = {
         N_("Take Point (TP)"),
         N_("Beaver Point (BP)"),
@@ -268,7 +266,9 @@ add_mw_money_rows(theorywidget * ptw, const cubeinfo * pci, float aarRates[2][2]
     /* money play */
     getMoneyPoints(aaarPoints, pci->fJacoby, pci->fBeavers, aarRates);
     for (i = 0; i < 2; ++i) {
-        store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ptw->apwMW[i])));
+        GtkListStore * store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ptw->apwMW[i])));
+        GtkTreeIter iter;
+
         for (j = 0; j < 7; j++) {
             asz[0] = g_strdup(gettext(aszMoneyPointLabel[j]));
             for (k = 0; k < 2; k++)
@@ -292,8 +292,6 @@ add_mw_match_rows(theorywidget * ptw, const cubeinfo * pci, float aarRates[2][2]
     float aaarPointsMatch[2][4][2];
     int afAutoRedouble[2];
     int afDead[2];
-    GtkListStore *store;
-    GtkTreeIter iter;
     const char *aszMatchPlayLabel[] = {
         N_("Take Point (TP)"),
         N_("Double point (DP)"),
@@ -303,7 +301,9 @@ add_mw_match_rows(theorywidget * ptw, const cubeinfo * pci, float aarRates[2][2]
 
 
     for (i = 0; i < 2; i++) {
-        store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ptw->apwMW[i])));
+        GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ptw->apwMW[i])));
+        GtkTreeIter iter;
+
         getMatchPoints(aaarPointsMatch, afAutoRedouble, afDead, pci, aarRates);
         for (j = 0; j < 4; j++) {
             asz[0] = g_strdup(gettext(aszMatchPlayLabel[j]));
@@ -348,7 +348,7 @@ TheoryUpdated(GtkWidget * UNUSED(pw), theorywidget * ptw)
     for (i = 0; i < 2; ++i)
         gtk_adjustment_set_upper(ptw->aapwRates[i][1], 100.0 - gtk_adjustment_get_value(ptw->aapwRates[i][0]));
 
-    SetCubeInfo(&ci, ci.nCube, 0, 0, ci.nMatchTo, ci.anScore, ci.fCrawford, ci.fJacoby, ci.fBeavers, ci.bgv);
+    SetCubeInfo(&ci, ci.nCube, 0, 0, ci.nMatchTo, ci.anScore, ci.fCrawford, ci.fJacoby, ci.fBeavers, ms.bgv);
 
     /* hide show widgets */
 
@@ -469,11 +469,10 @@ GraphDraw(GtkWidget * pwGraph, cairo_t * cr, theorywidget * ptw)
 
     GtkAllocation allocation;
     int i, x, y, cx, cy, iPlayer, ax[3];
-    char sz[4];
     PangoLayout *layout;
 
 #if ! GTK_CHECK_VERSION(3,0,0)
-    /* The gtk_locdef_* below don't use cr with GTK2. Avoid compiler warning in this case. */
+    /* The gtk_paint_* below don't use cr with GTK2. Avoid compiler warning in this case. */
     (void)cr;
 #endif
 
@@ -494,24 +493,38 @@ GraphDraw(GtkWidget * pwGraph, cairo_t * cr, theorywidget * ptw)
     iPlayer = pwGraph == ptw->apwGraph[1];
 
     for (i = 0; i <= 20; i++) {
-        gtk_locdef_paint_vline(gtk_widget_get_style(pwGraph), gtk_widget_get_window(pwGraph), cr, GTK_STATE_NORMAL,
+#if GTK_CHECK_VERSION(3,0,0)
+        gtk_render_line(gtk_widget_get_style_context(pwGraph), cr,
+                               (double)(x + cx * i / 20), (double)(y - 1),
+                               (double)(x + cx * i / 20), (double)((i & 3) ? y - 3 : y - 5));
+#else
+        gtk_paint_vline(gtk_widget_get_style(pwGraph), gtk_widget_get_window(pwGraph), GTK_STATE_NORMAL,
                                NULL, pwGraph, "tick", y - 1, (i & 3) ? y - 3 : y - 5, x + cx * i / 20);
-
+#endif
         if (!(i & 3)) {
             int width;
             int height;
-            sprintf(sz, "%d", i * 5);
+            gchar *sz = g_strdup_printf("%d", i * 5);
+
             pango_layout_set_text(layout, sz, -1);
+
+            g_free(sz);
+
             pango_layout_get_pixel_size(layout, &width, &height);
-            gtk_locdef_paint_layout(gtk_widget_get_style(pwGraph), gtk_widget_get_window(pwGraph), cr,
-                                    GTK_STATE_NORMAL, TRUE, NULL, pwGraph, "label",
-                                    x + cx * i / 20 - width / 2 /* FIXME */ , y - height - 1, layout);
+#if GTK_CHECK_VERSION(3,0,0)
+            gtk_render_layout(gtk_widget_get_style_context(pwGraph), cr,
+                                (double)(x + cx * i / 20 - width / 2) /* FIXME */, (double)(y - height - 1), layout);
+#else
+            gtk_paint_layout(gtk_widget_get_style(pwGraph), gtk_widget_get_window(pwGraph),
+                                GTK_STATE_NORMAL, TRUE, NULL, pwGraph, "label",
+                                x + cx * i / 20 - width / 2 /* FIXME */ , y - height - 1, layout);
+#endif
         }
     }
     g_object_unref(layout);
 
     for (i = 0; i < 3; i++)
-        ax[i] = (int) (x + cx * ptw->aar[iPlayer][i]);
+        ax[i] = x + (int) ((float) cx * ptw->aar[iPlayer][i]);
 
     gtk_locdef_paint_box(gtk_widget_get_style(pwGraph), gtk_widget_get_window(pwGraph), cr, GTK_STATE_NORMAL,
                          GTK_SHADOW_IN, NULL, pwGraph, "doubling-window", x, 12, cx, cy);
@@ -598,26 +611,30 @@ GTKShowTheory(const int fActivePage)
     GtkWidget *pwDialog, *pwNotebook;
 
     GtkWidget *pwOuterHBox, *pwVBox, *pwHBox;
-    GtkWidget *pwFrame, *pwTable, *pwAlign;
+    GtkWidget *pwFrame;
+#if GTK_CHECK_VERSION(3,0,0)
+    GtkWidget *pwGrid;
+#else
+    GtkWidget *pwTable;
+#endif
 
-    GtkWidget *pw, *pwx, *pwz;
+    GtkWidget *pw, *pwx, *pwz, *pwsb;
 
-    int i, j;
-    char sz[256];
-    int *pi;
     PangoFontDescription *font_desc;
 
     theorywidget *ptw;
 
+    int i, j;
+
     /* create dialog */
 
-    pwDialog = GTKCreateDialog(_("GNU Backgammon - Theory"), DT_INFO,
+    pwDialog = GTKCreateDialog(_("GNU Backgammon - Market window"), DT_INFO,
                                NULL, DIALOG_FLAG_MODAL | DIALOG_FLAG_NOTIDY, NULL, NULL);
 
     gtk_window_set_default_size(GTK_WINDOW(pwDialog), 660, 300);
 
-    ptw = malloc(sizeof(theorywidget));
-    g_object_set_data_full(G_OBJECT(pwDialog), "theorywidget", ptw, free);
+    ptw = g_malloc(sizeof(theorywidget));
+    g_object_set_data_full(G_OBJECT(pwDialog), "theorywidget", ptw, g_free);
 
 #if GTK_CHECK_VERSION(3,0,0)
     pwOuterHBox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 8);
@@ -630,6 +647,7 @@ GTKShowTheory(const int fActivePage)
 
 #if GTK_CHECK_VERSION(3,0,0)
     pwVBox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+    gtk_widget_set_valign(pwVBox, GTK_ALIGN_START);
 #else
     pwVBox = gtk_vbox_new(FALSE, 0);
 #endif
@@ -644,16 +662,15 @@ GTKShowTheory(const int fActivePage)
 #endif
     gtk_box_pack_start(GTK_BOX(pwVBox), pwHBox, FALSE, FALSE, 0);
 
-    gtk_container_add(GTK_CONTAINER(pwHBox), ptw->apwRadio[0] = gtk_radio_button_new_with_label(NULL, _("Match play")));
-    gtk_container_add(GTK_CONTAINER(pwHBox),
-                      ptw->apwRadio[1] =
-                      gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(ptw->apwRadio[0]), _("Money game")));
+    gtk_box_pack_start(GTK_BOX(pwHBox), ptw->apwRadio[0] = gtk_radio_button_new_with_label(NULL, _("Match play")), TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(pwHBox), ptw->apwRadio[1] =
+                      gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(ptw->apwRadio[0]), _("Money game")), TRUE, TRUE, 0);
 
     g_signal_connect(G_OBJECT(ptw->apwRadio[0]), "toggled", G_CALLBACK(TheoryUpdated), ptw);
     g_signal_connect(G_OBJECT(ptw->apwRadio[1]), "toggled", G_CALLBACK(TheoryUpdated), ptw);
 
 
-    gtk_container_add(GTK_CONTAINER(pwHBox), ptw->pwReset = gtk_button_new_with_label(_("Reset")));
+    gtk_box_pack_end(GTK_BOX(pwHBox), ptw->pwReset = gtk_button_new_with_label(_("Reset")), TRUE, TRUE, 0);
     g_signal_connect(G_OBJECT(ptw->pwReset), "clicked", G_CALLBACK(ResetTheory), ptw);
 
     /* match score widget */
@@ -668,35 +685,67 @@ GTKShowTheory(const int fActivePage)
 #endif
     gtk_container_add(GTK_CONTAINER(ptw->apwFrame[0]), pw);
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pwGrid = gtk_grid_new();
+    gtk_container_add(GTK_CONTAINER(pw), pwGrid);
+#else
     pwTable = gtk_table_new(2, 3, FALSE);
     gtk_container_add(GTK_CONTAINER(pw), pwTable);
+#endif
 
     for (i = 0; i < 2; i++) {
 
-        gtk_table_attach(GTK_TABLE(pwTable),
-                         pwx = gtk_label_new(ap[i].szName),
-                         0, 1, 0 + i, 1 + i, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
+        pwx = gtk_label_new(ap[i].szName);
+
 #if GTK_CHECK_VERSION(3,0,0)
+        gtk_grid_attach(GTK_GRID(pwGrid), pwx, 0, i, 1, 1);
         gtk_widget_set_halign(pwx, GTK_ALIGN_START);
         gtk_widget_set_valign(pwx, GTK_ALIGN_CENTER);
+        gtk_widget_set_hexpand(pwx, TRUE);
+        gtk_widget_set_vexpand(pwx, TRUE);
+#if GTK_CHECK_VERSION(3,12,0)
+        gtk_widget_set_margin_start(pwx, 4);
+        gtk_widget_set_margin_end(pwx, 4);
 #else
+        gtk_widget_set_margin_left(pwx, 4);
+        gtk_widget_set_margin_right(pwx, 4);
+#endif
+#else                 
+        gtk_table_attach(GTK_TABLE(pwTable), pwx,
+                         0, 1, 0 + i, 1 + i, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
         gtk_misc_set_alignment(GTK_MISC(pwx), 0, 0.5);
 #endif
 
         ptw->apwScoreAway[i] = GTK_ADJUSTMENT(gtk_adjustment_new(1, 1, 64, 1, 5, 0));
+        pwsb = gtk_spin_button_new(ptw->apwScoreAway[i], 1, 0);
+        pwx = gtk_label_new(_("-away"));
 
-        gtk_table_attach(GTK_TABLE(pwTable),
-                         gtk_spin_button_new(ptw->apwScoreAway[i],
-                                             1, 0),
-                         1, 2, 0 + i, 1 + i, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
-
-        gtk_table_attach(GTK_TABLE(pwTable),
-                         pwx = gtk_label_new(_("-away")),
-                         2, 3, 0 + i, 1 + i, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
 #if GTK_CHECK_VERSION(3,0,0)
+        gtk_grid_attach(GTK_GRID(pwGrid), pwsb, 1, i, 1, 1);
+        gtk_grid_attach(GTK_GRID(pwGrid), pwx, 2, i, 1, 1);
         gtk_widget_set_halign(pwx, GTK_ALIGN_START);
         gtk_widget_set_valign(pwx, GTK_ALIGN_CENTER);
+        gtk_widget_set_hexpand(pwsb, TRUE);
+        gtk_widget_set_vexpand(pwsb, TRUE);
+        gtk_widget_set_hexpand(pwx, TRUE);
+        gtk_widget_set_vexpand(pwx, TRUE);
+#if GTK_CHECK_VERSION(3,12,0)
+        gtk_widget_set_margin_start(pwsb, 4);
+        gtk_widget_set_margin_end(pwsb, 4);
+        gtk_widget_set_margin_start(pwx, 4);
+        gtk_widget_set_margin_end(pwx, 4);
 #else
+        gtk_widget_set_margin_left(pwsb, 4);
+        gtk_widget_set_margin_right(pwsb, 4);
+        gtk_widget_set_margin_left(pwx, 4);
+        gtk_widget_set_margin_right(pwx, 4);
+#endif
+
+#else
+        gtk_table_attach(GTK_TABLE(pwTable), pwsb,
+                         1, 2, 0 + i, 1 + i, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
+        gtk_table_attach(GTK_TABLE(pwTable), pwx,
+                         2, 3, 0 + i, 1 + i, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
         gtk_misc_set_alignment(GTK_MISC(pwx), 0, 0.5);
 #endif
 
@@ -728,7 +777,7 @@ GTKShowTheory(const int fActivePage)
     j = 1;
     for (i = 0; i < 7; i++) {
 
-        sprintf(sz, "%d", j);
+        gchar *sz = g_strdup_printf("%d", j);
 
         if (!i)
             gtk_container_add(GTK_CONTAINER(pwHBox), ptw->apwCube[i] = gtk_radio_button_new_with_label(NULL, sz));
@@ -737,6 +786,8 @@ GTKShowTheory(const int fActivePage)
                               ptw->apwCube[i] =
                               gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(ptw->apwCube[0]), sz));
 
+        g_free(sz);
+
         g_signal_connect(G_OBJECT(ptw->apwCube[i]), "toggled", G_CALLBACK(TheoryUpdated), ptw);
 
         j *= 2;
@@ -811,44 +862,82 @@ GTKShowTheory(const int fActivePage)
     gtk_container_add(GTK_CONTAINER(pwFrame), pwx);
 
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pwGrid = gtk_grid_new();
+    gtk_box_pack_start(GTK_BOX(pwx), pwGrid, FALSE, FALSE, 4);
+#else
     pwTable = gtk_table_new(3, 3, TRUE);
     gtk_box_pack_start(GTK_BOX(pwx), pwTable, FALSE, FALSE, 4);
+#endif
 
     for (i = 0; i < 2; i++) {
 
         /* player name */
 
-        gtk_table_attach(GTK_TABLE(pwTable),
-                         pw = gtk_label_new(ap[i].szName),
-                         0, 1, 1 + i, 2 + i, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
+        pw = gtk_label_new(ap[i].szName);
+
 #if GTK_CHECK_VERSION(3,0,0)
+        gtk_grid_attach(GTK_GRID(pwGrid), pw, 0, 1 + i, 1, 1);
         gtk_widget_set_halign(pw, GTK_ALIGN_START);
         gtk_widget_set_valign(pw, GTK_ALIGN_CENTER);
+        gtk_widget_set_hexpand(pw, TRUE);
+        gtk_widget_set_vexpand(pw, TRUE);
+#if GTK_CHECK_VERSION(3,12,0)
+        gtk_widget_set_margin_start(pw, 4);
+        gtk_widget_set_margin_end(pw, 4);
 #else
+        gtk_widget_set_margin_left(pw, 4);
+        gtk_widget_set_margin_right(pw, 4);
+#endif
+#else
+        gtk_table_attach(GTK_TABLE(pwTable), pw,
+                         0, 1, 1 + i, 2 + i, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
         gtk_misc_set_alignment(GTK_MISC(pw), 0, 0.5);
 #endif
 
         /* column title */
 
-        gtk_table_attach(GTK_TABLE(pwTable),
-                         pw = gtk_label_new(i ? _("bg rate(%)") :
-                                            _("gammon rate(%)")),
-                         1 + i, 2 + i, 0, 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
+        pw = gtk_label_new(i ? _("bg rate(%)") : _("gammon rate(%)"));
+
 #if GTK_CHECK_VERSION(3,0,0)
+        gtk_grid_attach(GTK_GRID(pwGrid), pw, 1 + i, 0, 1, 1);
         gtk_widget_set_halign(pw, GTK_ALIGN_START);
         gtk_widget_set_valign(pw, GTK_ALIGN_CENTER);
+        gtk_widget_set_hexpand(pw, TRUE);
+        gtk_widget_set_vexpand(pw, TRUE);
+#if GTK_CHECK_VERSION(3,12,0)
+        gtk_widget_set_margin_start(pw, 4);
+        gtk_widget_set_margin_end(pw, 4);
 #else
+        gtk_widget_set_margin_left(pw, 4);
+        gtk_widget_set_margin_right(pw, 4);
+#endif
+#else
+        gtk_table_attach(GTK_TABLE(pwTable), pw,
+                         1 + i, 2 + i, 0, 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
         gtk_misc_set_alignment(GTK_MISC(pw), 0, 0.5);
 #endif
 
         for (j = 0; j < 2; j++) {
 
             ptw->aapwRates[i][j] = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 100.0, 0.01, 1.0, 0));
+            pwsb = gtk_spin_button_new(ptw->aapwRates[i][j], 0.01, 2);
 
-            gtk_table_attach(GTK_TABLE(pwTable),
-                             gtk_spin_button_new(ptw->aapwRates[i][j],
-                                                 0.01, 2),
+#if GTK_CHECK_VERSION(3,0,0)
+            gtk_grid_attach(GTK_GRID(pwGrid), pwsb, j + 1, i + 1, 1, 1);
+            gtk_widget_set_hexpand(pwsb, TRUE);
+            gtk_widget_set_vexpand(pwsb, TRUE);
+#if GTK_CHECK_VERSION(3,12,0)
+            gtk_widget_set_margin_start(pwsb, 4);
+            gtk_widget_set_margin_end(pwsb, 4);
+#else
+            gtk_widget_set_margin_left(pwsb, 4);
+            gtk_widget_set_margin_right(pwsb, 4);
+#endif
+#else
+            gtk_table_attach(GTK_TABLE(pwTable), pwsb,
                              j + 1, j + 2, i + 1, i + 2, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 4, 0);
+#endif
 
             g_signal_connect(G_OBJECT(ptw->aapwRates[i][j]), "value-changed", G_CALLBACK(TheoryUpdated), ptw);
 
@@ -867,6 +956,8 @@ GTKShowTheory(const int fActivePage)
 
     for (i = 0; i <= MAXPLY; ++i) {
 
+        int *pi;
+
         gchar *sz = g_strdup_printf(_("%d ply"), i);
         if (!i)
             ptw->apwPly[i] = gtk_radio_button_new_with_label(NULL, sz);
@@ -874,7 +965,7 @@ GTKShowTheory(const int fActivePage)
             ptw->apwPly[i] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(ptw->apwPly[0]), sz);
         g_free(sz);
 
-        pi = (int *) g_malloc(sizeof(int));
+        pi = g_malloc(sizeof(int));
         *pi = i;
 
         g_object_set_data_full(G_OBJECT(ptw->apwPly[i]), "ply", pi, g_free);
@@ -905,13 +996,15 @@ GTKShowTheory(const int fActivePage)
 
     for (i = 0; i < 2; ++i) {
 
-        sprintf(sz, _("Market window for player %s"), ap[i].szName);
+        gchar *sz = g_strdup_printf( _("Market window for player %s"), ap[i].szName);
+
         pwFrame = gtk_frame_new(sz);
         gtk_box_pack_start(GTK_BOX(pwVBox), pwFrame, FALSE, FALSE, 0);
 
-
         gtk_container_add(GTK_CONTAINER(pwFrame), ptw->apwMW[i]);
 
+        g_free(sz);
+
     }
 
     /* window graph */
@@ -923,10 +1016,14 @@ GTKShowTheory(const int fActivePage)
     gtk_notebook_append_page(GTK_NOTEBOOK(pwNotebook), pwVBox, gtk_label_new(_("Window graph")));
 
     for (i = 0; i < 2; i++) {
-        sprintf(sz, _("Window graph for player %s"), ap[i].szName);
+        GtkWidget *pwAlign;
+        gchar *sz = g_strdup_printf( _("Window graph for player %s"), ap[i].szName);
+
         pwFrame = gtk_frame_new(sz);
         gtk_box_pack_start(GTK_BOX(pwVBox), pwFrame, FALSE, FALSE, 4);
 
+        g_free(sz);
+
         gtk_container_add(GTK_CONTAINER(pwFrame), pwAlign = gtk_alignment_new(0.5, 0.5, 1, 0));
         gtk_container_add(GTK_CONTAINER(pwAlign), ptw->apwGraph[i] = gtk_drawing_area_new());
         gtk_widget_set_name(ptw->apwGraph[i], "gnubg-doubling-window-graph");
