@@ -1,11 +1,11 @@
 /*
- * gtkuidefs.h
+ * Copyright (C) 2011 Michael Petch <mpetch@capp-sysware.com>
+ * Copyright (C) 2013-2021 the AUTHORS
  *
- * by Michael Petch <mpetch@capp-sysware.com>, 2011.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkuidefs.h,v 1.4 2017/05/25 20:02:09 plm Exp $
+ * $Id: gtkuidefs.h,v 1.6 2021/10/05 20:58:46 plm Exp $
  */
 
 #define GNUBG_MAIN_UI \
@@ -176,6 +175,7 @@
 	      "<menuitem name='DistributionOfRolls' action='DistributionOfRollsAction' />" \
 	      "<menuitem name='TemperatureMap' action='TemperatureMapAction' />" \
 	      "<menuitem name='TemperatureMapCube' action='TemperatureMapCubeAction' />" \
+	      "<menuitem name='ScoreMapCube' action='ScoreMapAction' />" \
 	      "<separator/>" \
 	      "<menuitem name='RaceTheory' action='RaceTheoryAction' />" \
 	      "<separator/>" \
