@@ -1,13 +1,11 @@
 /*
- * gtkwindows.c
- * by Jon Kinsey, 2006
+ * Copyright (C) 2006-2019 Jon Kinsey <jonkinsey@gmail.com>
+ * Copyright (C) 2006-2019 the AUTHORS
  *
- * functions to create windows
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: gtkwindows.c,v 1.55 2016/08/15 20:12:05 plm Exp $
+ * $Id: gtkwindows.c,v 1.65 2022/10/22 18:34:15 plm Exp $
  */
 
 #include "config.h"
@@ -33,7 +30,7 @@
 #include <glib/gi18n.h>
 typedef void (*dialog_func_ty) (GtkWidget *, void *);
 
-typedef struct _Warning {
+typedef struct {
     char *warningString;
     char *warningName;
     int warningEnabled;
@@ -44,9 +41,6 @@ static Warning warnings[WARN_NUM_WARNINGS] = {
     {
      N_("Press escape to exit full screen mode"),
      "fullscreenexit", TRUE, FALSE},
-    {
-     N_("This option will speed up the 3d drawing, but may not work correctly on all machines"),
-     "quickdraw", TRUE, FALSE},
     {
      N_("Drawing shadows is only supported on the latest graphics cards\n"
         "Disable this option if performance is poor"),
@@ -69,12 +63,12 @@ static Warning warnings[WARN_NUM_WARNINGS] = {
      "rollout", TRUE, FALSE}
 };
 
-static char *aszStockItem[NUM_DIALOG_TYPES] = {
-    GTK_STOCK_DIALOG_INFO,
-    GTK_STOCK_DIALOG_QUESTION,
-    GTK_STOCK_DIALOG_WARNING,
-    GTK_STOCK_DIALOG_WARNING,
-    GTK_STOCK_DIALOG_ERROR,
+static char *aszNamedIcon[NUM_DIALOG_TYPES] = {
+    "dialog-information",
+    "dialog-question",
+    "dialog-warning",
+    "dialog-warning",
+    "dialog-error",
     NULL
 };
 
@@ -86,7 +80,7 @@ quitter(GtkWidget * UNUSED(widget), GtkWidget * parent)
         gtk_window_present(GTK_WINDOW(parent));
 }
 
-typedef struct _CallbackStruct {
+typedef struct {
     void (*DialogFun) (GtkWidget *, void *);
     gpointer data;
 } CallbackStruct;
@@ -146,7 +140,7 @@ GTKCreateDialog(const char *szTitle, const dialogtype dt,
                 GtkWidget * parent, int flags, GCallback okFun, void *okFunData)
 {
     CallbackStruct *cbData;
-    GtkWidget *pwDialog, *pwHbox, *pwPixmap;
+    GtkWidget *pwDialog, *pwHbox;
     GtkAccelGroup *pag;
     int fQuestion = (dt == DT_QUESTION || dt == DT_AREYOUSURE);
 
@@ -173,12 +167,20 @@ GTKCreateDialog(const char *szTitle, const dialogtype dt,
     pag = gtk_accel_group_new();
     gtk_window_add_accel_group(GTK_WINDOW(pwDialog), pag);
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pwHbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+#else
     pwHbox = gtk_hbox_new(FALSE, 0);
+#endif
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(pwDialog))), pwHbox, TRUE, TRUE, 0);
 
     if (dt != DT_CUSTOM) {
-        pwPixmap = gtk_image_new_from_stock(aszStockItem[dt], GTK_ICON_SIZE_DIALOG);
+        GtkWidget *pwPixmap = gtk_image_new_from_icon_name(aszNamedIcon[dt], GTK_ICON_SIZE_DIALOG);
+#if GTK_CHECK_VERSION(3,0,0)
+        g_object_set(pwPixmap, "margin", 8, NULL);
+#else
         gtk_misc_set_padding(GTK_MISC(pwPixmap), 8, 8);
+#endif
         gtk_box_pack_start(GTK_BOX(pwHbox), pwPixmap, FALSE, FALSE, 0);
     }
 
@@ -301,7 +303,11 @@ GTKMessage(const char *sz, dialogtype dt)
     sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
     gtk_container_add(GTK_CONTAINER(sw), pwText);
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_widget_get_preferred_size(GTK_WIDGET(pwText), &req, NULL);
+#else
     gtk_widget_size_request(GTK_WIDGET(pwText), &req);
+#endif
     if (strlen(sz) > MAXSTRLEN || req.height > MAXWINSIZE) {
         int wz = MIN(MAXWINSIZE, req.height + 100);
         gtk_window_set_default_size(GTK_WINDOW(pwDialog), -1, wz);
@@ -324,9 +330,9 @@ GTKMessage(const char *sz, dialogtype dt)
 }
 
 extern int
-GTKGetInputYN(char *szPrompt)
+GTKGetInputYN(char *sz)
 {
-    return GTKMessage(szPrompt, DT_AREYOUSURE);
+    return GTKMessage(sz, DT_AREYOUSURE);
 }
 
 static char *inputString;
@@ -346,7 +352,12 @@ GTKGetInput(char *title, char *prompt, GtkWidget * parent)
     inputString = NULL;
     pwDialog = GTKCreateDialog(title, DT_QUESTION, parent, DIALOG_FLAG_MODAL, G_CALLBACK(GetInputOk), pwEntry);
 
-    gtk_container_add(GTK_CONTAINER(DialogArea(pwDialog, DA_MAIN)), pwHbox = gtk_hbox_new(FALSE, 0));
+#if GTK_CHECK_VERSION(3,0,0)
+    pwHbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+#else
+    pwHbox = gtk_hbox_new(FALSE, 0);
+#endif
+    gtk_container_add(GTK_CONTAINER(DialogArea(pwDialog, DA_MAIN)), pwHbox);
 
     gtk_box_pack_start(GTK_BOX(pwHbox), gtk_label_new(prompt), FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(pwHbox), pwEntry, FALSE, FALSE, 0);
@@ -387,11 +398,19 @@ GTKShowWarning(warningType warning, GtkWidget * pwParent)
                             warnings[warning].isWarningQuestion ? DT_AREYOUSURE : DT_WARNING, pwParent,
                             DIALOG_FLAG_MODAL, G_CALLBACK(WarningOK), (void *) warning);
 
+#if GTK_CHECK_VERSION(3,0,0)
+        pwv = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
         pwv = gtk_vbox_new(FALSE, 0);
+#endif
         gtk_container_add(GTK_CONTAINER(DialogArea(pwDialog, DA_MAIN)), pwv);
 
         pwMsg = gtk_label_new(gettext(warnings[warning].warningString));
+#if GTK_CHECK_VERSION(3,0,0)
+        g_object_set(pwMsg, "margin", 8, NULL);
+#else
         gtk_misc_set_padding(GTK_MISC(pwMsg), 8, 8);
+#endif
         gtk_box_pack_start(GTK_BOX(pwv), pwMsg, TRUE, TRUE, 0);
 
         buf = g_strdup_printf("<small>%s</small>", _("Don't show this again"));
@@ -433,6 +452,12 @@ SetWarningEnabled(warningType warning, int value)
     warnings[warning].warningEnabled = value;
 }
 
+extern int
+GetWarningEnabled(warningType warning)
+{
+    return warnings[warning].warningEnabled;
+}
+
 extern void
 PrintWarning(warningType warning)
 {
