@@ -1,11 +1,11 @@
 /*
- * html.c
+ * Copyright (C) 2002-2004 Joern Thyssen <jthyssen@dk.ibm.com>
+ * Copyright (C) 2002-2021 the AUTHORS
  *
- * by Joern Thyssen  <jthyssen@dk.ibm.com>, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: html.c,v 1.246 2018/04/02 19:36:15 plm Exp $
+ * $Id: html.c,v 1.264 2022/10/22 18:27:59 plm Exp $
  */
 
 #include "config.h"
@@ -40,14 +39,13 @@
 
 #include <glib.h>
 #include <glib/gstdio.h>
-#include "glib-ext.h"
 
 #ifdef WIN32
 #include <io.h>
 #endif
 #include "util.h"
 
-typedef enum _stylesheetclass {
+typedef enum {
     CLASS_MOVETABLE,
     CLASS_MOVEHEADER,
     CLASS_MOVENUMBER,
@@ -163,7 +161,7 @@ WriteStyleSheet(FILE * pf, const htmlexportcss hecss)
 
         fputs("\n"
               "/* CSS Stylesheet for " VERSION_STRING " */\n"
-              "/* $Id: html.c,v 1.246 2018/04/02 19:36:15 plm Exp $ */\n", pf);
+              "/* $Id: html.c,v 1.264 2022/10/22 18:27:59 plm Exp $ */\n", pf);
 
     fputs("/* This file is distributed as a part of the "
           "GNU Backgammon program. */\n"
@@ -385,16 +383,14 @@ printStatTableRow(FILE * pf, const char *format1, const char *format2, ...)
 
     va_list val;
     char *sz;
-    size_t l = 100 + strlen(format1) + 2 * strlen(format2);
 
     va_start(val, format2);
 
-    sprintf(sz = (char *) malloc(l),
-            "<tr>\n" "<td>%s</td>\n" "<td>%s</td>\n" "<td>%s</td>\n" "</tr>\n", format1, format2, format2);
+    sz = g_strdup_printf("<tr>\n" "<td>%s</td>\n" "<td>%s</td>\n" "<td>%s</td>\n" "</tr>\n", format1, format2, format2);
 
     vfprintf(pf, sz, val);
 
-    free(sz);
+    g_free(sz);
 
     va_end(val);
 
@@ -505,7 +501,7 @@ printHTMLBoardBBS(FILE * pf, matchstate * pms, int fTurn,
 
     TanBoard anBoard;
     unsigned int anPips[2];
-    int acOff[2];
+    int acOff[2] = { 15, 15 };
     int i, j;
     char sz[1024];
 
@@ -516,7 +512,6 @@ printHTMLBoardBBS(FILE * pf, matchstate * pms, int fTurn,
     PipCount((ConstTanBoard) anBoard, anPips);
 
     for (i = 0; i < 2; i++) {
-        acOff[i] = 15;
         for (j = 0; j < 25; j++)
             acOff[i] -= anBoard[i][j];
     }
@@ -760,7 +755,7 @@ printHTMLBoardF2H(FILE * pf, matchstate * pms, int fTurn,
 
         /* display arrow */
 
-        if (pms->fCubeOwner == -1 || pms->fCubeOwner)
+        if (pms->fCubeOwner)
             printImage(pf, szImageDir, fTurn ? "b-topdn" : "b-topup",
                        szExtension, "", hecss, HTML_EXPORT_TYPE_FIBS2HTML);
         else {
@@ -952,7 +947,7 @@ printHTMLBoardF2H(FILE * pf, matchstate * pms, int fTurn,
     printImage(pf, szImageDir, "b-indent", szExtension, "", hecss, HTML_EXPORT_TYPE_FIBS2HTML);
 
     PipCount((ConstTanBoard) anBoard, anPips);
-    fprintf(pf, _("Pip counts: %s %u, %s %u<br/>\n"), ap[0].szName, anPips[1], ap[1].szName, anPips[0]);
+    fprintf(pf, "%s %s %u, %s %u<br/>\n", _("Pip counts:"), ap[0].szName, anPips[1], ap[1].szName, anPips[0]);
 
     /* position ID */
 
@@ -1029,7 +1024,7 @@ printHTMLBoardGNU(FILE * pf, matchstate * pms, int fTurn,
 
     TanBoard anBoard;
     unsigned int anPips[2];
-    int acOff[2];
+    int acOff[2] = { 15, 15 };
 
     memcpy(anBoard, pms->anBoard, sizeof(anBoard));
 
@@ -1037,7 +1032,6 @@ printHTMLBoardGNU(FILE * pf, matchstate * pms, int fTurn,
         SwapSides(anBoard);
 
     for (i = 0; i < 2; i++) {
-        acOff[i] = 15;
         for (j = 0; j < 25; j++)
             acOff[i] -= anBoard[i][j];
     }
@@ -1374,7 +1368,7 @@ printHTMLBoardGNU(FILE * pf, matchstate * pms, int fTurn,
     fputs("<p>", pf);
 
     PipCount((ConstTanBoard) anBoard, anPips);
-    fprintf(pf, _("Pip counts: %s %u, %s %u<br/>\n"), ap[0].szName, anPips[1], ap[1].szName, anPips[0]);
+    fprintf(pf, "%s %s %u, %s %u<br/>\n", _("Pip counts:"), ap[0].szName, anPips[1], ap[1].szName, anPips[0]);
 
     /* position ID */
 
@@ -1564,7 +1558,7 @@ HTMLPrologue(FILE * pf, const matchstate * pms,
 
 
 /*
- * Print html header: dtd, head etc.
+ * Print html footer
  *
  * Input:
  *   pf: output file
@@ -1580,7 +1574,7 @@ HTMLEpilogue(FILE * pf, const matchstate * UNUSED(pms), char *aszLinks[4], const
     int fFirst;
     int i;
 
-    const char szVersion[] = "$Revision: 1.246 $";
+    const char szVersion[] = "$Revision: 1.264 $";
     int iMajor, iMinor;
 
     iMajor = atoi(strchr(szVersion, ' '));
@@ -1599,7 +1593,7 @@ HTMLEpilogue(FILE * pf, const matchstate * UNUSED(pms), char *aszLinks[4], const
                 fputs("<p>\n", pf);
                 fFirst = FALSE;
             }
-            fprintf(pf, "<a href=\"%s\">%s</a>\n", aszLinks[i], aszLinkText[i]);
+            fprintf(pf, "<a href=\"%s\">%s</a>\n", aszLinks[i], gettext(aszLinkText[i]));
         }
 
 
@@ -1636,7 +1630,7 @@ HTMLEpilogue(FILE * pf, const matchstate * UNUSED(pms), char *aszLinks[4], const
 
 
 /*
- * Print html header: dtd, head etc.
+ * Print html footer.
  *
  * Input:
  *   pf: output file
@@ -1650,7 +1644,7 @@ HTMLEpilogueComment(FILE * pf)
 
     time_t t;
 
-    const char szVersion[] = "$Revision: 1.246 $";
+    const char szVersion[] = "$Revision: 1.264 $";
     int iMajor, iMinor;
     char *pc;
 
@@ -1709,7 +1703,7 @@ HTMLPrintCubeAnalysisTable(FILE * pf,
     float r;
 
     int fActual, fClose, fMissed;
-    int fDisplay;
+    int fDisplayIt;
     int fAnno = FALSE;
 
     cubedecision cd;
@@ -1725,18 +1719,27 @@ HTMLPrintCubeAnalysisTable(FILE * pf,
     fClose = isCloseCubedecision(arDouble);
     fMissed = fDouble > -1 && isMissedDouble(arDouble, aarOutput, fDouble, pci);
 
-    fDisplay =
+    fDisplayIt =
         (fActual && exsExport.afCubeDisplay[EXPORT_CUBE_ACTUAL]) ||
         (fClose && exsExport.afCubeDisplay[EXPORT_CUBE_CLOSE]) ||
         (fMissed && exsExport.afCubeDisplay[EXPORT_CUBE_MISSED]) ||
         (exsExport.afCubeDisplay[stDouble]) || (exsExport.afCubeDisplay[stTake]);
 
-    if (!fDisplay)
+    if (!fDisplayIt)
         return;
 
     fputs("\n<!-- Cube Analysis -->\n\n", pf);
 
 
+    if (fTake >= 0) { /* take or drop */
+        InvertEvaluationR(aarOutput[0], pci);
+        InvertEvaluationR(aarOutput[1], pci);
+        if (pes->et == EVAL_ROLLOUT) {
+            InvertStdDev(aarStdDev[0]);
+            InvertStdDev(aarStdDev[1]);
+        }
+    }
+
     /* print alerts */
 
     if (fMissed) {
@@ -1882,10 +1885,12 @@ HTMLPrintCubeAnalysisTable(FILE * pf,
                 GetStyle(CLASS_CUBE_EQUITY, hecss),
                 OutputEquity(aarOutput[0][OUTPUT_EQUITY], pci, TRUE),
                 GetStyle(CLASS_CUBE_CUBELESS_TEXT, hecss),
-                _("Money"), GetStyle(CLASS_CUBE_EQUITY, hecss), OutputMoneyEquity(aarOutput[0], TRUE));
+                _("Money"), GetStyle(CLASS_CUBE_EQUITY, hecss),
+		OutputMoneyEquity(aarOutput[0], TRUE));
     else
-        fprintf(pf, " cubeless equity</td><td>%s</td><td>&nbsp;</td>\n", OutputMoneyEquity(aarOutput[0], TRUE));
-
+        fprintf(pf, " %s</td><td>%s</td><td>&nbsp;</td>\n",
+		_("cubeless equity"),
+		OutputMoneyEquity(aarOutput[0], TRUE));
 
     fprintf(pf, "</tr>\n");
 
@@ -1920,13 +1925,21 @@ HTMLPrintCubeAnalysisTable(FILE * pf,
     for (i = 0; i < 3; i++) {
 
         fprintf(pf, "<tr><td>%d.</td><td>%s</td>", i + 1, gettext(aszCube[ai[i]]));
+        if (fTake == -1) { /* double */
+            fprintf(pf, "<td %s>%s</td>", GetStyle(CLASS_CUBE_EQUITY, hecss), OutputEquity(arDouble[ai[i]], pci, TRUE));
 
-        fprintf(pf, "<td %s>%s</td>", GetStyle(CLASS_CUBE_EQUITY, hecss), OutputEquity(arDouble[ai[i]], pci, TRUE));
+            if (i)
+                fprintf(pf, "<td>%s</td>", OutputEquityDiff(arDouble[ai[i]], arDouble[OUTPUT_OPTIMAL], pci));
+            else
+                fputs("<td>&nbsp;</td>", pf);
+        } else { /* take or drop */
+            fprintf(pf, "<td %s>%s</td>", GetStyle(CLASS_CUBE_EQUITY, hecss), OutputEquity(-arDouble[ai[i]], pci, TRUE));
 
-        if (i)
-            fprintf(pf, "<td>%s</td>", OutputEquityDiff(arDouble[ai[i]], arDouble[OUTPUT_OPTIMAL], pci));
-        else
-            fputs("<td>&nbsp;</td>", pf);
+            if (i)
+                fprintf(pf, "<td>%s</td>", OutputEquityDiff(-arDouble[ai[i]], -arDouble[OUTPUT_OPTIMAL], pci));
+            else
+                fputs("<td>&nbsp;</td>", pf);
+        }
 
         fputs("</tr>\n", pf);
 
@@ -1980,6 +1993,15 @@ HTMLPrintCubeAnalysisTable(FILE * pf,
 
     }
 
+    if (fTake >= 0) { /* take or drop */
+        InvertEvaluationR(aarOutput[0], pci);
+        InvertEvaluationR(aarOutput[1], pci);
+        if (pes->et == EVAL_ROLLOUT) {
+            InvertStdDev(aarStdDev[0]);
+            InvertStdDev(aarStdDev[1]);
+        }
+    }
+
     if (pes->et == EVAL_ROLLOUT && exsExport.afCubeParameters[1]) {
 
         char *sz = g_strdup(OutputRolloutContext(NULL, &pes->rc));
@@ -2051,8 +2073,8 @@ HTMLPrintCubeAnalysis(FILE * pf, matchstate * pms, moverecord * pmr,
 
         dt = DoubleType(pms->fDoubled, pms->fMove, pms->fTurn);
         if (dt != DT_NORMAL) {
-            fprintf(pf, "<p><span %s> Cannot analyse doubles nor raccoons!</span></p>\n",
-                    GetStyle(CLASS_BLUNDER, hecss));
+            fprintf(pf, "<p><span %s>%s</span></p>\n",
+                    GetStyle(CLASS_BLUNDER, hecss), _("Cannot analyse beaver nor raccoons!"));
             break;
         }
         HTMLPrintCubeAnalysisTable(pf,
@@ -2070,8 +2092,8 @@ HTMLPrintCubeAnalysis(FILE * pf, matchstate * pms, moverecord * pmr,
 
         if (dt != DT_NORMAL) {
             dt = DT_NORMAL;
-            fprintf(pf, "<p><span %s> Cannot analyse doubles nor raccoons!</span></p>\n",
-                    GetStyle(CLASS_BLUNDER, hecss));
+            fprintf(pf, "<p><span %s>%s</span></p>\n",
+                    GetStyle(CLASS_BLUNDER, hecss), _("Cannot analyse beaver nor raccoons!"));
             break;
         }
         HTMLPrintCubeAnalysisTable(pf, pmr->CubeDecPtr->aarOutput, pmr->CubeDecPtr->aarStdDev, pmr->fPlayer, &pmr->CubeDecPtr->esDouble, &ci, TRUE, pmr->mt == MOVE_TAKE, SKILL_NONE,   /* FIXME: skill from prev. cube */
@@ -2107,10 +2129,7 @@ HTMLPrintMoveAnalysis(FILE * pf, matchstate * pms, moverecord * pmr,
                       const char *UNUSED(szImageDir), const char *UNUSED(szExtension),
                       const htmlexporttype UNUSED(het), const htmlexportcss hecss)
 {
-
-    char sz[64];
-    unsigned int i;
-    float rEq, rEqTop;
+    char sz[FORMATEDMOVESIZE];
 
     cubeinfo ci;
 
@@ -2173,7 +2192,9 @@ HTMLPrintMoveAnalysis(FILE * pf, matchstate * pms, moverecord * pmr,
 
     if (pmr->ml.cMoves) {
 
-        for (i = 0; i < pmr->ml.cMoves; i++) {
+        for (unsigned int i = 0; i < pmr->ml.cMoves; i++) {
+
+            float rEq, rEqTop;
 
             if (i >= exsExport.nMoves && i != pmr->n.iMove)
                 continue;
@@ -2310,8 +2331,8 @@ HTMLPrintMoveAnalysis(FILE * pf, matchstate * pms, moverecord * pmr,
 
                 case EVAL_ROLLOUT:
                     {
-                        char *sz = g_strdup(OutputRolloutContext(NULL, &pes->rc));
-                        char *pcS = sz, *pcE;
+                        char *szrc = g_strdup(OutputRolloutContext(NULL, &pes->rc));
+                        char *pcS = szrc, *pcE;
 
                         while ((pcE = strstr(pcS, "\n"))) {
 
@@ -2338,7 +2359,7 @@ HTMLPrintMoveAnalysis(FILE * pf, matchstate * pms, moverecord * pmr,
 
                         }
 
-                        g_free(sz);
+                        g_free(szrc);
 
                         break;
 
@@ -2401,9 +2422,6 @@ static void
 HTMLAnalysis(FILE * pf, matchstate * pms, moverecord * pmr,
              const char *szImageDir, const char *szExtension, const htmlexporttype het, const htmlexportcss hecss)
 {
-
-    char sz[1024];
-
     switch (pmr->mt) {
 
     case MOVE_NORMAL:
@@ -2413,11 +2431,13 @@ HTMLAnalysis(FILE * pf, matchstate * pms, moverecord * pmr,
         if (het == HTML_EXPORT_TYPE_FIBS2HTML)
             printImage(pf, szImageDir, "b-indent", szExtension, "", hecss, het);
 
-        if (pmr->n.anMove[0] >= 0)
+        if (pmr->n.anMove[0] >= 0) {
+            char sz[FORMATEDMOVESIZE];
+
             fprintf(pf,
                     _("%s%s moves %s"), bullet,
                     ap[pmr->fPlayer].szName, FormatMove(sz, (ConstTanBoard) pms->anBoard, pmr->n.anMove));
-        else if (!pmr->ml.cMoves)
+        } else if (!pmr->ml.cMoves)
             fprintf(pf, _("%s%s cannot move"), bullet, ap[pmr->fPlayer].szName);
 
         fputs("</p>\n", pf);
@@ -2442,7 +2462,7 @@ HTMLAnalysis(FILE * pf, matchstate * pms, moverecord * pmr,
             printImage(pf, szImageDir, "b-indent", szExtension, "", hecss, het);
 
         if (pmr->mt == MOVE_DOUBLE)
-            fprintf(pf, "%s%s doubles</p>\n", bullet, ap[pmr->fPlayer].szName);
+            fprintf(pf, "%s%s %s</p>\n", bullet, ap[pmr->fPlayer].szName, _("doubles"));
         else
             fprintf(pf,
                     "%s%s %s</p>\n", bullet,
@@ -2493,7 +2513,7 @@ HTMLDumpStatcontext(FILE * pf, const statcontext * psc,
         GList *list = formatGS(psc, nMatchTo, FORMATGS_CHEQUER);
         GList *pl;
 
-        printStatTableHeader(pf, hecss, _("Checker play statistics"));
+        printStatTableHeader(pf, hecss, _("Chequer play statistics"));
 
         for (pl = g_list_first(list); pl; pl = g_list_next(pl)) {
 
@@ -2648,7 +2668,6 @@ HTMLMatchInfo(FILE * pf, const matchinfo * pmi, const htmlexportcss UNUSED(hecss
 {
 
     int i;
-    char sz[80];
     struct tm tmx;
 
     if (!pmi->nYear &&
@@ -2677,11 +2696,12 @@ HTMLMatchInfo(FILE * pf, const matchinfo * pmi, const htmlexportcss UNUSED(hecss
     /* date */
 
     if (pmi->nYear) {
+        char sz[80];
 
         tmx.tm_year = pmi->nYear - 1900;
         tmx.tm_mon = pmi->nMonth - 1;
         tmx.tm_mday = pmi->nDay;
-        strftime(sz, sizeof(sz), "%B %d, %Y", &tmx);
+        strftime(sz, sizeof(sz), "%x", &tmx);
         HTMLPrintMI(pf, _("Date"), sz);
     }
 
@@ -2764,10 +2784,8 @@ ExportGameHTML(FILE * pf, listOLD * plGame, const char *szImageDir,
 
         case MOVE_NORMAL:
 
-            if (pmr->fPlayer != msExport.fMove) {
+            if (pmr->fPlayer != msExport.fMove)
                 SwapSides(msExport.anBoard);
-                msExport.fMove = pmr->fPlayer;
-            }
 
             msExport.fTurn = msExport.fMove = pmr->fPlayer;
             msExport.anDice[0] = pmr->anDice[0];
@@ -2873,7 +2891,7 @@ OpenCSS(const char *sz)
     if (g_file_test(pchCSS, G_FILE_TEST_EXISTS)) {
         outputf(_("gnubg.css is not written since it already exist in \"%s\"\n"), pchBase);
         pf = NULL;
-    } else if (!(pf = gnubg_g_fopen(pchCSS, "w"))) {
+    } else if (!(pf = g_fopen(pchCSS, "w"))) {
         outputerr(pchCSS);
     }
 
@@ -2889,16 +2907,15 @@ OpenCSS(const char *sz)
 static void
 check_for_html_images(gchar * path)
 {
-    gchar *folder;
-    gchar *img;
     char *url = exsExport.szHTMLPictureURL;
 
     if (url && g_path_is_absolute(url)) {
         if (!g_file_test(url, G_FILE_TEST_EXISTS))
             CommandExportHTMLImages(url);
     } else {
-        folder = g_path_get_dirname(path);
-        img = g_build_filename(folder, url, NULL);
+        gchar *folder = g_path_get_dirname(path);
+        gchar *img = g_build_filename(folder, url, NULL);
+
         if (!g_file_test(img, G_FILE_TEST_EXISTS))
             CommandExportHTMLImages(img);
         g_free(img);
@@ -2912,6 +2929,7 @@ CommandExportGameHtml(char *sz)
 {
 
     FILE *pf;
+    int fDontClose = FALSE;
 
     sz = NextToken(&sz);
 
@@ -2928,9 +2946,10 @@ CommandExportGameHtml(char *sz)
     if (!confirmOverwrite(sz, fConfirmSave))
         return;
 
-    if (!strcmp(sz, "-"))
+    if (!strcmp(sz, "-")) {
         pf = stdout;
-    else if (!(pf = gnubg_g_fopen(sz, "w"))) {
+        fDontClose = TRUE;
+    } else if (!(pf = g_fopen(sz, "w"))) {
         outputerr(sz);
         return;
     }
@@ -2943,7 +2962,7 @@ CommandExportGameHtml(char *sz)
                    exsExport.het, exsExport.hecss, getGameNumber(plGame), FALSE, NULL);
 
 
-    if (pf != stdout)
+    if (!fDontClose)
         fclose(pf);
 
     setDefaultFileName(sz);
@@ -2966,7 +2985,6 @@ CommandExportMatchHtml(char *sz)
     listOLD *pl;
     int nGames;
     char *aszLinks[4], *filenames[4];
-    char *szCurrent;
     int i, j;
 
     sz = NextToken(&sz);
@@ -2984,8 +3002,8 @@ CommandExportMatchHtml(char *sz)
     for (pl = lMatch.plNext, nGames = 0; pl != &lMatch; pl = pl->plNext, nGames++);
 
     for (pl = lMatch.plNext, i = 0; pl != &lMatch; pl = pl->plNext, i++) {
+        char *szCurrent = filename_from_iGame(sz, i);
 
-        szCurrent = filename_from_iGame(sz, i);
         filenames[0] = filename_from_iGame(sz, 0);
         aszLinks[0] = g_path_get_basename(filenames[0]);
         filenames[1] = aszLinks[1] = NULL;
@@ -3020,7 +3038,7 @@ CommandExportMatchHtml(char *sz)
 
         if (!strcmp(szCurrent, "-"))
             pf = stdout;
-        else if (!(pf = gnubg_g_fopen(szCurrent, "w"))) {
+        else if (!(pf = g_fopen(szCurrent, "w"))) {
             outputerr(szCurrent);
             for (j = 0; j < 4; j++) {
                 g_free(aszLinks[j]);
@@ -3081,7 +3099,7 @@ CommandExportPositionHtml(char *sz)
 
     if (!strcmp(sz, "-"))
         pf = stdout;
-    else if (!(pf = gnubg_g_fopen(sz, "w"))) {
+    else if (!(pf = g_fopen(sz, "w"))) {
         outputerr(sz);
         return;
     }
@@ -3136,8 +3154,8 @@ CommandExportPositionHtml(char *sz)
 static void
 ExportPositionGammOnLine(FILE * pf)
 {
-    int fHistory;
-    moverecord *pmr = get_current_moverecord(&fHistory);
+
+    moverecord *pmr = get_current_moverecord(NULL);
 
     if (!pmr) {
         outputerrf(_("Unable to export this position"));
@@ -3165,12 +3183,9 @@ ExportPositionGammOnLine(FILE * pf)
 
     printHTMLBoard(pf, &ms, ms.fTurn, "../Images/", "gif", HTML_EXPORT_TYPE_BBS, HTML_EXPORT_CSS_INLINE);
 
-    if (pmr) {
-        HTMLAnalysis(pf, &ms, pmr, "../Images/", "gif", HTML_EXPORT_TYPE_BBS, HTML_EXPORT_CSS_INLINE);
+    HTMLAnalysis(pf, &ms, pmr, "../Images/", "gif", HTML_EXPORT_TYPE_BBS, HTML_EXPORT_CSS_INLINE);
 
-        HTMLPrintComment(pf, pmr, HTML_EXPORT_CSS_INLINE);
-
-    }
+    HTMLPrintComment(pf, pmr, HTML_EXPORT_CSS_INLINE);
 
     HTMLEpilogueComment(pf);
 
@@ -3182,6 +3197,7 @@ CommandExportPositionGammOnLine(char *sz)
 {
 
     FILE *pf;
+    int fDontClose = FALSE;
 
     sz = NextToken(&sz);
 
@@ -3198,16 +3214,17 @@ CommandExportPositionGammOnLine(char *sz)
     if (!confirmOverwrite(sz, fConfirmSave))
         return;
 
-    if (!strcmp(sz, "-"))
+    if (!strcmp(sz, "-")) {
         pf = stdout;
-    else if (!(pf = gnubg_g_fopen(sz, "w"))) {
+        fDontClose = TRUE;
+    } else if (!(pf = g_fopen(sz, "w"))) {
         outputerr(sz);
         return;
     }
 
     ExportPositionGammOnLine(pf);
 
-    if (pf != stdout)
+    if (!fDontClose)
         fclose(pf);
 
 }
@@ -3229,6 +3246,11 @@ CommandExportPositionGOL2Clipboard(char *UNUSED(sz))
 
     pf = GetTemporaryFile(NULL, &tmpFile);
 
+    if (pf == NULL) {
+        outputerr(_("Error creating temporary file"));
+        return;
+    }
+
     /* generate file */
 
     ExportPositionGammOnLine(pf);
@@ -3236,14 +3258,14 @@ CommandExportPositionGOL2Clipboard(char *UNUSED(sz))
     /* find size of file */
 
     if (fseek(pf, 0L, SEEK_END)) {
-        outputerr("temporary file");
+        outputerr(_("Error reading temporary file"));
         return;
     }
 
     l = ftell(pf);
 
     if (fseek(pf, 0L, SEEK_SET)) {
-        outputerr("temporary file");
+        outputerr(_("Error reading temporary file"));
         return;
     }
 
@@ -3252,7 +3274,7 @@ CommandExportPositionGOL2Clipboard(char *UNUSED(sz))
     szClipboard = (char *) malloc(l + 1);
 
     if (fread(szClipboard, 1, l, pf) != (unsigned long) l) {
-        outputerr("temporary file");
+        outputerr(_("Error reading temporary file"));
     } else {
         szClipboard[l] = 0;
         TextToClipboard(szClipboard);
