@@ -1,11 +1,11 @@
 /*
- * latex.c
+ * Copyright (C) 2001-2002 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2002-2021 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 2001, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: latex.c,v 1.55 2015/07/31 23:40:25 mdpetch Exp $
+ * $Id: latex.c,v 1.61 2022/02/13 22:30:16 plm Exp $
  */
 
 #include "config.h"
@@ -30,7 +29,6 @@
 #include "export.h"
 #include "format.h"
 #include <glib/gstdio.h>
-#include "glib-ext.h"
 
 static const char *aszLuckTypeLaTeXAbbr[] = { "$--$", "$-$", "", "$+$", "$++$" };
 
@@ -122,10 +120,11 @@ LaTeXEpilogue(FILE * pf)
 }
 
 static void
-DrawLaTeXPoint(FILE * pf, int i, int fPlayer, int c)
+DrawLaTeXPoint(FILE * pf, int i, int fPlayer, unsigned int c)
 {
 
-    int j, x, y = 0;
+    unsigned int j;
+    int x, y = 0;
 
     if (i < 6)
         x = 320 - 20 * i;
@@ -147,7 +146,7 @@ DrawLaTeXPoint(FILE * pf, int i, int fPlayer, int c)
         if (j == 5 || (i == 24 && j == 3)) {
             fprintf(pf, "\\whiten\\path(%d,%d)(%d,%d)(%d,%d)(%d,%d)(%d,%d)\n"
                     "\\path(%d,%d)(%d,%d)(%d,%d)(%d,%d)(%d,%d)\n"
-                    "\\put(%d,%d){\\makebox(10,10){\\textsf{\\tiny %d}}}\n",
+                    "\\put(%d,%d){\\makebox(10,10){\\textsf{\\tiny %u}}}\n",
                     x - 5, y - 5, x + 5, y - 5, x + 5, y + 5, x - 5, y + 5,
                     x - 5, y - 5,
                     x - 5, y - 5, x + 5, y - 5, x + 5, y + 5, x - 5, y + 5, x - 5, y - 5, x - 5, y - 5, c);
@@ -172,7 +171,8 @@ static void
 PrintLaTeXBoard(FILE * pf, matchstate * pms, int fPlayer)
 {
 
-    int anOff[2] = { 15, 15 }, i, x, y;
+    unsigned int anOff[2] = { 15, 15 };
+    int i, x, y;
 
     /* FIXME print position ID and pip count, and the player on roll.
      * Print player names too? */
@@ -221,10 +221,10 @@ static void
 LaTeXEscape(FILE * pf, char *pch)
 {
 
-    /* Translation table from GNU recode, by Franois Pinard. */
+    /* Translation table from GNU recode, by Francois Pinard. */
     static struct translation {
-        unsigned char uch;
-        char *sz;
+        const unsigned char uch;
+        const char *sz;
     } at[] = {
         {
         10, "\n\n"},            /* \n */
@@ -336,7 +336,7 @@ PrintLaTeXComment(FILE * pf, char *pch)
 static void
 PrintLaTeXCubeAnalysis(FILE * pf, const matchstate * pms, int fPlayer,
                        float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
-                       float aarStdDev[2][NUM_ROLLOUT_OUTPUTS], const evalsetup * pes)
+                       float aarStdDev[2][NUM_ROLLOUT_OUTPUTS], const evalsetup * pes, int fTake)
 {
 
     cubeinfo ci;
@@ -349,7 +349,7 @@ PrintLaTeXCubeAnalysis(FILE * pf, const matchstate * pms, int fPlayer,
 
     /* FIXME use center and tabular environment instead of verbatim */
     fputs("{\\begin{quote}\\footnotesize\\begin{verbatim}\n", pf);
-    fputs(OutputCubeAnalysis(aarOutput, aarStdDev, pes, &ci), pf);
+    fputs(OutputCubeAnalysis(aarOutput, aarStdDev, pes, &ci, fTake), pf);
     fputs("\\end{verbatim}\\end{quote}}\n", pf);
 }
 
@@ -385,21 +385,24 @@ ExportGameLaTeX(FILE * pf, listOLD * plGame)
             dt = DT_NORMAL;
             fputs("\\clearpage\n", pf);
             fputs("\\noindent{\\Large ", pf);
-            if (pmr->g.nMatch)
-                fprintf(pf, _("%d point%s match (game %d)"),
-                        pmr->g.nMatch, (pmr->g.nMatch > 1 ? "s" : ""), pmr->g.i + 1);
-            else
+            if (pmr->g.nMatch) {
+                fprintf(pf, ngettext("%d point match", "%d points match", pmr->g.nMatch), pmr->g.nMatch);
+                fprintf(pf, _(" (game %d)"), pmr->g.i + 1);
+             } else
                 fprintf(pf, _("Money session (game %d)"), pmr->g.i + 1);
 
             fputs("}\n\n\\vspace{\\baselineskip}\n\n", pf);
 
             fprintf(pf, "\\noindent\n\\makebox[0.5\\textwidth][s]" "{\\large %s ", PlayerSymbol(0));
             LaTeXEscape(pf, ap[0].szName);
-            fprintf(pf, " (%d points)\\hfill}", pmr->g.anScore[0]);
-
+            fprintf(pf, " (");
+            fprintf(pf, ngettext("%d point", "%d points", pmr->g.anScore[0]), pmr->g.anScore[0]);
+            fprintf(pf, ")\\hfill}");
             fprintf(pf, "\\makebox[0.5\\textwidth][s]" "{\\large %s ", PlayerSymbol(1));
             LaTeXEscape(pf, ap[1].szName);
-            fprintf(pf, " (%d points)\\hfill}\n\n", pmr->g.anScore[1]);
+            fprintf(pf, " (");
+            fprintf(pf, ngettext("%d point", "%d points", pmr->g.anScore[1]), pmr->g.anScore[1]);
+            fprintf(pf, ")\\hfill}\n\n");
 
             break;
 
@@ -413,7 +416,7 @@ ExportGameLaTeX(FILE * pf, listOLD * plGame)
                 PrintLaTeXBoard(pf, &msExport, pmr->fPlayer);
 
             PrintLaTeXCubeAnalysis(pf, &msExport, pmr->fPlayer,
-                                   pmr->CubeDecPtr->aarOutput, pmr->CubeDecPtr->aarStdDev, &pmr->CubeDecPtr->esDouble);
+                                   pmr->CubeDecPtr->aarOutput, pmr->CubeDecPtr->aarStdDev, &pmr->CubeDecPtr->esDouble, -1);
             /* FIXME: output cube skill */
 
             sprintf(sz, "%s %u%u%s: ", PlayerSymbol(pmr->fPlayer),
@@ -441,13 +444,13 @@ ExportGameLaTeX(FILE * pf, listOLD * plGame)
         case MOVE_DOUBLE:
             dt = DoubleType(msExport.fDoubled, msExport.fMove, msExport.fTurn);
             if (dt != DT_NORMAL) {
-                fprintf(pf, "\\begin{center}\\emph{Cannot analyse doubles nor raccoons!}\\end{center}");
+                fprintf(pf, "\\begin{center}\\emph{%s}\\end{center}", _("Cannot analyse beaver nor raccoons!"));
                 break;
             }
             PrintLaTeXBoard(pf, &msExport, pmr->fPlayer);
 
             PrintLaTeXCubeAnalysis(pf, &msExport, pmr->fPlayer,
-                                   pmr->CubeDecPtr->aarOutput, pmr->CubeDecPtr->aarStdDev, &pmr->CubeDecPtr->esDouble);
+                                   pmr->CubeDecPtr->aarOutput, pmr->CubeDecPtr->aarStdDev, &pmr->CubeDecPtr->esDouble, -1);
 
             /* FIXME what about beavers? */
             fprintf(pf, "\\begin{center}%s %s%s\\end{center}\n\n",
@@ -460,7 +463,7 @@ ExportGameLaTeX(FILE * pf, listOLD * plGame)
         case MOVE_TAKE:
             if (dt != DT_NORMAL) {
                 dt = DT_NORMAL;
-                fprintf(pf, "\\begin{center}\\emph{Cannot analyse doubles nor raccoons!}\\end{center}");
+                fprintf(pf, "\\begin{center}\\emph{%s}\\end{center}", _("Cannot analyse beaver nor raccoons!"));
                 break;
             }
             fTook = TRUE;
@@ -474,7 +477,7 @@ ExportGameLaTeX(FILE * pf, listOLD * plGame)
         case MOVE_DROP:
             if (dt != DT_NORMAL) {
                 dt = DT_NORMAL;
-                fprintf(pf, "\\begin{center}\\emph{Cannot analyse doubles nor raccoons!}\\end{center}");
+                fprintf(pf, "\\begin{center}\\emph{%s}\\end{center}", _("Cannot analyse beaver nor raccoons!"));
                 break;
             }
             fprintf(pf, "\\begin{center}%s %s%s\\end{center}\n\n",
@@ -534,7 +537,7 @@ CommandExportGameLaTeX(char *sz)
 
     if (!strcmp(sz, "-"))
         pf = stdout;
-    else if ((pf = gnubg_g_fopen(sz, "w")) == 0) {
+    else if ((pf = g_fopen(sz, "w")) == 0) {
         outputerr(sz);
         return;
     }
@@ -558,6 +561,7 @@ CommandExportMatchLaTeX(char *sz)
 
     FILE *pf;
     listOLD *pl;
+    int fDontClose = FALSE;
 
     sz = NextToken(&sz);
 
@@ -569,9 +573,10 @@ CommandExportMatchLaTeX(char *sz)
     if (!confirmOverwrite(sz, fConfirmSave))
         return;
 
-    if (!strcmp(sz, "-"))
+    if (!strcmp(sz, "-")) {
         pf = stdout;
-    else if ((pf = gnubg_g_fopen(sz, "w")) == 0) {
+        fDontClose = TRUE;
+    } else if ((pf = g_fopen(sz, "w")) == 0) {
         outputerr(sz);
         return;
     }
@@ -585,7 +590,7 @@ CommandExportMatchLaTeX(char *sz)
 
     LaTeXEpilogue(pf);
 
-    if (pf != stdout)
+    if (!fDontClose)
         fclose(pf);
 
     setDefaultFileName(sz);
