@@ -1,10 +1,11 @@
 /*
- * Subset of inputs computed with SSE
+ * Copyright (C) 2006 Oystein Johansen <oystein@gnubg.org>
+ * Copyright (C) 2011-2018 the AUTHORS
  *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: inputs.c,v 1.10 2017/08/22 20:08:10 plm Exp $
+ * $Id: inputs.c,v 1.13 2021/06/09 22:20:25 plm Exp $
  */
 
 #include "config.h"
@@ -24,7 +24,9 @@
 #include "eval.h"
 
 #if defined(USE_SIMD_INSTRUCTIONS)
-#if defined(USE_AVX)
+#if defined(USE_NEON)
+#include <arm_neon.h>
+#elif defined(USE_AVX)
 #include <immintrin.h>
 #elif defined(USE_SSE2)
 #include <emmintrin.h>
@@ -38,72 +40,40 @@ typedef float float_vector[4];
 typedef SSE_ALIGN(float float_vec_aligned[sizeof(float_vector)/sizeof(float)]);
 
 SSE_ALIGN (static float_vec_aligned inpvec[16]) = {
-    /*  0 */  {
-    0.0, 0.0, 0.0, 0.0},
-        /*  1 */  {
-    1.0, 0.0, 0.0, 0.0},
-        /*  2 */  {
-    0.0, 1.0, 0.0, 0.0},
-        /*  3 */  {
-    0.0, 0.0, 1.0, 0.0},
-        /*  4 */  {
-    0.0, 0.0, 1.0, 0.5},
-        /*  5 */  {
-    0.0, 0.0, 1.0, 1.0},
-        /*  6 */  {
-    0.0, 0.0, 1.0, 1.5},
-        /*  7 */  {
-    0.0, 0.0, 1.0, 2.0},
-        /*  8 */  {
-    0.0, 0.0, 1.0, 2.5},
-        /*  9 */  {
-    0.0, 0.0, 1.0, 3.0},
-        /* 10 */  {
-    0.0, 0.0, 1.0, 3.5},
-        /* 11 */  {
-    0.0, 0.0, 1.0, 4.0},
-        /* 12 */  {
-    0.0, 0.0, 1.0, 4.5},
-        /* 13 */  {
-    0.0, 0.0, 1.0, 5.0},
-        /* 14 */  {
-    0.0, 0.0, 1.0, 5.5},
-        /* 15 */  {
-0.0, 0.0, 1.0, 6.0}};
+    /*  0 */  { 0.0, 0.0, 0.0, 0.0},
+    /*  1 */  { 1.0, 0.0, 0.0, 0.0},
+    /*  2 */  { 0.0, 1.0, 0.0, 0.0},
+    /*  3 */  { 0.0, 0.0, 1.0, 0.0},
+    /*  4 */  { 0.0, 0.0, 1.0, 0.5},
+    /*  5 */  { 0.0, 0.0, 1.0, 1.0},
+    /*  6 */  { 0.0, 0.0, 1.0, 1.5},
+    /*  7 */  { 0.0, 0.0, 1.0, 2.0},
+    /*  8 */  { 0.0, 0.0, 1.0, 2.5},
+    /*  9 */  { 0.0, 0.0, 1.0, 3.0},
+    /* 10 */  { 0.0, 0.0, 1.0, 3.5},
+    /* 11 */  { 0.0, 0.0, 1.0, 4.0},
+    /* 12 */  { 0.0, 0.0, 1.0, 4.5},
+    /* 13 */  { 0.0, 0.0, 1.0, 5.0},
+    /* 14 */  { 0.0, 0.0, 1.0, 5.5},
+    /* 15 */  { 0.0, 0.0, 1.0, 6.0}};
 
 SSE_ALIGN(static float_vec_aligned inpvecb[16]) = {
-    /*  0 */  {
-    0.0, 0.0, 0.0, 0.0},
-        /*  1 */  {
-    1.0, 0.0, 0.0, 0.0},
-        /*  2 */  {
-    1.0, 1.0, 0.0, 0.0},
-        /*  3 */  {
-    1.0, 1.0, 1.0, 0.0},
-        /*  4 */  {
-    1.0, 1.0, 1.0, 0.5},
-        /*  5 */  {
-    1.0, 1.0, 1.0, 1.0},
-        /*  6 */  {
-    1.0, 1.0, 1.0, 1.5},
-        /*  7 */  {
-    1.0, 1.0, 1.0, 2.0},
-        /*  8 */  {
-    1.0, 1.0, 1.0, 2.5},
-        /*  9 */  {
-    1.0, 1.0, 1.0, 3.0},
-        /* 10 */  {
-    1.0, 1.0, 1.0, 3.5},
-        /* 11 */  {
-    1.0, 1.0, 1.0, 4.0},
-        /* 12 */  {
-    1.0, 1.0, 1.0, 4.5},
-        /* 13 */  {
-    1.0, 1.0, 1.0, 5.0},
-        /* 14 */  {
-    1.0, 1.0, 1.0, 5.5},
-        /* 15 */  {
-1.0, 1.0, 1.0, 6.0}};
+    /*  0 */  { 0.0, 0.0, 0.0, 0.0},
+    /*  1 */  { 1.0, 0.0, 0.0, 0.0},
+    /*  2 */  { 1.0, 1.0, 0.0, 0.0},
+    /*  3 */  { 1.0, 1.0, 1.0, 0.0},
+    /*  4 */  { 1.0, 1.0, 1.0, 0.5},
+    /*  5 */  { 1.0, 1.0, 1.0, 1.0},
+    /*  6 */  { 1.0, 1.0, 1.0, 1.5},
+    /*  7 */  { 1.0, 1.0, 1.0, 2.0},
+    /*  8 */  { 1.0, 1.0, 1.0, 2.5},
+    /*  9 */  { 1.0, 1.0, 1.0, 3.0},
+    /* 10 */  { 1.0, 1.0, 1.0, 3.5},
+    /* 11 */  { 1.0, 1.0, 1.0, 4.0},
+    /* 12 */  { 1.0, 1.0, 1.0, 4.5},
+    /* 13 */  { 1.0, 1.0, 1.0, 5.0},
+    /* 14 */  { 1.0, 1.0, 1.0, 5.5},
+    /* 15 */  { 1.0, 1.0, 1.0, 6.0}};
 
 #if defined(USE_SIMD_INSTRUCTIONS)
 extern SIMD_AVX_STACKALIGN void
@@ -113,16 +83,29 @@ baseInputs(const TanBoard anBoard, float arInput[])
 
     const unsigned int *pB = &anBoard[0][0];
     float *pInput = &arInput[0];
-    register __m128 vec0;
-    register __m128 vec1;
-    register __m128 vec2;
-    register __m128 vec3;
-    register __m128 vec4;
-    register __m128 vec5;
-    register __m128 vec6;
-    register __m128 vec7;
+
+#if defined(HAVE_SSE)
+    __m128 vec0;
+    __m128 vec1;
+    __m128 vec2;
+    __m128 vec3;
+    __m128 vec4;
+    __m128 vec5;
+    __m128 vec6;
+    __m128 vec7;
+#elif defined(HAVE_NEON)
+    float32x4_t vec0;
+    float32x4_t vec1;
+    float32x4_t vec2;
+    float32x4_t vec3;
+    float32x4_t vec4;
+    float32x4_t vec5;
+    float32x4_t vec6;
+    float32x4_t vec7;
+#endif
 
     while (i--) {
+#if defined(HAVE_SSE)
         vec0 = _mm_load_ps(inpvec[*pB++]);
         _mm_store_ps(pInput, vec0);
         vec1 = _mm_load_ps(inpvec[*pB++]);
@@ -139,16 +122,40 @@ baseInputs(const TanBoard anBoard, float arInput[])
         _mm_store_ps(pInput += 4, vec6);
         vec7 = _mm_load_ps(inpvec[*pB++]);
         _mm_store_ps(pInput += 4, vec7);
+#else
+        vec0 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput, vec0);
+        vec1 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec1);
+        vec2 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec2);
+        vec3 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec3);
+        vec4 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec4);
+        vec5 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec5);
+        vec6 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec6);
+        vec7 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec7);
+#endif
         pInput += 4;
     }
 
     /* bar */
+#if defined(HAVE_SSE)
     vec0 = _mm_load_ps(inpvecb[*pB++]);
     _mm_store_ps(pInput, vec0);
+#else
+    vec0 = vld1q_f32(inpvecb[*pB++]);
+    vst1q_f32(pInput, vec0);
+#endif
     pInput += 4;
 
     i = 3;
     while (i--) {
+#if defined(HAVE_SSE)
         vec0 = _mm_load_ps(inpvec[*pB++]);
         _mm_store_ps(pInput, vec0);
         vec1 = _mm_load_ps(inpvec[*pB++]);
@@ -165,15 +172,34 @@ baseInputs(const TanBoard anBoard, float arInput[])
         _mm_store_ps(pInput += 4, vec6);
         vec7 = _mm_load_ps(inpvec[*pB++]);
         _mm_store_ps(pInput += 4, vec7);
+#else
+        vec0 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput, vec0);
+        vec1 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec1);
+        vec2 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec2);
+        vec3 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec3);
+        vec4 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec4);
+        vec5 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec5);
+        vec6 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec6);
+        vec7 = vld1q_f32(inpvec[*pB++]);
+        vst1q_f32(pInput += 4, vec7);
+#endif
         pInput += 4;
     }
 
     /* bar */
+#if defined(HAVE_SSE)
     vec0 = _mm_load_ps(inpvecb[*pB]);
     _mm_store_ps(pInput, vec0);
-
-#if defined(USE_AVX)
-    _mm256_zeroupper();
+#else
+    vec0 = vld1q_f32(inpvecb[*pB++]);
+    vst1q_f32(pInput, vec0);
 #endif
 
     return;
