@@ -1,7 +1,11 @@
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * Copyright (C) 1996-1999 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2004-2007 the AUTHORS
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -9,21 +13,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * list.h
- *
- * by Gary Wong, 1996
- * $Id: list.h,v 1.9 2013/06/16 02:16:24 mdpetch Exp $
+ * $Id: list.h,v 1.10 2021/06/09 21:11:26 plm Exp $
  */
 
 #ifndef LIST_H
 #define LIST_H
 
-typedef struct _list {
-    struct _list *plPrev;
-    struct _list *plNext;
+typedef struct list {
+    struct list *plPrev;
+    struct list *plNext;
     void *p;
 } listOLD;
 /* Renamed to listOLD - use GList instead (hopefullly replace existing usage eventually */
