@@ -1,11 +1,11 @@
 /*
- * neuralnet.c
+ * Copyright (C) 1998-2002 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2000-2019 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 1998, 1999, 2000, 2001, 2002.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: neuralnet.c,v 1.92 2018/05/12 20:59:35 plm Exp $
+ * $Id: neuralnet.c,v 1.95 2020/06/05 20:07:51 plm Exp $
  */
 
 #include "config.h"
@@ -270,7 +269,7 @@ NeuralNetLoad(neuralnet * pnn, FILE * pf)
     float *pr;
     char dummy[16];
 
-    if (fscanf(pf, "%u %u %u %s %f %f\n", &pnn->cInput, &pnn->cHidden,
+    if (fscanf(pf, "%u %u %u %15s %f %f\n", &pnn->cInput, &pnn->cHidden,
                &pnn->cOutput, dummy, &pnn->rBetaHidden,
                &pnn->rBetaOutput) < 5 || pnn->cInput < 1 ||
         pnn->cHidden < 1 || pnn->cOutput < 1 || pnn->rBetaHidden <= 0.0f || pnn->rBetaOutput <= 0.0f) {
@@ -309,7 +308,7 @@ NeuralNetLoadBinary(neuralnet * pnn, FILE * pf)
     int dummy;
 
 #define FREAD( p, c ) \
-    if( fread( (p), sizeof( *(p) ), (c), pf ) < (unsigned int)(c) ) return -1;
+    if ( fread( (p), sizeof( *(p) ), (c), pf ) < (unsigned int)(c) ) return -1
 
     FREAD(&pnn->cInput, 1);
     FREAD(&pnn->cHidden, 1);
@@ -342,7 +341,7 @@ NeuralNetSaveBinary(const neuralnet * pnn, FILE * pf)
 {
 
 #define FWRITE( p, c ) \
-    if( fwrite( (p), sizeof( *(p) ), (c), pf ) < (unsigned int)(c) ) return -1;
+    if ( fwrite( (p), sizeof( *(p) ), (c), pf ) < (unsigned int)(c) ) return -1
 
     FWRITE(&pnn->cInput, 1);
     FWRITE(&pnn->cHidden, 1);
@@ -384,7 +383,7 @@ check_for_cpuid(void)
 {
     int result;
 
-    asm volatile(
+    __asm__ __volatile__ (
 #if defined(ENVIRONMENT32) && defined(__PIC__)
         /* We have to be careful to not destroy ebx if using PIC on 32bit builds */
         "pushl %%ebx\n\t"
@@ -458,7 +457,7 @@ CheckSSE(void)
 
 #if defined(USE_AVX)
 
-    asm volatile(
+    __asm__ __volatile__ (
 #if defined(ENVIRONMENT32) && defined(__PIC__)
         /* We have to be careful to not destroy ebx if using PIC on 32bit builds */
         "pushl %%ebx\n\t"
@@ -505,7 +504,7 @@ CheckSSE(void)
 
 #else
 
-    asm volatile (
+    __asm__ __volatile__ (
 #if defined(ENVIRONMENT32) && defined(__PIC__)
         /* We have to be careful to not destroy ebx if using PIC on I386 */
         "pushl %%ebx\n\t"
