@@ -1,7 +1,11 @@
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * Copyright (C) 1999-2002 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2002-2017 the AUTHORS
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -9,13 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * neuralnet.h
- *
- * by Gary Wong, 1998
- * $Id: neuralnet.h,v 1.35 2017/02/18 16:19:44 plm Exp $
+ * $Id: neuralnet.h,v 1.37 2021/06/09 21:31:00 plm Exp $
  */
 
 #ifndef NEURALNET_H
@@ -24,7 +24,7 @@
 #include <stdio.h>
 #include "common.h"
 
-typedef struct _neuralnet {
+typedef struct {
     unsigned int cInput;
     unsigned int cHidden;
     unsigned int cOutput;
@@ -49,7 +49,7 @@ typedef enum {
     NNSTATE_DONE
 } NNStateType;
 
-typedef struct _NNState {
+typedef struct {
     NNStateType state;
     float *savedBase;
     float *savedIBase;
