@@ -1,11 +1,11 @@
 /*
- * makehyper.c
+ * Copyright (C) 2003 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2007-2019 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2003
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: makehyper.c,v 1.52 2018/07/01 20:48:21 plm Exp $
+ * $Id: makehyper.c,v 1.58 2022/01/27 22:14:47 plm Exp $
  */
 
 #include "config.h"
@@ -42,14 +41,14 @@
 static cubeinfo ci;
 static cubeinfo ciJacoby;
 
-typedef enum _hyperclass {
+typedef enum {
     HYPER_OVER,
     HYPER_BEAROFF,
     HYPER_CONTACT,
     HYPER_ILLEGAL
 } hyperclass;
 
-enum _xxx {
+enum {
     EQUITY_CUBELESS = 0,
     EQUITY_OWNED = 1,
     EQUITY_CENTER = 2,
@@ -57,7 +56,7 @@ enum _xxx {
     EQUITY_OPPONENT = 4
 };
 
-typedef struct _hyperequity {
+typedef struct {
 
     float arOutput[NUM_OUTPUTS];
     float arEquity[5];
@@ -153,7 +152,7 @@ StartGuessHyper(hyperequity ahe[], const int nC, bearoffcontext * UNUSED(pbc))
                 ++ai[1];
                 --ai[2];
 #if defined(HAVE_FUNC_ATTRIBUTE_FALLTHROUGH)
-                __attribute__((fallthrough));
+                __attribute__ ((fallthrough));
 #endif
             case HYPER_BEAROFF:
 
@@ -186,12 +185,12 @@ StartGuessHyper(hyperequity ahe[], const int nC, bearoffcontext * UNUSED(pbc))
         }
     }
 
-    printf("%-25s: %10d\n", _("Number of game-over positions"), ai[0]);
-    printf("%-25s: %10d\n", _("Number of non-contact positions"), ai[2]);
-    printf("%-25s: %10d\n", _("Number of contact positions"), ai[1]);
-    printf("%-25s: %10d\n", _("Total number of legal positions"), ai[0] + ai[2] + ai[1]);
-    printf("%-25s: %10d\n", _("Number of illegal positions"), ai[3]);
-    printf("%-25s: %10d\n", _("Total number of positions in file"), nPos * nPos);
+    g_print("%-25s: %10d\n", _("Number of game-over positions"), ai[0]);
+    g_print("%-25s: %10d\n", _("Number of non-contact positions"), ai[2]);
+    g_print("%-25s: %10d\n", _("Number of contact positions"), ai[1]);
+    g_print("%-25s: %10d\n", _("Total number of legal positions"), ai[0] + ai[2] + ai[1]);
+    g_print("%-25s: %10d\n", _("Number of illegal positions"), ai[3]);
+    g_print("%-25s: %10d\n", _("Total number of positions in file"), nPos * nPos);
 
 }
 
@@ -207,7 +206,7 @@ StartFromDatabase(hyperequity ahe[], const int nC, const char *szFilename)
     int i, j, k;
     float r;
 
-    if (!(pf = gnubg_g_fopen(szFilename, "r+b"))) {
+    if (!(pf = g_fopen(szFilename, "r+b"))) {
         perror(szFilename);
         exit(2);
     }
@@ -226,14 +225,14 @@ StartFromDatabase(hyperequity ahe[], const int nC, const char *szFilename)
 
             for (k = 0; k < NUM_OUTPUTS; ++k) {
                 us = ac[3 * k] | (ac[3 * k + 1]) << 8 | (ac[3 * k + 2]) << 16;
-                r = us / 16777215.0f;
+                r = (float) us / 16777215.0f;
                 g_assert(r >= 0 && r <= 1);
                 ahe[i * nPos + j].arOutput[k] = r;
             }
 
             for (k = 0; k < 4; ++k) {
                 us = ac[15 + 3 * k] | (ac[15 + 3 * k + 1]) << 8 | (ac[15 + 3 * k + 2]) << 16;
-                r = (us / 16777215.0f - 0.5f) * 6.0f;
+                r = ((float) us / 16777215.0f - 0.5f) * 6.0f;
                 g_assert(r >= -3 && r <= 3);
                 ahe[i * nPos + j].arEquity[k + 1] = r;
             }
@@ -476,7 +475,7 @@ CalcNewEquity(hyperequity ahe[], const int nC, float arNorm[])
 
     for (i = 0; i < nPos; ++i) {
 
-        printf("\r%d/%d              ", i + 1, nPos);
+        g_print("\r%d/%d              ", i + 1, nPos);
         fflush(stdout);
 
         for (j = 0; j < nPos; ++j) {
@@ -487,7 +486,7 @@ CalcNewEquity(hyperequity ahe[], const int nC, float arNorm[])
 
     }
 
-    printf("\n");
+    g_print("\n");
 
 }
 
@@ -530,7 +529,7 @@ WriteHyperFile(const char *szFilename, const hyperequity ahe[], const int nC)
     FILE *pf;
 
 
-    if (!(pf = gnubg_g_fopen(szFilename, "w+b"))) {
+    if (!(pf = g_fopen(szFilename, "w+b"))) {
         perror(szFilename);
         return;
     }
@@ -553,12 +552,11 @@ WriteHyperFile(const char *szFilename, const hyperequity ahe[], const int nC)
 
 }
 
+
 static void
 version(void)
 {
-
-    printf("$id\n");
-
+    g_print("makehyper $Revision: 1.58 $\n");
 }
 
 
@@ -584,17 +582,17 @@ main(int argc, char **argv)
 
     GOptionEntry ao[] = {
         {"chequers", 'c', 0, G_OPTION_ARG_INT, &nC,
-         "The number of chequers(0<C<4). Default is 3.", "C"},
+         N_("The number of chequers (0<C<4). Default is 3"), "C"},
         {"restart", 'r', 0, G_OPTION_ARG_FILENAME, &szRestart,
-         "Restart calculation of database from \"filename\"", "filename"},
+         N_("Restart calculation of database from \"filename\"."), "filename"},
         {"threshold", 't', 0, G_OPTION_ARG_STRING, &szEpsilon,
-         "The convergence threshold (T). Default is 1e-5", "T"},
+         N_("The convergence threshold (T). Default is 1e-5"), "T"},
         {"no-checkpoint", 'n', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &fCheckPoint,
-         "Do not write a checkpoint file after each iteration.", NULL},
+         N_("Do not write a checkpoint file after each iteration"), NULL},
         {"version", 'v', 0, G_OPTION_ARG_NONE, &show_version,
-         "Print version info and exit", NULL},
+         N_("Print version info and exit"), NULL},
         {"outfile", 'f', 0, G_OPTION_ARG_STRING, &szOutput,
-         "Output filename. Default is hyper<C>.bd.", "filename"},
+         N_("Output filename. Default is hyper<C>.bd"), "filename"},
         {NULL, 0, 0, (GOptionArg) 0, NULL, NULL, NULL}
     };
 
@@ -609,6 +607,8 @@ main(int argc, char **argv)
     bindtextdomain(PACKAGE, LOCALEDIR);
     textdomain(PACKAGE);
 
+    g_set_print_handler(print_utf8_to_locale);
+
     context = g_option_context_new(NULL);
     g_option_context_add_main_entries(context, ao, PACKAGE);
     g_option_context_parse(context, &argc, &argv, &error);
@@ -628,12 +628,12 @@ main(int argc, char **argv)
     if (szEpsilon)
         rEpsilon = (float) g_strtod(szEpsilon, NULL);
     if (rEpsilon > 1.0f || rEpsilon < 0.0f) {
-        g_printerr("Valid threadholds are 0.0 - 1.0\n");
+        g_printerr(_("Valid thresholds are 0.0 - 1.0\n"));
         exit(1);
     }
 
     if (nC < 1 || nC > 3) {
-        g_printerr("Illegal options. Try `makehyper --help' for usage " "information\n");
+        g_printerr(_("Illegal options. Try `makehyper --help' for usage information\n"));
         exit(1);
     }
 
@@ -644,38 +644,38 @@ main(int argc, char **argv)
 
     time(&t2);
 
-    printf(_("Number of chequers: %d\n"), nC);
+    g_print(_("Number of chequers: %d\n"), nC);
 
     nPos = Combination(25 + nC, nC);
 
-    printf("%-40s: %d\n", _("Total number of one sided positions"), nPos);
-    printf("%-40s: %d\n", _("Total number of two sided positions"), nPos * nPos);
-    printf("%-40s: %s %d\n", _("Estimated size of file"), _("bytes"), nPos * nPos * 28 + 40);
-    printf("%-40s: %s\n", _("Output file"), szOutput);
-    printf("%-40s: %e\n", _("Convergence threshold"), rEpsilon);
+    g_print("%-40s: %d\n", _("Total number of one sided positions"), nPos);
+    g_print("%-40s: %d\n", _("Total number of two sided positions"), nPos * nPos);
+    g_print("%-40s: %d %s\n", _("Estimated size of file"), nPos * nPos * 28 + 40,  _("bytes"));
+    g_print("%-40s: %s\n", _("Output file"), szOutput);
+    g_print("%-40s: %e\n", _("Convergence threshold"), rEpsilon);
 
     /* Iteration 0 */
 
     time(&t0);
 
-    printf(_("*** Obtain start guess ***\n"));
+    g_print(_("*** Obtain start guess ***\n"));
 
     SetCubeInfo(&ci, 1, -1, 0, 0, NULL, FALSE, FALSE, FALSE, VARIATION_HYPERGAMMON_1 + nC - 1);
     SetCubeInfo(&ciJacoby, 1, -1, 0, 0, NULL, FALSE, TRUE, FALSE, VARIATION_HYPERGAMMON_1 + nC - 1);
 
-    aheEquity = (hyperequity *) malloc(nPos * nPos * sizeof(hyperequity));
+    aheEquity = (hyperequity *) g_malloc(nPos * nPos * sizeof(hyperequity));
 
     if (!szRestart) {
-        printf(_("0-vector start guess\n"));
+        g_print(_("0-vector start guess\n"));
         StartGuessHyper(aheEquity, nC, pbc);
     } else {
-        printf(_("Start from file\n"));
+        g_print(_("Start from file\n"));
         StartFromDatabase(aheEquity, nC, szRestart);
     }
 
     time(&t1);
 
-    printf(_("Time for start guess: %d seconds\n"), (int) (t1 - t0));
+    g_print(_("Time for start guess: %d seconds\n"), (int) (t1 - t0));
 
     it = 1;
 
@@ -683,13 +683,13 @@ main(int argc, char **argv)
 
         time(&t0);
 
-        printf(_("*** Iteration %03d *** \n"), it);
+        g_print(_("*** Iteration %03d *** \n"), it);
 
         CalcNewEquity(aheEquity, nC, arNorm);
 
         rNorm = NormOO(arNorm, 10);
 
-        printf(_("norm of delta: %f\n"), rNorm);
+        g_print(_("norm of delta: %f\n"), rNorm);
 
         if (fCheckPoint) {
 
@@ -702,7 +702,7 @@ main(int argc, char **argv)
         }
 
         time(&t1);
-        printf(_("Time for iteration %03d: %d seconds\n"), it, (int) (t1 - t0));
+        g_print(_("Time for iteration %03d: %d seconds\n"), it, (int) (t1 - t0));
 
         ++it;
 
@@ -714,14 +714,14 @@ main(int argc, char **argv)
 
     time(&t1);
 
-    printf(_("Time for writing final file: %d seconds\n"), (int) (t1 - t0));
+    g_print(_("Time for writing final file: %d seconds\n"), (int) (t1 - t0));
 
-    free(aheEquity);
+    g_free(aheEquity);
     g_free(szOutput);
 
     time(&t3);
 
-    printf(_("Total time: %d seconds\n"), (int) (t3 - t2));
+    g_print(_("Total time: %d seconds\n"), (int) (t3 - t2));
 
     return 0;
 
