@@ -1,14 +1,11 @@
 /*
- * matchequity.c
- * by Joern Thyssen, 1999-2002
+ * Copyright (C) 1999-2004 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2000-2016 the AUTHORS
  *
- * Calculate matchequity table based on 
- * formulas from [insert the Managent Science ref here].
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: matchequity.c,v 1.98 2016/05/11 21:33:08 plm Exp $
+ * $Id: matchequity.c,v 1.108 2022/03/12 20:39:35 plm Exp $
  */
 
 #include "config.h"
@@ -43,22 +39,18 @@
 #include "backgammon.h"
 
 
-typedef struct _parameter {
-
+typedef struct {
     char *szName;
     float value;
-
 } parameter;
 
 
-typedef struct _metparameters {
-
+typedef struct {
     char *szName;
     listOLD lParameters;
-
 } metparameters;
 
-typedef struct _metdata {
+typedef struct {
 
     /* Data saved */
 
@@ -478,7 +470,7 @@ GetPoints(float arOutput[5], const cubeinfo * pci, float arCP[2])
         nDead *= 2;
     }
 
-    for (nCubeValue = nDead, n = nMax; nCubeValue >= nCube; nCubeValue >>= 1, n--) {
+    for (nCubeValue = nDead, n = nMax; n >= 0; nCubeValue >>= 1, n--) {
 
         /* Calculate dead and live cube cash points.
          * See notes by me (Joern Thyssen) available from the
@@ -691,12 +683,11 @@ ExtendMET(float aarMET[MAXSCORE][MAXSCORE], const int nMaxScore)
 
     float rStddev0, rStddev1, rGames, rSigma;
     int i, j;
-    int nScore0, nScore1;
 
     /* Extend match equity table */
     for (i = nMaxScore; i < MAXSCORE; i++) {
 
-        nScore0 = i + 1;
+        int nScore0 = i + 1;
 
         if (nScore0 > 10)
             rStddev0 = 1.77f;
@@ -705,9 +696,9 @@ ExtendMET(float aarMET[MAXSCORE][MAXSCORE], const int nMaxScore)
 
         for (j = 0; j <= i; j++) {
 
-            nScore1 = j + 1;
+            int nScore1 = j + 1;
 
-            rGames = (nScore0 + nScore1) / 2.0f;
+            rGames = (float)(nScore0 + nScore1) / 2.0f;
 
             if (nScore1 > 10)
                 rStddev1 = 1.77f;
@@ -716,7 +707,7 @@ ExtendMET(float aarMET[MAXSCORE][MAXSCORE], const int nMaxScore)
 
             rSigma = sqrtf(rStddev0 * rStddev0 + rStddev1 * rStddev1) * sqrtf(rGames);
 
-            if (6.0f * rSigma > nScore0 - nScore1)
+            if (6.0f * rSigma > (float) (nScore0 - nScore1))
                 aarMET[i][j] = NormalDistArea((float) (nScore0 - nScore1), 6.0f * rSigma, 0.0f, rSigma);
             else
                 aarMET[i][j] = 0.0f;
@@ -815,7 +806,7 @@ getDefaultMET(metdata * pmd)
     initMD(pmd);
 
     for (i = 0; i < 4; i++) {
-        pp = (parameter *) malloc(sizeof(parameter));
+        pp = (parameter *) g_malloc(sizeof(parameter));
         pp->szName = g_strdup(apPreCrawford[i].szName);
         pp->value = apPreCrawford[i].value;
         ListInsert(&pmd->mpPreCrawford.lParameters, pp);
@@ -823,7 +814,7 @@ getDefaultMET(metdata * pmd)
 
     for (j = 0; j < 2; j++) {
         for (i = 0; i < 3; i++) {
-            pp = (parameter *) malloc(sizeof(parameter));
+            pp = (parameter *) g_malloc(sizeof(parameter));
             pp->szName = g_strdup(apPostCrawford[i].szName);
             pp->value = apPostCrawford[i].value;
             ListInsert(&pmd->ampPostCrawford[j].lParameters, pp);
@@ -1029,7 +1020,7 @@ typedef enum {
     MP_PLAYER_BOTH = 2
 } metplayer;
 
-typedef struct _MatchEquityParser {
+typedef struct {
     parserstate state;
     tabletype type;
     metplayer player;
@@ -1089,20 +1080,20 @@ readMET(metdata * met, const char *filename)
     parser->j = 0;
 
     if (!(context = g_markup_parse_context_new(&markup_parser, (GMarkupParseFlags) 0, parser, met_parser_destroy))) {
-        g_warning("Can't create XML parser\n");
+        g_warning(_("Can't create XML parser\n"));
         g_free(parser);
         return -1;
     }
 
     /* Read the file */
     if (!g_file_get_contents(filename, &contents, &size, &error)) {
-        g_warning("Error reading XML file: %s\n", error->message);
+        g_warning(_("Error reading XML file: %s\n"), error->message);
         goto err;
     }
 
     /* Parse the content */
     if (!g_markup_parse_context_parse(context, contents, size, &error)) {
-        g_warning("Error parsing XML file: %s\n", error->message);
+        g_warning(_("Error parsing XML file: %s\n"), error->message);
         goto err;
     }
 
@@ -1170,7 +1161,7 @@ SetNameFromType(char **pName, tabletype type)
         *pName = g_strdup("zadeh");
         break;
     default:
-        g_error("Invalid met table type");
+        g_error(_("Invalid MET table type"));
     }
 }
 
@@ -1380,7 +1371,7 @@ static void
 met_parser_error(GMarkupParseContext * UNUSED(context), GError * UNUSED(error), gpointer user_data)
 {
     MatchEquityParser *parser = (MatchEquityParser *) user_data;
-    g_warning("An error occurred while parsing file: %s\n", parser->filename);
+    g_warning(_("An error occurred while parsing file: %s\n"), parser->filename);
 }
 
 /*
@@ -1696,20 +1687,19 @@ invertMET(void)
 {
 
     int i, j;
-    float r;
 
     for (i = 0; i < MAXSCORE; i++) {
 
+        /* post crawford entries */
+
+        float r = aafMETPostCrawford[0][i];
+        aafMETPostCrawford[0][i] = aafMETPostCrawford[1][i];
+        aafMETPostCrawford[1][i] = r;
+
         /* diagonal */
 
         aafMET[i][i] = 1.0f - aafMET[i][i];
 
-        /* post crawford entries */
-
-        r = aafMETPostCrawford[0][i];
-        aafMETPostCrawford[0][i] = aafMETPostCrawford[1][i];
-        aafMETPostCrawford[1][i] = r;
-
         /* off diagonal entries */
 
         for (j = 0; j < i; j++) {
@@ -1750,7 +1740,7 @@ getMEMultiple(const int nScore0, const int nScore1, const int nMatchTo,
 {
 
     int scores[2][DTLBP1 + 1];  /* the resulting match scores */
-    int i, max_res, s0, s1;
+    int i, max_res;
     int *score0, *score1;
     int mult[] = { 1, 2, 3, 4, 6 };
     float *p0, *p1, f;
@@ -1808,8 +1798,8 @@ getMEMultiple(const int nScore0, const int nScore1, const int nMatchTo,
 
     /* now go through the resulting scores, looking up the equities */
     for (i = 0; i < max_res; ++i) {
-        s0 = *score0++;
-        s1 = *score1++;
+        int s0 = *score0++;
+        int s1 = *score1++;
 
         if (unlikely(s0 < 0)) {
             /* player 0 wins */
