@@ -1,15 +1,11 @@
-
 /*
- * osr.c
+ * Copyright (C) 2002-2004 Joern Thyssen <jthyssen@dk.ibm.com>
+ * Copyright (C) 2004-2015 the AUTHORS
  *
- * one-sided race rollouts
- *
- * by Joern Thyssen <jthyssen@dk.ibm.com>, 2002.
- * (after inspiration from osr.cc from fibs2html <fibs2html.sourceforge.net>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,10 +13,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: osr.c,v 1.40 2018/04/23 21:49:59 plm Exp $
+ *
+ * Inspired from osr.cc from fibs2html <fibs2html.sourceforge.net>
+ *
+ * $Id: osr.c,v 1.45 2022/10/22 18:27:59 plm Exp $
  */
 
 /*lint -e514 */
@@ -41,22 +39,18 @@
 static unsigned long mt[MT_ARRAY_N];
 static int mti = MT_ARRAY_N + 1;
 
-static unsigned int
-OSRQuasiRandomDice(const unsigned int iTurn, const unsigned int iGame, const unsigned int cGames,
-                   unsigned int anDice[2])
+static void
+OSRQuasiRandomDice(const unsigned int iTurn, const unsigned int iGame, const unsigned int cGames, unsigned int anDice[2])
 {
     if (!iTurn && !(cGames % 36)) {
         anDice[0] = (iGame % 6) + 1;
         anDice[1] = ((iGame / 6) % 6) + 1;
-        return TRUE;
     } else if (iTurn == 1 && !(cGames % 1296)) {
         anDice[0] = ((iGame / 36) % 6) + 1;
         anDice[1] = ((iGame / 216) % 6) + 1;
-        return TRUE;
     } else {
-        anDice[0] = (genrand_int32(&mti, mt) % 6) + 1;
-        anDice[1] = (genrand_int32(&mti, mt) % 6) + 1;
-        return (anDice[0] > 0 && anDice[1] > 0);
+        anDice[0] = (unsigned int) (genrand_int32(&mti, mt) % 6) + 1;
+        anDice[1] = (unsigned int) (genrand_int32(&mti, mt) % 6) + 1;
     }
 }
 
@@ -66,9 +60,8 @@ OSRQuasiRandomDice(const unsigned int iTurn, const unsigned int iGame, const uns
 static void
 getBearoffProbs(const unsigned int n, unsigned short int aaProb[32])
 {
-    g_assert(pbc1);
     if (BearoffDist(pbc1, n, NULL, NULL, NULL, aaProb, NULL))
-        puts("BearoffDist failed?");
+        g_assert_not_reached();
 }
 
 
@@ -338,9 +331,7 @@ static void
 FindBestMoveOSR4(unsigned int anBoard[25], const unsigned int nDice, unsigned int *pnOut)
 {
     unsigned int nd = 4;
-    unsigned int i, j, n = 0;
-    unsigned int first, any;
-    unsigned int lc;
+    unsigned int i, n = 0;
 
     /* check for exact bear-ins */
 
@@ -384,6 +375,9 @@ FindBestMoveOSR4(unsigned int anBoard[25], const unsigned int nDice, unsigned in
 #endif
 
     if (*pnOut > 0 && nd > 0) {
+        unsigned int first;
+        unsigned int lc;
+
         first = TRUE;
 
         /* find rearest chequer */
@@ -457,6 +451,8 @@ FindBestMoveOSR4(unsigned int anBoard[25], const unsigned int nDice, unsigned in
 
     if (!*pnOut) {              /* all chequers inside home quadrant */
         while (nd) {
+            unsigned int any;
+
             if (anBoard[nDice - 1]) {
                 /* perfect bear-off */
                 --anBoard[nDice - 1];
@@ -497,7 +493,7 @@ FindBestMoveOSR4(unsigned int anBoard[25], const unsigned int nDice, unsigned in
             /* FIXME: fill gaps? */
 
             for (i = 0; nd && i < 6; i++) {
-                j = 5 - i;
+                unsigned int j = 5 - i;
 
                 while (anBoard[j] && nd) {
 
@@ -568,8 +564,7 @@ osr(unsigned int anBoard[25], const unsigned int iGame, const unsigned int nGame
 
     while (nOut) {
         /* roll dice */
-        if (OSRQuasiRandomDice(iTurn, iGame, nGames, anDice) == FALSE)
-            g_warning("Error in function OSRQuasiRandomDice");
+        OSRQuasiRandomDice(iTurn, iGame, nGames, anDice);
 
         if (anDice[0] < anDice[1])
             swap_us(anDice, anDice + 1);
@@ -606,7 +601,6 @@ rollOSR(const unsigned int nGames, const unsigned int anBoard[25], const unsigne
     unsigned int an[25];
     unsigned short int anProb[32];
     unsigned int i;
-    unsigned int n, m;
     unsigned int iGame;
 
     int *anCounts = (int *) g_alloca(nMaxGammonProbs * sizeof(int));
@@ -619,6 +613,7 @@ rollOSR(const unsigned int nGames, const unsigned int anBoard[25], const unsigne
     /* perform rollouts */
 
     for (iGame = 0; iGame < nGames; ++iGame) {
+        unsigned int n, m;
 
         memcpy(an, anBoard, sizeof(an));
 
@@ -648,7 +643,7 @@ rollOSR(const unsigned int nGames, const unsigned int anBoard[25], const unsigne
     /* scale resulting probabilities */
 
     for (i = 0; i < (unsigned int) nMaxProbs; ++i) {
-        arProbs[i] /= nGames;
+        arProbs[i] /= (float)nGames;
         /* printf ( "arProbs[%d]=%f\n", i, arProbs[ i ] ); */
     }
 
@@ -656,7 +651,7 @@ rollOSR(const unsigned int nGames, const unsigned int anBoard[25], const unsigne
      * (prob. of getting inside home quadrant in i rolls */
 
     for (i = 0; i < nMaxGammonProbs; ++i) {
-        arGammonProbs[i] = 1.0f * anCounts[i] / nGames;
+        arGammonProbs[i] = (float)anCounts[i] / (float)nGames;
         /* printf ( "arGammonProbs[%d]=%f\n", i, arGammonProbs[ i ] ); */
     }
 
@@ -684,9 +679,8 @@ osp(const unsigned int anBoard[25], const unsigned int nGames,
     unsigned int an[25], float arProbs[MAX_PROBS], float arGammonProbs[MAX_GAMMON_PROBS])
 {
 
-    int i, n;
+    int i;
     unsigned int nTotal, nOut;
-    unsigned short int anProb[32];
 
     /* copy board into an, and find total number of chequers left,
      * and number of chequers outside home */
@@ -709,6 +703,8 @@ osp(const unsigned int anBoard[25], const unsigned int nGames,
     else {
         /* chequers inside home: use BEAROFF2 */
 
+        unsigned short int anProb[32];
+
         /* no gammon possible */
         for (i = 0; i < MAX_GAMMON_PROBS; ++i)
             arGammonProbs[i] = 0.0f;
@@ -726,7 +722,7 @@ osp(const unsigned int anBoard[25], const unsigned int nGames,
         getBearoffProbs(PositionBearoff(anBoard, pbc1->nPoints, pbc1->nChequers), anProb);
 
         for (i = 0; i < 32; ++i) {
-            n = MIN(i, MAX_PROBS - 1);
+            int n = MIN(i, MAX_PROBS - 1);
             arProbs[n] += anProb[i] / 65535.0f;
             /* printf ( "arProbs[%d]=%f\n", n, arProbs[n] ); */
         }
@@ -744,9 +740,8 @@ bgProb(const unsigned int anBoard[25],
 {
 
     unsigned int nTotPipsHome = 0;
-    unsigned int i, j;
-    float r, s;
-    unsigned short int anProb[32];
+    unsigned int i;
+    float r;
 
     /* total pips before out of opponent's home quadrant */
 
@@ -759,11 +754,13 @@ bgProb(const unsigned int anBoard[25],
 
         /* ( nTotal + 3 ) / 4 - 1: number of rolls before opponent is off. */
         /* (nTotPipsHome + 2) / 3: numbers of rolls before I'm out of
-         * opponent's home quadrant (with consequtive 2-1's) */
+         * opponent's home quadrant (with consecutive 2-1's) */
 
         if ((nTotal + 3) / 4 - 1 <= (nTotPipsHome + 2) / 3) {
             /* backgammon is possible */
 
+            unsigned short int anProb[32];
+
             /* get "bear-off" prob (for getting out of opp.'s home quadr.) */
 
             /* FIXME: this ignores chequers on the bar */
@@ -774,7 +771,8 @@ bgProb(const unsigned int anBoard[25],
 
                 if (arProbs[i] > 0.0f) {
 
-                    s = 0.0f;
+                    float s = 0.0f;
+                    unsigned int j;
 
                     for (j = i + !fOnRoll; j < 32; ++j)
                         s += anProb[j] / 65535.0f;
@@ -814,7 +812,7 @@ raceProbs(const TanBoard anBoard, const unsigned int nGames, float arOutput[NUM_
     TanBoard an;
     float aarProbs[2][MAX_PROBS];
     float aarGammonProbs[2][MAX_PROBS];
-    float arG[2], arBG[2];
+    float arG[2] = { 0.0f, 0.0f }, arBG[2] = { 0.0f, 0.0f };
 
     unsigned int anTotal[2];
 
@@ -826,7 +824,7 @@ raceProbs(const TanBoard anBoard, const unsigned int nGames, float arOutput[NUM_
 
     init_genrand(0, &mti, mt);
 
-    for (i = 0; i < 5; ++i)
+    for (i = 0; i < NUM_OUTPUTS; ++i)
         arOutput[i] = 0.0f;
 
     for (i = 0; i < 2; ++i)
@@ -858,9 +856,6 @@ raceProbs(const TanBoard anBoard, const unsigned int nGames, float arOutput[NUM_
 
     for (i = 0; i < 2; ++i) {
 
-        arG[i] = 0.0f;
-        arBG[i] = 0.0f;
-
         if (anTotal[!i] == 15) {
 
             /* gammon and backgammon possible */
@@ -901,7 +896,7 @@ raceProbs(const TanBoard anBoard, const unsigned int nGames, float arOutput[NUM_
         for (i = 0; i < 2; ++i) {
             arMu[i] = 0.0f;
             for (j = 0; j < MAX_PROBS; ++j)
-                arMu[i] += 1.0f * j * aarProbs[i][j];
+                arMu[i] +=  (float)j * aarProbs[i][j];
 
         }
 
