@@ -1,11 +1,11 @@
 /*
- * output.h
+ * Copyright (C) 1999-2002 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2015-2022 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 1999, 2000, 2001, 2002.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,15 +13,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: output.h,v 1.3 2015/10/08 00:06:41 mdpetch Exp $
+ * $Id: output.h,v 1.7 2022/05/22 21:35:51 plm Exp $
  */
 
 #ifndef OUTPUT_H
 #define OUTPUT_H
+
 #include <stdarg.h>
+#include <glib.h>
 #include "common.h"
 
 /* Initialize output module */
@@ -32,20 +33,33 @@ extern void output(const char *sz);
 extern void outputl(const char *sz);
 /* Write a character to stdout/status bar/popup window */
 extern void outputc(const char ch);
+/* Write an error message, perror() style */
+extern void outputerr(const char *sz);
+
+#if defined(HAVE_FUNC_ATTRIBUTE_FORMAT)
 /* Write a string to stdout/status bar/popup window, printf style */
 extern void outputf(const char *sz, ...)
     __attribute__ ((format(printf, 1, 2)));
 /* Write a string to stdout/status bar/popup window, vprintf style */
 extern void outputv(const char *sz, va_list val)
     __attribute__ ((format(printf, 1, 0)));
-/* Write an error message, perror() style */
-extern void outputerr(const char *sz);
 /* Write an error message, fprintf() style */
 extern void outputerrf(const char *sz, ...)
     __attribute__ ((format(printf, 1, 2)));
 /* Write an error message, vfprintf() style */
 extern void outputerrv(const char *sz, va_list val)
     __attribute__ ((format(printf, 1, 0)));
+#else
+/* Write a string to stdout/status bar/popup window, printf style */
+extern void outputf(const char *sz, ...);
+/* Write a string to stdout/status bar/popup window, vprintf style */
+extern void outputv(const char *sz, va_list val);
+/* Write an error message, fprintf() style */
+extern void outputerrf(const char *sz, ...);
+/* Write an error message, vfprintf() style */
+extern void outputerrv(const char *sz, va_list val);
+#endif
+
 /* Signifies that all output for the current command is complete */
 extern void outputx(void);
 /* Temporarily disable outputx() calls */
@@ -63,4 +77,13 @@ extern int cOutputDisabled;
 extern int cOutputPostponed;
 extern int foutput_on;
 
+#if defined(USE_PYTHON)
+/* output-family functions allocate with g_strdup_printf(), caller g_free()s */
+extern char* szMemOutput;
+
+extern int foutput_to_mem;
+#endif
+
+extern void print_utf8_to_locale(const gchar *sz);
+
 #endif
