@@ -1,16 +1,29 @@
 /*
- * positionid.c
+ * Copyright (C) 1998-2002 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2000-2014 the AUTHORS
  *
- * by Gary Wong <gary@cs.arizona.edu>, 1998-1999.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * An implementation of the position "external" key/IDs at:
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *   http://www.gnu.org/manual/gnubg/html_node/A-technical-description-of-the-Position-ID.html
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * Please see that page for more information.
  *
- * For internal use, we use another "key", much faster to encode or decode
- * that is essentially the raw board squeezed down to 4 bits/point.
+ * An implementation of the legacy position key/ID described at:
+ *   https://www.gnu.org/software/gnubg/manual/gnubg.html#gnubg-tech_postionid
+ *
+ * This encoding creates IDs as compact as possible.
+ *
+ * For internal use in moves generation and evaluation we now use another
+ * key, much faster to encode or decode, that is essentially the raw board
+ * squeezed down to 4 bits/point.
  *
  * This library also calculates bearoff IDs, which are enumerations of the
  *
@@ -21,20 +34,7 @@
  * combinations of (up to c) chequers among 6 points.
  *
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * $Id: positionid.c,v 1.54 2014/10/07 07:14:28 plm Exp $
+ * $Id: positionid.c,v 1.57 2021/12/18 22:06:37 plm Exp $
  */
 
 #include "config.h"
@@ -198,11 +198,10 @@ oldPositionFromKey(TanBoard anBoard, const oldpositionkey * pkey)
 static char *
 oldPositionIDFromKey(const oldpositionkey * pkey)
 {
-
     unsigned char const *puch = pkey->auch;
     static char szID[L_POSITIONID + 1];
     char *pch = szID;
-    static char aszBase64[65] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+    static const char aszBase64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
     int i;
 
     for (i = 0; i < 3; i++) {
@@ -225,7 +224,6 @@ oldPositionIDFromKey(const oldpositionkey * pkey)
 extern char *
 PositionIDFromKey(const positionkey * pkey)
 {
-
     TanBoard anBoard;
     oldpositionkey okey;
 
@@ -238,7 +236,6 @@ PositionIDFromKey(const positionkey * pkey)
 extern char *
 PositionID(const TanBoard anBoard)
 {
-
     oldpositionkey key;
 
     oldPositionKey(anBoard, &key);
@@ -268,9 +265,9 @@ PositionFromXG(TanBoard anBoard, const char *pos)
         if (pos[i] >= 'A' && pos[i] <= 'P') {
             if (p0 > -1)
                 anBoard[0][p0] = 0;
-            anBoard[1][p1] = pos[i] - 'A' + 1;
+            anBoard[1][p1] = (unsigned int) (pos[i] - 'A' + 1);
         } else if (pos[i] >= 'a' && pos[i] <= 'p') {
-            anBoard[0][p0] = pos[i] - 'a' + 1;
+            anBoard[0][p0] = (unsigned int) (pos[i] - 'a' + 1);
             if (p1 > -1)
                 anBoard[1][p1] = 0;
         } else if (pos[i] == '-') {
@@ -354,13 +351,13 @@ extern unsigned char
 Base64(const unsigned char ch)
 {
     if (ch >= 'A' && ch <= 'Z')
-        return ch - 'A';
+        return (unsigned char)(ch - 'A');
 
     if (ch >= 'a' && ch <= 'z')
-        return (ch - 'a') + 26;
+        return (unsigned char)(ch - 'a' + 26);
 
     if (ch >= '0' && ch <= '9')
-        return (ch - '0') + 52;
+        return (unsigned char)(ch - '0' + 52);
 
     if (ch == '+')
         return 62;
@@ -460,6 +457,11 @@ PositionBearoff(const unsigned int anBoard[], unsigned int nPoints, unsigned int
 {
     unsigned int i, fBits, j;
 
+    if (nPoints == 0) {
+        g_assert_not_reached();
+        return 0;
+    }
+
     for (j = nPoints - 1, i = 0; i < nPoints; i++)
         j += anBoard[i];
 
@@ -512,7 +514,14 @@ extern unsigned short
 PositionIndex(unsigned int g, const unsigned int anBoard[6])
 {
     unsigned int i, fBits;
-    unsigned int j = g - 1;
+    unsigned int j;
+
+    if (g == 0) {
+        g_assert_not_reached();
+        return 0;
+    }
+
+    j = g - 1;
 
     for (i = 0; i < g; i++)
         j += anBoard[i];
