@@ -1,11 +1,11 @@
 /*
- * relational.c
+ * Copyright (C) 2004 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2004-2021 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2004.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: relational.c,v 1.73 2015/01/11 16:10:21 plm Exp $
+ * $Id: relational.c,v 1.84 2021/03/10 20:44:47 plm Exp $
  */
 
 #include "config.h"
@@ -51,15 +50,18 @@ RelationalMatchExists(DBProvider * pdb)
 static int
 GameOver(void)
 {
-    int anFinalScore[2];
-    int nMatch;
     const listOLD *firstGame = lMatch.plNext->p;
+
     if (firstGame) {
         const moverecord *pmr = firstGame->plNext->p;
         if (pmr) {
+            int nMatch;
+
             g_assert(pmr->mt == MOVE_GAMEINFO);
             nMatch = pmr->g.nMatch;
             if (ms.nMatchTo) {  /* Match - check someone has won */
+                int anFinalScore[2];
+
                 return (getFinalScore(anFinalScore) && ((anFinalScore[0] >= nMatch)
                                                         || (anFinalScore[1] >= nMatch)));
             } else {            /* Session - check game over */
@@ -126,9 +128,9 @@ MatchResult(int nMatchTo)
     int result = 0;
     int anFinalScore[2];
     if (nMatchTo && getFinalScore(anFinalScore)) {
-        if (anFinalScore[0] > nMatchTo)
+        if (anFinalScore[0] >= nMatchTo)
             result = -1;
-        else if (anFinalScore[1] > nMatchTo)
+        else if (anFinalScore[1] >= nMatchTo)
             result = 1;
     }
     return result;
@@ -148,6 +150,8 @@ Ratio(float a, int b)
 	g_string_append_printf(value, "'%s', ", g_ascii_dtostr(tmpf, G_ASCII_DTOSTR_BUF_SIZE, y));}
 #define APPENDI(x,y) {g_string_append_printf(column, "%s, ", x); \
 	g_string_append_printf(value, "'%i', ", y);}
+#define APPENDU(x,y) {g_string_append_printf(column, "%s, ", x); \
+        g_string_append_printf(value, "'%u', ", y);}
 
 static int
 AddStats(DBProvider * pdb, int gm_id, int player_id, int player, const char *table, int nMatchTo, statcontext * sc)
@@ -196,7 +200,7 @@ AddStats(DBProvider * pdb, int gm_id, int player_id, int player, const char *tab
     APPENDF("chequer_error_total", sc->arErrorCheckerplay[player][1]);
     APPENDF("chequer_error_per_move_normalised", Ratio(sc->arErrorCheckerplay[player][0], unforced));
     APPENDF("chequer_error_per_move", Ratio(sc->arErrorCheckerplay[player][1], unforced));
-    APPENDI("chequer_rating", GetRating(Ratio(scMatch.arErrorCheckerplay[player][0], unforced)));
+    APPENDU("chequer_rating", GetRating(Ratio(scMatch.arErrorCheckerplay[player][0], unforced)));
     APPENDI("very_lucky_rolls", sc->anLuck[player][LUCK_VERYGOOD]);
     APPENDI("lucky_rolls", sc->anLuck[player][LUCK_GOOD]);
     APPENDI("unmarked_rolls", sc->anLuck[player][LUCK_NONE]);
@@ -206,7 +210,7 @@ AddStats(DBProvider * pdb, int gm_id, int player_id, int player, const char *tab
     APPENDF("luck_total", sc->arLuck[player][1]);
     APPENDF("luck_per_move_normalised", Ratio(sc->arLuck[player][0], totalmoves));
     APPENDF("luck_per_move", Ratio(sc->arLuck[player][1], totalmoves));
-    APPENDI("luck_rating", getLuckRating(Ratio(sc->arLuck[player][0], totalmoves)));
+    APPENDU("luck_rating", getLuckRating(Ratio(sc->arLuck[player][0], totalmoves)));
     APPENDI("total_cube_decisions", sc->anTotalCube[player]);
     APPENDI("close_cube_decisions", sc->anCloseCube[player]);
     APPENDI("doubles", sc->anDouble[player]);
@@ -230,28 +234,28 @@ AddStats(DBProvider * pdb, int gm_id, int player_id, int player, const char *tab
     APPENDF("error_wrong_doubles_above_tg", sc->arErrorWrongDoubleTG[player][1]);
     APPENDF("error_wrong_takes", sc->arErrorWrongTake[player][1]);
     APPENDF("error_wrong_passes", sc->arErrorWrongPass[player][1]);
-    APPENDF("cube_error_total_normalised", errorskill * sc->anCloseCube[player]);
-    APPENDF("cube_error_total", errorcost * sc->anCloseCube[player]);
+    APPENDF("cube_error_total_normalised", errorskill * (float) sc->anCloseCube[player]);
+    APPENDF("cube_error_total", errorcost * (float) sc->anCloseCube[player]);
     APPENDF("cube_error_per_move_normalised", errorskill);
     APPENDF("cube_error_per_move", errorcost);
-    APPENDI("cube_rating", GetRating(errorskill));;
-    APPENDF("overall_error_total_normalised", errorskill * sc->anCloseCube[player] + sc->arErrorCheckerplay[player][0]);
-    APPENDF("overall_error_total", errorcost * sc->anCloseCube[player] + sc->arErrorCheckerplay[player][1]);
+    APPENDU("cube_rating", GetRating(errorskill));
+    APPENDF("overall_error_total_normalised", errorskill * (float) sc->anCloseCube[player] + sc->arErrorCheckerplay[player][0]);
+    APPENDF("overall_error_total", errorcost * (float) sc->anCloseCube[player] + sc->arErrorCheckerplay[player][1]);
     APPENDF("overall_error_per_move_normalised",
-            Ratio(errorskill * sc->anCloseCube[player] +
+            Ratio(errorskill * (float) sc->anCloseCube[player] +
                   sc->arErrorCheckerplay[player][0], sc->anCloseCube[player] + unforced));
     APPENDF("overall_error_per_move",
-            Ratio(errorcost * sc->anCloseCube[player] +
+            Ratio(errorcost * (float) sc->anCloseCube[player] +
                   sc->arErrorCheckerplay[player][1], sc->anCloseCube[player] + unforced));
-    APPENDI("overall_rating",
+    APPENDU("overall_rating",
             GetRating(Ratio
-                      (errorskill * sc->anCloseCube[player] +
+                      (errorskill * (float) sc->anCloseCube[player] +
                        sc->arErrorCheckerplay[player][0], sc->anCloseCube[player] + unforced)));
     APPENDF("actual_result", sc->arActualResult[player]);
     APPENDF("luck_adjusted_result", sc->arLuckAdj[player]);
     APPENDI("snowie_moves", totalmoves + scMatch.anTotalMoves[!player]);
     APPENDF("snowie_error_rate_per_move",
-            Ratio(errorskill * scMatch.anCloseCube[player] +
+            Ratio(errorskill * (float) scMatch.anCloseCube[player] +
                   scMatch.arErrorCheckerplay[player][0], totalmoves + scMatch.anTotalMoves[!player]));
     /*time */
     APPENDI("time_penalties", 0);
@@ -278,10 +282,10 @@ AddStats(DBProvider * pdb, int gm_id, int player_id, int player, const char *tab
     /* for money sessions only */
     if (scMatch.fDice && !nMatchTo && scMatch.nGames > 1) {
 
-        APPENDF("actual_advantage", scMatch.arActualResult[player] / scMatch.nGames);
-        APPENDF("actual_advantage_ci", 1.95996f * sqrt(scMatch.arVarianceActual[player] / scMatch.nGames));
-        APPENDF("luck_adjusted_advantage", scMatch.arLuckAdj[player] / scMatch.nGames);
-        APPENDF("luck_adjusted_advantage_ci", 1.95996f * sqrt(scMatch.arVarianceLuckAdj[player] / scMatch.nGames));
+        APPENDF("actual_advantage", scMatch.arActualResult[player] / (float) scMatch.nGames);
+        APPENDF("actual_advantage_ci", 1.95996f * sqrtf(scMatch.arVarianceActual[player] / (float) scMatch.nGames));
+        APPENDF("luck_adjusted_advantage", scMatch.arLuckAdj[player] / (float) scMatch.nGames);
+        APPENDF("luck_adjusted_advantage_ci", 1.95996f * sqrtf(scMatch.arVarianceLuckAdj[player] / (float) scMatch.nGames));
     }
 
     g_string_truncate(column, column->len - 2);
@@ -375,17 +379,32 @@ static void
 AddGames(DBProvider * pdb, int session_id, int player_id0, int player_id1)
 {
     int gamenum = 0;
-    listOLD *plGame, *pl = lMatch.plNext;
-    while ((plGame = pl->p) != NULL) {
+    listOLD *plg, *pl = lMatch.plNext;
+    while ((plg = pl->p) != NULL) {
         int game_id = GetNextId(pdb, "game");
-        moverecord *pmr = plGame->plNext->p;
+        int result = 0;
+        moverecord *pmr = plg->plNext->p;
         xmovegameinfo *pmgi = &pmr->g;
+        char *buf;
 
-        char *buf = g_strdup_printf("INSERT INTO game(game_id, session_id, player_id0, player_id1, "
+        switch(pmgi->fWinner) {
+            case 0:
+                result = pmgi->nPoints;
+                break;
+            case 1:
+                result = -pmgi->nPoints;
+                break;
+            case -1:
+                break;
+            default:
+                g_assert_not_reached();
+        }
+
+        buf = g_strdup_printf("INSERT INTO game(game_id, session_id, player_id0, player_id1, "
                                     "score_0, score_1, result, added, game_number, crawford) "
                                     "VALUES (%d, %d, %d, %d, %d, %d, %d, CURRENT_TIMESTAMP, %d, %d )",
                                     game_id, session_id, player_id0, player_id1,
-                                    pmgi->anScore[0], pmgi->anScore[1], pmgi->nPoints, ++gamenum, pmr->g.fCrawfordGame);
+                                    pmgi->anScore[0], pmgi->anScore[1], result, ++gamenum, pmr->g.fCrawfordGame);
 
         if (pdb->UpdateCommand(buf)) {
             AddStats(pdb, game_id, player_id0, 0, "gamestat", ms.nMatchTo, &(pmgi->sc));
@@ -400,7 +419,7 @@ extern void
 CommandRelationalAddMatch(char *sz)
 {
     DBProvider *pdb;
-    char *buf, *buf2, *date;
+    char *buf, *date;
     char warnings[1024] = "";
     int session_id, existing_id, player_id0, player_id1;
     char *arg = NULL;
@@ -433,6 +452,8 @@ CommandRelationalAddMatch(char *sz)
     }
     existing_id = RelationalMatchExists(pdb);
     if (existing_id != -1) {
+        char *buf2;
+
         if (!quiet && !GetInputYN(_("Match exists, overwrite?")))
             return;
 
@@ -464,7 +485,7 @@ CommandRelationalAddMatch(char *sz)
     }
 
     if (mi.nYear)
-        date = g_strdup_printf("%04d-%02d-%02d", mi.nYear, mi.nMonth, mi.nDay);
+        date = g_strdup_printf("%04u-%02u-%02u", mi.nYear, mi.nMonth, mi.nDay);
     else
         date = NULL;
 
@@ -530,8 +551,6 @@ relational_player_stats_get(const char *player0, const char *player1)
     DBProvider *pdb = NULL;
     char *query[2];
     int i;
-    char *buf;
-    RowSet *rs;
     statcontext *psc;
 
     g_return_val_if_fail(player0, NULL);
@@ -569,7 +588,7 @@ relational_player_stats_get(const char *player0, const char *player1)
 
     IniStatcontext(psc);
     for (i = 0; i < 2; ++i) {
-        buf = g_strdup_printf("SUM(total_moves),"
+        char *buf = g_strdup_printf("SUM(total_moves),"
                               "SUM(unforced_moves),"
                               "SUM(total_cube_decisions),"
                               "SUM(close_cube_decisions),"
@@ -599,8 +618,9 @@ relational_player_stats_get(const char *player0, const char *player1)
                               "SUM(error_wrong_takes_normalised),"
                               "SUM(error_wrong_passes_normalised),"
                               "SUM(luck_total_normalised)" "from matchstat " "%s", query[i]);
-        rs = pdb->Select(buf);
+        RowSet *rs = pdb->Select(buf);
         g_free(buf);
+
         if ((!rs) || !strtol(rs->data[1][0], NULL, 0))
             return NULL;
         psc->anTotalMoves[i] = (int) strtol(rs->data[1][0], NULL, 0);
@@ -646,7 +666,7 @@ relational_player_stats_get(const char *player0, const char *player1)
 extern void
 CommandRelationalShowDetails(char *sz)
 {
-    gchar output[4096];
+    gchar buffer[STATCONTEXT_MAXSIZE];
     statcontext *psc;
 
     gchar *player0 = NextToken(&sz);
@@ -665,16 +685,16 @@ CommandRelationalShowDetails(char *sz)
         return;
     }
 
-    DumpStatcontext(output, psc, player0, player1 ? player1 : _("Opponents"), 0);
+    DumpStatcontext(buffer, psc, player0, player1 ? player1 : _("Opponents"), 0);
     g_free(psc);
 #if USE_GTK
     if (fX) {
-        GTKTextWindow(output, _("Player statistics"), DT_INFO, NULL);
+        GTKTextWindow(buffer, _("Player statistics"), DT_INFO, NULL);
     } else
 #endif
     {
         outputl(_("Player statistics\n\n"));
-        outputl(output);
+        outputl(buffer);
     }
 }
 
@@ -760,11 +780,10 @@ CommandRelationalEraseAll(char *UNUSED(sz))
 extern void
 CommandRelationalSelect(char *sz)
 {
-    unsigned int i, j;
     RowSet *rs;
 
     if (!sz || !*sz) {
-        outputl(_("You must specify a sql query to run."));
+        outputl(_("You must specify a SQL query to run."));
         return;
     }
 
@@ -774,6 +793,7 @@ CommandRelationalSelect(char *sz)
 
     if (rs->rows == 0) {
         outputl(_("No rows found.\n"));
+        FreeRowset(rs);
         return;
     }
 #if USE_GTK
@@ -781,7 +801,7 @@ CommandRelationalSelect(char *sz)
         GtkShowQuery(rs);
     else
 #endif
-        for (i = 0; i < rs->rows; i++) {
+        for (unsigned int i = 0; i < rs->rows; i++) {
             if (i == 1) {       /* Underline headings */
                 char *line, *p;
                 unsigned int k;
@@ -791,7 +811,7 @@ CommandRelationalSelect(char *sz)
                     if (k != 0)
                         totalwidth += 2;
                 }
-                line = malloc(totalwidth + 1);
+                line = g_malloc(totalwidth + 1);
                 memset(line, '-', totalwidth);
                 p = line;
                 for (k = 0; k < rs->cols - 1; k++) {
@@ -801,10 +821,10 @@ CommandRelationalSelect(char *sz)
                 }
                 line[totalwidth] = '\0';
                 outputl(line);
-                free(line);
+                g_free(line);
             }
 
-            for (j = 0; j < rs->cols; j++) {
+            for (unsigned int j = 0; j < rs->cols; j++) {
                 if (j > 0)
                     output(" | ");
 
