@@ -1,11 +1,11 @@
 /*
- * relational.h
+ * Copyright (C) 2004 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2004-2009 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2004.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,19 +13,18 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: relational.h,v 1.15 2013/06/16 02:16:20 mdpetch Exp $
+ * $Id: relational.h,v 1.17 2019/12/24 09:02:43 plm Exp $
  */
 
 #ifndef RELATIONAL_H
 #define RELATIONAL_H
 
-#include <dbprovider.h>
 #include <stddef.h>
 #include <sys/types.h>
 #include "analysis.h"
+#include "dbprovider.h"
 
 #define DB_VERSION 1
 
