@@ -1,11 +1,11 @@
 /*
- * render.h
+ * Copyright (C) 1997-2002 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2003-2020 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 1997-2002.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: render.h,v 1.40 2017/04/02 20:19:19 plm Exp $
+ * $Id: render.h,v 1.45 2021/11/08 22:40:39 plm Exp $
  */
 
 #ifndef RENDER_H
@@ -25,25 +24,16 @@
 #include "gnubg-types.h"
 
 #if defined(USE_BOARD3D)
-#include "types3d.h"
-struct _Material {
-    float ambientColour[4];
-    float diffuseColour[4];
-    float specularColour[4];
-    int shine;
-    int alphaBlend;
-    TextureInfo *textureInfo;
-    Texture *pTexture;
-};
+#include "board3d/types3d.h"
 #endif
 
-typedef enum _woodtype {
+typedef enum {
     WOOD_ALDER, WOOD_ASH, WOOD_BASSWOOD, WOOD_BEECH, WOOD_CEDAR,
     WOOD_EBONY, WOOD_FIR, WOOD_MAPLE, WOOD_OAK, WOOD_PINE, WOOD_REDWOOD,
     WOOD_WALNUT, WOOD_WILLOW, WOOD_PAINT
 } woodtype;
 
-typedef struct _renderdata {
+typedef struct {
     woodtype wt;
     float aarColour[2][4];     /* RGBA for each player */
     float aarDiceColour[2][4];  /* RGB(A) of dice for each player */
@@ -76,14 +66,12 @@ typedef struct _renderdata {
 
     int animateRoll;            /* Animate dice rolls */
     int animateFlag;            /* Animate resignation flag */
-    int closeBoardOnExit;       /* Animate board close on quit */
-    int quickDraw;              /* Cut down 3d drawing to improve performance */
     unsigned int curveAccuracy; /* Round curve approximation accuracy */
     lighttype lightType;        /* Positional/Directional light source */
     float lightPos[3];          /* x,y,z pos of light source */
     int lightLevels[3];         /* amibient/diffuse/specular light levels */
-    int boardAngle;             /* Angle board is tilted at */
-    int skewFactor;             /* FOV adjustment */
+    float boardAngle;           /* Angle board is tilted at */
+    float skewFactor;           /* FOV adjustment */
     int planView;               /* Ortho view? */
     float diceSize;             /* How big are the dice */
     int roundedEdges;           /* Rounded board edges? */
@@ -104,7 +92,7 @@ typedef struct _renderdata {
 #endif
 } renderdata;
 
-typedef struct _renderimages {
+typedef struct {
     unsigned char *ach, *achChequer[2], *achChequerLabels, *achDice[2], *achPip[2], *achCube, *achCubeFaces;
     unsigned char *achResign;
     unsigned char *achResignFaces;
@@ -169,10 +157,18 @@ extern void CalculateArea(renderdata * prd, unsigned char *puch, int nStride,
                           renderimages * pri, TanBoard anBoard,
                           int anOff[2], const unsigned int anDice[2],
                           int anDicePosition[2][2],
-                          int fDiceColour, int anCubePosition[2],
+                          int fDiceColour, const int anCubePosition[2],
                           int nLogCube, int nCubeOrientation,
-                          int anResignPosition[2],
+                          const int anResignPosition[2],
                           int fResign, int fResignOrientation,
                           int anArrowPosition[2], int fPlaying, int nPlayer, int x, int y, int cx, int cy);
 
+#if defined(USE_BOARD3D)
+extern gboolean widget3dValid;
+extern gboolean display_is_2d(const renderdata* prd);
+extern gboolean display_is_3d(const renderdata* prd);
+extern void SuspendDiceRolling(renderdata* prd);
+extern void ResumeDiceRolling(renderdata* prd);
+#endif
+
 #endif
