@@ -1,11 +1,11 @@
 /*
- * renderprefs.h
+ * Copyright (C) 2000-2003 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2003-2009 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 2000, 2003
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: renderprefs.h,v 1.11 2013/06/16 02:16:20 mdpetch Exp $
+ * $Id: renderprefs.h,v 1.13 2019/11/03 15:25:19 plm Exp $
  */
 
 #ifndef RENDERPREFS_H
@@ -33,7 +32,7 @@ extern void CopyAppearance(renderdata * prd);
 extern void RenderPreferencesParam(renderdata * prd, const char *szParam, char *szValue);
 extern void SaveRenderingSettings(FILE * pf);
 
-#if USE_BOARD3D
+#if defined(USE_BOARD3D)
 char *WriteMaterial(Material * pMat);
 char *WriteMaterialDice(renderdata * prd, int num);
 #endif
