@@ -1,11 +1,11 @@
 /*
- * rollout.c
+ * Copyright (C) 1999-2003 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2000-2021 the AUTHORS
  *
- * by Gary Wong <gary@cs.arizona.edu>, 1999, 2000, 2001.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: rollout.c,v 1.255 2018/06/16 15:02:37 plm Exp $
+ * $Id: rollout.c,v 1.262 2021/11/21 20:16:11 plm Exp $
  */
 
 #include "config.h"
@@ -30,7 +29,6 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <time.h>
-#include "glib-ext.h"
 
 #include "backgammon.h"
 #if defined(USE_GTK)
@@ -144,7 +142,7 @@ log_game_start(const char *name, const cubeinfo * pci, int fCubeful, TanBoard an
         }
     }
 
-    if ((logfp = gnubg_g_fopen(name, "w")) == 0)
+    if ((logfp = g_fopen(name, "w")) == 0)
         return NULL;
 
     fprintf(logfp, "(;FF[4]GM[6]CA[UTF-8]AP[GNU Backgammon:%s]MI"
@@ -222,13 +220,12 @@ RolloutDice(int iTurn, int iGame,
     if (fInitial && !iTurn) {
         /* rollout of initial position: no doubles allowed */
         if (fRotate) {
-            unsigned int j;
 
             if (!iGame)
-                nSkip = 0;
+                MT_SafeSet(&nSkip, 0);
 
-            for (;; nSkip++) {
-                j = dicePerms->aaanPermutation[0][0][(iGame + nSkip) % 36];
+            for (;; MT_SafeInc(&nSkip)) {
+                unsigned int j = dicePerms->aaanPermutation[0][0][(iGame + MT_SafeGet(&nSkip)) % 36];
 
                 anDice[0] = j / 6 + 1;
                 anDice[1] = j % 6 + 1;
@@ -253,7 +250,7 @@ RolloutDice(int iTurn, int iGame,
          k;                     /* 36**i */
 
         for (i = 0, j = 0, k = 1; i < 6 && i <= (unsigned int) iTurn; i++, k *= 36)
-            j = dicePerms->aaanPermutation[i][iTurn][((iGame + nSkip) / k + j) % 36];
+            j = dicePerms->aaanPermutation[i][iTurn][((iGame + MT_SafeGet(&nSkip)) / k + j) % 36];
 
         anDice[0] = j / 6 + 1;
         anDice[1] = j % 6 + 1;
@@ -267,11 +264,11 @@ extern void
 ClosedBoard(int afClosedBoard[2], const TanBoard anBoard)
 {
 
-    int i, j, n;
+    int i, j;
 
     for (i = 0; i < 2; i++) {
+        int n = 0;
 
-        n = 0;
         for (j = 0; j < 6; j++) {
             if (anBoard[i][j] > 1)
                 n++;
@@ -353,6 +350,8 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
 
     int nLateEvals = prc->fLateEvals ? prc->nLate : 0x7fffffff;
 
+    int useVarRedn = prc->fVarRedn;
+
     /* Make local copy of cubeinfo struct, since it
      * may be modified */
     cubeinfo *pciLocal = g_alloca(cci * sizeof(cubeinfo));
@@ -379,7 +378,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
     /* local pointers to the eval contexts to use */
     evalcontext *pecCube[2], *pecChequer[2];
 
-    if (prc->fVarRedn) {
+    if (useVarRedn) {
 
         /*
          * Create evaluation context one ply deep
@@ -520,7 +519,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
                         if (aarsStatistics) {
                             MT_SafeInc(&aarsStatistics[ici][pci->fMove].acDoubleDrop[LogCubeClamped(pci->nCube)]);
                             MT_SafeInc(&aarsStatistics[ici][pci->fMove].acWin[LogCubeClamped(pci->nCube)]);
-                        };
+                        }
 
                         break;
 
@@ -579,7 +578,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
 
                 /* Find best move :-) */
 
-                if (prc->fVarRedn) {
+                if (useVarRedn) {
 
                     /* Variance reduction */
 
@@ -667,7 +666,7 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
 
                         r = arMean[OUTPUT_CUBEFUL_EQUITY] - aaar[anDice[0] - 1][anDice[1] - 1]
                             [OUTPUT_CUBEFUL_EQUITY];
-                        aarVarRedn[ici][OUTPUT_CUBEFUL_EQUITY] += r * pci->nCube / aci[ici].nCube;
+                        aarVarRedn[ici][OUTPUT_CUBEFUL_EQUITY] += r * (float) (pci->nCube / aci[ici].nCube);
                     }
 
                 } else {
@@ -811,16 +810,16 @@ BasicCubefulRollout(unsigned int aanBoard[][2][25],
          * all variance reduction terms */
 
         if (!pci->nMatchTo)
-            aarOutput[ici][OUTPUT_CUBEFUL_EQUITY] *= pci->nCube / aci[ici].nCube;
+            aarOutput[ici][OUTPUT_CUBEFUL_EQUITY] *= (float) (pci->nCube / aci[ici].nCube);
 
-        if (prc->fVarRedn)
+        if (useVarRedn)
             for (i = 0; i < NUM_ROLLOUT_OUTPUTS; i++)
                 aarOutput[ici][i] += aarVarRedn[ici][i];
 
         /* multiply money equities */
 
         if (!pci->nMatchTo)
-            aarOutput[ici][OUTPUT_CUBEFUL_EQUITY] *= aci[ici].nCube / nBasisCube;
+            aarOutput[ici][OUTPUT_CUBEFUL_EQUITY] *= (float) (aci[ici].nCube / nBasisCube);
 
 
 
@@ -941,13 +940,13 @@ check_jsds(int *active)
 {
     int alt;
     float v, s, denominator;
-    rolloutcontext *prc;
 
     for (alt = 0; alt < ro_alternatives; ++alt) {
 
         /* 1) For each move, calculate the cubeful (or cubeless if that's what we're doing)
          * equity */
-        prc = &ro_apes[alt]->rc;
+        rolloutcontext *prc = &ro_apes[alt]->rc;
+
         if (prc->fCubeful) {
             v = aarMu[alt][OUTPUT_CUBEFUL_EQUITY];
             s = aarSigma[alt][OUTPUT_CUBEFUL_EQUITY];
@@ -1068,21 +1067,21 @@ check_sds(int *active)
     int alt;
     for (alt = 0; alt < ro_alternatives; ++alt) {
         float s;
-        int output;
+        int ioutput;
         int err_too_big = 0;
         rolloutcontext *prc;
         if (fNoMore[alt] || altGameCount[alt] < (rcRollout.nMinimumGames))
             continue;
         prc = &ro_apes[alt]->rc;
-        for (output = OUTPUT_EQUITY; output < NUM_ROLLOUT_OUTPUTS; output++) {
-            if (output == OUTPUT_EQUITY) {      /* cubeless */
+        for (ioutput = OUTPUT_EQUITY; ioutput < NUM_ROLLOUT_OUTPUTS; ioutput++) {
+            if (ioutput == OUTPUT_EQUITY) {     /* cubeless */
                 if (!ms.nMatchTo) {     /* money game */
-                    s = fabsf(aarSigma[alt][output]);
+                    s = fabsf(aarSigma[alt][ioutput]);
                     if (ro_fCubeRollout) {
-                        s *= aciLocal[alt].nCube / aciLocal[0].nCube;
+                        s *= (float) (aciLocal[alt].nCube / aciLocal[0].nCube);
                     }
                 } else {        /* match play */
-                    s = fabsf(se_mwc2eq(se_eq2mwc(aarSigma[alt][output],
+                    s = fabsf(se_mwc2eq(se_eq2mwc(aarSigma[alt][ioutput],
                                                   &aciLocal[alt]), &aciLocal[(ro_fCubeRollout ? 0 : alt)]));
                 }
             } else {
@@ -1090,9 +1089,9 @@ check_sds(int *active)
                     continue;
                 /* cubeful */
                 if (!ms.nMatchTo) {     /* money game */
-                    s = fabsf(aarSigma[alt][output]);
+                    s = fabsf(aarSigma[alt][ioutput]);
                 } else {
-                    s = fabsf(se_mwc2eq(aarSigma[alt][output], &aciLocal[(ro_fCubeRollout ? 0 : alt)]));
+                    s = fabsf(se_mwc2eq(aarSigma[alt][ioutput], &aciLocal[(ro_fCubeRollout ? 0 : alt)]));
                 }
             }
 
@@ -1100,7 +1099,7 @@ check_sds(int *active)
                 err_too_big = 1;
                 break;
             }
-        }                       /* for (output = OUTPUT_EQUITY; output < NUM_ROLLOUT_OUTPUTS; output++) */
+        }                       /* for (ioutput = OUTPUT_EQUITY; ioutput < NUM_ROLLOUT_OUTPUTS; ioutput++) */
 
         if (!err_too_big) {
             fNoMore[alt] = 1;
@@ -1152,7 +1151,7 @@ RolloutLoopMT(void *UNUSED(unused))
             if (prc->fRotate)
                 QuasiRandomSeed(&dicePerms, (int) prc->nSeed);
 
-            nSkip = 0;          /* not multi-thread safe do quasi random dice for initial positions */
+            MT_SafeSet(&nSkip, 0);      /* not multi-thread safe do quasi random dice for initial positions */
 
             /* ... and the RNG */
             if (prc->rngRollout != RNG_MANUAL)
@@ -1187,18 +1186,17 @@ RolloutLoopMT(void *UNUSED(unused))
 
             /* apply the results */
             for (j = 0; j < NUM_ROLLOUT_OUTPUTS; j++) {
-                float rMuNew, rDelta;
+                float rMuNew;
 
                 aarResult[alt][j] += aar[j];
-                rMuNew = aarResult[alt][j] / (altGameCount[alt]);
+                rMuNew = aarResult[alt][j] / (float) altGameCount[alt];
 
                 if (altGameCount[alt] > 1) {    /* for i == 0 aarVariance is not defined */
-
-                    rDelta = rMuNew - aarMu[alt][j];
+                    float rDelta = rMuNew - aarMu[alt][j];
 
                     aarVariance[alt][j] =
-                        aarVariance[alt][j] * (1.0f - 1.0f / (altGameCount[alt] - 1)) +
-                        (altGameCount[alt]) * rDelta * rDelta;
+                        aarVariance[alt][j] * (1.0f - 1.0f / (float) (altGameCount[alt] - 1)) +
+                        (float) (altGameCount[alt]) * rDelta * rDelta;
                 }
 
                 aarMu[alt][j] = rMuNew;
@@ -1210,7 +1208,7 @@ RolloutLoopMT(void *UNUSED(unused))
                         aarMu[alt][j] = 1.0f;
                 }
 
-                aarSigma[alt][j] = sqrtf(aarVariance[alt][j] / (altGameCount[alt]));
+                aarSigma[alt][j] = sqrtf(aarVariance[alt][j] / (float) altGameCount[alt]);
             }                   /* for (j = 0; j < NUM_ROLLOUT_OUTPUTS; j++ ) */
 
             /* For normal alternatives nGamesDone and altGameCount will be equal. For cube decisions,
@@ -1251,7 +1249,7 @@ RolloutLoopMT(void *UNUSED(unused))
         multi_debug("exclusive release: rollout cycle update");
         MT_Release();
     }
-    free(rngctxMTRollout);
+    g_free(rngctxMTRollout);
 }
 
 static rolloutprogressfunc *ro_pfProgress;
@@ -1262,13 +1260,13 @@ UpdateProgress(gpointer UNUSED(unused))
 {
     if (fShowProgress && ro_alternatives > 0) {
         int alt;
-        rolloutcontext *prc;
 
         multi_debug("exclusive lock: update progress");
         MT_Exclusive();
 
         for (alt = 0; alt < ro_alternatives; ++alt) {
-            prc = &ro_apes[alt]->rc;
+            rolloutcontext *prc = &ro_apes[alt]->rc;
+
             (*ro_pfProgress) (aarMu, aarSigma, prc, aciLocal, initial_game_count, altGameCount[alt] - 1, alt,
                               ajiJSD[alt].nRank + 1, ajiJSD[alt].rJSD, fNoMore[alt], show_jsds, ro_fCubeRollout,
                               ro_pUserData);
@@ -1390,7 +1388,6 @@ RolloutGeneral(ConstTanBoard * apBoard,
             }
         } else {
             int nGames = prc->nGamesDone;
-            float r;
 
             previous_rollouts++;
 
@@ -1407,10 +1404,12 @@ RolloutGeneral(ConstTanBoard * apBoard,
                 nFirstTrial = nGames;
             /* restore internal variables from input values */
             for (j = 0; j < NUM_ROLLOUT_OUTPUTS; ++j) {
+                float r;
+
                 r = aarMu[alt][j] = (*apOutput[alt])[j];
-                aarResult[alt][j] = r * nGames;
+                aarResult[alt][j] = r * (float) nGames;
                 r = aarSigma[alt][j] = (*apStdDev[alt])[j];
-                aarVariance[alt][j] = r * r * nGames;
+                aarVariance[alt][j] = r * r * (float) nGames;
             }
         }
 
@@ -1642,13 +1641,11 @@ GeneralCubeDecisionR(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
                      const TanBoard anBoard,
                      cubeinfo * pci, rolloutcontext * prc, evalsetup * pes, rolloutprogressfunc * pf, void *p)
 {
-
-
     evalsetup esLocal;
     evalsetup(*apes[2]);
     cubeinfo aci[2];
     const cubeinfo(*apci[2]);
-    int cGames;
+    int nTrials;
     int afCubeDecTop[] = { FALSE, FALSE };      /* no cube decision in 
                                                  * iTurn = 0 */
     ConstTanBoard apBoard[2];
@@ -1691,16 +1688,14 @@ GeneralCubeDecisionR(float aarOutput[2][NUM_ROLLOUT_OUTPUTS],
         prc->fCubeful = TRUE;
     }
 
-
-    if ((cGames = RolloutGeneral(apBoard, apOutput, apStdDev,
-                                 aarsStatistics, apes, apci, apCubeDecTop, 2, FALSE, TRUE, pf, p)) <= 0)
+    if ((nTrials = RolloutGeneral(apBoard, apOutput, apStdDev,
+                                  aarsStatistics, apes, apci, apCubeDecTop, 2, FALSE, TRUE, pf, p)) <= 0)
         return -1;
 
-    pes->rc.nGamesDone = cGames;
-    pes->rc.nSkip = nSkip;
+    pes->rc.nGamesDone = nTrials;
+    pes->rc.nSkip = MT_SafeGet(&nSkip);
 
     return 0;
-
 }
 
 
@@ -1815,12 +1810,9 @@ getResignEquities(float arResign[NUM_ROLLOUT_OUTPUTS], cubeinfo * pci, int nResi
 extern int
 ScoreMoveRollout(move ** ppm, cubeinfo ** ppci, int cMoves, rolloutprogressfunc * pfRolloutProgress, void * pUserData)
 {
-
-    const cubeinfo *pci;
     int fCubeDecTop = TRUE;
     int i;
     int nGamesDone;
-    rolloutcontext *prc;
 
     TanBoard *anBoard = g_alloca(cMoves * 2 * 25 * sizeof(int));
     ConstTanBoard *apBoard = g_alloca(cMoves * sizeof(int *));
@@ -1850,7 +1842,8 @@ ScoreMoveRollout(move ** ppm, cubeinfo ** ppci, int cMoves, rolloutprogressfunc
     }
 
     nGamesDone = RolloutGeneral(apBoard,
-                                apOutput, apStdDev, NULL, apes, apci, apCubeDecTop, cMoves, TRUE, FALSE, pfRolloutProgress, pUserData);
+                                apOutput, apStdDev, NULL, apes, apci, apCubeDecTop, cMoves, TRUE, FALSE,
+                                pfRolloutProgress, pUserData);
     /* put fMove back again */
     for (i = 0; i < cMoves; ++i) {
         aci[i].fMove = !aci[i].fMove;
@@ -1860,6 +1853,8 @@ ScoreMoveRollout(move ** ppm, cubeinfo ** ppci, int cMoves, rolloutprogressfunc
         return -1;
 
     for (i = 0; i < cMoves; ++i) {
+        const cubeinfo *pci;
+        rolloutcontext *prc;
 
         /* Score for move:
          * rScore is the primary score (cubeful/cubeless)
@@ -1881,4 +1876,18 @@ ScoreMoveRollout(move ** ppm, cubeinfo ** ppci, int cMoves, rolloutprogressfunc
     return 0;
 }
 
+extern void
+InvertStdDev(float ar[NUM_ROLLOUT_OUTPUTS])
+{
+    float r;
+
+    r = ar[OUTPUT_WINGAMMON];
+    ar[OUTPUT_WINGAMMON] = ar[OUTPUT_LOSEGAMMON];
+    ar[OUTPUT_LOSEGAMMON] = r;
+
+    r = ar[OUTPUT_WINBACKGAMMON];
+    ar[OUTPUT_WINBACKGAMMON] = ar[OUTPUT_LOSEBACKGAMMON];
+    ar[OUTPUT_LOSEBACKGAMMON] = r;
+}
+
 #endif
