@@ -1,11 +1,11 @@
 /*
- * sgf.h
+ * Copyright (C) 2000-2001 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2006-2008 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 2000
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: sgf.h,v 1.10 2013/06/16 02:16:20 mdpetch Exp $
+ * $Id: sgf.h,v 1.12 2021/10/30 14:13:24 plm Exp $
  */
 
 #ifndef SGF_H
@@ -25,7 +24,7 @@
 #include "list.h"
 #include <stdio.h>
 
-typedef struct _property {
+typedef struct {
     char ach[2];                /* 2 character tag; ach[ 1 ] = 0 for 1 character tags */
     listOLD *pl;                /* Values */
 } property;
