@@ -1,11 +1,11 @@
 /*
- * show.c
+ * Copyright (C) 1999-2003 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 1999-2021 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 1999, 2000, 2001, 2002.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: show.c,v 1.289 2018/04/10 22:11:41 plm Exp $
+ * $Id: show.c,v 1.306 2022/03/12 20:14:14 plm Exp $
  */
 
 #include "config.h"
@@ -48,7 +47,7 @@
 #include "openurl.h"
 #include "multithread.h"
 
-#if USE_GTK
+#if defined(USE_GTK)
 #include "gtkboard.h"
 #include "gtkgame.h"
 #include "gtktheory.h"
@@ -57,10 +56,12 @@
 #include "gtkmet.h"
 #include "gtkrolls.h"
 #include "gtktempmap.h"
+#include "gtkscoremap.h"
 #include "gtkoptions.h"
+#include "gtkcube.h"
 #endif
 
-#ifdef WIN32
+#if defined(WIN32)
 #include <io.h>
 #endif
 
@@ -83,7 +84,8 @@ ShowMoveFilter(const movefilter * pmf, const int ply)
         return;
     }
 
-    outputf(_(" and up to %d more moves within equity %0.3g\n"), pmf->Extra, pmf->Threshold);
+    outputf(_(" and up to %d more moves within equity %0.3g"), pmf->Extra, pmf->Threshold);
+    outputf("\n");
 }
 
 
@@ -95,7 +97,7 @@ ShowMoveFilters(movefilter aamf[MAX_FILTER_PLIES][MAX_FILTER_PLIES])
 
     for (i = 0; i < MAX_FILTER_PLIES; ++i) {
 
-        outputf("      Move filter for %d ply:\n", i + 1);
+        outputf(_("      Move filter for %d ply:\n"), i + 1);
 
         for (j = 0; j <= i; ++j) {
             outputf("        ");
@@ -172,6 +174,11 @@ show_evals(const char *text,
     if (!fLateEvals)
         return;
 
+    if (late == NULL) {
+        g_assert_not_reached();
+        return;
+    }
+
     outputf(_("%s after %d plies:\n"), text, nLate);
     if (fPlayersAreSame)
         ShowEvaluation(late);
@@ -202,24 +209,20 @@ show_movefilters(movefilter aaamf[2][MAX_FILTER_PLIES][MAX_FILTER_PLIES])
 static void
 ShowRollout(rolloutcontext * prc)
 {
-
-    int fDoTruncate = 0;
-    int fLateEvals = 0;
+    int fDoTruncate = prc->fDoTruncate;
+    int fLateEvals = FALSE;
     int nTruncate = prc->nTruncate;
     int nLate = prc->nLate;
-    int fPlayersAreSame = 1;
-    int fCubeEqualChequer = 1;
+    int fPlayersSameSettings = TRUE;
+    int fCubeChequerSameSettings = TRUE;
 
-    if (prc->fDoTruncate && (nTruncate > 0))
-        fDoTruncate = 1;
-
-    if (prc->fLateEvals && (!fDoTruncate || ((nTruncate > nLate) && (nLate > 2))))
-        fLateEvals = 1;
+    if (prc->fLateEvals && (!fDoTruncate || (nTruncate > nLate) ))
+        fLateEvals = TRUE;
 
     outputf(ngettext("%u game will be played per rollout.\n",
                      "%u games will be played per rollout.\n", prc->nTrials), prc->nTrials);
 
-    if (!fDoTruncate || nTruncate < 1)
+    if (!fDoTruncate)
         outputl(_("No truncation."));
     else
         outputf(ngettext("Truncation after %d ply.\n", "Truncation after %d plies.\n", nTruncate), nTruncate);
@@ -252,22 +255,22 @@ ShowRollout(rolloutcontext * prc)
         (fLateEvals &&
          (EvalCmp(prc->aecChequerLate, prc->aecChequerLate + 1, 1) ||
           EvalCmp(prc->aecCubeLate, prc->aecCubeLate + 1, 1))))
-        fPlayersAreSame = 0;
+        fPlayersSameSettings = FALSE;
 
     /* see if the cube and chequer evals are the same */
     if (EvalCmp(prc->aecChequer, prc->aecCube, 2) ||
         (fLateEvals && (EvalCmp(prc->aecChequerLate, prc->aecCubeLate, 2))))
-        fCubeEqualChequer = 0;
+        fCubeChequerSameSettings = FALSE;
 
-    if (fCubeEqualChequer) {
+    if (fCubeChequerSameSettings) {
         /* simple summary - show_evals will deal with player differences */
         show_evals(_("Evaluation parameters:"), prc->aecChequer,
-                   prc->aecChequerLate, fPlayersAreSame, fLateEvals, nLate);
+                   prc->aecChequerLate, fPlayersSameSettings, fLateEvals, nLate);
     } else {
         /* Cube different from Chequer */
         show_evals(_("Chequer play parameters:"), prc->aecChequer,
-                   prc->aecChequerLate, fPlayersAreSame, fLateEvals, nLate);
-        show_evals(_("Cube decision parameters:"), prc->aecCube, prc->aecCubeLate, fPlayersAreSame, fLateEvals, nLate);
+                   prc->aecChequerLate, fPlayersSameSettings, fLateEvals, nLate);
+        show_evals(_("Cube decision parameters:"), prc->aecCube, prc->aecCubeLate, fPlayersSameSettings, fLateEvals, nLate);
     }
 
     if (fLateEvals) {
@@ -281,8 +284,8 @@ ShowRollout(rolloutcontext * prc)
     }
 
     if (fDoTruncate) {
-        show_evals(_("Truncation point Chequer play evaluation:"), &prc->aecChequerTrunc, 0, 1, 0, 0);
-        show_evals(_("Truncation point Cube evaluation:"), &prc->aecCubeTrunc, 0, 1, 0, 0);
+        show_evals(_("Truncation point Chequer play evaluation:"), &prc->aecChequerTrunc, NULL, TRUE, FALSE, 0);
+        show_evals(_("Truncation point Cube evaluation:"), &prc->aecCubeTrunc, NULL, TRUE, FALSE, 0);
     }
 
     if (prc->fStopOnSTD) {
@@ -323,15 +326,13 @@ ShowEvalSetup(evalsetup * pes)
 static void
 ShowPaged(char **ppch)
 {
-
-    int i, nRows = 0;
-    char *pchLines;
-#ifdef TIOCGWINSZ
-    struct winsize ws;
-#endif
-
     if (isatty(STDIN_FILENO)) {
-#ifdef TIOCGWINSZ
+        int i, nRows = 0;
+        char *pchLines;
+
+#if defined(TIOCGWINSZ)
+        struct winsize ws;
+
         if (!(ioctl(STDIN_FILENO, TIOCGWINSZ, &ws)))
             nRows = ws.ws_row;
 #endif
@@ -442,8 +443,6 @@ CommandShowBoard(char *sz)
 {
 
     TanBoard an;
-    char szOut[2048];
-    char *ap[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
 
     if (!*sz) {
         if (ms.gs == GAME_NONE)
@@ -458,22 +457,24 @@ CommandShowBoard(char *sz)
     if (ParsePosition(an, &sz, NULL) < 0)
         return;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX)
         game_set(BOARD(pwBoard), an, TRUE, "", "", 0, 0, 0, 0, 0, FALSE, anChequers[ms.bgv]);
     else
 #endif
-        outputl(DrawBoard(szOut, (ConstTanBoard) an, TRUE, ap, MatchIDFromMatchState(&ms), anChequers[ms.bgv]));
+    {
+        char szOut[2048];
+        char *asz[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+
+        outputl(DrawBoard(szOut, (ConstTanBoard) an, TRUE, asz, MatchIDFromMatchState(&ms), anChequers[ms.bgv]));
+    }
 }
 
-extern
-    void
+extern void
 CommandShowFullBoard(char *sz)
 {
 
     TanBoard an;
-    char szOut[2048];
-    char *apch[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
 
     if (!*sz) {
         if (ms.gs == GAME_NONE)
@@ -488,21 +489,26 @@ CommandShowFullBoard(char *sz)
     if (ParsePosition(an, &sz, NULL) < 0)
         return;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX)
         game_set(BOARD(pwBoard), an, ms.fTurn,
                  ap[1].szName, ap[0].szName, ms.nMatchTo,
                  ms.anScore[1], ms.anScore[0], ms.anDice[0], ms.anDice[1], FALSE, anChequers[ms.bgv]);
     else
 #endif
+    {
+        char szOut[2048];
+        char *apch[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+       
         outputl(DrawBoard(szOut, (ConstTanBoard) an, TRUE, apch, MatchIDFromMatchState(&ms), anChequers[ms.bgv]));
+    }
 }
 
 
 extern void
 CommandShowDelay(char *UNUSED(sz))
 {
-#if USE_GTK
+#if defined(USE_GTK)
     if (nDelay)
         outputf(_("The delay is set to %u ms.\n"), nDelay);
     else
@@ -518,6 +524,7 @@ CommandShowAliases(char *UNUSED(sz))
     outputf(_("Aliases for player 1 when importing MAT files is set to \"%s\".\n "), player1aliases);
 }
 
+#if CACHE_STATS
 extern void
 CommandShowCache(char *UNUSED(sz))
 {
@@ -525,7 +532,7 @@ CommandShowCache(char *UNUSED(sz))
 
     EvalCacheStats(c, cLookup, cHit);
 
-    outputf("%10u regular eval entries used %10u lookups %10u hits", c[0], cLookup[0], cHit[0]);
+    outputf(_("%10u regular eval entries used %10u lookups %10u hits"), c[0], cLookup[0], cHit[0]);
 
     if (cLookup[0])
         outputf(" (%4.1f%%).", (float) cHit[0] * 100.0f / (float) cLookup[0]);
@@ -534,7 +541,7 @@ CommandShowCache(char *UNUSED(sz))
 
     outputc('\n');
 
-    outputf("%10u pruning eval entries used %10u lookups %10u hits", c[1], cLookup[1], cHit[1]);
+    outputf(_("%10u pruning eval entries used %10u lookups %10u hits"), c[1], cLookup[1], cHit[1]);
 
     if (cLookup[1])
         outputf(" (%4.1f%%).", (float) cHit[1] * 100.0f / (float) cLookup[1]);
@@ -543,12 +550,13 @@ CommandShowCache(char *UNUSED(sz))
 
     outputc('\n');
 }
+#endif
 
 extern void
 CommandShowCalibration(char *UNUSED(sz))
 {
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowCalibration();
         return;
@@ -628,7 +636,7 @@ extern void
 CommandShowCopying(char *UNUSED(sz))
 {
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX)
         ShowList(aszCopying, _("Copying"), NULL);
     else
@@ -755,7 +763,7 @@ extern void
 CommandShowMatchInfo(char *UNUSED(sz))
 {
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKMatchInfo();
         return;
@@ -813,7 +821,7 @@ CommandShowPipCount(char *sz)
 
     outputf(_("The pip counts are: %s %u, %s %u.\n"), ap[ms.fMove].szName, anPips[1], ap[!ms.fMove].szName, anPips[0]);
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX && fFullScreen) {    /* Display in dialog box in full screen mode (urgh) */
         output(" ");
         outputx();
@@ -836,7 +844,7 @@ CommandShowPlayer(char *UNUSED(sz))
             break;
         case PLAYER_GNU:
             outputf(_("gnubg:\n"));
-            outputl(_("    Checker play:"));
+            outputl(_("    Chequer play:"));
             ShowEvalSetup(&ap[i].esChequer);
             outputl(_("    Move filters:"));
             ShowMoveFilters(ap[i].aamf);
@@ -891,14 +899,14 @@ extern void
 CommandShowScore(char *UNUSED(sz))
 {
 
-    outputf((ms.cGames == 1 ? _("The score (after %d game) is: %s %d, %s %d")
-             : _("The score (after %d games) is: %s %d, %s %d")),
+    outputf(ngettext("The score (after %d game) is: %s %d, %s %d",
+           "The score (after %d games) is: %s %d, %s %d", ms.cGames),
             ms.cGames, ap[0].szName, ms.anScore[0], ap[1].szName, ms.anScore[1]);
 
     if (ms.nMatchTo > 0) {
-        outputf(ms.nMatchTo == 1 ?
-                _(" (match to %d point%s).\n") :
-                _(" (match to %d points%s).\n"),
+        outputf(ngettext(" (match to %d point%s)",
+                         " (match to %d points%s)",
+                         ms.nMatchTo),
                 ms.nMatchTo, ms.fCrawford ? _(", Crawford game") : (ms.fPostCrawford ? _(", post-Crawford play") : ""));
     } else {
         if (ms.fJacoby)
@@ -967,7 +975,7 @@ CommandShowBuildInfo(char *UNUSED(sz))
 {
     const char *pch;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX)
         GTKShowBuildInfo(pwMain, NULL);
 #endif
@@ -982,21 +990,17 @@ extern void
 CommandShowScoreSheet(char *UNUSED(sz))
 {
     size_t i, width1, width2;
-    char *data[2];
+    gchar *data0 = NULL, *data1 = NULL;
     listOLD *pl;
 
-    if (ms.gs == GAME_NONE) {
-        outputl(_("No game in progress (type `new game' to start one)."));
-        return;
-    }
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowScoreSheet();
         return;
     }
 #endif
 
-    output(_("Score Sheet - "));
+    outputf("%s - ", _("Score Sheet"));
     if (ms.nMatchTo > 0)
         outputf(ms.nMatchTo == 1 ? _("Match to %d point") : _("Match to %d points"), ms.nMatchTo);
     else
@@ -1012,24 +1016,23 @@ CommandShowScoreSheet(char *UNUSED(sz))
         outputc('-');
     output("\n");
 
-    data[0] = malloc(50);
-    data[1] = malloc(50);
-
     for (pl = lMatch.plNext; pl->p; pl = pl->plNext) {
         int score[2];
-        listOLD *plGame = pl->plNext->p;
+        listOLD *plg = pl->plNext->p;
+
+        if (plg) {
+            moverecord *pmr = plg->plNext->p;
 
-        if (plGame) {
-            moverecord *pmr = plGame->plNext->p;
             score[0] = pmr->g.anScore[0];
             score[1] = pmr->g.anScore[1];
         } else {
             moverecord *pmr;
-            listOLD *plGame = pl->p;
-            if (!plGame) {
+
+            plg = pl->p;
+            if (!plg) {
                 continue;
             } else {
-                pmr = plGame->plNext->p;
+                pmr = plg->plNext->p;
                 score[0] = pmr->g.anScore[0];
                 score[1] = pmr->g.anScore[1];
                 if (pmr->g.fWinner == -1) {
@@ -1041,13 +1044,13 @@ CommandShowScoreSheet(char *UNUSED(sz))
                     score[pmr->g.fWinner] += pmr->g.nPoints;
             }
         }
-        sprintf(data[0], "%d", score[0]);
-        sprintf(data[1], "%d", score[1]);
-        outputf("%*s | %s\n", (int) width1, data[0], data[1]);
+        data0 = g_strdup_printf("%d", score[0]);
+        data1 = g_strdup_printf("%d", score[1]);
+        outputf("%*s | %s\n", (int) width1, data0, data1);
     }
 
-    free(data[0]);
-    free(data[1]);
+    g_free(data0);
+    g_free(data1);
 
     output("\n");
     outputx();
@@ -1056,7 +1059,7 @@ CommandShowScoreSheet(char *UNUSED(sz))
 extern void
 CommandShowCredits(char *UNUSED(sz))
 {
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKCommandShowCredits(NULL, NULL);
         return;
@@ -1071,7 +1074,7 @@ extern void
 CommandShowWarranty(char *UNUSED(sz))
 {
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX)
         ShowList(aszWarranty, _("Warranty"), NULL);
     else
@@ -1120,7 +1123,7 @@ CommandShowKleinman(char *sz)
     if (ParsePosition(an, &sz, NULL) < 0)
         return;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowRace(an);
         return;
@@ -1150,11 +1153,11 @@ show_thorp(TanBoard an, char *sz)
     sprintf(strchr(sz, 0), _("Thorp Count Leader(+1/10)    L: %.1f\n"), adjusted);
     sprintf(strchr(sz, 0), _("Thorp Count Trailer          T: %d\n\n"), nTrailer);
 
-    if (nTrailer >= (adjusted + 2))
+    if ((float) nTrailer >= (adjusted + 2))
         sprintf(strchr(sz, 0), _("Double, Drop (since L <= T - 2)\n"));
-    else if (nTrailer >= (adjusted - 1))
+    else if ((float) nTrailer >= (adjusted - 1))
         sprintf(strchr(sz, 0), _("Redouble, Take (since L <= T + 1 )\n"));
-    else if (nTrailer >= (adjusted - 2))
+    else if ((float) nTrailer >= (adjusted - 2))
         sprintf(strchr(sz, 0), _("Double, Take (since L <= T + 2)\n"));
     else
         sprintf(strchr(sz, 0), _("No Double, Take (since L > T + 2)\n"));
@@ -1174,7 +1177,7 @@ CommandShowThorp(char *sz)
     if (ParsePosition(an, &sz, NULL) < 0)
         return;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowRace(an);
         return;
@@ -1221,7 +1224,7 @@ CommandShow8912(char *sz)
     if (ParsePosition(anBoard, &sz, NULL) < 0)
         return;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowRace(anBoard);
         return;
@@ -1268,7 +1271,7 @@ CommandShowKeith(char *sz)
     if (ParsePosition(an, &sz, NULL) < 0)
         return;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowRace(an);
         return;
@@ -1288,7 +1291,7 @@ show_isight(TanBoard an, char *sz)
     sprintf(sz, _("Isight Count Leader          L: %d\n"), pn[1]);
     sprintf(strchr(sz, 0), _("Isight Count Trailer         T: %d\n\n"), pn[0]);
 
-    gwc = 80.0f - pn[1] / 3.0f + 2.0f * (pn[0] - pn[1]);
+    gwc = 80.0f - (float) pn[1] / 3.0f + 2.0f * (float) (pn[0] - pn[1]);
 
     sprintf(strchr(sz, 0), _("Estimated GWC: %4.1f%% (80 - L/3 + 2*(L-T))\n\n"), gwc);
 
@@ -1316,7 +1319,7 @@ CommandShowIsight(char *sz)
     if (ParsePosition(an, &sz, NULL) < 0)
         return;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowRace(an);
         return;
@@ -1351,7 +1354,7 @@ CommandShowGammonValues(char *UNUSED(sz))
 
         return;
     }
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowTheory(1);
         return;
@@ -1386,7 +1389,7 @@ writeMET(float aafMET[][MAXSCORE], const int nRows, const int nCols, const int f
         outputf(_(" %3i-away "), i + 1);
 
         for (j = 0; j < nCols; j++)
-            outputf(" %8.4f ", fInvert ? 100.0f * (1.0 - GET_MET(i, j, aafMET)) : GET_MET(i, j, aafMET) * 100.0);
+            outputf(" %8.4f ", fInvert ? 100.0f * (1.0f - GET_MET(i, j, aafMET)) : GET_MET(i, j, aafMET) * 100.0f);
         output("\n");
     }
     output("\n");
@@ -1408,7 +1411,7 @@ EffectivePipCount(const float arMu[2], const unsigned int anPips[2])
     outputf("%-20.20s   %7s  %7s\n", "", _("EPC"), _("Wastage"));
     for (i = 0; i < 2; ++i)
         outputf("%-20.20s   %7.3f  %7.3f\n",
-                ap[i].szName, arMu[ms.fMove ? i : !i] * x, arMu[ms.fMove ? i : !i] * x - anPips[ms.fMove ? i : !i]);
+                ap[i].szName, arMu[ms.fMove ? i : !i] * x, arMu[ms.fMove ? i : !i] * x - (float) anPips[ms.fMove ? i : !i]);
 
     outputf(_("\n" "EPC = Avg. rolls * %5.3f\n" "Wastage = EPC - Pips\n\n"), x);
 
@@ -1433,7 +1436,7 @@ CommandShowOneSidedRollout(char *sz)
         return;
 
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowRace(anBoard);
         return;
@@ -1479,7 +1482,7 @@ CommandShowMatchEquityTable(char *sz)
         anScore[0] = anScore[1] = -1;
 
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowMatchEquityTable(n, anScore);
         return;
@@ -1521,7 +1524,7 @@ CommandShowOutput(char *UNUSED(sz))
             _("Game winning chances will be shown as percentages.\n") :
             _("Game winning chances will be shown as probabilities.\n"));
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (!fX)
 #endif
         outputf(fOutputRawboard ? _("Boards will be shown in raw format.\n") : _("Boards will be shown in ASCII.\n"));
@@ -1530,7 +1533,7 @@ CommandShowOutput(char *UNUSED(sz))
 extern void
 CommandShowVersion(char *UNUSED(sz))
 {
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowVersion();
         return;
@@ -1550,18 +1553,17 @@ CommandShowMarketWindow(char *sz)
 
     float arOutput[NUM_OUTPUTS];
     float arDP1[2], arDP2[2], arCP1[2], arCP2[2];
-    float rDTW, rDTL, rNDW, rNDL, rDP, rRisk, rGain, r;
 
     float aarRates[2][2];
 
-    int i, fAutoRedouble[2], afDead[2], anNormScore[2];
+    int anNormScore[2];
 
     if (ms.gs != GAME_PLAYING) {
         outputl(_("No game in progress (type `new game' to start one)."));
 
         return;
     }
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowTheory(0);
         return;
@@ -1582,6 +1584,8 @@ CommandShowMarketWindow(char *sz)
 
         /* read the others */
 
+        float r;
+
         aarRates[1][0] = ((r = ParseReal(&sz)) > 0.0f) ? r : 0.0f;
         aarRates[0][1] = ((r = ParseReal(&sz)) > 0.0f) ? r : 0.0f;
         aarRates[1][1] = ((r = ParseReal(&sz)) > 0.0f) ? r : 0.0f;
@@ -1598,7 +1602,7 @@ CommandShowMarketWindow(char *sz)
 
         /* Check that ratios are 0 <= ratio <= 1 */
 
-        for (i = 0; i < 2; i++) {
+        for (int i = 0; i < 2; i++) {
             if (aarRates[i][0] > 1.0f) {
                 outputf(_("illegal gammon ratio for player %i: %f\n"), i, aarRates[i][0]);
                 return;
@@ -1629,19 +1633,23 @@ CommandShowMarketWindow(char *sz)
 
 
 
-    for (i = 0; i < 2; i++)
+    for (int i = 0; i < 2; i++)
         outputf(_("Player %-25s: gammon rate %6.2f%%, bg rate %6.2f%%\n"),
-                ap[i].szName, aarRates[i][0] * 100.0, aarRates[i][1] * 100.0);
+                ap[i].szName, aarRates[i][0] * 100.0f, aarRates[i][1] * 100.0f);
 
 
     if (ms.nMatchTo) {
 
-        for (i = 0; i < 2; i++)
+        int fAutoRedouble[2], afDead[2];
+
+        for (int i = 0; i < 2; i++)
             anNormScore[i] = ms.nMatchTo - ms.anScore[i];
 
         GetPoints(arOutput, &ci, arCP2);
 
-        for (i = 0; i < 2; i++) {
+        for (int i = 0; i < 2; i++) {
+
+            float rDTW, rDTL, rNDW, rNDL, rDP, rRisk, rGain;
 
             fAutoRedouble[i] = (anNormScore[i] - 2 * ms.nCube <= 0) && (anNormScore[!i] - 2 * ms.nCube > 0);
 
@@ -1765,7 +1773,7 @@ CommandShowMarketWindow(char *sz)
 
         output("\n\n");
 
-        for (i = 0; i < 2; i++) {
+        for (int i = 0; i < 2; i++) {
 
             outputf(_("Player %s market window:\n\n"), ap[i].szName);
 
@@ -1774,12 +1782,12 @@ CommandShowMarketWindow(char *sz)
             else
                 output(_("Dead cube: "));
 
-            outputf("%6.2f%% - %6.2f%%\n", 100. * arDP1[i], 100. * arCP1[i]);
+            outputf("%6.2f%% - %6.2f%%\n", 100.0f * arDP1[i], 100.0f * arCP1[i]);
 
             if (fAutoRedouble[i])
-                outputf(_("Dead cube (opponent redoubles):" "%6.2f%% - %6.2f%%\n\n"), 100. * arDP2[i], 100. * arCP2[i]);
+                outputf(_("Dead cube (opponent redoubles):" "%6.2f%% - %6.2f%%\n\n"), 100.0f * arDP2[i], 100.0f * arCP2[i]);
             else if (!afDead[i])
-                outputf(_("Live cube:" "%6.2f%% - %6.2f%%\n\n"), 100. * arDP2[i], 100. * arCP2[i]);
+                outputf(_("Live cube:" "%6.2f%% - %6.2f%%\n\n"), 100.0f * arDP2[i], 100.0f * arCP2[i]);
 
         }
 
@@ -1806,16 +1814,14 @@ CommandShowMarketWindow(char *sz)
 
         float aaarPoints[2][7][2];
 
-        int i, j;
-
         getMoneyPoints(aaarPoints, ci.fJacoby, ci.fBeavers, aarRates);
 
-        for (i = 0; i < 2; i++) {
+        for (int i = 0; i < 2; i++) {
 
             outputf(_("\nPlayer %s cube parameters:\n\n"), ap[i].szName);
             outputf("%-27s  %7s      %s\n", _("Cube parameter"), _("Dead Cube"), _("Live Cube"));
 
-            for (j = 0; j < 7; j++)
+            for (int j = 0; j < 7; j++)
                 outputf("%-27s  %7.3f%%     %7.3f%%\n",
                         gettext(aszMoneyPointLabel[j]), aaarPoints[i][j][0] * 100.0f, aaarPoints[i][j][1] * 100.0f);
 
@@ -1832,7 +1838,7 @@ CommandShowExport(char *UNUSED(sz))
 
     int i;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowExport(&exsExport);
         return;
@@ -2019,7 +2025,7 @@ extern void
 CommandShowRolls(char *sz)
 {
 
-#if USE_GTK
+#if defined(USE_GTK)
     int nDepth = ParseNumber(&sz);
 #else
     (void) sz;                  /* suppress unused parameter compiler warning */
@@ -2030,7 +2036,7 @@ CommandShowRolls(char *sz)
 
         return;
     }
-#if USE_GTK
+#if defined(USE_GTK)
 
     if (fX) {
         static evalcontext ec0ply = { TRUE, 0, FALSE, TRUE, 0.0 };
@@ -2054,7 +2060,7 @@ CommandShowTemperatureMap(char *sz)
 
         return;
     }
-#if USE_GTK
+#if defined(USE_GTK)
 
     if (fX) {
 
@@ -2070,8 +2076,14 @@ CommandShowTemperatureMap(char *sz)
                 int i;
                 gchar *asz[2];
 
-                for (i = 0; i < 2; ++i)
+                for (i = 0; i < 2; ++i) {
                     memcpy(&ams[i], &ms, sizeof(matchstate));
+                    /* if MoneyEval is enabled, we evaluate in money play */
+                    if (cubeTempMapAtMoney) {
+                        ams[i].nMatchTo = 0;
+                        ams[i].fJacoby = cubeTempMapJacoby;
+                    }
+                }
 
                 ams[1].nCube *= 2;
                 ams[1].fCubeOwner = !ams[1].fMove;
@@ -2103,6 +2115,63 @@ CommandShowTemperatureMap(char *sz)
 
 }
 
+// defined in backgammon.h
+extern void
+CommandShowScoreMap(char *sz)
+{
+    if (ms.gs != GAME_PLAYING) {
+        outputl(_("No game in progress (type `new game' to start one)."));
+
+        return;
+    }
+#if defined(USE_GTK)
+
+    if (fX) {
+        
+// TODO: don't show score map if not a double. Perhaps start with doShowScoreMap=ms.fDoubled
+        int doShowScoreMap=TRUE; 
+        if ( sz && *sz && (strncmp(sz, "=move", 5) ==0) ) { //strncmp: returns 0 if same content => we enter if NOT move
+            GTKShowScoreMap(&ms, 0);
+        } else { 
+            cubeinfo ci;
+            GetMatchStateCubeInfo(&ci, &ms);
+            if (!GetDPEq(NULL, NULL, &ci)) {
+
+                /* cube is available 
+
+                matchstate ams[1];
+                int i;
+                gchar *asz[1];
+
+                memcpy(&ams[0], &ms, sizeof(matchstate));
+
+                asz[0] = g_malloc(200);
+                GetMatchStateCubeInfo(&ci, &ams[0]);
+                FormatCubePosition(asz[0], &ci);
+
+                GTKShowScoreMap(ams, 1, asz, FALSE);
+
+                for (i = 0; i < 2; ++i)
+                    g_free(asz[i]);
+
+            } else {*/
+                outputl(_("Cube is not available."));
+                doShowScoreMap=FALSE;
+            }
+        }
+        if (doShowScoreMap)
+            GTKShowScoreMap(&ms, 1); //NULL, FALSE);
+
+        return;
+    }
+#else
+    (void) sz;                  /* suppress unused parameter compiler warning */
+#endif
+
+    CommandNotImplemented(NULL);
+
+}
+
 extern void
 CommandShowVariation(char *UNUSED(sz))
 {
@@ -2192,7 +2261,7 @@ CommandShowBearoff(char *sz)
 
     if (ParsePosition(an, &sz, NULL) < 0)
         return;
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKShowRace(an);
         return;
@@ -2214,7 +2283,6 @@ CommandShowMatchResult(char *UNUSED(sz))
     xmovegameinfo *pmgi;
     statcontext *psc;
     listOLD *pl;
-    float r;
 
     updateStatisticsMatch(&lMatch);
 
@@ -2223,6 +2291,8 @@ CommandShowMatchResult(char *UNUSED(sz))
 
     for (pl = lMatch.plNext; pl != &lMatch; pl = pl->plNext, ++n) {
 
+        float r;
+
         pmr = (moverecord *) ((listOLD *) pl->p)->plNext->p;
         pmgi = &pmr->g;
         g_assert(pmr->mt == MOVE_GAMEINFO);
@@ -2233,8 +2303,8 @@ CommandShowMatchResult(char *UNUSED(sz))
             if (ms.nMatchTo)
                 outputf("%10d %9.2f%% %9.2f%%\n",
                         n,
-                        100.0 * (0.5f + psc->arActualResult[0]),
-                        100.0 * (0.5f + psc->arActualResult[0] - psc->arLuck[0][1] + psc->arLuck[1][1]));
+                        100.0f * (0.5f + psc->arActualResult[0]),
+                        100.0f * (0.5f + psc->arActualResult[0] - psc->arLuck[0][1] + psc->arLuck[1][1]));
             else
                 outputf("%10d %9.3f%% %9.3f%%\n",
                         n, psc->arActualResult[0], psc->arActualResult[0] - psc->arLuck[0][1] + psc->arLuck[1][1]);
@@ -2253,17 +2323,19 @@ CommandShowMatchResult(char *UNUSED(sz))
     }
 
     if (ms.nMatchTo)
-        outputf("%10s %9.2f%% %9.2f%%\n", _("Final"), 100.0 * (0.5f + arSum[0]), 100.0 * (0.5f + arSum[1]));
+        outputf("%10s %9.2f%% %9.2f%%\n", _("Final"), 100.0f * (0.5f + arSum[0]), 100.0f * (0.5f + arSum[1]));
     else
         outputf("%10s %+9.3f %+9.3f\n", _("Sum"), arSum[0], arSum[1]);
 
     if (n && !ms.nMatchTo) {
-        outputf("%10s %+9.3f %+9.3f\n", _("Average"), arSum[0] / n, arSum[1] / n);
-        outputf("%10s %9.3f %9.3f\n", "95%CI",
+		float fn = (float)n;
+        outputf("%10s %+9.3f %+9.3f\n", _("Average"), arSum[0] / fn, arSum[1] / fn);
+        /* xgettext: no-c-format */
+        outputf("%10s %9.3f %9.3f\n", _("95%CI"),
                 1.95996f *
-                sqrt(arSumSquared[0] / n -
-                     arSum[0] * arSum[0] / (n * n)) / sqrt(n),
-                1.95996f * sqrt(arSumSquared[1] / n - arSum[1] * arSum[1] / (n * n)) / sqrt(n));
+                sqrtf(arSumSquared[0] / fn -
+                     arSum[0] * arSum[0] / (fn * fn)) / sqrtf(fn),
+                1.95996f * sqrtf(arSumSquared[1] / fn - arSum[1] * arSum[1] / (fn * fn)) / sqrtf(fn));
     }
 
 }
