@@ -1,11 +1,11 @@
 /*
- * text.c
+ * Copyright (C) 2002-2004 Joern Thyssen <jthyssen@dk.ibm.com>
+ * Copyright (C) 2002-2019 the AUTHORS
  *
- * by Joern Thyssen  <jthyssen@dk.ibm.com>, 2002
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: text.c,v 1.114 2017/04/04 04:49:32 plm Exp $
+ * $Id: text.c,v 1.128 2022/03/12 20:14:14 plm Exp $
  */
 
 #include "config.h"
@@ -28,7 +27,6 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <stdarg.h>
-#include "glib-ext.h"
 
 #include "backgammon.h"
 #include "drawboard.h"
@@ -39,7 +37,6 @@
 #include "relational.h"
 
 
-
 /* "Color" of chequers */
 
 static void
@@ -58,15 +55,9 @@ printTextBoard(FILE * pf, const matchstate * pms)
     apch[0] = szPlayer0;
     apch[6] = szPlayer1;
 
-    if (pms->anScore[0] == 1)
-        sprintf(apch[1] = szScore0, _("%d point"), pms->anScore[0]);
-    else
-        sprintf(apch[1] = szScore0, _("%d points"), pms->anScore[0]);
+    sprintf(apch[1] = szScore0, ngettext("%d point", "%d points", pms->anScore[0]), pms->anScore[0]);
 
-    if (pms->anScore[1] == 1)
-        sprintf(apch[5] = szScore1, _("%d point"), pms->anScore[1]);
-    else
-        sprintf(apch[5] = szScore1, _("%d points"), pms->anScore[1]);
+    sprintf(apch[5] = szScore1, ngettext("%d point", "%d points", pms->anScore[1]), pms->anScore[1]);
 
     if (pms->fDoubled) {
         apch[pms->fTurn ? 4 : 2] = szCube;
@@ -90,13 +81,15 @@ printTextBoard(FILE * pf, const matchstate * pms)
         if (pms->fCubeOwner < 0) {
             apch[3] = szCube;
 
+            /* Using ngettext() below looks awkward, but it matters in case of multiple plurals, as in many eastern european languages */
+
             if (pms->nMatchTo)
                 if (pms->nMatchTo == 1)
-                    sprintf(szCube, _("1 point match"));
+                    sprintf(szCube, ngettext("%d point match", "%d points match", pms->nMatchTo), pms->nMatchTo);
                 else if (pms->fCrawford)
-                    sprintf(szCube, _("%d point match (Crawford game)"), pms->nMatchTo);
+                    sprintf(szCube, ngettext("%d point match (Crawford game)", "%d points match (Crawford game)", pms->nMatchTo), pms->nMatchTo);
                 else
-                    sprintf(szCube, _("%d point match (Cube: %d)"), pms->nMatchTo, pms->nCube);
+                    sprintf(szCube, ngettext("%d point match (Cube: %d)", "%d points match (Cube: %d)", pms->nMatchTo), pms->nMatchTo, pms->nCube);
             else
                 sprintf(szCube, _("(Cube: %d)"), pms->nCube);
         } else {
@@ -111,7 +104,7 @@ printTextBoard(FILE * pf, const matchstate * pms)
             apch[pms->fCubeOwner ? 6 : 0] = szCube;
 
             if (pms->nMatchTo)
-                sprintf(apch[3] = szMatch, _("%d point match"), pms->nMatchTo);
+                sprintf(apch[3] = szMatch, ngettext("%d point match", "%d points match", pms->nMatchTo), pms->nMatchTo);
         }
     }
 
@@ -129,13 +122,13 @@ printTextBoard(FILE * pf, const matchstate * pms)
 
     PipCount((ConstTanBoard) anBoard, anPips);
 
-    fprintf(pf, "Pip counts: O %u, X %u\n\n", anPips[0], anPips[1]);
+    fprintf(pf, "%s O %u, X %u\n\n", _("Pip counts:"), anPips[0], anPips[1]);
 
 }
 
 
 /*
- * Print html header for board: move or cube decision 
+ * Print text header for board: move or cube decision 
  *
  * Input:
  *   pf: output file
@@ -152,7 +145,6 @@ TextBoardHeader(GString * gsz, const matchstate * pms, const int UNUSED(iGame),
         g_string_append_printf(gsz, _("Move number %d: "), iMove + 1);
 
     if (pms->fResigned)
-
         /* resignation */
 
         g_string_append_printf(gsz,
@@ -162,14 +154,12 @@ TextBoardHeader(GString * gsz, const matchstate * pms, const int UNUSED(iGame),
                                ap[pms->fTurn].szName, pms->fResigned * pms->nCube);
 
     else if (pms->anDice[0] && pms->anDice[1])
-
         /* chequer play decision */
 
         g_string_append_printf(gsz, _(" %s to play %u%u\n\n"), ap[pms->fMove].szName, pms->anDice[0], pms->anDice[1]
             );
 
     else if (pms->fDoubled)
-
         /* take decision */
 
         g_string_append_printf(gsz, _(" %s doubles to %d\n\n"), ap[!(pms->fTurn)].szName, pms->nCube * 2);
@@ -183,7 +173,7 @@ TextBoardHeader(GString * gsz, const matchstate * pms, const int UNUSED(iGame),
 
 
 /*
- * Print html header: dtd, head etc.
+ * Print text header
  *
  * Input:
  *   pf: output file
@@ -195,10 +185,10 @@ extern void
 TextPrologue(GString * gsz, const matchstate * pms, const int UNUSED(iGame))
 {
 
-    g_string_append_printf(gsz, pms->cGames == 1 ?
-                           _("The score (after %d game) is: %s %d, %s %d") :
-                           _("The score (after %d games) is: %s %d, %s %d"),
-                           pms->cGames, ap[0].szName, pms->anScore[0], ap[1].szName, pms->anScore[1]);
+    g_string_append_printf(gsz,
+                           ngettext("The score (after %d game) is: %s %d, %s %d",
+                                    "The score (after %d games) is: %s %d, %s %d", pms->cGames), pms->cGames,
+                           ap[0].szName, pms->anScore[0], ap[1].szName, pms->anScore[1]);
 
     if (pms->nMatchTo > 0) {
         g_string_append_printf(gsz,
@@ -209,13 +199,13 @@ TextPrologue(GString * gsz, const matchstate * pms, const int UNUSED(iGame))
             g_string_append(gsz, _(", post-Crawford play"));
     }
 
-    g_string_append(gsz, "\n");
+    g_string_append(gsz, "\n\n");
 
 }
 
 
 /*
- * Print html header: dtd, head etc.
+ * Print text footer
  *
  * Input:
  *   pf: output file
@@ -229,7 +219,7 @@ TextEpilogue(FILE * pf, const matchstate * UNUSED(pms))
 
     time_t t;
 
-    const char szVersion[] = "$Revision: 1.114 $";
+    const char szVersion[] = "$Revision: 1.128 $";
     int iMajor, iMinor;
 
     iMajor = atoi(strchr(szVersion, ' '));
@@ -237,7 +227,7 @@ TextEpilogue(FILE * pf, const matchstate * UNUSED(pms))
 
     time(&t);
 
-    fprintf(pf, _("Output generated %s" "by %s "), ctime(&t), VERSION_STRING);
+    fprintf(pf, _("Output generated %s by %s "), ctime(&t), VERSION_STRING);
 
     fprintf(pf, _("(Text Export version %d.%d)\n\n"), iMajor, iMinor);
 
@@ -267,7 +257,7 @@ TextPrintCubeAnalysisTable(GString * gsz,
 {
 
     int fActual, fClose, fMissed;
-    int fDisplay;
+    int fDisplayIt;
     float arDouble[4];
 
     /* check if cube analysis should be printed */
@@ -281,13 +271,13 @@ TextPrintCubeAnalysisTable(GString * gsz,
     fClose = isCloseCubedecision(arDouble);
     fMissed = isMissedDouble(arDouble, aarOutput, fDouble, pci);
 
-    fDisplay =
+    fDisplayIt =
         (fActual && exsExport.afCubeDisplay[EXPORT_CUBE_ACTUAL]) ||
         (fClose && exsExport.afCubeDisplay[EXPORT_CUBE_CLOSE]) ||
         (fMissed && exsExport.afCubeDisplay[EXPORT_CUBE_MISSED]) ||
         (exsExport.afCubeDisplay[stDouble]) || (exsExport.afCubeDisplay[stTake]);
 
-    if (!fDisplay)
+    if (!fDisplayIt)
         return;
 
     g_string_append(gsz, OutputCubeAnalysisFull(aarOutput, aarStdDev, pes, pci, fDouble, fTake, stDouble, stTake));
@@ -339,7 +329,8 @@ TextPrintCubeAnalysis(GString * gsz, const matchstate * pms, moverecord * pmr)
 
         dt = DoubleType(pms->fDoubled, pms->fMove, pms->fTurn);
         if (dt != DT_NORMAL) {
-            g_string_append(gsz, _("Cannot analyse beaver nor raccoons!\n"));
+            g_string_append(gsz, _("Cannot analyse beaver nor raccoons!"));
+            g_string_append(gsz, "\n");
             break;
         }
         TextPrintCubeAnalysisTable(gsz,
@@ -356,7 +347,8 @@ TextPrintCubeAnalysis(GString * gsz, const matchstate * pms, moverecord * pmr)
 
         if (dt != DT_NORMAL) {
             dt = DT_NORMAL;
-            g_string_append(gsz, _("Cannot analyse beaver nor raccoons!\n"));
+            g_string_append(gsz, _("Cannot analyse beaver nor raccoons!"));
+            g_string_append(gsz, "\n");
             break;
         }
         TextPrintCubeAnalysisTable(gsz, pmr->CubeDecPtr->aarOutput, pmr->CubeDecPtr->aarStdDev, pmr->fPlayer, &pmr->CubeDecPtr->esDouble, &ci, TRUE, pmr->mt == MOVE_TAKE, SKILL_NONE,  /* FIXME: skill from prev. cube */
@@ -391,10 +383,7 @@ TextPrintCubeAnalysis(GString * gsz, const matchstate * pms, moverecord * pmr)
 static void
 TextPrintMoveAnalysis(GString * gsz, const matchstate * pms, moverecord * pmr)
 {
-
-    char szBuf[1024];
-    char sz[64];
-    unsigned int i;
+    char sz[FORMATEDMOVESIZE];
 
     cubeinfo ci;
 
@@ -447,8 +436,11 @@ TextPrintMoveAnalysis(GString * gsz, const matchstate * pms, moverecord * pmr)
         g_string_append_printf(gsz, ":\n");
 
     if (pmr->ml.cMoves) {
+        unsigned int i;
 
         for (i = 0; i < pmr->ml.cMoves; i++) {
+            char szBuf[1024];
+
             if (i >= exsExport.nMoves && i != pmr->n.iMove)
                 continue;
 
@@ -500,18 +492,17 @@ extern void
 TextAnalysis(GString * gsz, const matchstate * pms, moverecord * pmr)
 {
 
-
-    char sz[1024];
-
     switch (pmr->mt) {
 
     case MOVE_NORMAL:
 
-        if (pmr->n.anMove[0] >= 0)
+        if (pmr->n.anMove[0] >= 0) {
+            char sz[FORMATEDMOVESIZE];
+
             g_string_append_printf(gsz,
                                    _("* %s moves %s"),
                                    ap[pmr->fPlayer].szName, FormatMove(sz, pms->anBoard, pmr->n.anMove));
-        else if (!pmr->ml.cMoves)
+        } else if (!pmr->ml.cMoves)
             g_string_append_printf(gsz, _("* %s cannot move"), ap[pmr->fPlayer].szName);
 
         g_string_append(gsz, "\n");
@@ -529,7 +520,7 @@ TextAnalysis(GString * gsz, const matchstate * pms, moverecord * pmr)
     case MOVE_DROP:
 
         if (pmr->mt == MOVE_DOUBLE)
-            g_string_append_printf(gsz, "* %s doubles\n\n", ap[pmr->fPlayer].szName);
+            g_string_append_printf(gsz, "* %s %s\n\n", ap[pmr->fPlayer].szName, _("doubles"));
         else
             g_string_append_printf(gsz,
                                    "* %s %s\n\n",
@@ -551,7 +542,7 @@ TextAnalysis(GString * gsz, const matchstate * pms, moverecord * pmr)
 static void
 TextDumpStatcontext(GString * gsz, const statcontext * psc, int nMatchTo)
 {
-    char sz[4096];
+    char sz[STATCONTEXT_MAXSIZE];
 
     DumpStatcontext(sz, psc, ap[0].szName, ap[1].szName, nMatchTo);
     g_string_append(gsz, sz);
@@ -584,10 +575,8 @@ TextMatchInfo(FILE * pf, const matchinfo * pmi)
 {
 
     int i;
-    char sz[80];
-    struct tm tmx;
 
-    fputs(_("Match Information:\n\n"), pf);
+    fputs(_("Match Information:\n"), pf);
 
     /* ratings */
 
@@ -600,6 +589,8 @@ TextMatchInfo(FILE * pf, const matchinfo * pmi)
     /* date */
 
     if (pmi->nYear) {
+        struct tm tmx;
+        char sz[80];
 
         tmx.tm_year = pmi->nYear - 1900;
         tmx.tm_mon = pmi->nMonth - 1;
@@ -633,7 +624,7 @@ TextMatchInfo(FILE * pf, const matchinfo * pmi)
 }
 
 /*
- * Export a game in HTML
+ * Export a game in text
  *
  * Input:
  *   pf: output file
@@ -655,7 +646,6 @@ ExportGameText(FILE * pf, listOLD * plGame, const int iGame, const int fLastGame
     xmovegameinfo *pmgi = NULL;
     GString *gsz;
     listOLD *pl_hint = NULL;
-    statcontext *psc_rel;
 
     msOrig.nMatchTo = 0;
 
@@ -684,8 +674,10 @@ ExportGameText(FILE * pf, listOLD * plGame, const int iGame, const int fLastGame
             fputs(gsz->str, pf);
             g_string_free(gsz, TRUE);
 
-            if (exsExport.fIncludeMatchInfo)
+            if (exsExport.fIncludeMatchInfo) {
                 TextMatchInfo(pf, &mi);
+                fputs("\n", pf);
+            }
 
             msOrig = msExport;
             pmgi = &pmr->g;
@@ -699,10 +691,8 @@ ExportGameText(FILE * pf, listOLD * plGame, const int iGame, const int fLastGame
 
         case MOVE_NORMAL:
 
-            if (pmr->fPlayer != msExport.fMove) {
+            if (pmr->fPlayer != msExport.fMove)
                 SwapSides(msExport.anBoard);
-                msExport.fMove = pmr->fPlayer;
-            }
 
             msExport.fTurn = msExport.fMove = pmr->fPlayer;
             msExport.anDice[0] = pmr->anDice[0];
@@ -777,6 +767,8 @@ ExportGameText(FILE * pf, listOLD * plGame, const int iGame, const int fLastGame
     }
 
     if (fLastGame) {
+        statcontext *psc_rel = relational_player_stats_get(ap[0].szName, ap[1].szName);
+
         gsz = g_string_new(NULL);
         if (msOrig.nMatchTo)
             g_string_append_printf(gsz, _("Match statistics\n\n"));
@@ -784,7 +776,6 @@ ExportGameText(FILE * pf, listOLD * plGame, const int iGame, const int fLastGame
             g_string_append_printf(gsz, _("Session statistics\n\n"));
         TextDumpStatcontext(gsz, &scTotal, msOrig.nMatchTo);
 
-        psc_rel = relational_player_stats_get(ap[0].szName, ap[1].szName);
         if (psc_rel) {
             g_string_append_printf(gsz, _("\nStatistics from database\n\n"));
             TextDumpStatcontext(gsz, psc_rel, 0);
@@ -802,6 +793,7 @@ CommandExportGameText(char *sz)
 {
 
     FILE *pf;
+    int fDontClose = FALSE;
 
     sz = NextToken(&sz);
 
@@ -811,23 +803,24 @@ CommandExportGameText(char *sz)
     }
 
     if (!sz || !*sz) {
-        outputl(_("You must specify a file to export to (see `help export " "game text')."));
+        outputl(_("You must specify a file to export to (see `help export game text')."));
         return;
     }
 
     if (!confirmOverwrite(sz, fConfirmSave))
         return;
 
-    if (!strcmp(sz, "-"))
+    if (!strcmp(sz, "-")) {
         pf = stdout;
-    else if ((pf = gnubg_g_fopen(sz, "w")) == 0) {
+        fDontClose = TRUE;
+    } else if ((pf = g_fopen(sz, "w")) == 0) {
         outputerr(sz);
         return;
     }
 
     ExportGameText(pf, plGame, getGameNumber(plGame), FALSE);
 
-    if (pf != stdout)
+    if (!fDontClose)
         fclose(pf);
 
     setDefaultFileName(sz);
@@ -840,13 +833,13 @@ CommandExportMatchText(char *sz)
     FILE *pf;
     listOLD *pl;
     int nGames;
-    char *szCurrent;
     int i;
+    int fDontClose = FALSE;
 
     sz = NextToken(&sz);
 
     if (!sz || !*sz) {
-        outputl(_("You must specify a file to export to (see `help export " "match text')."));
+        outputl(_("You must specify a file to export to (see `help export match text')."));
         return;
     }
 
@@ -855,7 +848,7 @@ CommandExportMatchText(char *sz)
 
     for (pl = lMatch.plNext, i = 0; pl != &lMatch; pl = pl->plNext, i++) {
 
-        szCurrent = filename_from_iGame(sz, i);
+        char *szCurrent = filename_from_iGame(sz, i);
 
         if (!i) {
 
@@ -869,9 +862,10 @@ CommandExportMatchText(char *sz)
         }
 
 
-        if (!strcmp(szCurrent, "-"))
+        if (!strcmp(szCurrent, "-")) {
             pf = stdout;
-        else if ((pf = gnubg_g_fopen(szCurrent, "w")) == 0) {
+            fDontClose = TRUE;
+        } else if ((pf = g_fopen(szCurrent, "w")) == 0) {
             outputerr(szCurrent);
             g_free(szCurrent);
             return;
@@ -879,7 +873,7 @@ CommandExportMatchText(char *sz)
 
         ExportGameText(pf, pl->p, i, i == nGames - 1);
 
-        if (pf != stdout)
+        if (!fDontClose)
             fclose(pf);
 
         g_free(szCurrent);
@@ -898,6 +892,7 @@ CommandExportPositionText(char *sz)
     moverecord *pmr;
     int iMove;
     GString *gsz;
+    int fDontClose = FALSE;
 
     sz = NextToken(&sz);
 
@@ -907,7 +902,7 @@ CommandExportPositionText(char *sz)
     }
 
     if (!sz || !*sz) {
-        outputl(_("You must specify a file to export to (see `help export " "position text')."));
+        outputl(_("You must specify a file to export to (see `help export position text')."));
         return;
     }
     pmr = get_current_moverecord(&fHistory);
@@ -915,9 +910,10 @@ CommandExportPositionText(char *sz)
     if (!confirmOverwrite(sz, fConfirmSave))
         return;
 
-    if (!strcmp(sz, "-"))
+    if (!strcmp(sz, "-")) {
         pf = stdout;
-    else if ((pf = gnubg_g_fopen(sz, "w")) == 0) {
+        fDontClose = TRUE;
+    } else if ((pf = g_fopen(sz, "w")) == 0) {
         outputerr(sz);
         return;
     }
@@ -959,7 +955,7 @@ CommandExportPositionText(char *sz)
 
     TextEpilogue(pf, &ms);
 
-    if (pf != stdout)
+    if (!fDontClose)
         fclose(pf);
 
     setDefaultFileName(sz);
