@@ -1,13 +1,11 @@
 /*
- * timer.c
- * by Jon Kinsey, 2003
+ * Copyright (C) 2003 Jon Kinsey <jonkinsey@gmail.com>
+ * Copyright (C) 2004-2019 the AUTHORS
  *
- * Accurate win32 timer or default timer
- *
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,18 +13,20 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: timer.c,v 1.20 2018/06/16 14:51:38 plm Exp $
+ * $Id: timer.c,v 1.22 2019/12/08 20:30:35 plm Exp $
  */
 
 #include "config.h"
 
 #include <time.h>
+
+#if !HAVE_CLOCK_GETTIME
 #if HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
+#endif
 
 #include "backgammon.h"
 
@@ -61,6 +61,17 @@ get_time()
     return timer.QuadPart / perFreq;
 }
 
+#elif HAVE_CLOCK_GETTIME
+
+extern double
+get_time(void)
+{                               /* Return elapsed time in milliseconds */
+    struct timespec ts;
+    clock_gettime(CLOCK_MONOTONIC, &ts);
+
+    return 1000.0 * (double) ts.tv_sec + 0.000001 * (double) ts.tv_nsec;
+}
+
 #else
 
 extern double
